@@ -60,7 +60,7 @@
  * 
  * MAXCODE and MAXABET constants are defined in config.h
  */   
-extern char  Alphabet[MAXCODE]; /* "ACDEFGHIKLMNPQRSTVWYBZX" for example */
+extern char  Alphabet[MAXCODE+1]; /* "ACDEFGHIKLMNPQRSTVWYBZX" for example */
 extern int   Alphabet_type;     /* hmmNUCLEIC or hmmAMINO                */
 extern int   Alphabet_size;     /* uniq alphabet size: 4 or 20           */
 extern int   Alphabet_iupac;    /* total size of alphabet + IUPAC degen. */
