@@ -12,10 +12,21 @@
 #include <limits.h>
 #include <float.h>
 #include <assert.h>
+#include <time.h>
 
 #include <unistd.h>	  
 
 
+#ifdef COMPILE_VERBOSITY
+#define COMPILE_VERBOSITY_FLAG 1
+#else
+#define COMPILE_VERBOSITY_FLAG 0
+#endif
+
+#define VERBOSITY_CHECK(level,verbosity) ((COMPILE_VERBOSITY_FLAG == 1)&(verbosity > level))
+
+
+#define PTHREAD
 /**** OK some system wide defines now - used all over the place ****/
 
 #define MAXLINE 512 /* generalised maximum input line */
@@ -24,8 +35,11 @@
 #ifndef BOOLEANDEFINED
 
 typedef int boolean;
+#ifndef TRUE
 #define TRUE 1
 #define FALSE 0
+#endif
+
 
 #define BOOLEANDEFINED
 #endif
@@ -46,6 +60,10 @@ typedef int boolean;
 #include "wiseoverlay.h"
 #include "commandline.h"
 
+/* stream wrappers */
+#include "wisestreaminterface.h"
+
+
 #ifdef WISE_MEMORY_WATCH
 #ifndef CKALLOC_GUARD
 #undef  ckalloc
@@ -53,5 +71,6 @@ typedef int boolean;
 #endif
 #endif
 
+#define error warn
 
 #endif /* TOP_BASE.H loaded */
