@@ -13,10 +13,10 @@ static boolean isinit = FALSE;
  *
  *
  */
-# line 17 "wiserandom.dy"
+# line 20 "wiserandom.dy"
 void init_random(void)
 {
-  srand48((long) time(NULL));
+  srandom((long) time(NULL));
   isinit = TRUE;
 }
 
@@ -32,7 +32,7 @@ void init_random(void)
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 28 "wiserandom.dy"
+# line 31 "wiserandom.dy"
 int random_integer(int l)
 {
   double rand;
@@ -52,19 +52,26 @@ int random_integer(int l)
  * Return [UNKN ]  Undocumented return value [double]
  *
  */
-# line 41 "wiserandom.dy"
+# line 44 "wiserandom.dy"
 double random_0_to_1(void)
 {
   double ret;
+  long rnd;
+
   if( isinit == FALSE)
     init_random();
   isinit = TRUE;
-  
-  ret = drand48(); 
+
+  rnd = random();
+
+  ret = (double)rnd / (double)LONG_MAX;
+
+/*  fprintf(stderr," got... %D %f\n",rnd,ret);*/
+
   return ret;
 }
 	
-# line 60 "wiserandom.c"
+# line 67 "wiserandom.c"
 
 #ifdef _cplusplus
 }
