@@ -1207,6 +1207,9 @@ APIfunc * APIfunc_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->fi = NULL;  
     out->only_C = FALSE; 
@@ -1229,6 +1232,7 @@ APIfunc * APIfunc_alloc(void)
  */
 APIfunc * free_APIfunc(APIfunc * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1237,10 +1241,18 @@ APIfunc * free_APIfunc(APIfunc * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->fi != NULL) 
@@ -1498,6 +1510,9 @@ APIObject * APIObject_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->destructor = NULL;  
     out->hard_linker = NULL; 
@@ -1524,6 +1539,7 @@ APIObject * APIObject_alloc(void)
  */
 APIObject * free_APIObject(APIObject * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1533,10 +1549,18 @@ APIObject * free_APIObject(APIObject * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->destructor != NULL) 
@@ -1970,6 +1994,9 @@ dynAPI * dynAPI_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->non_obj = NULL; 
     out->non_len = out->non_maxlen = 0;  
     out->obj = NULL; 
@@ -1993,6 +2020,7 @@ dynAPI * dynAPI_alloc(void)
  */
 dynAPI * free_dynAPI(dynAPI * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -2002,10 +2030,18 @@ dynAPI * free_dynAPI(dynAPI * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->non_obj != NULL)    {  
       for(i=0;i<obj->non_len;i++)    {  
         if( obj->non_obj[i] != NULL) 
