@@ -179,17 +179,20 @@ boolean write_thread_loop(DYNFILE * dfp,DPImplementation * dpi,GenericMatrix * g
   boolean tdb,qdb;
   char * pre;
 
+  int do_dummy = 0;
   tdb= qdb = FALSE;
+
+
   if( gm->qtype == NULL || gm->ttype == NULL ) {
     warn("No database types - must have dbtypes for pthreads support");
-    return FALSE;
+    do_dummy =1 ;
   }
 
 
   if( gm->qtype != NULL && gm->qtype->is_database == TRUE) 
     if( gm->qtype->is_thread_safe == FALSE) {
       warn("Query is a database but not using a thread safe library. Can't use it!");
-      return FALSE;
+      do_dummy = 1;
     } else {
       qdb = TRUE;
     }
@@ -197,11 +200,22 @@ boolean write_thread_loop(DYNFILE * dfp,DPImplementation * dpi,GenericMatrix * g
   if( gm->ttype != NULL && gm->ttype->is_database == TRUE) 
     if( gm->ttype->is_thread_safe == FALSE) {
       warn("Target is a database but not using a thread safe library. Can't use it!");
-      return FALSE;
+      do_dummy = 1;
     } else {
       tdb = TRUE;
     }
   
+
+  if( do_dummy == 1 ) {
+    fi = FuncInfo_named_from_varstr(FI_INTERNAL,"thread_loop_%s",gm->name);
+    add_line_to_Ftext(fi->ft,"dummy loop code foreach thread for %s",gm->name);
+    start_function_FuncInfo(fi,dfp,"void * thread_loop_%s(void * ptr)",gm->name);
+    expr(dfp,"fatal(\"dummy thread loop function\");");
+    close_function(dfp);
+    add_break(dfp);
+    return FALSE;
+  }
+
   
   fi = FuncInfo_named_from_varstr(FI_INTERNAL,"thread_loop_%s",gm->name);
   add_line_to_Ftext(fi->ft,"Infinite loop code foreach thread for %s",gm->name);
@@ -544,7 +558,7 @@ boolean write_thread_loop(DYNFILE * dfp,DPImplementation * dpi,GenericMatrix * g
  * Return [UNKN ]  allocated string (must free) with chained-args [char *]
  *
  */
-# line 531 "dbthread.dy"
+# line 545 "dbthread.dy"
 char * get_pre_chainstr_GenericMatrix(const char * pre,const GenericMatrix * gm)
 {
   char buffer[MAXLINE];
@@ -569,7 +583,7 @@ char * get_pre_chainstr_GenericMatrix(const char * pre,const GenericMatrix * gm)
       
     
 
-# line 565 "dbthread.c"
+# line 579 "dbthread.c"
 
 #ifdef _cplusplus
 }
