@@ -813,6 +813,9 @@ Aln2DisplayField * Aln2DisplayField_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->string = NULL;  
     out->length = NULL;  
     out->direction = NULL;   
@@ -840,6 +843,7 @@ Aln2DisplayField * Aln2DisplayField_alloc(void)
  */
 Aln2DisplayField * free_Aln2DisplayField(Aln2DisplayField * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -848,10 +852,18 @@ Aln2DisplayField * free_Aln2DisplayField(Aln2DisplayField * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->string != NULL) 
       ckfree(obj->string);   
     if( obj->length != NULL) 
@@ -909,6 +921,9 @@ Index2Display * Index2Display_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->eval = NULL;    
     out->is_start = FALSE;   
@@ -932,6 +947,7 @@ Index2Display * Index2Display_alloc(void)
  */
 Index2Display * free_Index2Display(Index2Display * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -940,10 +956,18 @@ Index2Display * free_Index2Display(Index2Display * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->eval != NULL)   
@@ -1364,6 +1388,9 @@ Label2Display * Label2Display_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->label = NULL;   
     out->direction = NULL;   
     out->a2df = NULL;    
@@ -1390,6 +1417,7 @@ Label2Display * Label2Display_alloc(void)
  */
 Label2Display * free_Label2Display(Label2Display * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1399,10 +1427,18 @@ Label2Display * free_Label2Display(Label2Display * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->label != NULL)  
       ckfree(obj->label);    
     if( obj->direction != NULL)  
@@ -1837,6 +1873,9 @@ Sequence2Display * Sequence2Display_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->number = 0; 
     out->l2d = NULL; 
     out->len = out->maxlen = 0;  
@@ -1865,6 +1904,7 @@ Sequence2Display * Sequence2Display_alloc(void)
  */
 Sequence2Display * free_Sequence2Display(Sequence2Display * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1874,10 +1914,18 @@ Sequence2Display * free_Sequence2Display(Sequence2Display * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->l2d != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->l2d[i] != NULL) 
@@ -1945,6 +1993,9 @@ Aln2DisplayResource * Aln2DisplayResource_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->type = NULL;    
     out->name = NULL;    
     out->arg = NULL; 
@@ -1967,6 +2018,7 @@ Aln2DisplayResource * Aln2DisplayResource_alloc(void)
  */
 Aln2DisplayResource * free_Aln2DisplayResource(Aln2DisplayResource * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1975,10 +2027,18 @@ Aln2DisplayResource * free_Aln2DisplayResource(Aln2DisplayResource * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->type != NULL)   
       ckfree(obj->type);     
     if( obj->name != NULL)   
@@ -2401,6 +2461,9 @@ Aln2Display * Aln2Display_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->a2dr = NULL;    
     out->res_len = out->res_maxlen = 0;  
@@ -2426,6 +2489,7 @@ Aln2Display * Aln2Display_alloc(void)
  */
 Aln2Display * free_Aln2Display(Aln2Display * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -2435,10 +2499,18 @@ Aln2Display * free_Aln2Display(Aln2Display * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->a2dr != NULL)   {  
