@@ -41,6 +41,9 @@ extern "C" {
 
 struct Aln2DisplayField {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * string;   
     char * length;   
     char * direction;    
@@ -59,6 +62,9 @@ typedef struct Aln2DisplayField Aln2DisplayField;
 
 struct Index2Display {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * name;     
     char * eval;     
     boolean is_start;    
@@ -73,6 +79,9 @@ typedef struct Index2Display Index2Display;
 
 struct Label2Display {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * label;    
     char * direction;    
     Aln2DisplayField ** a2df;    
@@ -92,6 +101,9 @@ typedef struct Label2Display Label2Display;
 
 struct Sequence2Display {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     int number;  
     Label2Display ** l2d;    
     int len;/* len for above l2d  */ 
@@ -113,6 +125,9 @@ typedef struct Sequence2Display Sequence2Display;
 
 struct Aln2DisplayResource {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * type;     
     char * name;     
     char * arg;  
@@ -126,6 +141,9 @@ typedef struct Aln2DisplayResource Aln2DisplayResource;
 
 struct Aln2Display {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * name;     
     Aln2DisplayResource ** a2dr;     
     int res_len;/* len for above a2dr  */ 
