@@ -548,6 +548,9 @@ OmState * OmState_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->type = OmState_TYPE_UNKNOWN;    
 
@@ -569,6 +572,7 @@ OmState * OmState_alloc(void)
  */
 OmState * free_OmState(OmState * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -577,10 +581,18 @@ OmState * free_OmState(OmState * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
 
@@ -632,6 +644,9 @@ OmUnit * OmUnit_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->c_string = NULL;    
     out->base = 0;   
     out->length = 0; 
@@ -655,6 +670,7 @@ OmUnit * OmUnit_alloc(void)
  */
 OmUnit * free_OmUnit(OmUnit * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -663,10 +679,18 @@ OmUnit * free_OmUnit(OmUnit * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->c_string != NULL)   
       ckfree(obj->c_string);     
 
@@ -718,6 +742,9 @@ OmTrans * OmTrans_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->dx = 0; 
     out->dy = 0; 
     out->t_prf = -1; 
@@ -750,6 +777,7 @@ OmTrans * OmTrans_alloc(void)
  */
 OmTrans * free_OmTrans(OmTrans * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -758,10 +786,18 @@ OmTrans * free_OmTrans(OmTrans * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     /* obj->from is linked in */ 
     /* obj->to is linked in */ 
     if( obj->tprf_unit != NULL)  
@@ -817,6 +853,9 @@ OmTransFunc * OmTransFunc_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->me = NULL;  
 
@@ -838,6 +877,7 @@ OmTransFunc * OmTransFunc_alloc(void)
  */
 OmTransFunc * free_OmTransFunc(OmTransFunc * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -846,10 +886,18 @@ OmTransFunc * free_OmTransFunc(OmTransFunc * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->me != NULL) 
@@ -1433,6 +1481,9 @@ OneModel * OneModel_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->state_num = 0;  
     out->semistate_num = 0;  
@@ -1463,6 +1514,7 @@ OneModel * OneModel_alloc(void)
  */
 OneModel * free_OneModel(OneModel * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1472,10 +1524,18 @@ OneModel * free_OneModel(OneModel * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->trans != NULL)  {  
