@@ -13,156 +13,7 @@ extern "C" {
 #define CellStateLISTLENGTH  64
 #define GenericMatrixLISTLENGTH 64
 #define CellExprLISTLENGTH 64
-
-
-/***** RCS Info *****************************************/
-/*
-   $Id: dyna2.dy,v 1.2 2001/09/04 14:57:35 birney Exp $
-
-   $Log: dyna2.dy,v $
-   Revision 1.2  2001/09/04 14:57:35  birney
-   monkeying around with the dyc compiler. It is not pretty down here. debugging support just about ready for serious playing
-
-   Revision 1.1.1.1  2001/06/18 13:59:55  birney
-   moved wise2 to ensembl cvs repository
-
-   Revision 1.6  1999/05/10 14:29:02  birney
-   lots of additions
-
-   Revision 1.5  1999/05/03 14:52:55  birney
-   added in alot of fixes to dyanmite.
-
-   added in def boolean's in commandline
-
-   added in seqalign/pwmdna stuff and linked it to hmmer bridge.
-
-   Revision 1.4  1999/01/20 14:42:08  birney
-   serious change of the dynamite compiler - lots of stuff gone...
-
-   Revision 1.3  1998/11/19 09:12:31  birney
-   changed dyc so that non threaded applications are not written...
-
-# Revision 1.2  1998/09/08  13:26:32  birney
-# added protection to maxlen db searching code
-#
-# Revision 1.1.1.1  1998/08/28  09:30:56  birney
-# Wise2
-#
- * Revision 1.34  1998/07/27  11:39:36  birney
- * before compugen merge
- *
- * Revision 1.33  1997/08/02  14:38:04  birney
- * fixed parse error problems (chaining back to calling function)
- *
- * Revision 1.32  1997/07/24  15:38:33  birney
- * added SOURCE_POS for !top etc
- * also some documentation
- *
- * Revision 1.31  1997/07/11  10:54:41  birney
- * Many things:
- *   strengthened parsing errors
- *   made typed system in particular logical->real mapping
- *   removed search structure
- *
- * Revision 1.30  1997/06/16  16:52:05  birney
- * moved some things over to using get_watched string. Got parser (yacc
- * styke) in
- *
- * Revision 1.29  1997/06/10  13:55:37  birney
- * Lots of things: moved to teh dynfile system of keeping files around, addedfunction information, started to remodelled the matrix part
- *
- * Revision 1.28  1997/04/23  17:15:45  birney
- * added CDNA line as a valid logical type
- *
- * Revision 1.27  1997/03/09  15:11:45  birney
- * added CKS checks for warning messages
- *
- * Revision 1.26  1996/12/06  08:59:11  birney
- * fixed logical protein work to ComplexSequence
- *
- * Revision 1.25  1996/11/24  18:56:34  birney
- * added start/end extract systems.
- *
- * Revision 1.24  1996/11/21  18:38:09  birney
- * added collapsable labels
- *
- * Revision 1.23  1996/11/12  16:43:21  birney
- * checked some of hte linear functions stuff...
- *
- * Revision 1.22  1996/11/12  13:56:52  birney
- * removed trailing comma on enum list
- *
- * Revision 1.21  1996/11/11  22:18:58  birney
- * added support for atomistic representation of dynamic
- * programming matrices.
- *
- * Revision 1.20  1996/11/10  12:17:17  birney
- * started to split lines. I need to resculpt the CellExpr part
- * of Dynamite though...
- *
- * Revision 1.19  1996/11/08  13:52:35  birney
- * changed stacking functions
- *
- * Revision 1.18  1996/10/28  17:50:49  birney
- * added specialtospecial handling
- *
- * Revision 1.17  1996/10/28  10:53:39  birney
- * added genericmatrix specialtospecial tag
- *
- * Revision 1.16  1996/10/25  17:19:51  birney
- * added source offset checker for 0,0 offsets and other nasties
- *
- * Revision 1.15  1996/10/21  16:02:41  birney
- * slightly better logical type management, but still not good at all
- *
- * Revision 1.14  1996/10/20  18:00:16  birney
- * added matrix/state andsource stack messages
- *
- * Revision 1.13  1996/10/18  14:31:07  birney
- * removed offending alnholder definition
- *
- * Revision 1.12  1996/10/18  10:40:40  birney
- * added footprints as both +ve and -ve
- *
- * Revision 1.11  1996/10/09  14:40:10  birney
- * added transition label support: reads them in,
- * propagates state defaults and checks if full labels
- * are there....
- *
- * Revision 1.10  1996/10/09  09:37:31  birney
- * changed indentation... lost log_full_error to warn
- *
- * Revision 1.9  1996/05/18  13:32:27  birney
- * added basic tests for "Search ability"
- *
- * Revision 1.8  1996/05/04  19:52:32  birney
- * added footprint sutff
- *
- * Revision 1.7  1996/04/28  16:37:12  birney
- * variable structure moved to an independant file
- *
- * Revision 1.6  1996/04/27  11:54:56  birney
- * added basic MatrixVariable support ---
- *
- * Revision 1.5  1996/04/24  16:35:54  birney
- * revised search structure, still have to deal with calc lines
- *
- * Revision 1.4  1996/04/23  14:50:18  birney
- * added a (faulty) search structure
- *
- * Revision 1.3  1996/03/19  18:00:19  birney
- * added some special to special issues
- * ,
- *
- * Revision 1.2  1996/03/03  15:18:38  birney
- * compiles cleanly for 3.x
- *
- * Revision 1.1  1996/03/03  15:03:36  birney
- * Initial revision
- *
-
-*/
-/********************************************************/
+#define CellSignatureSetLISTLENGTH 64
 
 
 #define STANDARDGENERICMATRIX   231
@@ -199,8 +50,42 @@ enum source_positon {
 #define SOURCE_BOTTOM_BIT 4
 #define SOURCE_RIGHT_BIT  8
 
+struct CellSignature {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int offi;    
+    int offj;    
+    } ;  
+/* CellSignature defined */ 
+#ifndef DYNAMITE_DEFINED_CellSignature
+typedef struct CellSignature CellSignature;
+#define DYNAMITE_DEFINED_CellSignature
+#endif
+
+
+struct CellSignatureSet {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    CellSignature ** sig;    
+    int len;/* len for above sig  */ 
+    int maxlen; /* maxlen for above sig */ 
+    } ;  
+/* CellSignatureSet defined */ 
+#ifndef DYNAMITE_DEFINED_CellSignatureSet
+typedef struct CellSignatureSet CellSignatureSet;
+#define DYNAMITE_DEFINED_CellSignatureSet
+#endif
+
+
 struct CellSource {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * state_source;     
     int offi;    
     int offj;    
@@ -212,6 +97,7 @@ struct CellSource {
     char * target_label;     
     int  position;   
     int trans_no;   /*  unique number for this transition */ 
+    int from_state_no;   
     } ;  
 /* CellSource defined */ 
 #ifndef DYNAMITE_DEFINED_CellSource
@@ -222,6 +108,9 @@ typedef struct CellSource CellSource;
 
 struct CellState {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * name;     
     char * def_score;    
     char * calc_expr;    
@@ -244,6 +133,7 @@ struct CellState {
     char * target_label;     
     int position;    
     ExprTree * etr;  
+    int state_number;    
     } ;  
 /* CellState defined */ 
 #ifndef DYNAMITE_DEFINED_CellState
@@ -254,6 +144,9 @@ typedef struct CellState CellState;
 
 struct CollapsableLabel {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * query;    
     char * target;   
     } ;  
@@ -266,6 +159,9 @@ typedef struct CollapsableLabel CollapsableLabel;
 
 struct ExternVariable {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * name;     
     char * type;     
     } ;  
@@ -278,6 +174,9 @@ typedef struct ExternVariable ExternVariable;
 
 struct GenericMatrix {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * name;     
     int type;    
     CellState ** state;  
@@ -409,6 +308,140 @@ boolean check_source_positions(GenericMatrix * gm);
 ExternVariable * read_ExternVariable_line(char * line);
 
 
+/* Function:  hard_link_CellSignature(obj)
+ *
+ * Descrip:    Bumps up the reference count of the object
+ *             Meaning that multiple pointers can 'own' it
+ *
+ *
+ * Arg:        obj [UNKN ] Object to be hard linked [CellSignature *]
+ *
+ * Return [UNKN ]  Undocumented return value [CellSignature *]
+ *
+ */
+CellSignature * hard_link_CellSignature(CellSignature * obj);
+
+
+/* Function:  CellSignature_alloc(void)
+ *
+ * Descrip:    Allocates structure: assigns defaults if given 
+ *
+ *
+ *
+ * Return [UNKN ]  Undocumented return value [CellSignature *]
+ *
+ */
+CellSignature * CellSignature_alloc(void);
+
+
+/* Function:  free_CellSignature(obj)
+ *
+ * Descrip:    Free Function: removes the memory held by obj
+ *             Will chain up to owned members and clear all lists
+ *
+ *
+ * Arg:        obj [UNKN ] Object that is free'd [CellSignature *]
+ *
+ * Return [UNKN ]  Undocumented return value [CellSignature *]
+ *
+ */
+CellSignature * free_CellSignature(CellSignature * obj);
+
+
+/* Function:  add_CellSignatureSet(obj,add)
+ *
+ * Descrip:    Adds another object to the list. It will expand the list if necessary
+ *
+ *
+ * Arg:        obj [UNKN ] Object which contains the list [CellSignatureSet *]
+ * Arg:        add [OWNER] Object to add to the list [CellSignature *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean add_CellSignatureSet(CellSignatureSet * obj,CellSignature * add);
+
+
+/* Function:  flush_CellSignatureSet(obj)
+ *
+ * Descrip:    Frees the list elements, sets length to 0
+ *             If you want to save some elements, use hard_link_xxx
+ *             to protect them from being actually destroyed in the free
+ *
+ *
+ * Arg:        obj [UNKN ] Object which contains the list  [CellSignatureSet *]
+ *
+ * Return [UNKN ]  Undocumented return value [int]
+ *
+ */
+int flush_CellSignatureSet(CellSignatureSet * obj);
+
+
+/* Function:  CellSignatureSet_alloc_std(void)
+ *
+ * Descrip:    Equivalent to CellSignatureSet_alloc_len(CellSignatureSetLISTLENGTH)
+ *
+ *
+ *
+ * Return [UNKN ]  Undocumented return value [CellSignatureSet *]
+ *
+ */
+CellSignatureSet * CellSignatureSet_alloc_std(void);
+
+
+/* Function:  CellSignatureSet_alloc_len(len)
+ *
+ * Descrip:    Allocates len length to all lists
+ *
+ *
+ * Arg:        len [UNKN ] Length of lists to allocate [int]
+ *
+ * Return [UNKN ]  Undocumented return value [CellSignatureSet *]
+ *
+ */
+CellSignatureSet * CellSignatureSet_alloc_len(int len);
+
+
+/* Function:  hard_link_CellSignatureSet(obj)
+ *
+ * Descrip:    Bumps up the reference count of the object
+ *             Meaning that multiple pointers can 'own' it
+ *
+ *
+ * Arg:        obj [UNKN ] Object to be hard linked [CellSignatureSet *]
+ *
+ * Return [UNKN ]  Undocumented return value [CellSignatureSet *]
+ *
+ */
+CellSignatureSet * hard_link_CellSignatureSet(CellSignatureSet * obj);
+
+
+/* Function:  CellSignatureSet_alloc(void)
+ *
+ * Descrip:    Allocates structure: assigns defaults if given 
+ *
+ *
+ *
+ * Return [UNKN ]  Undocumented return value [CellSignatureSet *]
+ *
+ */
+CellSignatureSet * CellSignatureSet_alloc(void);
+
+
+/* Function:  free_CellSignatureSet(obj)
+ *
+ * Descrip:    Free Function: removes the memory held by obj
+ *             Will chain up to owned members and clear all lists
+ *
+ *
+ * Arg:        obj [UNKN ] Object that is free'd [CellSignatureSet *]
+ *
+ * Return [UNKN ]  Undocumented return value [CellSignatureSet *]
+ *
+ */
+CellSignatureSet * free_CellSignatureSet(CellSignatureSet * obj);
+
+
 /* Function:  hard_link_CellSource(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -835,6 +868,7 @@ GenericMatrix * free_GenericMatrix(GenericMatrix * obj);
 
   /* Unplaced functions */
   /* There has been no indication of the use of these functions */
+CellSignatureSet * CellSignatureSet_from_GenericMatrix(GenericMatrix * gm);
 boolean can_do_threads(GenericMatrix * gm);
 CellState * CellState_from_str(GenericMatrix * gm,char * str);
 boolean check_cell_refs(GenericMatrix * gm);
@@ -870,6 +904,10 @@ ParseError parse_calc_line_GenericMatrix(GenericMatrix * gm,MethodTypeSet * mts,
 boolean calc_window(GenericMatrix * gm);
 boolean calc_footprint(GenericMatrix * gm);
 boolean cross_reference_state_and_source(GenericMatrix * gm);
+void swap_CellSignatureSet(CellSignature ** list,int i,int j) ;
+void qsort_CellSignatureSet(CellSignature ** list,int left,int right,int (*comp)(CellSignature * ,CellSignature * ));
+void sort_CellSignatureSet(CellSignatureSet * obj,int (*comp)(CellSignature *, CellSignature *));
+boolean expand_CellSignatureSet(CellSignatureSet * obj,int len);
 void swap_CellState(CellSource ** list,int i,int j) ;
 void qsort_CellState(CellSource ** list,int left,int right,int (*comp)(CellSource * ,CellSource * ));
 void sort_CellState(CellState * obj,int (*comp)(CellSource *, CellSource *));
