@@ -38,7 +38,7 @@ void one_shot_aln_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation * dpi)
   /** started function, now see what basematrix says about memory **/
 
 
-  expr(dfp,"int total;");
+  expr(dfp,"long long total;");
   expr(dfp,"%s * mat;",gm->name);
   expr(dfp,"PackAln * out;");
   expr(dfp,"DebugMatrix * de");
@@ -66,6 +66,27 @@ void one_shot_aln_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation * dpi)
   closebrace(dfp);
   add_break(dfp);
 
+  expr(dfp,"if( dpenv != NULL )");
+  startbrace(dfp);
+   expr(dfp,"if( strategy == DPIM_Explicit)");
+   startbrace(dfp);
+    expr(dfp,"if( (mat=allocate_Expl_%s(%s,dpri)) == NULL )",gm->name,chainstr);
+    startbrace(dfp);
+    warn_expr(dfp,"Unable to allocate large %s version",gm->name);
+    expr(dfp,"return NULL;");
+    closebrace(dfp);
+    expr(dfp,"calculate_dpenv_%s(mat,dpenv)",gm->name);   
+    expr(dfp,"out =  PackAln_read_Expl_%s(mat);",gm->name);
+   closebrace(dfp);
+   expr(dfp,"else");
+   startbrace(dfp);
+   expr(dfp,"mat = allocate_%s_only(%s)",gm->name,chainstr);
+   expr(dfp,"calculate_shatter_%s(mat,dpenv)",gm->name);
+   expr(dfp,"out = PackAln_read_Shatter_%s(mat)",gm->name);
+   closebrace(dfp);
+  closebrace(dfp);
+  expr(dfp,"else");
+  startbrace(dfp);
   expr(dfp,"if( strategy == DPIM_Linear )",gm->len);
   startbrace(dfp);
   add_block_comment(dfp,"use small implementation");
@@ -79,7 +100,7 @@ void one_shot_aln_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation * dpi)
   expr(dfp,"else");
   startbrace(dfp);
   add_block_comment(dfp,"use Large implementation");
-  expr(dfp,"if( (mat=allocate_Expl_%s(%s)) == NULL )",gm->name,chainstr);
+  expr(dfp,"if( (mat=allocate_Expl_%s(%s,dpri)) == NULL )",gm->name,chainstr);
   startbrace(dfp);
   warn_expr(dfp,"Unable to allocate large %s version",gm->name);
   expr(dfp,"return NULL;");
@@ -90,16 +111,24 @@ void one_shot_aln_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation * dpi)
     expr(dfp,"de = create_debug_%s();",gm->name);
     expr(dfp,"de->matrix = (void*) mat;");
     expr(dfp,"explicit_debug_%s(de,0);",gm->name);
-    expr(dfp,"free_DebugMatrix(de)");
+    expr(dfp,"out =  PackAln_read_Expl_%s(mat);",gm->name);
+
+    expr(dfp,"if( dpri->paldebug == TRUE ) ");
+    hang_expr(dfp,"show_PackAln_Debug(de,out,stdout);");
+    expr(dfp,"free_DebugMatrix(de);");
   } else {
     expr(dfp,"fatal(\"Asked for dydebug, but dynamite file not compiled with -g. Need to recompile dynamite source\");");
   }
   closebrace(dfp);
   expr(dfp,"else");
-  hang_expr(dfp,"calculate_%s(mat)",gm->name);   
+  startbrace(dfp);
+  expr(dfp,"calculate_%s(mat)",gm->name);   
   expr(dfp,"out =  PackAln_read_Expl_%s(mat);",gm->name);
   closebrace(dfp);
-  
+
+  closebrace(dfp);
+  closebrace(dfp);
+
   add_break(dfp);
 
   expr(dfp,"mat = free_%s(mat)",gm->name);
@@ -123,7 +152,7 @@ void one_shot_aln_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation * dpi)
  * Arg:         gm [UNKN ] Undocumented argument [const GenericMatrix *]
  *
  */
-# line 136 "dynafunc.dy"
+# line 165 "dynafunc.dy"
 void recalculate_PackAln_func(DYNFILE * dfp,const GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -228,7 +257,7 @@ void recalculate_PackAln_func(DYNFILE * dfp,const GenericMatrix * gm)
  * Arg:         gm [READ ] generic matrix structure [const GenericMatrix *]
  *
  */
-# line 237 "dynafunc.dy"
+# line 266 "dynafunc.dy"
 void alloc_expl_func_GenericMatrix(DYNFILE * dfp,const GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -247,8 +276,11 @@ void alloc_expl_func_GenericMatrix(DYNFILE * dfp,const GenericMatrix * gm)
   arg_str = get_argstr_GenericMatrix(gm);
   add_args_GenericMatrix_FuncInfo(fi,gm);
 
+  
+
+
+  start_function_FuncInfo(fi,dfp,"%s * allocate_Expl_%s(%s,DPRunImpl * dpri)",gm->name,gm->name,arg_str);
 
-  start_function_FuncInfo(fi,dfp,"%s * allocate_Expl_%s(%s)",gm->name,gm->name,arg_str);
 
   /*** clean up ***/
   ckfree(arg_str);
@@ -267,13 +299,30 @@ void alloc_expl_func_GenericMatrix(DYNFILE * dfp,const GenericMatrix * gm)
 
   expr(dfp,"if( out == NULL )");
   hang_expr(dfp,"return NULL;");
-  
+
+  expr(dfp,"if( dpri->should_cache == TRUE )");
+  startbrace(dfp);
+  expr(dfp,"if( dpri->cache != NULL )");
+  startbrace(dfp);
+  expr(dfp,"if( dpri->cache->maxleni >= (out->lenj+%d)*%d && dpri->cache->maxlenj >= (out->leni+%d))",gm->window_j,gm->len,gm->window_i);
+  hang_expr(dfp,"out->basematrix = hard_link_BaseMatrix(dpri->cache)");
+  expr(dfp,"else");
+  hang_expr(dfp,"dpri->cache = free_BaseMatrix(dpri->cache)");
+  closebrace(dfp);
+  closebrace(dfp);
+
+  expr(dfp,"if( out->basematrix == NULL )");
+  startbrace(dfp);
   expr(dfp,"if( (out->basematrix = BaseMatrix_alloc_matrix_and_specials((out->lenj+%d)*%d,(out->leni+%d),%d,out->lenj+%d)) == NULL)",gm->window_j,gm->len,gm->window_i,gm->spec_len,gm->window_j);
   startbrace(dfp);
   expr(dfp,"warn(\"Explicit matrix %s cannot be allocated, (asking for %%d by %%d main cells)\",out->leni,out->lenj);",gm->name);
   expr(dfp,"free_%s(out)",gm->name);
   expr(dfp,"return NULL;");
   closebrace(dfp);
+  closebrace(dfp);
+
+  expr(dfp,"if( dpri->should_cache == TRUE && dpri->cache == NULL)");
+  hang_expr(dfp,"dpri->cache = hard_link_BaseMatrix(out->basematrix)");
 
   expr(dfp,"out->basematrix->type = BASEMATRIX_TYPE_EXPLICIT;");
   expr(dfp,"init_%s(out)",gm->name);
@@ -303,7 +352,7 @@ void alloc_expl_func_GenericMatrix(DYNFILE * dfp,const GenericMatrix * gm)
  * Arg:         gm [UNKN ] generic matrix structure [GenericMatrix *]
  *
  */
-# line 308 "dynafunc.dy"
+# line 357 "dynafunc.dy"
 void write_safe_alloc_function(DYNFILE * dfp,GenericMatrix * gm)
 {
   int i;
@@ -367,7 +416,7 @@ void write_safe_alloc_function(DYNFILE * dfp,GenericMatrix * gm)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 366 "dynafunc.dy"
+# line 415 "dynafunc.dy"
 boolean add_args_GenericMatrix_FuncInfo(FuncInfo * fi,const GenericMatrix * gm)
 {
   int i;
@@ -404,7 +453,7 @@ boolean add_args_GenericMatrix_FuncInfo(FuncInfo * fi,const GenericMatrix * gm)
  * Return [UNKN ]  allocated string (must free) with chained-args [char *]
  *
  */
-# line 398 "dynafunc.dy"
+# line 447 "dynafunc.dy"
 char * get_chainstr_GenericMatrix(const GenericMatrix * gm)
 {
   char buffer[MAXLINE];
@@ -435,7 +484,7 @@ char * get_chainstr_GenericMatrix(const GenericMatrix * gm)
  * Return [UNKN ]  allocated string (must free) with args [char *]
  *
  */
-# line 424 "dynafunc.dy"
+# line 473 "dynafunc.dy"
 char * get_argstr_GenericMatrix(const GenericMatrix * gm)
 {
   char buffer[MAXLINE];
@@ -453,7 +502,7 @@ char * get_argstr_GenericMatrix(const GenericMatrix * gm)
   return stringalloc(buffer);
 }
  
-# line 441 "dynafunc.dy"
+# line 490 "dynafunc.dy"
 void write_dotter_dump(DYNFILE * dfp,GenericMatrix * gm)
 {
 
@@ -511,7 +560,7 @@ gm->query->name,gm->query_len,gm->query->name,gm->query_len,gm->target->name,gm-
 }
   
 
-# line 498 "dynafunc.dy"
+# line 547 "dynafunc.dy"
 void write_memory_macros(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int i;
@@ -529,7 +578,7 @@ void write_memory_macros(DYNFILE * dfp,GenericMatrix * gm)
   add_break(dfp);
 }
 
-# line 515 "dynafunc.dy"
+# line 564 "dynafunc.dy"
 void write_search_macros(DYNFILE * dfp,const GenericMatrix * gm,DPImplementation * dpi)
 {
 
@@ -550,7 +599,48 @@ void write_search_macros(DYNFILE * dfp,const GenericMatrix * gm,DPImplementation
   add_break(dfp);
 }
 
-# line 535 "dynafunc.dy"
+# line 584 "dynafunc.dy"
+void write_expl_access_funcs(DYNFILE * dfp,GenericMatrix * gm)
+{
+  FuncInfo * fi;
+
+  fi = FuncInfo_named_from_varstr(FI_INTERNAL,"%s_explicit_access_main",gm->name);
+  start_function_FuncInfo(fi,dfp,"int %s_explicit_access_main(%s * mat,int i,int j,int state)",gm->name,gm->name);
+  expr(dfp,"return %s_EXPL_MATRIX(mat,i,j,state);",gm->name);
+  close_function(dfp);
+  add_break(dfp);
+
+  fi = FuncInfo_named_from_varstr(FI_INTERNAL,"%s_explicit_access_special",gm->name);
+  start_function_FuncInfo(fi,dfp,"int %s_explicit_access_special(%s * mat,int i,int j,int state)",gm->name,gm->name);
+  expr(dfp,"return %s_EXPL_SPECIAL(mat,i,j,state);",gm->name);
+  close_function(dfp);
+  add_break(dfp);
+}
+  
+
+# line 602 "dynafunc.dy"
+void write_expl_read_func(DYNFILE * dfp,GenericMatrix * gm)
+{
+
+  FuncInfo * fi;
+
+  fi = FuncInfo_named_from_varstr(FI_CALLABLE,"PackAln_read_Expl_%s",gm->name);
+  add_line_to_Ftext(fi->ft,"Reads off PackAln from explicit matrix structure",gm->name);
+
+  start_function_FuncInfo(fi,dfp,"PackAln * PackAln_read_Expl_%s(%s * mat)",gm->name,gm->name);
+  expr(dfp,"%s_access_func_holder holder",gm->name);
+  add_break(dfp);
+  expr(dfp,"holder.access_main    = %s_explicit_access_main;",gm->name);
+  expr(dfp,"holder.access_special = %s_explicit_access_special;",gm->name);
+
+  expr(dfp,"return PackAln_read_generic_%s(mat,holder);",gm->name);
+ 
+  close_function(dfp);
+  add_break(dfp);
+}
+
+
+# line 623 "dynafunc.dy"
 void write_basic_read_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int i;
@@ -566,10 +656,10 @@ void write_basic_read_func(DYNFILE * dfp,GenericMatrix * gm)
   
   st = i;
 
-  fi = FuncInfo_named_from_varstr(FI_CALLABLE,"PackAln_read_Expl_%s",gm->name);
+  fi = FuncInfo_named_from_varstr(FI_CALLABLE,"PackAln_read_generic_%s",gm->name);
   add_line_to_Ftext(fi->ft,"Reads off PackAln from explicit matrix structure",gm->name);
 
-  start_function_FuncInfo(fi,dfp,"PackAln * PackAln_read_Expl_%s(%s * mat)",gm->name,gm->name);
+  start_function_FuncInfo(fi,dfp,"PackAln * PackAln_read_generic_%s(%s * mat,%s_access_func_holder h)",gm->name,gm->name,gm->name);
   
   expr(dfp,"register PackAln * out;");
   expr(dfp,"int i;");
@@ -583,11 +673,9 @@ void write_basic_read_func(DYNFILE * dfp,GenericMatrix * gm)
   
   add_break(dfp);
   
-  expr(dfp,"if( mat->basematrix->type != BASEMATRIX_TYPE_EXPLICIT)");
-  startbrace(dfp);
-  expr(dfp,"warn(\"In %s_basic_read You have asked for an alignment from a non-explicit matrix: c'est impossible [current type is %%d - %%s]\", mat->basematrix->type,basematrix_type_to_string(mat->basematrix->type));",gm->name);
-  expr(dfp,"return NULL");
-  closebrace(dfp);
+  expr(dfp,"assert(mat)");
+  expr(dfp,"assert(h.access_main)");
+  expr(dfp,"assert(h.access_special)");
   
   add_break(dfp);
   expr(dfp,"out = PackAln_alloc_std()");
@@ -598,7 +686,7 @@ void write_basic_read_func(DYNFILE * dfp,GenericMatrix * gm)
   
   /*** Find the end of the matrix ***/
   
-  expr(dfp,"out->score =  find_end_%s(mat,&i,&j,&state,&isspecial);",gm->name);
+  expr(dfp,"out->score =  find_end_%s(mat,&i,&j,&state,&isspecial,h);",gm->name);
   
   add_break(dfp);
   
@@ -632,9 +720,9 @@ void write_basic_read_func(DYNFILE * dfp,GenericMatrix * gm)
   
   add_break(dfp);
   expr(dfp,"if( isspecial == TRUE )");		
-  hang_expr(dfp,"max_calc_special_%s(mat,i,j,state,isspecial,&i,&j,&state,&isspecial,&cellscore)",gm->name);
+  hang_expr(dfp,"max_calc_special_%s(mat,i,j,state,isspecial,&i,&j,&state,&isspecial,&cellscore,h)",gm->name);
   expr(dfp,"else");
-  hang_expr(dfp,"max_calc_%s(mat,i,j,state,isspecial,&i,&j,&state,&isspecial,&cellscore)",gm->name);	
+  hang_expr(dfp,"max_calc_%s(mat,i,j,state,isspecial,&i,&j,&state,&isspecial,&cellscore,h)",gm->name);	
 
   
   /*** check we haven't boobied ***/
@@ -681,7 +769,7 @@ void write_basic_read_func(DYNFILE * dfp,GenericMatrix * gm)
   add_break(dfp);
 }
 
-# line 665 "dynafunc.dy"
+# line 751 "dynafunc.dy"
 void write_pal_to_ars_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -733,7 +821,7 @@ void write_pal_to_ars_func(DYNFILE * dfp,GenericMatrix * gm)
 }
 
 
-# line 716 "dynafunc.dy"
+# line 802 "dynafunc.dy"
 void write_alncconvert_make_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   int i;
@@ -839,7 +927,7 @@ void write_alncconvert_make_func(DYNFILE * dfp,GenericMatrix * gm)
 
 }
 
-# line 821 "dynafunc.dy"
+# line 907 "dynafunc.dy"
 void write_aln_conversion_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -871,7 +959,7 @@ void write_aln_conversion_func(DYNFILE * dfp,GenericMatrix * gm)
  * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
  *
  */
-# line 846 "dynafunc.dy"
+# line 932 "dynafunc.dy"
 void init_matrix_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -933,7 +1021,7 @@ void init_matrix_func(DYNFILE * dfp,GenericMatrix * gm)
 }
        
 
-# line 907 "dynafunc.dy"
+# line 993 "dynafunc.dy"
 void find_end_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int i;
@@ -951,22 +1039,23 @@ void find_end_func(DYNFILE * dfp,GenericMatrix * gm)
   }
   
   fi = FuncInfo_named_from_varstr(FI_INTERNAL,"find_end_%s",gm->name);
-  start_function_FuncInfo(fi,dfp,"int find_end_%s(%s * mat,int * ri,int * rj,int * state,boolean * isspecial)",gm->name,gm->name);
+  start_function_FuncInfo(fi,dfp,"int find_end_%s(%s * mat,int * ri,int * rj,int * state,boolean * isspecial,%s_access_func_holder h)",gm->name,gm->name,gm->name);
 	
-  expr(dfp,"register int j");
-  expr(dfp,"register int max;");
-  expr(dfp,"register int maxj");
+  expr(dfp,"int j");
+  expr(dfp,"int max;");
+  expr(dfp,"int maxj");
+  expr(dfp,"int temp");
   
   add_break(dfp);
   
-  expr(dfp,"max = %s_EXPL_SPECIAL(mat,0,mat->%s->%s-1,%s);",gm->name,gm->target->name,gm->target_len,gm->special[i]->name);
+  expr(dfp,"max = (*h.access_special)(mat,0,mat->%s->%s-1,%s);",gm->target->name,gm->target_len,gm->special[i]->name);
   expr(dfp,"maxj = mat->%s->%s-1",gm->target->name,gm->target_len);
   expr(dfp,"for(j= mat->%s->%s-2 ;j >= 0 ;j--)",gm->target->name,gm->target_len);
   
   startbrace(dfp);
-  expr(dfp,"if( %s_EXPL_SPECIAL(mat,0,j,%s) > max )",gm->name,gm->special[i]->name);
+  expr(dfp,"if( (temp =(*h.access_special)(mat,0,j,%s)) > max )",gm->special[i]->name);
   startbrace(dfp);
-  expr(dfp,"max = %s_EXPL_SPECIAL(mat,0,j,%s);",gm->name,gm->special[i]->name);
+  expr(dfp,"max = temp;");
   expr(dfp,"maxj = j;");
   closebrace(dfp);
   closebrace(dfp);
@@ -994,7 +1083,7 @@ void find_end_func(DYNFILE * dfp,GenericMatrix * gm)
 }
 	
 
-# line 967 "dynafunc.dy"
+# line 1054 "dynafunc.dy"
 void debug_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int i;
@@ -1026,7 +1115,7 @@ void debug_func(DYNFILE * dfp,GenericMatrix * gm)
   return;
 }
 	
-# line 998 "dynafunc.dy"
+# line 1085 "dynafunc.dy"
 void write_special_max_calc_func_debug(DYNFILE * dfp,GenericMatrix * gm,int debug)
 {
   register int i;
@@ -1038,7 +1127,7 @@ void write_special_max_calc_func_debug(DYNFILE * dfp,GenericMatrix * gm,int debu
   fi = FuncInfo_named_from_varstr(FI_INTERNAL,"max_calc_special_%s",gm->name);
 
   
-  start_function_FuncInfo(fi,dfp,"int max_calc_special_%s(%s * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore)",gm->name,gm->name);
+  start_function_FuncInfo(fi,dfp,"int max_calc_special_%s(%s * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,%s_access_func_holder h)",gm->name,gm->name,gm->name);
   
   expr(dfp,"register int temp;");
   expr(dfp,"register int cscore;");
@@ -1057,7 +1146,7 @@ void write_special_max_calc_func_debug(DYNFILE * dfp,GenericMatrix * gm,int debu
   
   add_break(dfp);
   
-  expr(dfp,"cscore = %s_%s(mat,i,j,state);",gm->name,special_tag);
+  expr(dfp,"cscore = (*h.access_special)(mat,i,j,state);");
   
   if( debug)
     expr(dfp,"fprintf(stderr,\"Looking for score %%d\\n\",cscore);");
@@ -1089,7 +1178,7 @@ void write_special_max_calc_func_debug(DYNFILE * dfp,GenericMatrix * gm,int debu
 	if( debug )
 	  expr(dfp,"fprintf(stderr,\"Trying %%d for case %s\\n\",temp)",cell->source[j]->state_source); 
 
-	expr(dfp,"if( temp == %s_%s(mat,i - %d,j - %d,%s) )",gm->name,matrix_tag,cell->source[j]->offi,
+	expr(dfp,"if( temp == (*h.access_main)(mat,i - %d,j - %d,%s) )",cell->source[j]->offi,
 	       cell->source[j]->offj,cell->source[j]->state_source);
   
 	startbrace(dfp);
@@ -1105,11 +1194,11 @@ void write_special_max_calc_func_debug(DYNFILE * dfp,GenericMatrix * gm,int debu
 	/*** if they want the score, calc the difference in the number and give it back ***/
 	expr(dfp,"if( cellscore != NULL)");
 	startbrace(dfp);
-	expr(dfp,"*cellscore = cscore - %s_%s(mat,i-%d,j-%d,%s)",gm->name,matrix_tag,cell->source[j]->offi,
+	expr(dfp,"*cellscore = cscore - (*h.access_main)(mat,i-%d,j-%d,%s)",cell->source[j]->offi,
 	     cell->source[j]->offj,cell->source[j]->state_source); 
 	closebrace(dfp);
 	
-	expr(dfp,"return %s_%s(mat,i - %d,j - %d,%s) ",gm->name,matrix_tag,cell->source[j]->offi,
+	expr(dfp,"return (*h.access_main)(mat,i - %d,j - %d,%s) ",cell->source[j]->offi,
 	     cell->source[j]->offj,cell->source[j]->state_source); 
 	closebrace(dfp);
 	closebrace(dfp); /** this closes the for brace ***/
@@ -1125,8 +1214,8 @@ void write_special_max_calc_func_debug(DYNFILE * dfp,GenericMatrix * gm,int debu
 	if( debug )
 	  expr(dfp,"fprintf(stderr,\"Trying %%d for case %s\\n\",temp)",cell->source[j]->state_source); 
 
-	expr(dfp,"if( temp == %s_%s(mat,i - %d,j - %d,%s) )",gm->name,special_tag,cell->source[j]->offi,
-	       cell->source[j]->offj,cell->source[j]->state_source);
+	expr(dfp,"if( temp == (*h.access_special)(mat,i - %d,j - %d,%s) )",cell->source[j]->offi,
+	     cell->source[j]->offj,cell->source[j]->state_source);
           
 
 	startbrace(dfp);
@@ -1141,11 +1230,11 @@ void write_special_max_calc_func_debug(DYNFILE * dfp,GenericMatrix * gm,int debu
 	/*** if they want the score, calc the difference in the number and give it back ***/
 	expr(dfp,"if( cellscore != NULL)");
 	startbrace(dfp);
-	expr(dfp,"*cellscore = cscore - %s_%s(mat,i-%d,j-%d,%s)",gm->name,special_tag,cell->source[j]->offi,
+	expr(dfp,"*cellscore = cscore - (*h.access_special)(mat,i-%d,j-%d,%s)",cell->source[j]->offi,
 	     cell->source[j]->offj,cell->source[j]->state_source); 
 	closebrace(dfp);
       
-	expr(dfp,"return %s_%s(mat,i - %d,j - %d,%s) ",gm->name,matrix_tag,cell->source[j]->offi,
+	expr(dfp,"return (*h.access_special)(mat,i - %d,j - %d,%s) ",cell->source[j]->offi,
 	     cell->source[j]->offj,cell->source[j]->state_source); 
 	closebrace(dfp);
       
@@ -1167,7 +1256,7 @@ void write_special_max_calc_func_debug(DYNFILE * dfp,GenericMatrix * gm,int debu
   add_break(dfp);
 }
 
-# line 1138 "dynafunc.dy"
+# line 1225 "dynafunc.dy"
 void write_special_max_calc_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   write_special_max_calc_func_debug(dfp,gm,0);
@@ -1187,18 +1276,30 @@ void write_special_max_calc_func(DYNFILE * dfp,GenericMatrix * gm)
 
    ***/
 
-# line 1157 "dynafunc.dy"
-void write_max_calc_block(DYNFILE * dfp,GenericMatrix * gm,char * matrix_tag,char * special_tag,boolean use_special)
+# line 1244 "dynafunc.dy"
+void write_max_calc_block(DYNFILE * dfp,GenericMatrix * gm,char * matrix_tag,char * special_tag,boolean use_special,boolean use_holder)
 {
   register int i;
   register int j;
+  char matrix_access[128];
+  char special_access[128];
+
+
+  if( use_holder == TRUE ) {
+    sprintf(matrix_access,"(*h.access_main)");
+    sprintf(special_access,"(*h.access_special)");
+  } else {
+    sprintf(matrix_access,"%s_%s",gm->name,matrix_tag);
+    sprintf(special_access,"%s_%s",gm->name,special_tag);
+  }
+
 
   add_break(dfp);
   add_block_comment(dfp,"Then you have to select the correct switch statement to figure out the readoff     ");
   add_block_comment(dfp,"Somewhat odd - reverse the order of calculation and return as soon as it is correct");
   
   
-  expr(dfp,"cscore = %s_%s(mat,i,j,state);",gm->name,matrix_tag);
+  expr(dfp,"cscore = %s(mat,i,j,state);",matrix_access);
   expr(dfp,"switch(state)");
   
   startbrace_tag(dfp,"Switch state ");
@@ -1223,9 +1324,9 @@ void write_max_calc_block(DYNFILE * dfp,GenericMatrix * gm,char * matrix_tag,cha
 	   cell->calc_expr == NULL ? "0" : cell->calc_expr);
       
       if( cell->source[j]->isspecial == TRUE)
-	expr(dfp,"if( temp == %s_%s(mat,i - %d,j - %d,%s) )",gm->name,special_tag,cell->source[j]->offi,
+	expr(dfp,"if( temp == %s(mat,i - %d,j - %d,%s) )",special_access,cell->source[j]->offi,
 	     cell->source[j]->offj,cell->source[j]->state_source);
-      else	expr(dfp,"if( temp == %s_%s(mat,i - %d,j - %d,%s) )",gm->name,matrix_tag,cell->source[j]->offi,
+      else	expr(dfp,"if( temp == %s(mat,i - %d,j - %d,%s) )",matrix_access,cell->source[j]->offi,
 		     cell->source[j]->offj,cell->source[j]->state_source);
       
       startbrace(dfp);
@@ -1241,13 +1342,13 @@ void write_max_calc_block(DYNFILE * dfp,GenericMatrix * gm,char * matrix_tag,cha
       startbrace(dfp);
 
       
-      expr(dfp,"*cellscore = cscore - %s_%s(mat,i-%d,j-%d,%s);",gm->name,
-	   cell->source[j]->isspecial == TRUE ? special_tag : matrix_tag,cell->source[j]->offi,
+      expr(dfp,"*cellscore = cscore - %s(mat,i-%d,j-%d,%s);",
+	   cell->source[j]->isspecial == TRUE ? special_access : matrix_access,cell->source[j]->offi,
 	   cell->source[j]->offj,cell->source[j]->state_source); 
       closebrace(dfp);
       
       
-      expr(dfp,"return %s_%s(mat,i - %d,j - %d,%s); ",gm->name,matrix_tag,cell->source[j]->offi,
+      expr(dfp,"return %s(mat,i - %d,j - %d,%s); ",matrix_access,cell->source[j]->offi,
 	   cell->source[j]->offj,cell->source[j]->state_source);
 
       if( cell->source[j]->position != SOURCE_POS_ALL) {
@@ -1272,14 +1373,14 @@ void write_max_calc_block(DYNFILE * dfp,GenericMatrix * gm,char * matrix_tag,cha
 
 
 
-# line 1241 "dynafunc.dy"
+# line 1340 "dynafunc.dy"
 void write_max_calc_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
 
   fi = FuncInfo_named_from_varstr(FI_INTERNAL,"max_calc_%s",gm->name);
 
-  start_function_FuncInfo(fi,dfp,"int max_calc_%s(%s * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore)",gm->name,gm->name);
+  start_function_FuncInfo(fi,dfp,"int max_calc_%s(%s * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,%s_access_func_holder h)",gm->name,gm->name,gm->name);
   
   expr(dfp,"register int temp;");
   expr(dfp,"register int cscore;");
@@ -1297,7 +1398,7 @@ void write_max_calc_func(DYNFILE * dfp,GenericMatrix * gm)
   expr(dfp,"return -1");
   closebrace(dfp);
 
-  write_max_calc_block(dfp,gm,"EXPL_MATRIX","EXPL_SPECIAL",TRUE);
+  write_max_calc_block(dfp,gm,"EXPL_MATRIX","EXPL_SPECIAL",TRUE,TRUE);
   
   close_function(dfp);
   
@@ -1310,7 +1411,7 @@ void write_max_calc_func(DYNFILE * dfp,GenericMatrix * gm)
 
 static char source_buffer[125];
 
-# line 1278 "dynafunc.dy"
+# line 1377 "dynafunc.dy"
 char * source_allowed_statement(int position,int offi,int offj)
 {
   switch(position) {
@@ -1340,7 +1441,7 @@ char * source_allowed_statement(int position,int offi,int offj)
 
 
 
-# line 1307 "dynafunc.dy"
+# line 1406 "dynafunc.dy"
 void write_score_block_debug(DYNFILE * dfp,GenericMatrix * gm,char * matrixtag,char * pointertag,char * specialtag,boolean use_special,int debug)
 {
   register int i;
@@ -1584,7 +1685,7 @@ void write_score_block_debug(DYNFILE * dfp,GenericMatrix * gm,char * matrixtag,c
 }
 
 
-# line 1550 "dynafunc.dy"
+# line 1649 "dynafunc.dy"
 void write_score_block(DYNFILE * dfp,GenericMatrix * gm,char * matrixtag,char * pointertag,char * specialtag,boolean use_special)
 {
   write_score_block_debug(dfp,gm,matrixtag,pointertag,specialtag,use_special,0);
@@ -1601,7 +1702,7 @@ void write_score_block(DYNFILE * dfp,GenericMatrix * gm,char * matrixtag,char *
  * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
  *
  */
-# line 1560 "dynafunc.dy"
+# line 1659 "dynafunc.dy"
 void matrix_calculate_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -1685,7 +1786,183 @@ void matrix_calculate_func(DYNFILE * dfp,GenericMatrix * gm)
   
 }	
 
-# line 1643 "dynafunc.dy"
+
+
+/* Function:  matrix_calculate_func_dpenv(dfp,gm)
+ *
+ * Descrip:    makes calculate_xxx functions, which
+ *             is for explicit matrix implementations
+ *
+ *
+ * Arg:        dfp [UNKN ] Undocumented argument [DYNFILE *]
+ * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
+ *
+ */
+# line 1748 "dynafunc.dy"
+void matrix_calculate_func_dpenv(DYNFILE * dfp,GenericMatrix * gm)
+{
+  FuncInfo * fi;
+  ArgInfo * ai;
+  int i;
+
+  fi = FuncInfo_named_from_varstr(FI_CALLABLE,"calculate_dpenv_%s",gm->name);
+  add_line_to_Ftext(fi->ft,"This function calculates the %s matrix when in explicit mode, subject to the envelope",gm->name);
+
+
+  /** ArgInfo_in_FuncInfo_from_varstr automatically adds ai to fi **/
+  
+  ai =  ArgInfo_in_FuncInfo_from_varstr(fi,"mat");
+  ai->desc=stringallocf("%s which contains explicit basematrix memory",gm->name);
+
+
+  start_function_FuncInfo(fi,dfp,"boolean calculate_dpenv_%s(%s * mat,DPEnvelope * dpenv)",gm->name,gm->name);
+  expr(dfp,"int i;");
+  expr(dfp,"int j;");
+  expr(dfp,"int k;");
+  expr(dfp,"int starti;");
+  expr(dfp,"int startj;");
+  expr(dfp,"int endi;");
+  expr(dfp,"int endj;");
+  expr(dfp,"int tot;");
+  expr(dfp,"int num;");
+  expr(dfp,"int should_calc;");
+  add_break(dfp);
+  
+  expr(dfp,"if( mat->basematrix->type != BASEMATRIX_TYPE_EXPLICIT )");
+  startbrace(dfp);
+  warn_expr(dfp,"in calculate_%s, passed a non Explicit matrix type, cannot calculate!",gm->name);
+  expr(dfp,"return FALSE;");
+  closebrace(dfp);
+  add_break(dfp);
+
+  
+  /*** make num and tot ***/
+  
+  expr(dfp,"prepare_DPEnvelope(dpenv)");
+
+  expr(dfp,"starti = dpenv->starti");
+  expr(dfp,"if( starti < 0 )");
+  hang_expr(dfp,"starti = 0");
+
+  expr(dfp,"startj = dpenv->startj");
+  expr(dfp,"if( startj < 0 )");
+  hang_expr(dfp,"startj = 0");
+
+  expr(dfp,"endi = dpenv->endi");
+  expr(dfp,"if( endi > mat->leni )");
+  hang_expr(dfp,"endi = mat->leni");
+  
+  expr(dfp,"endj = dpenv->endj");
+  expr(dfp,"if( endj > mat->lenj )");
+  hang_expr(dfp,"endj = mat->lenj");
+  
+
+  expr(dfp,"tot = (endi-starti) * (endj-startj);");
+  expr(dfp,"num = 0;");
+
+  add_break(dfp);
+  expr(dfp,"for(j=startj-%d;j<endj;j++)",gm->window_j);
+  startbrace(dfp);
+  expr(dfp,"for(i=%d;i<mat->leni;i++)",gm->window_i);
+  startbrace(dfp);
+  for(i=0;i<gm->len;i++) {
+    expr(dfp,"%s_EXPL_MATRIX(mat,i,j,%s) = %s;",gm->name,gm->state[i]->name,gm->state[i]->def_score);
+  }
+  closebrace(dfp);
+  closebrace(dfp);
+
+
+  expr(dfp,"for(j=-%d;j<mat->lenj;j++)",gm->window_j);
+  startbrace(dfp);
+  for(i=0;i<gm->spec_len;i++) {
+    expr(dfp,"%s_EXPL_SPECIAL(mat,i,j,%s) = %s;",gm->name,gm->special[i]->name,gm->special[i]->def_score);
+  }
+  closebrace(dfp);
+
+
+  /*** see if there any specials to specials to do ***/
+  
+  add_break(dfp);
+  expr(dfp,"start_reporting(\"%s Matrix calculation: \");",gm->name);
+  
+  expr(dfp,"for(j=startj;j<endj;j++)");
+  startbrace(dfp);
+  expr(dfp,"auto int score");
+  expr(dfp,"auto int temp");
+  expr(dfp,"for(i=starti;i<endi;i++)");
+  startbrace(dfp);
+
+  add_block_comment(dfp,"Check if is in envelope - code identical to is_in_DPEnvelope, but aggressively inlined here for speed");
+
+  expr(dfp,"should_calc = 0;");
+  expr(dfp,"for(k=0;k<dpenv->len;k++)");
+  startbrace(dfp);
+  expr(dfp,"auto DPUnit * u;");
+  expr(dfp,"u = dpenv->dpu[k];");
+  expr(dfp,"switch(u->type)");
+  startbrace(dfp);
+  expr(dfp,"case DPENV_RECT :");
+  startcase(dfp);
+  expr(dfp,"if( i >= u->starti && j >= u->startj && i <= (u->starti+u->height) && j <= (u->startj+u->length))");
+  hang_expr(dfp,"should_calc = 1");
+  expr(dfp,"break;");
+  closecase(dfp);
+  expr(dfp,"case DPENV_DIAG :");
+  startcase(dfp);
+  expr(dfp,"if(  abs( (i-j) - (u->starti-u->startj)) <= u->height && i+j >= u->starti+u->startj && i+j+u->length >= u->starti+u->startj)");
+  hang_expr(dfp,"should_calc = 1");
+  expr(dfp,"break;");
+  closecase(dfp);
+  closebrace(dfp);
+  expr(dfp,"if( should_calc == 1 )");
+  hang_expr(dfp,"break;");
+  closebrace(dfp);
+
+  expr(dfp,"if( should_calc == 0)");
+  startbrace(dfp);
+  for(i=0;i<gm->len;i++) {
+    expr(dfp,"%s_EXPL_MATRIX(mat,i,j,%s) = %s;",gm->name,gm->state[i]->name,gm->state[i]->def_score);
+  }
+  expr(dfp,"continue;");
+  closebrace(dfp);
+
+
+  add_break(dfp);
+
+
+  expr(dfp,"if( num%%1000 == 0 )");
+  hang_expr(dfp,"log_full_error(REPORT,0,\"[%%7d] Cells %%2d%%%%%%%%\",num,num*100/tot);");
+  
+
+/**  expr(dfp,"(void) do_cell_%s(mat,i,j)",gm->name); **/
+
+  expr(dfp,"num++;"); 
+
+  write_score_block(dfp,gm,"EXPL_MATRIX","mat","EXPL_SPECIAL",TRUE);
+
+
+  closebrace(dfp);
+  /**** if there are any specials do them here ****/
+
+  write_special_block(dfp,gm,"EXPL_MATRIX","EXPL_SPECIAL",NULL);
+  
+  closebrace(dfp);
+  
+  /*** stop reporting ***/
+  
+  expr(dfp,"stop_reporting()");
+    
+  expr(dfp,"return TRUE");
+  
+  close_function(dfp);
+  
+  add_break(dfp);
+  
+}	
+
+
+
+# line 1911 "dynafunc.dy"
 void do_special_function(DYNFILE * dfp,GenericMatrix * gm)
 {
 
@@ -1720,7 +1997,7 @@ void do_special_function(DYNFILE * dfp,GenericMatrix * gm)
  * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
  *
  */
-# line 1671 "dynafunc.dy"
+# line 1939 "dynafunc.dy"
 void do_cell_function(DYNFILE * dfp,GenericMatrix * gm)
 {
   start_function(dfp,"int do_cell_%s(%s * mat,int i,int j)",gm->name,gm->name);
@@ -1755,7 +2032,7 @@ void do_cell_function(DYNFILE * dfp,GenericMatrix * gm)
  * Arg:        bestscore [READ ] if not NULL, name of scoped variable for updating bestscore [char *]
  *
  */
-# line 1703 "dynafunc.dy"
+# line 1971 "dynafunc.dy"
 void write_special_block(DYNFILE * dfp,GenericMatrix * gm,char * matrix,char * special,char * bestscore)
 {
   register int i;
@@ -1853,7 +2130,7 @@ source->calc_expr,state->calc_expr == NULL ? "0" : state->calc_expr);
 
 }
 
-# line 1800 "dynafunc.dy"
+# line 2068 "dynafunc.dy"
 void write_search_distributor_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation * dpi)
 {
   int i;
@@ -2059,27 +2336,49 @@ void write_search_distributor_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementa
 
 
 
-# line 2005 "dynafunc.dy"
+# line 2273 "dynafunc.dy"
 void write_GenericMatrix_header(DYNFILE * dfp,GenericMatrix * gm,DPImplementation * dpi)
 {
-	write_StructHolder_header(dfp,gm->sh);
-	if( dpi->do_threads == TRUE ) {
-	  if( can_do_threads(gm) == FALSE ) {
-	    warn("Asking for threads, but the types in this matrix %s are not thread safe",gm->name);
-	  } else {
-	    write_thread_struct(dfp,dpi,gm);
-	  }
-	}
+  StructHolder * temp;
+  StructElement * se;
+  char buffer[MAXLINE];
+
+  write_StructHolder_header(dfp,gm->sh);
+  if( dpi->do_threads == TRUE ) {
+    if( can_do_threads(gm) == FALSE ) {
+      warn("Asking for threads, but the types in this matrix %s are not thread safe",gm->name);
+    } else {
+      write_thread_struct(dfp,dpi,gm);
+    }
+  }
+  
+  temp = StructHolder_alloc_std();
+  sprintf(buffer,"%s_access_func_holder",gm->name);
+  temp->name = stringalloc(buffer);
+
+  sprintf(buffer,"int (*access_main)(%s*,int,int,int)",gm->name);
+  se = basic_add_StructElement(temp,"access_main","int");
+  se->element_type = stringalloc(buffer);
+  se->isfunc = TRUE;
+
+  sprintf(buffer,"int (*access_special)(%s*,int,int,int)",gm->name);
+  se = basic_add_StructElement(temp,"access_special","int");
+  se->element_type = stringalloc(buffer);
+  se->isfunc = TRUE;
+
+  write_StructHolder_header(dfp,temp);
+
+  free_StructHolder(temp);
 
 /**	write_StructHolder_header(gm->search_sh,ofp); **/
+  
+  /**
+     load_file(ofp);
 
-/**
-	load_file(ofp);
-
-	write_memory_macros(gm);
-
-	close_file();
-**/
+     write_memory_macros(gm);
+     
+     close_file();
+  **/
 
 }
 
@@ -2093,7 +2392,7 @@ void write_GenericMatrix_header(DYNFILE * dfp,GenericMatrix * gm,DPImplementatio
 
 ******************************************/
 
-# line 2038 "dynafunc.dy"
+# line 2328 "dynafunc.dy"
 void write_GenericMatrix_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation * dpi)
 {
 
@@ -2121,6 +2420,8 @@ void write_GenericMatrix_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation
     write_debug_funcs(dfp,gm);
   }
 
+  write_shatter_functions(dfp,gm,dpi);
+
 
   write_search_distributor_func(dfp,gm,dpi);
 
@@ -2169,8 +2470,15 @@ void write_GenericMatrix_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation
 
   write_alncconvert_make_func(dfp,gm);
   
+  write_expl_read_func(dfp,gm);
+
+  write_expl_access_funcs(dfp,gm);
+
+
   write_basic_read_func(dfp,gm);
+
   
+
   find_end_func(dfp,gm);
   
   debug_func(dfp,gm);
@@ -2180,6 +2488,8 @@ void write_GenericMatrix_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation
   write_special_max_calc_func(dfp,gm);
 
   matrix_calculate_func(dfp,gm);
+
+  matrix_calculate_func_dpenv(dfp,gm);
   
   write_simplealloc_function(dfp,gm->sh);
   write_free_function(dfp,gm->sh);
@@ -2190,7 +2500,7 @@ void write_GenericMatrix_func(DYNFILE * dfp,GenericMatrix * gm,DPImplementation
   return;
 }	
 
-# line 2172 "dynafunc.c"
+# line 2480 "dynafunc.c"
 
 #ifdef _cplusplus
 }
