@@ -56,10 +56,14 @@ void write_dc_functions(DYNFILE * dfp,GenericMatrix * gm)
 
 /*  write_shadow_start_end_alloc(dfp,gm); */
 
-  write_start_end_build(dfp,gm);
+/*  write_start_end_build(dfp,gm);*/
 
   write_start_end_find_end(dfp,gm);
 
+  optimised_shadow_GenericMatrix(dfp,gm);
+
+/*  heavy_optimised_shadow_GenericMatrix(dfp,gm);*/
+
   write_start_end_init(dfp,gm);
 
 }
@@ -77,7 +81,7 @@ void write_dc_functions(DYNFILE * dfp,GenericMatrix * gm)
  * Arg:         gm [READ ] generic matrix structure [const GenericMatrix *]
  *
  */
-# line 87 "dynashadow.dy"
+# line 91 "dynashadow.dy"
 void alloc_small_func_GenericMatrix(DYNFILE * dfp,const GenericMatrix * gm)
 {
   char * arg_str;
@@ -146,7 +150,7 @@ void alloc_small_func_GenericMatrix(DYNFILE * dfp,const GenericMatrix * gm)
  * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
  *
  */
-# line 149 "dynashadow.dy"
+# line 153 "dynashadow.dy"
 void make_small_calculate_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -213,7 +217,7 @@ void make_small_calculate_func(DYNFILE * dfp,GenericMatrix * gm)
   /** got to figure out start/end first **/
 
   expr(dfp,"start_reporting(\"Find start end points: \");");
-  expr(dfp,"dc_start_end_calculate_%s(mat,dpenv);",gm->name);
+  expr(dfp,"dc_optimised_start_end_calc_%s(mat,dpenv);",gm->name);
   
   expr(dfp,"score = start_end_find_end_%s(mat,&endj);",gm->name);
   expr(dfp,"out->score = score;");
@@ -369,7 +373,7 @@ void make_small_calculate_func(DYNFILE * dfp,GenericMatrix * gm)
  * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
  *
  */
-# line 365 "dynashadow.dy"
+# line 369 "dynashadow.dy"
 void one_shot_AlnRangeSet_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -390,7 +394,7 @@ void one_shot_AlnRangeSet_func(DYNFILE * dfp,GenericMatrix * gm)
   add_break(dfp);
   
   expr(dfp,"start_reporting(\"Find start end points: \");");
-  expr(dfp,"dc_start_end_calculate_%s(mat,NULL);",gm->name);
+  expr(dfp,"dc_optimised_start_end_calc_%s(mat,NULL);",gm->name);
   expr(dfp,"log_full_error(REPORT,0,\"Calculated\");");
 
   add_break(dfp);
@@ -403,7 +407,7 @@ void one_shot_AlnRangeSet_func(DYNFILE * dfp,GenericMatrix * gm)
 }
   
   
-# line 398 "dynashadow.dy"
+# line 402 "dynashadow.dy"
 void write_AlnRangeSet_build_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int s;
@@ -458,7 +462,7 @@ void write_AlnRangeSet_build_func(DYNFILE * dfp,GenericMatrix * gm)
   add_break(dfp);
 }
 
-# line 452 "dynashadow.dy"
+# line 456 "dynashadow.dy"
 void write_AlnRange_build_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int s;
@@ -547,7 +551,7 @@ void write_AlnRange_build_func(DYNFILE * dfp,GenericMatrix * gm)
  * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
  *
  */
-# line 534 "dynashadow.dy"
+# line 538 "dynashadow.dy"
 void write_dc_PackAln_build_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int i;
@@ -622,7 +626,7 @@ void write_dc_PackAln_build_func(DYNFILE * dfp,GenericMatrix * gm)
   /** got to figure out start/end first **/
 
   expr(dfp,"start_reporting(\"Find start end points: \");");
-  expr(dfp,"dc_start_end_calculate_%s(mat);",gm->name);
+  expr(dfp,"dc_optimised_start_end_calc_%s(mat);",gm->name);
   
   expr(dfp,"score = start_end_find_end_%s(mat,&endj);",gm->name);
   expr(dfp,"out->score = score;");
@@ -751,7 +755,7 @@ void write_dc_PackAln_build_func(DYNFILE * dfp,GenericMatrix * gm)
  * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
  *
  */
-# line 731 "dynashadow.dy"
+# line 735 "dynashadow.dy"
 void write_full_dc_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   ArgInfo * ai;
@@ -890,7 +894,7 @@ void write_full_dc_func(DYNFILE * dfp,GenericMatrix * gm)
   add_break(dfp);
 }
 
-# line 869 "dynashadow.dy"
+# line 873 "dynashadow.dy"
 void write_single_dc_pass_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -920,7 +924,7 @@ void write_single_dc_pass_func(DYNFILE * dfp,GenericMatrix * gm)
 }
 
 
-# line 898 "dynashadow.dy"
+# line 902 "dynashadow.dy"
 void write_push_dc_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   int s;
@@ -1082,7 +1086,7 @@ void write_push_dc_func(DYNFILE * dfp,GenericMatrix * gm)
   add_break(dfp);
 }
 
-# line 1059 "dynashadow.dy"
+# line 1063 "dynashadow.dy"
 void write_init_dc_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   int s;
@@ -1119,7 +1123,7 @@ void write_init_dc_func(DYNFILE * dfp,GenericMatrix * gm)
 }
 
   
-# line 1095 "dynashadow.dy"
+# line 1099 "dynashadow.dy"
 void write_up_to_dc_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -1175,7 +1179,7 @@ void write_up_to_dc_func(DYNFILE * dfp,GenericMatrix * gm)
 }
   
 
-# line 1150 "dynashadow.dy"
+# line 1154 "dynashadow.dy"
 void write_follow_on_dc_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   int i;
@@ -1215,7 +1219,7 @@ void write_follow_on_dc_func(DYNFILE * dfp,GenericMatrix * gm)
   hang_expr(dfp,"log_full_error(REPORT,0,\"[%%d%%%%%%%% done]After  mid-j %%5d Cells done %%d%%%%%%%%\",perc_done,startj,(num*100)/total)");
 
 
-  write_main_shadow_block(dfp,gm,"DC_SHADOW_MATRIX","mat","DC_SHADOW_SPECIAL","DC_SHADOW_MATRIX_SP","DC_SHADOW_SPECIAL_SP",7,FALSE,FALSE,0);
+  write_main_shadow_block(dfp,gm,"DC_SHADOW_MATRIX","mat","DC_SHADOW_SPECIAL","DC_SHADOW_MATRIX_SP","DC_SHADOW_SPECIAL_SP",7,FALSE,FALSE,0,FALSE);
   closebrace(dfp);
 
   /** no specials, as we are guarenteed to be outside them **/
@@ -1228,7 +1232,7 @@ void write_follow_on_dc_func(DYNFILE * dfp,GenericMatrix * gm)
 
 }
 
-# line 1202 "dynashadow.dy"
+# line 1206 "dynashadow.dy"
 void write_special_strip_read_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   int ss;
@@ -1317,7 +1321,7 @@ void write_special_strip_read_func(DYNFILE * dfp,GenericMatrix * gm)
 }
 
 
-# line 1290 "dynashadow.dy"
+# line 1294 "dynashadow.dy"
 void write_hidden_read_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -1397,7 +1401,7 @@ void write_hidden_read_func(DYNFILE * dfp,GenericMatrix * gm)
 
   
 
-# line 1369 "dynashadow.dy"
+# line 1373 "dynashadow.dy"
 void write_hidden_calc_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -1451,7 +1455,7 @@ void write_hidden_calc_func(DYNFILE * dfp,GenericMatrix * gm)
 }
   
 
-# line 1422 "dynashadow.dy"
+# line 1426 "dynashadow.dy"
 void write_hidden_init_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int s;
@@ -1486,7 +1490,7 @@ void write_hidden_init_func(DYNFILE * dfp,GenericMatrix * gm)
 }
 
 
-# line 1456 "dynashadow.dy"
+# line 1460 "dynashadow.dy"
 void write_hidden_max_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -1510,7 +1514,7 @@ void write_hidden_max_func(DYNFILE * dfp,GenericMatrix * gm)
   expr(dfp,"return -1;");
   closebrace(dfp);
 
-  write_max_calc_block(dfp,gm,"HIDDEN_MATRIX","HIDDEN_SPECIAL",FALSE);
+  write_max_calc_block(dfp,gm,"HIDDEN_MATRIX","HIDDEN_SPECIAL",FALSE,FALSE);
   
   close_function(dfp);
   
@@ -1519,7 +1523,7 @@ void write_hidden_max_func(DYNFILE * dfp,GenericMatrix * gm)
 
 }
 
-# line 1488 "dynashadow.dy"
+# line 1492 "dynashadow.dy"
 void write_matrix_to_special_max_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int i;
@@ -1614,7 +1618,7 @@ void write_matrix_to_special_max_func(DYNFILE * dfp,GenericMatrix * gm)
 }
   
 
-# line 1582 "dynashadow.dy"
+# line 1586 "dynashadow.dy"
 void write_special_strip_max_func(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int i;
@@ -1737,9 +1741,11 @@ void write_special_strip_max_func(DYNFILE * dfp,GenericMatrix * gm)
 
 
 
-# line 1704 "dynashadow.dy"
+# line 1708 "dynashadow.dy"
 void write_shadow_dc_macros(DYNFILE * dfp,GenericMatrix * gm)
 {
+  int steve_space;
+
   macro(dfp,"#define %s_HIDDEN_MATRIX(thismatrix,i,j,state) (thismatrix->basematrix->matrix[(j-hiddenj+%d)][(i+%d)*%d+state])",gm->name,gm->window_j,gm->window_i,gm->len);
 
   macro(dfp,"#define %s_DC_SHADOW_MATRIX(thismatrix,i,j,state) (thismatrix->basematrix->matrix[((j+%d)*8) %% %d][(i+%d)*%d+state])",gm->name,gm->window_j+1,(gm->window_j+1)*8,gm->window_i,gm->len);
@@ -1749,6 +1755,21 @@ void write_shadow_dc_macros(DYNFILE * dfp,GenericMatrix * gm)
   macro(dfp,"#define %s_DC_SHADOW_SPECIAL(thismatrix,i,j,state) (thismatrix->basematrix->specmatrix[state*8][(j+%d)])",gm->name,gm->window_j);
   macro(dfp,"#define %s_DC_SHADOW_MATRIX_SP(thismatrix,i,j,state,shadow) (thismatrix->basematrix->matrix[((((j+%d)*8)+(shadow+1)) %% %d)][(i+%d)*%d + state])",gm->name,gm->window_j+1,(gm->window_j+1)*8,gm->window_i,gm->len);
   macro(dfp,"#define %s_DC_SHADOW_SPECIAL_SP(thismatrix,i,j,state,shadow) (thismatrix->basematrix->specmatrix[state*8 +shadow+1][(j+%d)])",gm->name,gm->window_j);
+
+  /* we need 8 shadow pointers per matrix cell; one for the score 7 for different shadow implementations*/
+  steve_space = gm->len * 8;
+
+  macro(dfp,"#define %s_DC_OPT_SHADOW_MATRIX(thismatrix,i,j,state) (score_pointers[(((j+%d)%% %d) * (leni+1) * %d) + ((i+%d) * %d) + (state)])",gm->name,gm->window_j,gm->window_j,gm->len,gm->window_i,gm->len);
+
+  /* the +1 is because + 0 is the score */
+  macro(dfp,"#define %s_DC_OPT_SHADOW_MATRIX_SP(thismatrix,i,j,state,shadow) (shadow_pointers[(((j+%d)%% %d) * (leni+1) * %d) + ((i+%d) * %d) + (state * %d) + shadow+1])",gm->name,gm->window_j,gm->window_j,steve_space,gm->window_i,steve_space,8);
+
+
+  /* currently for the optimised shadow we use the same special matrix layout */
+  macro(dfp,"#define %s_DC_OPT_SHADOW_SPECIAL(thismatrix,i,j,state) (thismatrix->basematrix->specmatrix[state*8][(j+%d)])",gm->name,gm->window_j);
+  macro(dfp,"#define %s_DC_OPT_SHADOW_SPECIAL_SP(thismatrix,i,j,state,shadow) (thismatrix->basematrix->specmatrix[state*8 +shadow+1][(j+%d)])",gm->name,gm->window_j);
+
+  
 }
 
 
@@ -1761,7 +1782,7 @@ void write_shadow_dc_macros(DYNFILE * dfp,GenericMatrix * gm)
  * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
  *
  */
-# line 1721 "dynashadow.dy"
+# line 1742 "dynashadow.dy"
 void write_shadow_dc_alloc(DYNFILE * dfp,GenericMatrix * gm)
 {
   FuncInfo * fi;
@@ -1809,7 +1830,7 @@ void write_shadow_dc_alloc(DYNFILE * dfp,GenericMatrix * gm)
    ***/
 
 
-# line 1768 "dynashadow.dy"
+# line 1789 "dynashadow.dy"
 void write_start_end_init(DYNFILE * dfp,GenericMatrix * gm)
 {
   register int s;
@@ -1863,7 +1884,7 @@ void write_start_end_init(DYNFILE * dfp,GenericMatrix * gm)
 }
 
 
-# line 1821 "dynashadow.dy"
+# line 1842 "dynashadow.dy"
 void write_start_end_find_end(DYNFILE * dfp,GenericMatrix * gm)
 {
 
@@ -1928,7 +1949,323 @@ void write_start_end_find_end(DYNFILE * dfp,GenericMatrix * gm)
 }
 	
 
-# line 1885 "dynashadow.dy"
+/* Function:  heavy_optimised_shadow_GenericMatrix(dfp,gm)
+ *
+ * Descrip:    heavily optimised start end calc function
+ *
+ *
+ * Arg:        dfp [UNKN ] Undocumented argument [DYNFILE *]
+ * Arg:         gm [UNKN ] Undocumented argument [GenericMatrix *]
+ *
+ */
+# line 1909 "dynashadow.dy"
+void heavy_optimised_shadow_GenericMatrix(DYNFILE * dfp,GenericMatrix * gm)
+{
+  int i;
+  int j;
+  int k;
+  FuncInfo * fi;
+
+  char * matrixtag = "DC_OPT_SHADOW_MATRIX";
+  char * shadow_main_tag = "DC_OPT_SHADOW_MATRIX_SP";
+  char * shadow_special_tag = "DC_OPT_SHADOW_SPECIAL_SP";
+  char * specialtag = "DC_OPT_SHADOW_SPECIAL";
+
+
+  fi = FuncInfo_named_from_varstr(FI_INTERNAL,"dc_heavy_optimised_start_end_calc_%s",gm->name);
+  add_line_to_Ftext(fi->ft,"Calculates special strip, leaving start/end/score points in shadow matrix");
+  add_line_to_Ftext(fi->ft,"Works off specially laid out memory from steve searle");
+	
+
+  start_function_FuncInfo(fi,dfp,"boolean dc_heavy_optimised_start_end_calc_%s(%s *mat,DPEnvelope * dpenv)",gm->name,gm->name);
+
+  expr(dfp,"int i;");
+  expr(dfp,"int j;");
+  expr(dfp,"int k;");
+
+
+  expr(dfp,"int leni;");
+  expr(dfp,"int lenj;");
+  expr(dfp,"int localshadow[7]");
+  expr(dfp,"long int total");
+  expr(dfp,"long int num=0");
+  expr(dfp,"int * score_pointers");
+  expr(dfp,"int * shadow_pointers");
+
+
+  for(i=0;i<gm->len;i++) {
+    expr(dfp,"int score_%s;",gm->state[i]->name);
+    expr(dfp,"int temp_%s;",gm->state[i]->name);
+    expr(dfp,"int * localsp_%s;",gm->state[i]->name);
+    for(j=0;j<gm->state[i]->len;j++) {
+      expr(dfp,"int score_%s_%d;",gm->state[i]->name,j);
+    }
+  }
+
+  add_break(dfp);
+
+  expr(dfp,"leni = mat->%s->%s;",gm->query->name,gm->query_len);
+  expr(dfp,"lenj = mat->%s->%s;",gm->target->name,gm->target_len);
+
+  add_break(dfp);
+
+  expr(dfp,"score_pointers = (int *) calloc (%d * (leni + %d) * %d,sizeof(int));",gm->window_j,gm->window_i,gm->len);
+  expr(dfp,"shadow_pointers = (int *) calloc (%d * (leni + %d) * %d * 8,sizeof(int));",gm->window_j,gm->window_i,gm->len);
+
+
+  add_break(dfp);
+
+  expr(dfp,"for(j=0;j<lenj;j++)");
+  startbrace_tag(dfp,"for each j strip");
+  expr(dfp,"for(i=0;i<leni;i++)");
+  startbrace_tag(dfp,"for each i position in strip");
+  expr(dfp,"num++");
+
+
+  expr(dfp,"if( dpenv != NULL && is_in_DPEnvelope(dpenv,i,j) == FALSE )");
+  startbrace_tag(dfp,"Is not in envelope");
+
+  for(i=0;i<gm->len;i++) {
+    expr(dfp,"%s_DC_OPT_SHADOW_MATRIX(mat,i,j,%s) = NEGI",gm->name,gm->state[i]->name);
+  }
+  expr(dfp,"continue;");
+  closebrace(dfp);
+
+
+  expr(dfp,"if( num%%1000 == 0)");
+  hang_expr(dfp,"log_full_error(REPORT,0,\"%%6d Cells done [%%2d%%%%%%%%]\",num,num*100/total);");
+
+  add_break(dfp);
+
+
+  /* main block */
+
+  add_block_comment(dfp,"Retrieve all the calculations first, and put into local variables");
+  
+  for(i=0;i<gm->len;i++) {
+    for(j=0;j<gm->state[i]->len;j++) {
+      expr(dfp,"score_%s_%d = %s_%s(mat,i-%d,j-%d,%s) + %s + (%s)",
+	   gm->state[i]->name,j,
+	   gm->name,
+	   gm->state[i]->source[j]->isspecial == TRUE ? specialtag : matrixtag, 
+	   gm->state[i]->source[j]->offi,gm->state[i]->source[j]->offj,
+	   gm->state[i]->source[j]->state_source,
+	   gm->state[i]->source[j]->calc_expr,
+	   gm->state[i]->calc_expr == NULL ? "0" : gm->state[i]->calc_expr
+	   );
+    }
+  }
+  
+  add_break(dfp);
+
+  add_block_comment(dfp,"now do the if/then/else store");
+
+  for(i=0;i<gm->len;i++) {
+    add_block_comment(dfp,"handling state %s",gm->state[i]->name);
+
+    expr(dfp,"score_%s   = score_%s_0;",gm->state[i]->name,gm->state[i]->name);
+    expr(dfp,"localsp_%s = &(%s_%s(mat,i - %d,j - %d,%s,0));",gm->state[i]->name,
+	 gm->name,gm->state[i]->source[0]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,
+	 gm->state[i]->source[0]->offi,gm->state[i]->source[0]->offj,gm->state[i]->source[0]->state_source);
+
+    for(j=1;j<gm->state[i]->len;j++) {
+      expr(dfp,"if( score_%s_%d > score_%s)",gm->state[i]->name,j,gm->state[i]->name);
+      startbrace(dfp);
+      expr(dfp,"score_%s = score_%s_%d;",gm->state[i]->name,gm->state[i]->name,j);
+      if( gm->state[i]->source[j]->isspecial == TRUE ) {
+	add_block_comment(dfp,"This source is a special, push top shadow pointers");
+	expr(dfp,"localshadow[0] = i;");
+	expr(dfp,"localshadow[1] = j;");
+	expr(dfp,"localshadow[2] = %s;",gm->state[i]->name);
+	expr(dfp,"localshadow[3] = -1;");
+	expr(dfp,"localshadow[4] = -1;");
+	expr(dfp,"localshadow[5] = -1;");
+	expr(dfp,"localshadow[6] = score_%s;",gm->state[i]->name);
+	expr(dfp,"localsp_%s = localshadow;",gm->state[i]->name);
+      } else {
+	expr(dfp,"localsp_%s = &(%s_%s(mat,i -%d,j - %d,%s,0));",gm->state[i]->name,
+	     gm->name,shadow_main_tag,
+	     gm->state[i]->source[j]->offi,gm->state[i]->source[j]->offj,gm->state[i]->source[j]->state_source);
+      }
+      closebrace(dfp);
+
+    } /* end of loop over all sources */
+
+
+    expr(dfp,"%s_%s(mat,i,j,%s) = score_%s",gm->name,matrixtag,gm->state[i]->name,gm->state[i]->name);
+    expr(dfp,"for(k=0;k<7;k++)");
+    hang_expr(dfp,"%s_%s(mat,i,j,%s,k) = localsp_%s[k]",gm->name,shadow_main_tag,gm->state[i]->name,gm->state[i]->name);
+
+    for(j=0;j<gm->spec_len;j++) {
+      auto CellState * specstate;
+      specstate = gm->special[j];
+      
+      
+      for(k=0;k<specstate->len;k++) {
+	if( strcmp(specstate->source[k]->state_source,gm->state[i]->name) == 0) {
+	  /********************************/
+	  /* is a special source!         */
+	  /********************************/
+	  add_break(dfp);
+	  add_block_comment(dfp,"state %s is a source for special %s",gm->state[i]->name,specstate->name);
+	  expr(dfp,"temp_%s = score_%s + (%s) + (%s) ",gm->state[i]->name,gm->state[i]->name,specstate->source[k]->calc_expr,specstate->calc_expr == NULL ? "0" : specstate->calc_expr );
+	  expr(dfp,"if( temp_%s > %s_%s(mat,i,j,%s) ) ",gm->state[i]->name,gm->name,specialtag,specstate->name);
+	  startbrace(dfp);
+	  expr(dfp,"%s_%s(mat,i,j,%s) = temp_%s",gm->name,specialtag,specstate->name,gm->state[i]->name);
+	  
+	  add_block_comment(dfp,"Have to push only bottem half of system here");
+	  expr(dfp,"for(k=0;k<3;k++)");
+	  hang_expr(dfp,"%s_%s(mat,i,j,%s,k) = %s_%s(mat,i,j,%s,k);",
+		    gm->name,shadow_special_tag,
+		    specstate->name, gm->name,
+		    specstate->source[k]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,
+		    gm->state[i]->name);
+	  
+	  /*** sort of hacky ***/
+	  expr(dfp,"%s_%s(mat,i,j,%s,6) = temp_%s;",
+	       gm->name,shadow_special_tag,
+	       specstate->name, gm->state[i]->name);
+
+
+
+	  expr(dfp,"%s_%s(mat,i,j,%s,3) = i;",gm->name,shadow_special_tag,specstate->name);
+	  expr(dfp,"%s_%s(mat,i,j,%s,4) = j;",gm->name,shadow_special_tag,specstate->name);
+	  expr(dfp,"%s_%s(mat,i,j,%s,5) = %s;",gm->name,shadow_special_tag,specstate->name,gm->state[i]->name);
+	  
+	  closebrace(dfp);
+	}
+      }
+    } /* end of loop over all special states */
+
+    add_block_comment(dfp,"Finished if/then/else loop for state %s",gm->state[i]->name);
+
+  } /* end of loop over states */
+
+  add_break(dfp);
+
+  closebrace(dfp);
+  
+  if( gm->specialtospecial == TRUE ) 
+    write_special_shadow_block(dfp,gm,"DC_OPT_SHADOW_MATRIX","mat","DC_OPT_SHADOW_SPECIAL","DC_OPT_SHADOW_MATRIX_SP","DC_OPT_SHADOW_SPECIAL_SP",7,TRUE,0);
+
+  closebrace(dfp);
+
+  add_break;
+
+  expr(dfp,"free(score_pointers);");
+  expr(dfp,"free(shadow_pointers);");
+
+  expr(dfp,"return TRUE");
+
+  close_function(dfp);
+
+  add_break(dfp);
+
+}
+
+
+/* Function:  optimised_shadow_GenericMatrix(dfp,*gm)
+ *
+ * Descrip:    Makes the optimised shadow matrix routine,
+ *             worked out by Steve Searle - memory access
+ *             is put into one array so that the routine
+ *             is faster
+ *
+ *
+ * Arg:        dfp [UNKN ] Undocumented argument [DYNFILE *]
+ * Arg:        *gm [UNKN ] Undocumented argument [const GenericMatrix]
+ *
+ */
+# line 2120 "dynashadow.dy"
+void optimised_shadow_GenericMatrix(DYNFILE * dfp,const GenericMatrix *gm)
+{
+  int i;
+  FuncInfo * fi;
+
+  fi = FuncInfo_named_from_varstr(FI_INTERNAL,"dc_optimised_start_end_calc_%s",gm->name);
+  add_line_to_Ftext(fi->ft,"Calculates special strip, leaving start/end/score points in shadow matrix");
+  add_line_to_Ftext(fi->ft,"Works off specially laid out memory from steve searle");
+	
+
+  start_function_FuncInfo(fi,dfp,"boolean dc_optimised_start_end_calc_%s(%s *mat,DPEnvelope * dpenv)",gm->name,gm->name);
+
+  expr(dfp,"int i;");
+  expr(dfp,"int j;");
+  expr(dfp,"int k;");
+  expr(dfp,"int score;");
+  expr(dfp,"int temp;");
+  expr(dfp,"int leni;");
+  expr(dfp,"int lenj;");
+  expr(dfp,"int localshadow[7]");
+  expr(dfp,"long int total");
+  expr(dfp,"long int num=0");
+  expr(dfp,"int * score_pointers");
+  expr(dfp,"int * shadow_pointers");
+  expr(dfp,"int * localsp;");
+  
+  expr(dfp,"leni = mat->%s->%s;",gm->query->name,gm->query_len);
+  expr(dfp,"lenj = mat->%s->%s;",gm->target->name,gm->target_len);
+  expr(dfp,"total = leni * lenj;");
+
+  add_break(dfp);
+
+  expr(dfp,"score_pointers = (int *) calloc (%d * (leni + %d) * %d,sizeof(int));",gm->window_j,gm->window_i,gm->len);
+  expr(dfp,"shadow_pointers = (int *) calloc (%d * (leni + %d) * %d * 8,sizeof(int));",gm->window_j,gm->window_i,gm->len);
+
+
+  add_break(dfp);
+
+  expr(dfp,"for(j=0;j<lenj;j++)");
+  startbrace_tag(dfp,"for each j strip");
+  expr(dfp,"for(i=0;i<leni;i++)");
+  startbrace_tag(dfp,"for each i position in strip");
+  expr(dfp,"num++");
+
+
+  expr(dfp,"if( dpenv != NULL && is_in_DPEnvelope(dpenv,i,j) == FALSE )");
+  startbrace_tag(dfp,"Is not in envelope");
+
+  for(i=0;i<gm->len;i++) {
+    expr(dfp,"%s_DC_OPT_SHADOW_MATRIX(mat,i,j,%s) = NEGI",gm->name,gm->state[i]->name);
+  }
+  expr(dfp,"continue;");
+  closebrace(dfp);
+
+
+  expr(dfp,"if( num%%1000 == 0)");
+  hang_expr(dfp,"log_full_error(REPORT,0,\"%%6d Cells done [%%2d%%%%%%%%]\",num,num*100/total);");
+
+  add_break(dfp);
+
+  write_main_shadow_block(dfp,gm,"DC_OPT_SHADOW_MATRIX","mat","DC_OPT_SHADOW_SPECIAL","DC_OPT_SHADOW_MATRIX_SP","DC_OPT_SHADOW_SPECIAL_SP",7,TRUE,TRUE,0,TRUE);
+
+  add_break(dfp);
+
+  closebrace(dfp);
+
+  if( gm->specialtospecial == TRUE ) 
+    write_special_shadow_block(dfp,gm,"DC_OPT_SHADOW_MATRIX","mat","DC_OPT_SHADOW_SPECIAL","DC_OPT_SHADOW_MATRIX_SP","DC_OPT_SHADOW_SPECIAL_SP",7,TRUE,0);
+
+  closebrace(dfp);
+
+  add_break;
+
+  expr(dfp,"free(score_pointers);");
+  expr(dfp,"free(shadow_pointers);");
+
+  /*  expr(dfp,"free(mat->basematrix->optimised_shadow);"); */
+
+  expr(dfp,"return TRUE");
+
+  close_function(dfp);
+
+  add_break(dfp);
+  
+}
+
+
+
+# line 2208 "dynashadow.dy"
 void write_start_end_build(DYNFILE * dfp,GenericMatrix * gm)
 {
   int i;
@@ -1987,7 +2324,7 @@ void write_start_end_build(DYNFILE * dfp,GenericMatrix * gm)
 
   add_break(dfp);
 
-  write_main_shadow_block(dfp,gm,"DC_SHADOW_MATRIX","mat","DC_SHADOW_SPECIAL","DC_SHADOW_MATRIX_SP","DC_SHADOW_SPECIAL_SP",7,TRUE,TRUE,0);
+  write_main_shadow_block(dfp,gm,"DC_SHADOW_MATRIX","mat","DC_SHADOW_SPECIAL","DC_SHADOW_MATRIX_SP","DC_SHADOW_SPECIAL_SP",7,TRUE,TRUE,0,FALSE);
 
   add_break(dfp);
 
@@ -2007,7 +2344,7 @@ void write_start_end_build(DYNFILE * dfp,GenericMatrix * gm)
 
 
 
-# line 1963 "dynashadow.dy"
+# line 2286 "dynashadow.dy"
 void write_start_end_macros(DYNFILE * dfp,GenericMatrix * gm)
 {
   macro(dfp,"#define %s_SE_SHADOW_MATRIX(thismatrix,i,j,state) (thismatrix->basematrix->matrix[((j+%d)*2) %% %d][(i+%d)*%d+state])",gm->name,gm->window_j+1,(gm->window_j+1)*2,gm->window_i,gm->len);
@@ -2019,7 +2356,7 @@ void write_start_end_macros(DYNFILE * dfp,GenericMatrix * gm)
 }
 
 
-# line 1974 "dynashadow.dy"
+# line 2297 "dynashadow.dy"
 void write_shadow_start_end_alloc(DYNFILE * dfp,GenericMatrix * gm)
 {
   start_function(dfp,"%s * shadow_se_alloc_%s(int leni,int lenj)",gm->name,gm->name);
@@ -2055,7 +2392,7 @@ void write_shadow_start_end_alloc(DYNFILE * dfp,GenericMatrix * gm)
 }
   
 
-/* Function:  write_main_shadow_block(dfp,gm,matrixtag,pointer_tag,specialtag,shadow_main_tag,shadow_special_tag,shadow_length,shadow_on_special,use_special,debug)
+/* Function:  write_main_shadow_block(dfp,gm,matrixtag,pointer_tag,specialtag,shadow_main_tag,shadow_special_tag,shadow_length,shadow_on_special,use_special,debug,use_shadow_pointer)
  *
  * Descrip:    The core inner loop for shadow methods. Pretty terrifying stuff in here.
  *
@@ -2091,10 +2428,11 @@ void write_shadow_start_end_alloc(DYNFILE * dfp,GenericMatrix * gm)
  * Arg:         shadow_on_special [UNKN ] Undocumented argument [boolean]
  * Arg:               use_special [UNKN ] Undocumented argument [boolean]
  * Arg:                     debug [UNKN ] Undocumented argument [int]
+ * Arg:        use_shadow_pointer [UNKN ] Undocumented argument [int]
  *
  */
-# line 2032 "dynashadow.dy"
-void write_main_shadow_block(DYNFILE * dfp,GenericMatrix * gm,char * matrixtag,char * pointer_tag,char * specialtag,char * shadow_main_tag,char * shadow_special_tag,int shadow_length,boolean shadow_on_special,boolean use_special,int debug)
+# line 2355 "dynashadow.dy"
+void write_main_shadow_block(DYNFILE * dfp,GenericMatrix * gm,char * matrixtag,char * pointer_tag,char * specialtag,char * shadow_main_tag,char * shadow_special_tag,int shadow_length,boolean shadow_on_special,boolean use_special,int debug,int use_shadow_pointer)
 {
   register int i;
   register int j;
@@ -2106,7 +2444,7 @@ void write_main_shadow_block(DYNFILE * dfp,GenericMatrix * gm,char * matrixtag,c
     int temp
     int k
     int localshadow[SHADOW_LENGTH]
-
+    int * localsp
     ***/
 
   if( shadow_on_special == TRUE && shadow_length != 7)
@@ -2116,7 +2454,7 @@ void write_main_shadow_block(DYNFILE * dfp,GenericMatrix * gm,char * matrixtag,c
     fatal("Look, you can't shaodw on special but not being using the specials... get otta here");
   }
 
-for(i=0;i<gm->len;i++) {
+  for(i=0;i<gm->len;i++) {
     auto CellState * state;
     state = gm->state[i];
     
@@ -2141,24 +2479,21 @@ for(i=0;i<gm->len;i++) {
     
 
     /*** must also shift shadown positions ***/
-    add_block_comment(dfp,"shift first shadow numbers");
 
-   
+    if( use_shadow_pointer == FALSE ) {
+      add_block_comment(dfp,"shift first shadow numbers");
 
-    expr(dfp,"for(k=0;k<%d;k++)",shadow_length);
+      expr(dfp,"for(k=0;k<%d;k++)",shadow_length);
 
-
-/**** now using local shadow system
-    hang_expr(dfp,"%s_%s(mat,i,j,%s,k) = %s_%s(mat,i - %d,j - %d,%s,k);",gm->name,shadow_main_tag,state->name,
-	 gm->name,state->source[0]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,
-	      state->source[0]->offi,state->source[0]->offj,state->source[0]->state_source);
-****/
-
-
-    hang_expr(dfp,"localshadow[k] = %s_%s(mat,i - %d,j - %d,%s,k);",
-	 gm->name,state->source[0]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,
-	      state->source[0]->offi,state->source[0]->offj,state->source[0]->state_source);
-    
+      hang_expr(dfp,"localshadow[k] = %s_%s(mat,i - %d,j - %d,%s,k);",
+		gm->name,state->source[0]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,
+		state->source[0]->offi,state->source[0]->offj,state->source[0]->state_source);
+    } else {
+      add_block_comment(dfp,"assign local shadown pointer");
+      expr(dfp,"localsp = &(%s_%s(mat,i - %d,j - %d,%s,0));",
+	   gm->name,state->source[0]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,
+	   state->source[0]->offi,state->source[0]->offj,state->source[0]->state_source);
+    }
 
 
     /**** ok this is to stop underflow, but is v.v.v. hacky ****/
@@ -2228,6 +2563,7 @@ for(i=0;i<gm->len;i++) {
       startbrace(dfp);
       expr(dfp,"score = temp;");
       /** ok for shadow matrix should put things in here */
+
       if( shadow_on_special == TRUE && state->source[j]->isspecial == TRUE) {
 	add_block_comment(dfp,"This state [%s] is a special for %s... push top shadow pointers here",state->source[j]->state_source,state->name);
 	expr(dfp,"localshadow[0]= i;");
@@ -2237,13 +2573,18 @@ for(i=0;i<gm->len;i++) {
 	expr(dfp,"localshadow[4]= (-1);");
 	expr(dfp,"localshadow[5]= (-1);");
 	expr(dfp,"localshadow[6]= score;");
+	if( use_shadow_pointer == TRUE ) {
+	  expr(dfp,"localsp = localshadow;");
+	}
       }
       else {
-	expr(dfp,"for(k=0;k<%d;k++)",shadow_length);
 
-	hang_expr(dfp,"localshadow[k] = %s_%s(mat,i - %d,j - %d,%s,k);",
-		  gm->name,state->source[0]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,
-		  state->source[j]->offi,state->source[j]->offj,state->source[j]->state_source);
+	if( use_shadow_pointer == FALSE ) {
+	  expr(dfp,"for(k=0;k<%d;k++)",shadow_length);
+	  
+	  hang_expr(dfp,"localshadow[k] = %s_%s(mat,i - %d,j - %d,%s,k);",
+		    gm->name,state->source[0]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,
+		    state->source[j]->offi,state->source[j]->offj,state->source[j]->state_source);
 
 /*** now using local shadow system
 	hang_expr(dfp,"%s_%s(%s,i,j,%s,k) = %s_%s(%s,i - %d,j - %d,%s,k);",
@@ -2251,6 +2592,12 @@ for(i=0;i<gm->len;i++) {
 		  gm->name,state->source[j]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,pointer_tag,state->source[j]->offi,
 		  state->source[j]->offj,state->source[j]->state_source);
 ***/
+	} else {
+	  add_block_comment(dfp,"assign local shadown pointer");
+	  expr(dfp,"localsp = &(%s_%s(mat,i - %d,j - %d,%s,0));",
+	       gm->name,state->source[j]->isspecial == TRUE ? shadow_special_tag : shadow_main_tag,
+	       state->source[j]->offi,state->source[j]->offj,state->source[j]->state_source);
+	}
 
       }
 
@@ -2285,9 +2632,13 @@ for(i=0;i<gm->len;i++) {
     /* put away localshadow    */
     /***************************/
 
-    expr(dfp,"for(k=0;k<%d;k++)",shadow_length);
-    hang_expr(dfp,"%s_%s(%s,i,j,%s,k) = localshadow[k];",gm->name,shadow_main_tag,pointer_tag,state->name);
-    
+    if( use_shadow_pointer == FALSE ) {
+      expr(dfp,"for(k=0;k<%d;k++)",shadow_length);
+      hang_expr(dfp,"%s_%s(%s,i,j,%s,k) = localshadow[k];",gm->name,shadow_main_tag,pointer_tag,state->name);
+    } else {
+      expr(dfp,"for(k=0;k<%d;k++)",shadow_length);
+      hang_expr(dfp,"%s_%s(%s,i,j,%s,k) = localsp[k];",gm->name,shadow_main_tag,pointer_tag,state->name);
+    }
 
     add_block_comment(dfp,"Now figure out if any specials need this score");
 
@@ -2369,7 +2720,7 @@ for(i=0;i<gm->len;i++) {
 
 
 
-# line 2307 "dynashadow.dy"
+# line 2643 "dynashadow.dy"
 void write_special_shadow_block(DYNFILE * dfp,GenericMatrix * gm,char * matrix,char * pointer_tag,char * special,char *  shadow_main_tag,char * shadow_special_tag,int shadow_length,boolean shadow_on_special,int debug)
 {
   register int i;
@@ -2424,7 +2775,7 @@ source->calc_expr,state->calc_expr == NULL ? "0" : state->calc_expr);
 }
 
 
-# line 2412 "dynashadow.c"
+# line 2759 "dynashadow.c"
 
 #ifdef _cplusplus
 }
