@@ -814,8 +814,23 @@ void close_struct(DYNFILE * dfp,char * str, ...)
 
 }
 	
+# line 750 "dynfile.dy"
+void struct_macro(DYNFILE * dfp,char * str, ...)
+{
+  char buffer[MAXLINE];
+  va_list ap;
 
-# line 751 "dynfile.dy"
+  flush_line_to_header(dfp);
+
+  va_start(ap,str);
+  vsprintf(buffer,str,ap);
+  va_end(ap);
+  strcat(dfp->line,buffer);
+
+  pad_line(dfp);
+}
+
+# line 765 "dynfile.dy"
 void struct_expr(DYNFILE * dfp,char * str, ...)
 {
   char buffer[MAXLINE];
@@ -841,7 +856,7 @@ void struct_expr(DYNFILE * dfp,char * str, ...)
 
  /*** Comments ***/
 
-# line 776 "dynfile.dy"
+# line 790 "dynfile.dy"
 void add_block_comment(DYNFILE * dfp,char * str, ...)
 {
   char buffer[MAXLINE];
@@ -863,7 +878,7 @@ void add_block_comment(DYNFILE * dfp,char * str, ...)
   
 }
 
-# line 797 "dynfile.dy"
+# line 811 "dynfile.dy"
 void add_end_comment(DYNFILE * dfp,char * str, ...)
 {
   char buffer[MAXLINE];
@@ -881,7 +896,7 @@ void add_end_comment(DYNFILE * dfp,char * str, ...)
   flush_line(dfp);
 }
 
-# line 814 "dynfile.dy"
+# line 828 "dynfile.dy"
 void add_end_comment_header(DYNFILE * dfp,char * str, ...)
 {
   char buffer[MAXLINE];
@@ -900,7 +915,7 @@ void add_end_comment_header(DYNFILE * dfp,char * str, ...)
 }
 	
 
-# line 832 "dynfile.dy"
+# line 846 "dynfile.dy"
 void set_commentstart(DYNFILE * dfp,int s)
 {
   
@@ -908,7 +923,7 @@ void set_commentstart(DYNFILE * dfp,int s)
 }
 	
 			
-# line 839 "dynfile.dy"
+# line 853 "dynfile.dy"
 void current_indent(DYNFILE * dfp)
 {
   register int i;
@@ -925,14 +940,14 @@ void current_indent(DYNFILE * dfp)
   
 }
 	
-# line 855 "dynfile.dy"
+# line 869 "dynfile.dy"
 void startcase(DYNFILE * dfp)
 {
   flush_line(dfp);
   dfp->bracelevel++;
 }
 
-# line 861 "dynfile.dy"
+# line 875 "dynfile.dy"
 void closecase(DYNFILE * dfp)
 {
   flush_line(dfp);
@@ -942,7 +957,7 @@ void closecase(DYNFILE * dfp)
 
 
 
-# line 870 "dynfile.dy"
+# line 884 "dynfile.dy"
 void startbrace_tag(DYNFILE * dfp,char * str)
 {
   strcat(dfp->line," { ");
@@ -960,7 +975,7 @@ void startbrace_tag(DYNFILE * dfp,char * str)
   
 }
 
-# line 887 "dynfile.dy"
+# line 901 "dynfile.dy"
 void startbrace(DYNFILE * dfp)
 {
   strcat(dfp->line," { ");
@@ -968,7 +983,7 @@ void startbrace(DYNFILE * dfp)
   dfp->bracelevel++;
 }
 
-# line 894 "dynfile.dy"
+# line 908 "dynfile.dy"
 void closebrace(DYNFILE * dfp)
 {
   flush_line(dfp);
@@ -985,7 +1000,7 @@ void closebrace(DYNFILE * dfp)
 }
 
 
-# line 910 "dynfile.dy"
+# line 924 "dynfile.dy"
 void fputs_func_DYNFILE(char * str,DYNFILE * dfp)
 {
   fputs(str,dfp->func);
@@ -997,13 +1012,13 @@ void fputs_func_DYNFILE(char * str,DYNFILE * dfp)
  /** I/O stuff **/
 
 
-# line 921 "dynfile.dy"
+# line 935 "dynfile.dy"
 DYNFILE * open_std_no_html_dynfile(char * name)
 {
   return open_std_dynfile(name,NULL);
 }
 
-# line 926 "dynfile.dy"
+# line 940 "dynfile.dy"
 DYNFILE * open_std_dynfile(char * name,char * html_directory)
 {
   DYNFILE * dfp;
@@ -1072,7 +1087,7 @@ DYNFILE * open_std_dynfile(char * name,char * html_directory)
   return dfp;
 }
 
-# line 994 "dynfile.dy"
+# line 1008 "dynfile.dy"
 void close_dynfile(DYNFILE * dfp)
 {
   
@@ -1104,7 +1119,7 @@ void close_dynfile(DYNFILE * dfp)
     
 
 
-# line 1080 "dynfile.c"
+# line 1095 "dynfile.c"
 /* Function:  swap_DYNFILE(list,i,j)
  *
  * Descrip:    swap function: an internal for qsort_DYNFILE
@@ -1351,6 +1366,9 @@ DYNFILE * DYNFILE_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->desc_type = DYN_DESC_EDDY;  
     /* line[1024] is an array: no default possible */ 
     out->commentstart = 0;   
@@ -1383,6 +1401,7 @@ DYNFILE * DYNFILE_alloc(void)
  */
 DYNFILE * free_DYNFILE(DYNFILE * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1392,10 +1411,18 @@ DYNFILE * free_DYNFILE(DYNFILE * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     /* obj->*tag[128] is linked in */ 
     if( obj->sourceroot != NULL) 
       ckfree(obj->sourceroot);   
