@@ -566,6 +566,9 @@ Fblock * Fblock_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->line = NULL;    
     out->len = out->maxlen = 0;  
 
@@ -587,6 +590,7 @@ Fblock * Fblock_alloc(void)
  */
 Fblock * free_Fblock(Fblock * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -596,10 +600,18 @@ Fblock * free_Fblock(Fblock * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->line != NULL)   {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->line[i] != NULL)    
@@ -860,6 +872,9 @@ Ftext * Ftext_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->fb = NULL;  
     out->len = out->maxlen = 0;  
 
@@ -881,6 +896,7 @@ Ftext * Ftext_alloc(void)
  */
 Ftext * free_Ftext(Ftext * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -890,10 +906,18 @@ Ftext * free_Ftext(Ftext * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->fb != NULL) {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->fb[i] != NULL)  
