@@ -376,6 +376,9 @@ Code * Code_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->lines = NULL;   
     out->len = out->maxlen = 0;  
 
@@ -397,6 +400,7 @@ Code * Code_alloc(void)
  */
 Code * free_Code(Code * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -406,10 +410,18 @@ Code * free_Code(Code * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->lines != NULL)  {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->lines[i] != NULL)   
@@ -466,6 +478,9 @@ InDeclare * InDeclare_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->type = NULL;    
 
@@ -487,6 +502,7 @@ InDeclare * InDeclare_alloc(void)
  */
 InDeclare * free_InDeclare(InDeclare * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -495,10 +511,18 @@ InDeclare * free_InDeclare(InDeclare * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->type != NULL)   
@@ -552,6 +576,9 @@ InRequire * InRequire_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->type = NULL;    
     out->help = NULL;    
@@ -575,6 +602,7 @@ InRequire * InRequire_alloc(void)
  */
 InRequire * free_InRequire(InRequire * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -583,10 +611,18 @@ InRequire * free_InRequire(InRequire * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->type != NULL)   
@@ -1011,6 +1047,9 @@ Input * Input_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->decl = NULL;    
     out->decl_len = out->decl_maxlen = 0;    
     out->req = NULL; 
@@ -1035,6 +1074,7 @@ Input * Input_alloc(void)
  */
 Input * free_Input(Input * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1044,10 +1084,18 @@ Input * free_Input(Input * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->decl != NULL)   {  
       for(i=0;i<obj->decl_len;i++)   {  
         if( obj->decl[i] != NULL)    
