@@ -583,6 +583,9 @@ Transition * Transition_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->to = NULL;  
     out->from = NULL;    
     out->offi = 0;   
@@ -610,6 +613,7 @@ Transition * Transition_alloc(void)
  */
 Transition * free_Transition(Transition * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -618,10 +622,18 @@ Transition * free_Transition(Transition * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->to != NULL) 
       ckfree(obj->to);   
     if( obj->from != NULL)   
@@ -885,6 +897,9 @@ TransitionSet * TransitionSet_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->trans = NULL;   
     out->len = out->maxlen = 0;  
 
@@ -906,6 +921,7 @@ TransitionSet * TransitionSet_alloc(void)
  */
 TransitionSet * free_TransitionSet(TransitionSet * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -915,10 +931,18 @@ TransitionSet * free_TransitionSet(TransitionSet * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->trans != NULL)  {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->trans[i] != NULL)   
