@@ -243,6 +243,9 @@ LabelInstance * LabelInstance_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->state = NULL;   
     out->source = NULL;  
     out->offi = 0;   
@@ -267,6 +270,7 @@ LabelInstance * LabelInstance_alloc(void)
  */
 LabelInstance * free_LabelInstance(LabelInstance * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -275,10 +279,18 @@ LabelInstance * free_LabelInstance(LabelInstance * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->state != NULL)  
       ckfree(obj->state);    
     if( obj->source != NULL) 
@@ -538,6 +550,9 @@ Label * Label_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->type = 0;   
     out->li = NULL;  
@@ -561,6 +576,7 @@ Label * Label_alloc(void)
  */
 Label * free_Label(Label * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -570,10 +586,18 @@ Label * free_Label(Label * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->li != NULL) {  
@@ -999,6 +1023,9 @@ LabelMaster * LabelMaster_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->query = NULL;   
     out->query_len = out->query_maxlen = 0;  
     out->target = NULL;  
@@ -1022,6 +1049,7 @@ LabelMaster * LabelMaster_alloc(void)
  */
 LabelMaster * free_LabelMaster(LabelMaster * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1031,10 +1059,18 @@ LabelMaster * free_LabelMaster(LabelMaster * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->query != NULL)  {  
       for(i=0;i<obj->query_len;i++)  {  
         if( obj->query[i] != NULL)   
