@@ -582,6 +582,9 @@ MethodArg * MethodArg_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->type = NULL;    
 
 
@@ -602,6 +605,7 @@ MethodArg * MethodArg_alloc(void)
  */
 MethodArg * free_MethodArg(MethodArg * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -610,10 +614,18 @@ MethodArg * free_MethodArg(MethodArg * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->type != NULL)   
       ckfree(obj->type);     
 
@@ -869,6 +881,9 @@ Method * Method_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->logical = NULL; 
     out->real = NULL;    
     out->retstr = NULL;  
@@ -895,6 +910,7 @@ Method * Method_alloc(void)
  */
 Method * free_Method(Method * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -904,10 +920,18 @@ Method * free_Method(Method * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->logical != NULL)    
       ckfree(obj->logical);  
     if( obj->real != NULL)   
@@ -972,6 +996,9 @@ Type * Type_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->is_database = FALSE;    
     out->is_thread_safe = FALSE; 
     out->logical = NULL; 
@@ -1005,6 +1032,7 @@ Type * Type_alloc(void)
  */
 Type * free_Type(Type * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1013,10 +1041,18 @@ Type * free_Type(Type * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->logical != NULL)    
       ckfree(obj->logical);  
     if( obj->real != NULL)   
@@ -1620,6 +1656,9 @@ MethodTypeSet * MethodTypeSet_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->me = NULL;  
     out->me_len = out->me_maxlen = 0;    
     out->ty = NULL;  
@@ -1645,6 +1684,7 @@ MethodTypeSet * MethodTypeSet_alloc(void)
  */
 MethodTypeSet * free_MethodTypeSet(MethodTypeSet * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1654,10 +1694,18 @@ MethodTypeSet * free_MethodTypeSet(MethodTypeSet * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->me != NULL) {  
       for(i=0;i<obj->me_len;i++) {  
         if( obj->me[i] != NULL)  
