@@ -349,6 +349,9 @@ CommonSubExpression * CommonSubExpression_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->id = 0; 
     out->type = 0;   
     out->number = 0; 
@@ -371,6 +374,7 @@ CommonSubExpression * CommonSubExpression_alloc(void)
  */
 CommonSubExpression * free_CommonSubExpression(CommonSubExpression * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -379,10 +383,18 @@ CommonSubExpression * free_CommonSubExpression(CommonSubExpression * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     /* obj->expr is linked in */ 
 
 
@@ -637,6 +649,9 @@ CommonSubExpressionSet * CommonSubExpressionSet_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->cse = NULL; 
     out->len = out->maxlen = 0;  
 
@@ -658,6 +673,7 @@ CommonSubExpressionSet * CommonSubExpressionSet_alloc(void)
  */
 CommonSubExpressionSet * free_CommonSubExpressionSet(CommonSubExpressionSet * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -667,10 +683,18 @@ CommonSubExpressionSet * free_CommonSubExpressionSet(CommonSubExpressionSet * ob
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->cse != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->cse[i] != NULL) 
