@@ -503,6 +503,9 @@ ScopeUnit * ScopeUnit_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->app = NULL; 
     out->type = NULL;    
@@ -528,6 +531,7 @@ ScopeUnit * ScopeUnit_alloc(void)
  */
 ScopeUnit * free_ScopeUnit(ScopeUnit * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -536,10 +540,18 @@ ScopeUnit * free_ScopeUnit(ScopeUnit * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->app != NULL)    
@@ -801,6 +813,9 @@ Scope * Scope_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->su = NULL;  
     out->len = out->maxlen = 0;  
 
@@ -822,6 +837,7 @@ Scope * Scope_alloc(void)
  */
 Scope * free_Scope(Scope * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -831,10 +847,18 @@ Scope * free_Scope(Scope * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->su != NULL) {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->su[i] != NULL)  
