@@ -113,9 +113,13 @@ void write_StructHolder_typedef(DYNFILE * dfp,StructHolder * sh,FILE * ofp)
 
   start_struct(dfp,"struct %s%s",dfp->package_name == NULL ? "" : dfp->package_name,sh->name);
 
-  if( is_hard_link == TRUE ) 
+  if( is_hard_link == TRUE ) {
     struct_expr(dfp,"int dynamite_hard_link;");
-  
+    struct_macro(dfp,"#ifdef PTHREAD");
+    struct_expr(dfp,"pthread_mutex_t dynamite_mutex");
+    struct_macro(dfp,"#endif");
+  }
+
   for(i=0;i<sh->len;i++)
     write_StructElement_typedef(dfp,sh->el[i]);
 	
@@ -138,7 +142,7 @@ void write_StructHolder_typedef(DYNFILE * dfp,StructHolder * sh,FILE * ofp)
 
 }
 
-# line 174 "wisec.dy"
+# line 178 "wisec.dy"
 void write_StructElement_typedef(DYNFILE * dfp,StructElement * se)
 {
   if( se->isfunc == TRUE )
@@ -174,7 +178,7 @@ void write_StructElement_typedef(DYNFILE * dfp,StructElement * se)
 	
 }
 
-# line 209 "wisec.dy"
+# line 213 "wisec.dy"
 char * free_function(char * type)
 {
   char * runner;
@@ -183,6 +187,11 @@ char * free_function(char * type)
 
   temp = depointer_element(type);
 
+  if( temp == NULL ) {
+    warn("Unable to depointer %s",type);
+    return NULL;
+  }
+
   if( strchr(temp,'*') != NULL)
     return stringalloc("ckfree");
 
@@ -204,7 +213,7 @@ char * free_function(char * type)
 }
 
 
-# line 238 "wisec.dy"
+# line 247 "wisec.dy"
 void write_hard_link_function(DYNFILE * dfp,StructHolder * sh)
 {
   FuncInfo * fi;
@@ -231,7 +240,7 @@ void write_hard_link_function(DYNFILE * dfp,StructHolder * sh)
   add_break(dfp);
 }
 
-# line 264 "wisec.dy"
+# line 273 "wisec.dy"
 void write_free_function(DYNFILE * dfp,StructHolder * sh)
 {
   FuncInfo * fi;
@@ -255,6 +264,7 @@ void write_free_function(DYNFILE * dfp,StructHolder * sh)
 
 
   start_function_FuncInfo(fi,dfp,"%s * free_%s(%s * obj)",sh->name,sh->name,sh->name);
+  expr(dfp,"int return_early = 0;");
   if( (islist=is_listfunc(sh)) == TRUE)
     {
       expr(dfp,"int i;");
@@ -276,11 +286,21 @@ void write_free_function(DYNFILE * dfp,StructHolder * sh)
 
 
   if( is_hard_link == TRUE ) {
+    macro(dfp,"#ifdef PTHREAD");
+    expr(dfp,"assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0);");
+    macro(dfp,"#endif");
     expr(dfp,"if( obj->dynamite_hard_link > 1) ");
     startbrace(dfp);
+    expr(dfp,"return_early = 1;");
     expr(dfp,"obj->dynamite_hard_link--;");
-    expr(dfp,"return NULL;");
     closebrace(dfp);
+
+    macro(dfp,"#ifdef PTHREAD");
+    expr(dfp,"assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);");
+    macro(dfp,"#endif");
+    expr(dfp,"if( return_early == 1)");
+    hang_expr(dfp,"return NULL;");
+
   }
 
   for(i=0;i<sh->len;i++)
@@ -353,7 +373,7 @@ void write_free_function(DYNFILE * dfp,StructHolder * sh)
 }
 
 
-# line 385 "wisec.dy"
+# line 405 "wisec.dy"
 boolean is_simple_type(StructElement * se)
 {
   if( se->isfunc == TRUE )
@@ -364,7 +384,7 @@ boolean is_simple_type(StructElement * se)
   return is_simple_type_string(se->element_type);
 }
 
-# line 395 "wisec.dy"
+# line 415 "wisec.dy"
 boolean is_simple_type_string(char * str)
 {
   if( strcmp(str,"int") == 0)
@@ -382,7 +402,7 @@ boolean is_simple_type_string(char * str)
 }
 
 
-# line 412 "wisec.dy"
+# line 432 "wisec.dy"
 boolean is_string_type(StructElement * se)
 {
   if( se->isfunc == TRUE )
@@ -394,7 +414,7 @@ boolean is_string_type(StructElement * se)
   return FALSE;
 }
 
-# line 423 "wisec.dy"
+# line 443 "wisec.dy"
 boolean is_binary_dump_type(StructElement * se)
 {
   if( se->isfunc == TRUE)
@@ -408,7 +428,7 @@ boolean is_binary_dump_type(StructElement * se)
   return TRUE;
 }
 
-# line 436 "wisec.dy"
+# line 456 "wisec.dy"
 void write_binary_dump_function(DYNFILE * dfp,StructHolder * sh)
 {
   register int i;
@@ -505,7 +525,7 @@ void write_binary_dump_function(DYNFILE * dfp,StructHolder * sh)
   add_break(dfp);
 }
 
-# line 532 "wisec.dy"
+# line 552 "wisec.dy"
 void write_binary_read_function(DYNFILE * dfp,StructHolder * sh)
 {
   register int i;
@@ -624,7 +644,7 @@ void write_binary_read_function(DYNFILE * dfp,StructHolder * sh)
   add_break(dfp);
 }
 
-# line 650 "wisec.dy"
+# line 670 "wisec.dy"
 void write_expand_matrix_function(DYNFILE * dfp,StructHolder * sh)
 {
   FuncInfo * fi;
@@ -705,7 +725,7 @@ void write_expand_matrix_function(DYNFILE * dfp,StructHolder * sh)
   return; 
 }
 
-# line 730 "wisec.dy"
+# line 750 "wisec.dy"
 void write_alloc_matrix_function(DYNFILE * dfp,StructHolder * sh)
 {
   FuncInfo * fi;
@@ -804,7 +824,7 @@ void write_alloc_matrix_function(DYNFILE * dfp,StructHolder * sh)
 
 
 
-# line 828 "wisec.dy"
+# line 848 "wisec.dy"
 void write_list_sort_function(DYNFILE * dfp,StructHolder * sh,StructElement * temp)
 {
   FuncInfo * fi;
@@ -909,7 +929,7 @@ void write_list_sort_function(DYNFILE * dfp,StructHolder * sh,StructElement * te
   add_break(dfp);
 }	
 
-# line 932 "wisec.dy"
+# line 952 "wisec.dy"
 void write_list_flush_function(DYNFILE * dfp,StructHolder * sh,StructElement * temp)
 {
   FuncInfo * fi;
@@ -954,7 +974,7 @@ void write_list_flush_function(DYNFILE * dfp,StructHolder * sh,StructElement * t
 }
   
 
-# line 976 "wisec.dy"
+# line 996 "wisec.dy"
 void write_list_add_function(DYNFILE * dfp,StructHolder * sh,StructElement * temp)
 {
   FuncInfo * fi;
@@ -996,7 +1016,7 @@ void write_list_add_function(DYNFILE * dfp,StructHolder * sh,StructElement * tem
 }
 
 
-# line 1017 "wisec.dy"
+# line 1037 "wisec.dy"
 void write_list_expand_function(DYNFILE * dfp,StructHolder * sh,StructElement * temp)
 {
   FuncInfo * fi;
@@ -1053,7 +1073,7 @@ void write_list_expand_function(DYNFILE * dfp,StructHolder * sh,StructElement *
   add_break(dfp);
 }
 
-# line 1073 "wisec.dy"
+# line 1093 "wisec.dy"
 void write_alloc_std_function(DYNFILE * dfp,StructHolder * sh)
 {
   FuncInfo * fi;
@@ -1067,7 +1087,7 @@ void write_alloc_std_function(DYNFILE * dfp,StructHolder * sh)
   add_break(dfp);
 }
 
-# line 1086 "wisec.dy"
+# line 1106 "wisec.dy"
 void write_alloc_len_function(DYNFILE * dfp,StructHolder * sh)
 {
   FuncInfo * fi;
@@ -1125,7 +1145,7 @@ void write_alloc_len_function(DYNFILE * dfp,StructHolder * sh)
 } 
 
 
-# line 1143 "wisec.dy"
+# line 1163 "wisec.dy"
 char * depointer_element(char * element)
 {
   char buffer[MAXLINE];
@@ -1147,7 +1167,7 @@ char * depointer_element(char * element)
   return stringalloc(buffer);
 }
 
-# line 1164 "wisec.dy"
+# line 1184 "wisec.dy"
 boolean isarray(StructElement * se)
 {
   if( strchr(se->name,'[') != NULL )
@@ -1155,7 +1175,7 @@ boolean isarray(StructElement * se)
   return FALSE;
 }
 
-# line 1171 "wisec.dy"
+# line 1191 "wisec.dy"
 void write_simplealloc_function(DYNFILE * dfp,StructHolder * sh)
 {
   FuncInfo * fi;
@@ -1192,6 +1212,9 @@ void write_simplealloc_function(DYNFILE * dfp,StructHolder * sh)
 
   if( is_hard_link == TRUE ) {
     expr(dfp,"out->dynamite_hard_link = 1;");
+    macro(dfp,"#ifdef PTHREAD");
+    expr(dfp,"pthread_mutex_init(&(out->dynamite_mutex),NULL)");
+    macro(dfp,"#endif");
   }
 
   for(i=0;i<sh->len;i++)
@@ -1236,7 +1259,7 @@ void write_simplealloc_function(DYNFILE * dfp,StructHolder * sh)
 }
 
 
-# line 1251 "wisec.dy"
+# line 1274 "wisec.dy"
 boolean is_listfunc(StructHolder * sh)
 {
   register int i;
@@ -1249,7 +1272,7 @@ boolean is_listfunc(StructHolder * sh)
 }
 
 
-# line 1263 "wisec.dy"
+# line 1286 "wisec.dy"
 boolean is_matrixfunc(StructHolder * sh)
 {
   register int i;
@@ -1262,7 +1285,7 @@ boolean is_matrixfunc(StructHolder * sh)
 
 /* IO */
 
-# line 1275 "wisec.dy"
+# line 1298 "wisec.dy"
 char * def_from_element(char * str)
 {
   char * runner;
@@ -1289,7 +1312,7 @@ char * def_from_element(char * str)
   return stringalloc("0");
 }
 
-# line 1301 "wisec.dy"
+# line 1324 "wisec.dy"
 char * get_word_from_bang(char * str,char ** word)
 {
 
@@ -1320,7 +1343,7 @@ char * get_word_from_bang(char * str,char ** word)
   return str;
 }
 
-# line 1331 "wisec.dy"
+# line 1354 "wisec.dy"
 boolean read_StructHolder_elements(StructHolder * sh,FILE * ifp)
 {
   char buffer[MAXLINE];
@@ -1497,13 +1520,13 @@ boolean read_StructHolder_elements(StructHolder * sh,FILE * ifp)
   return TRUE;
 }
 
-# line 1507 "wisec.dy"
+# line 1530 "wisec.dy"
 void show_StructElement(StructElement * se,FILE * ofp)
 {
   fprintf(ofp,"Name [%s] Type [%s]\n",se->name,se->element_type);
 } 
 
-# line 1512 "wisec.dy"
+# line 1535 "wisec.dy"
 void show_StructHolder(StructHolder   * sh,FILE * ofp)
 {
   register int i;
@@ -1512,7 +1535,7 @@ void show_StructHolder(StructHolder   * sh,FILE * ofp)
     show_StructElement(sh->el[i],ofp);
 }
 
-# line 1520 "wisec.dy"
+# line 1543 "wisec.dy"
 StructElement * basic_add_StructElement(StructHolder * sh,char * name,char * element)
 {
   StructElement * se;
@@ -1531,7 +1554,7 @@ StructElement * basic_add_StructElement(StructHolder * sh,char * name,char * ele
 }
 
 
-# line 1533 "wisec.c"
+# line 1556 "wisec.c"
 /* Function:  hard_link_StructElement(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -1574,6 +1597,9 @@ StructElement * StructElement_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->element_type = NULL;    
     out->comment = NULL; 
@@ -1605,6 +1631,7 @@ StructElement * StructElement_alloc(void)
  */
 StructElement * free_StructElement(StructElement * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1613,10 +1640,18 @@ StructElement * free_StructElement(StructElement * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->element_type != NULL)   
@@ -1880,6 +1915,9 @@ StructHolder * StructHolder_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->el = NULL;  
     out->len = out->maxlen = 0;  
     out->name = NULL;    
@@ -1904,6 +1942,7 @@ StructHolder * StructHolder_alloc(void)
  */
 StructHolder * free_StructHolder(StructHolder * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1913,10 +1952,18 @@ StructHolder * free_StructHolder(StructHolder * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->el != NULL) {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->el[i] != NULL)  
