@@ -3,6 +3,68 @@ extern "C" {
 #endif
 #include "aln.h"
 
+
+/* Function:  collapsed_AlnBlock(alb,comp_row)
+ *
+ * Descrip:    This function builds a new "collapsed" AlnBlock
+ *             on the similarity of one AlnSeq row 
+ *
+ *
+ * Arg:             alb [UNKN ] Undocumented argument [AlnBlock *]
+ * Arg:        comp_row [UNKN ] Undocumented argument [int]
+ *
+ * Return [UNKN ]  Undocumented return value [AlnBlock *]
+ *
+ */
+# line 111 "aln.dy"
+AlnBlock * collapsed_AlnBlock(AlnBlock * alb,int comp_row)
+{
+  AlnBlock * out;
+  AlnColumn * temp = NULL;
+  AlnColumn * prev = NULL;
+  AlnColumn * ori;
+  int i;
+
+  out = AlnBlock_alloc();
+  out->start = NULL;
+
+  assert(alb);
+  assert(comp_row >= 0);
+  assert(out);
+
+  for(ori=alb->start;ori != NULL;ori = ori->next) {
+    if( temp != NULL && strcmp(temp->alu[comp_row]->text_label,ori->alu[comp_row]->text_label) == 0 ) { 
+      /* same column; eat it up */
+      for(i=0;i<ori->len;i++) {
+	temp->alu[i]->end        = ori->alu[i]->end;
+	temp->alu[i]->score[0]   += ori->alu[i]->score[0];
+      }
+      
+    } else {
+      /* new column */
+      prev = temp;
+      temp = AlnColumn_alloc_len(ori->len);
+      for(i=0;i<ori->len;i++) {
+	temp->alu[i]= AlnUnit_alloc();
+	temp->alu[i]->text_label = ori->alu[i]->text_label;
+	temp->alu[i]->start      = ori->alu[i]->start;
+	temp->alu[i]->end        = ori->alu[i]->end;
+	temp->alu[i]->score[0]   = ori->alu[i]->score[0];
+      }
+      temp->len = ori->len;
+      if( prev == NULL ) {
+	out->start = temp;
+      } else {
+	prev->next = temp;
+      }
+    } 
+      
+
+  }
+
+  return out;
+}
+
 /* Function:  add_to_anchored_AlnBlock(growing,add)
  *
  * Descrip:    This function assummes that the first AlnSequence
@@ -25,7 +87,7 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 120 "aln.dy"
+# line 173 "aln.dy"
 boolean add_to_anchored_AlnBlock(AlnBlock * growing,AlnBlock * add)
 {
   AlnColumn * anc;
@@ -185,7 +247,7 @@ boolean add_to_anchored_AlnBlock(AlnBlock * growing,AlnBlock * add)
  * Return [UNKN ]  Undocumented return value [AlnBlock *]
  *
  */
-# line 271 "aln.dy"
+# line 324 "aln.dy"
 AlnBlock * single_unit_AlnBlock(int len,char * label)
 {
   AlnBlock * out;
@@ -248,7 +310,7 @@ AlnBlock * single_unit_AlnBlock(int len,char * label)
  * Return [UNKN ]  Undocumented return value [AlnBlockList *]
  *
  */
-# line 325 "aln.dy"
+# line 378 "aln.dy"
 AlnBlockList * split_AlnBlock(AlnBlock * alb,boolean (*is_spacer_column)(const AlnColumn * alc))
 {
   AlnBlockList * out;
@@ -375,7 +437,7 @@ AlnBlockList * split_AlnBlock(AlnBlock * alb,boolean (*is_spacer_column)(const A
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 443 "aln.dy"
+# line 496 "aln.dy"
 int score_line_from_AlnBlock(AlnBlock * alb,int seqno)
 {
   AlnUnit * alu;
@@ -406,7 +468,7 @@ int score_line_from_AlnBlock(AlnBlock * alb,int seqno)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 469 "aln.dy"
+# line 522 "aln.dy"
 boolean at_end_AlnColumn(AlnColumn * alc)
 {
   if( alc->next == NULL )
@@ -426,7 +488,7 @@ boolean at_end_AlnColumn(AlnColumn * alc)
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 482 "aln.dy"
+# line 535 "aln.dy"
 int bio_start_AlnUnit(AlnUnit * alu)
 {
   if( alu->seq == NULL ) {
@@ -448,7 +510,7 @@ int bio_start_AlnUnit(AlnUnit * alu)
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 497 "aln.dy"
+# line 550 "aln.dy"
 int bio_end_AlnUnit(AlnUnit * alu)
 {
   if( alu->seq == NULL ) {
@@ -482,7 +544,7 @@ int bio_end_AlnUnit(AlnUnit * alu)
  * Return [UNKN ]  Undocumented return value [AlnColumn *]
  *
  */
-# line 525 "aln.dy"
+# line 578 "aln.dy"
 AlnColumn * swallow_AlnColumn_multiple(AlnColumn * master,AlnColumn * eaten,boolean (*comp_func)(char *,char *))
 {
   register int i;
@@ -511,7 +573,7 @@ AlnColumn * swallow_AlnColumn_multiple(AlnColumn * master,AlnColumn * eaten,bool
  * Return [UNKN ]  number of columns eaten [int]
  *
  */
-# line 549 "aln.dy"
+# line 602 "aln.dy"
 int swallow_AlnColumn_number(AlnColumn * master,AlnColumn * eaten,int num,boolean (*comp_func)(char *,char *))
 {
   register int i;
@@ -541,7 +603,7 @@ int swallow_AlnColumn_number(AlnColumn * master,AlnColumn * eaten,int num,boolea
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 573 "aln.dy"
+# line 626 "aln.dy"
 boolean swallow_AlnColumn(AlnColumn * master,AlnColumn * eaten,boolean (*comp_func)(char *,char *))
 {
   register int i;
@@ -578,7 +640,7 @@ boolean swallow_AlnColumn(AlnColumn * master,AlnColumn * eaten,boolean (*comp_fu
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 601 "aln.dy"
+# line 654 "aln.dy"
 boolean can_swallow_AlnColumn(AlnColumn * master,AlnColumn * eaten,boolean (*comp_func)(char *,char *))
 {
   register int i;
@@ -610,7 +672,7 @@ boolean can_swallow_AlnColumn(AlnColumn * master,AlnColumn * eaten,boolean (*com
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 624 "aln.dy"
+# line 677 "aln.dy"
 boolean identical_labels_in_AlnColumn(AlnColumn * one,AlnColumn * two,boolean (*comp_func)(char *,char *))
 {
   register int i;
@@ -640,7 +702,7 @@ boolean identical_labels_in_AlnColumn(AlnColumn * one,AlnColumn * two,boolean (*
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 645 "aln.dy"
+# line 698 "aln.dy"
 boolean identical_labels_in_AlnUnits(AlnUnit * one,AlnUnit * two,boolean (*comp_func)(char *,char *))
 {
   if( (*comp_func)(one->text_label,two->text_label) == TRUE )
@@ -664,7 +726,7 @@ boolean identical_labels_in_AlnUnits(AlnUnit * one,AlnUnit * two,boolean (*comp_
  * Arg:        insert [UNKN ] Undocumented argument [AlnColumn *]
  *
  */
-# line 661 "aln.dy"
+# line 714 "aln.dy"
 void replace_and_free_AlnColumn_with_one(AlnColumn * start,AlnColumn * end,AlnColumn * insert)
 {
   replace_AlnColumn_with_one(start,end,insert);
@@ -687,7 +749,7 @@ void replace_and_free_AlnColumn_with_one(AlnColumn * start,AlnColumn * end,AlnCo
  * Arg:        insert [UNKN ] Undocumented argument [AlnColumn *]
  *
  */
-# line 676 "aln.dy"
+# line 729 "aln.dy"
 void replace_AlnColumn_with_one(AlnColumn * start,AlnColumn * end,AlnColumn * insert)
 {
   start->next = insert;
@@ -709,7 +771,7 @@ void replace_AlnColumn_with_one(AlnColumn * start,AlnColumn * end,AlnColumn * in
  * Arg:        insert [UNKN ] Undocumented argument [AlnColumn *]
  *
  */
-# line 691 "aln.dy"
+# line 744 "aln.dy"
 void insert_AlnColumn(AlnColumn * start,AlnColumn * insert)
 {
   insert->next = start->next;
@@ -732,7 +794,7 @@ void insert_AlnColumn(AlnColumn * start,AlnColumn * insert)
  * Return [UNKN ]  Undocumented return value [AlnColumn *]
  *
  */
-# line 704 "aln.dy"
+# line 757 "aln.dy"
 AlnColumn * go_back_n_AlnColumn(AlnBlock * alb,AlnColumn * start,int n)
 {
   /*** really quite hacky ****/
@@ -779,7 +841,7 @@ AlnColumn * go_back_n_AlnColumn(AlnBlock * alb,AlnColumn * start,int n)
  * Arg:        ofp [UNKN ] File stream to dump to [FILE *]
  *
  */
-# line 747 "aln.dy"
+# line 800 "aln.dy"
 void dump_ascii_AlnBlock(AlnBlock * alb,FILE * ofp)
 {
   int i;
@@ -795,7 +857,7 @@ void dump_ascii_AlnBlock(AlnBlock * alb,FILE * ofp)
     fprintf(ofp,"\n");
   }
 
-  fprintf(ofp,"\\");
+  fprintf(ofp,"\\\n");
 
 }
 
@@ -811,7 +873,7 @@ void dump_ascii_AlnBlock(AlnBlock * alb,FILE * ofp)
  * Arg:                    ofp [UNKN ] File stream to dump to [FILE *]
  *
  */
-# line 773 "aln.dy"
+# line 826 "aln.dy"
 void mapped_ascii_AlnBlockList(AlnBlockList *al,double (*score_to_double)(int),FILE * ofp)
 {
   int i;
@@ -836,7 +898,7 @@ void mapped_ascii_AlnBlockList(AlnBlockList *al,double (*score_to_double)(int),F
  * Arg:                    ofp [UNKN ] File stream to dump to [FILE *]
  *
  */
-# line 792 "aln.dy"
+# line 845 "aln.dy"
 void mapped_ascii_AlnBlock(AlnBlock * alb,double (*score_to_double)(int),int do_cumlative,FILE * ofp)
 {
   int i;
@@ -878,7 +940,7 @@ void mapped_ascii_AlnBlock(AlnBlock * alb,double (*score_to_double)(int),int do_
  * Return [UNKN ]  Undocumented return value [AlnBlock *]
  *
  */
-# line 828 "aln.dy"
+# line 881 "aln.dy"
 AlnBlock * read_ascii_dump_AlnBlock(FILE * ifp)
 {
   char buffer[MAXLINE];
@@ -918,7 +980,7 @@ AlnBlock * read_ascii_dump_AlnBlock(FILE * ifp)
  * Return [UNKN ]  Undocumented return value [AlnColumn *]
  *
  */
-# line 863 "aln.dy"
+# line 916 "aln.dy"
 AlnColumn * read_dumped_ascii_AlnColumn_line(char * line)
 {
   char ** base, **bstr;
@@ -962,7 +1024,7 @@ AlnColumn * read_dumped_ascii_AlnColumn_line(char * line)
  * Arg:        ofp [UNKN ] output [FILE *]
  *
  */
-# line 904 "aln.dy"
+# line 957 "aln.dy"
 void show_flat_AlnBlock(AlnBlock * alb,FILE * ofp)
 {
   AlnColumn * alc;
@@ -983,7 +1045,7 @@ void show_flat_AlnBlock(AlnBlock * alb,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 919 "aln.dy"
+# line 972 "aln.dy"
 void show_flat_AlnColumn(AlnColumn * alc,FILE * ofp)
 {
  register int i;
@@ -1004,7 +1066,7 @@ void show_flat_AlnColumn(AlnColumn * alc,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 934 "aln.dy"
+# line 987 "aln.dy"
 void show_flat_AlnUnit(AlnUnit * alu,FILE * ofp)
 {
   fprintf(ofp,"[%4d-%4d] [%s]\n",alu->start,alu->end,alu->text_label == NULL ? "No Label" : alu->text_label);
@@ -1023,7 +1085,7 @@ void show_flat_AlnUnit(AlnUnit * alu,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [AlnUnit *]
  *
  */
-# line 945 "aln.dy"
+# line 998 "aln.dy"
 AlnUnit * read_flat_AlnUnit_line(char * line,int * ret_pos)
 {
   AlnUnit * out;
@@ -1064,7 +1126,7 @@ AlnUnit * read_flat_AlnUnit_line(char * line,int * ret_pos)
  * Return [UNKN ]  Undocumented return value [AlnColumn *]
  *
  */
-# line 978 "aln.dy"
+# line 1031 "aln.dy"
 AlnColumn * get_second_end_AlnColumn(AlnBlock * alb)
 {
   AlnColumn * end = NULL;
@@ -1088,7 +1150,7 @@ AlnColumn * get_second_end_AlnColumn(AlnBlock * alb)
  * Return [UNKN ]  Undocumented return value [AlnColumn *]
  *
  */
-# line 994 "aln.dy"
+# line 1047 "aln.dy"
 AlnColumn * get_end_AlnColumn(AlnBlock * alb)
 {
   AlnColumn * end;
@@ -1110,7 +1172,7 @@ AlnColumn * get_end_AlnColumn(AlnBlock * alb)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 1008 "aln.dy"
+# line 1061 "aln.dy"
 boolean link_AlnUnits_AlnBlock(AlnBlock * alb)
 {
   AlnSequence * aseq;
@@ -1149,7 +1211,7 @@ boolean link_AlnUnits_AlnBlock(AlnBlock * alb)
  * Return [UNKN ]  Undocumented return value [AlnColumn *]
  *
  */
-# line 1040 "aln.dy"
+# line 1093 "aln.dy"
 AlnColumn * new_pairwise_AlnColumn(void)
 {
   AlnColumn * out;
@@ -1172,7 +1234,7 @@ AlnColumn * new_pairwise_AlnColumn(void)
  * Return [UNKN ]  Undocumented return value [AlnColumn *]
  *
  */
-# line 1056 "aln.dy"
+# line 1109 "aln.dy"
 AlnColumn * free_AlnColumn(AlnColumn * obj)
 {
 
@@ -1207,7 +1269,7 @@ AlnColumn * free_AlnColumn(AlnColumn * obj)
  * Return [UNKN ]  Undocumented return value [AlnUnit *]
  *
  */
-# line 1084 "aln.dy"
+# line 1137 "aln.dy"
 AlnUnit * free_AlnUnit(AlnUnit * obj)
 {
   if( obj == NULL )
@@ -1226,7 +1288,7 @@ AlnUnit * free_AlnUnit(AlnUnit * obj)
 }
 
 
-# line 1164 "aln.c"
+# line 1224 "aln.c"
 /* Function:  hard_link_AlnUnit(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -1269,6 +1331,9 @@ AlnUnit * AlnUnit_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->start = 0;  
     out->end = 0;    
     out->label = 0;  
@@ -1526,6 +1591,9 @@ AlnColumn * AlnColumn_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->alu = NULL; 
     out->len = out->maxlen = 0;  
 
@@ -1576,6 +1644,9 @@ AlnSequence * AlnSequence_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->start = NULL;   
     out->data_type = 0;  
     out->bio_start = 1;  
@@ -1599,6 +1670,7 @@ AlnSequence * AlnSequence_alloc(void)
  */
 AlnSequence * free_AlnSequence(AlnSequence * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1607,10 +1679,18 @@ AlnSequence * free_AlnSequence(AlnSequence * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->start != NULL)  
       free_AlnUnit(obj->start);  
     /* obj->data is linked in */ 
@@ -1867,6 +1947,9 @@ AlnBlock * AlnBlock_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->start = NULL;   
     out->seq = NULL; 
     out->len = out->maxlen = 0;  
@@ -1891,6 +1974,7 @@ AlnBlock * AlnBlock_alloc(void)
  */
 AlnBlock * free_AlnBlock(AlnBlock * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1900,10 +1984,18 @@ AlnBlock * free_AlnBlock(AlnBlock * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->start != NULL)  
       free_AlnColumn(obj->start);    
     if( obj->seq != NULL)    {  
@@ -2166,6 +2258,9 @@ AlnBlockList * AlnBlockList_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->alb = NULL; 
     out->len = out->maxlen = 0;  
 
@@ -2187,6 +2282,7 @@ AlnBlockList * AlnBlockList_alloc(void)
  */
 AlnBlockList * free_AlnBlockList(AlnBlockList * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -2196,10 +2292,18 @@ AlnBlockList * free_AlnBlockList(AlnBlockList * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->alb != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->alb[i] != NULL) 
