@@ -40,6 +40,9 @@ typedef struct Wise2_AlnSequence Wise2_AlnSequence;
  */
 struct Wise2_AlnUnit {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     int start;  /*  start position in the sequence */ 
     int end;    /*  end position in the sequence */ 
     int label;  /*  not used */ 
@@ -69,6 +72,9 @@ typedef struct Wise2_AlnUnit Wise2_AlnUnit;
  */
 struct Wise2_AlnColumn {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     AlnUnit ** alu; /*  list of the AlnUnits in this column */ 
     int len;/* len for above alu  */ 
     int maxlen; /* maxlen for above alu */ 
@@ -94,6 +100,9 @@ typedef struct Wise2_AlnColumn Wise2_AlnColumn;
  */
 struct Wise2_AlnSequence {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     AlnUnit * start;    /*  the first AlnUnit of this sequence */ 
     int data_type;  /*  not used */ 
     void * data;    /*  not used - don't use! */ 
@@ -122,6 +131,9 @@ typedef struct Wise2_AlnSequence Wise2_AlnSequence;
  */
 struct Wise2_AlnBlock {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     AlnColumn * start;  /*  the first AlnColumn in the alignment */ 
     AlnSequence ** seq; /*  a list of AlnSequences in the alignment */ 
     int len;/* len for above seq  */ 
@@ -144,6 +156,9 @@ typedef struct Wise2_AlnBlock Wise2_AlnBlock;
  */
 struct Wise2_AlnBlockList {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     AlnBlock ** alb;     
     int len;/* len for above alb  */ 
     int maxlen; /* maxlen for above alb */ 
@@ -165,6 +180,22 @@ typedef struct Wise2_AlnBlockList Wise2_AlnBlockList;
 
 
 
+/* Function:  collapsed_AlnBlock(alb,comp_row)
+ *
+ * Descrip:    This function builds a new "collapsed" AlnBlock
+ *             on the similarity of one AlnSeq row 
+ *
+ *
+ * Arg:             alb [UNKN ] Undocumented argument [AlnBlock *]
+ * Arg:        comp_row [UNKN ] Undocumented argument [int]
+ *
+ * Return [UNKN ]  Undocumented return value [AlnBlock *]
+ *
+ */
+AlnBlock * Wise2_collapsed_AlnBlock(AlnBlock * alb,int comp_row);
+#define collapsed_AlnBlock Wise2_collapsed_AlnBlock
+
+
 /* Function:  add_to_anchored_AlnBlock(growing,add)
  *
  * Descrip:    This function assummes that the first AlnSequence
@@ -966,78 +997,78 @@ AlnBlockList * Wise2_free_AlnBlockList(AlnBlockList * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
-AlnSequence * Wise2_access_seq_AlnBlock(AlnBlock * obj,int i);
-#define access_seq_AlnBlock Wise2_access_seq_AlnBlock
+AlnColumn * Wise2_access_start_AlnBlock(AlnBlock * obj);
+#define access_start_AlnBlock Wise2_access_start_AlnBlock
 AlnUnit * Wise2_access_alu_AlnColumn(AlnColumn * obj,int i);
 #define access_alu_AlnColumn Wise2_access_alu_AlnColumn
 int Wise2_length_alu_AlnColumn(AlnColumn * obj);
 #define length_alu_AlnColumn Wise2_length_alu_AlnColumn
-int Wise2_access_start_AlnUnit(AlnUnit * obj);
-#define access_start_AlnUnit Wise2_access_start_AlnUnit
-int Wise2_access_bio_end_AlnSequence(AlnSequence * obj);
-#define access_bio_end_AlnSequence Wise2_access_bio_end_AlnSequence
-boolean Wise2_replace_end_AlnUnit(AlnUnit * obj,int end);
-#define replace_end_AlnUnit Wise2_replace_end_AlnUnit
-boolean Wise2_replace_label_AlnUnit(AlnUnit * obj,int label);
-#define replace_label_AlnUnit Wise2_replace_label_AlnUnit
 int Wise2_access_label_AlnUnit(AlnUnit * obj);
 #define access_label_AlnUnit Wise2_access_label_AlnUnit
 boolean Wise2_replace_text_label_AlnUnit(AlnUnit * obj,char * text_label);
 #define replace_text_label_AlnUnit Wise2_replace_text_label_AlnUnit
-char * Wise2_access_text_label_AlnUnit(AlnUnit * obj);
-#define access_text_label_AlnUnit Wise2_access_text_label_AlnUnit
-AlnColumn * Wise2_access_start_AlnBlock(AlnBlock * obj);
-#define access_start_AlnBlock Wise2_access_start_AlnBlock
-boolean Wise2_replace_next_AlnUnit(AlnUnit * obj,AlnUnit * next);
-#define replace_next_AlnUnit Wise2_replace_next_AlnUnit
-boolean Wise2_replace_length_AlnBlock(AlnBlock * obj,int length);
-#define replace_length_AlnBlock Wise2_replace_length_AlnBlock
-AlnUnit * Wise2_access_next_AlnUnit(AlnUnit * obj);
-#define access_next_AlnUnit Wise2_access_next_AlnUnit
-boolean Wise2_replace_score_AlnBlock(AlnBlock * obj,int score);
-#define replace_score_AlnBlock Wise2_replace_score_AlnBlock
-boolean Wise2_replace_in_column_AlnUnit(AlnUnit * obj,boolean in_column);
-#define replace_in_column_AlnUnit Wise2_replace_in_column_AlnUnit
-boolean Wise2_replace_next_AlnColumn(AlnColumn * obj,AlnColumn * next);
-#define replace_next_AlnColumn Wise2_replace_next_AlnColumn
-boolean Wise2_access_in_column_AlnUnit(AlnUnit * obj);
-#define access_in_column_AlnUnit Wise2_access_in_column_AlnUnit
-boolean Wise2_replace_start_AlnUnit(AlnUnit * obj,int start);
-#define replace_start_AlnUnit Wise2_replace_start_AlnUnit
-boolean Wise2_replace_seq_AlnUnit(AlnUnit * obj,AlnSequence * seq);
-#define replace_seq_AlnUnit Wise2_replace_seq_AlnUnit
-boolean Wise2_replace_bio_end_AlnSequence(AlnSequence * obj,int bio_end);
-#define replace_bio_end_AlnSequence Wise2_replace_bio_end_AlnSequence
-AlnSequence * Wise2_access_seq_AlnUnit(AlnUnit * obj);
-#define access_seq_AlnUnit Wise2_access_seq_AlnUnit
-int Wise2_access_bio_start_AlnSequence(AlnSequence * obj);
-#define access_bio_start_AlnSequence Wise2_access_bio_start_AlnSequence
-boolean Wise2_replace_start_AlnSequence(AlnSequence * obj,AlnUnit * start);
-#define replace_start_AlnSequence Wise2_replace_start_AlnSequence
-int Wise2_access_length_AlnBlock(AlnBlock * obj);
-#define access_length_AlnBlock Wise2_access_length_AlnBlock
-AlnUnit * Wise2_access_start_AlnSequence(AlnSequence * obj);
-#define access_start_AlnSequence Wise2_access_start_AlnSequence
-AlnColumn * Wise2_access_next_AlnColumn(AlnColumn * obj);
-#define access_next_AlnColumn Wise2_access_next_AlnColumn
-boolean Wise2_replace_data_type_AlnSequence(AlnSequence * obj,int data_type);
-#define replace_data_type_AlnSequence Wise2_replace_data_type_AlnSequence
 boolean Wise2_replace_start_AlnBlock(AlnBlock * obj,AlnColumn * start);
 #define replace_start_AlnBlock Wise2_replace_start_AlnBlock
-int Wise2_access_data_type_AlnSequence(AlnSequence * obj);
-#define access_data_type_AlnSequence Wise2_access_data_type_AlnSequence
-int Wise2_access_score_AlnBlock(AlnBlock * obj);
-#define access_score_AlnBlock Wise2_access_score_AlnBlock
-boolean Wise2_replace_data_AlnSequence(AlnSequence * obj,void * data);
-#define replace_data_AlnSequence Wise2_replace_data_AlnSequence
+boolean Wise2_replace_bio_end_AlnSequence(AlnSequence * obj,int bio_end);
+#define replace_bio_end_AlnSequence Wise2_replace_bio_end_AlnSequence
+char * Wise2_access_text_label_AlnUnit(AlnUnit * obj);
+#define access_text_label_AlnUnit Wise2_access_text_label_AlnUnit
 int Wise2_length_seq_AlnBlock(AlnBlock * obj);
 #define length_seq_AlnBlock Wise2_length_seq_AlnBlock
-void * Wise2_access_data_AlnSequence(AlnSequence * obj);
-#define access_data_AlnSequence Wise2_access_data_AlnSequence
+boolean Wise2_replace_next_AlnUnit(AlnUnit * obj,AlnUnit * next);
+#define replace_next_AlnUnit Wise2_replace_next_AlnUnit
+int Wise2_access_length_AlnBlock(AlnBlock * obj);
+#define access_length_AlnBlock Wise2_access_length_AlnBlock
+AlnUnit * Wise2_access_next_AlnUnit(AlnUnit * obj);
+#define access_next_AlnUnit Wise2_access_next_AlnUnit
+int Wise2_access_score_AlnBlock(AlnBlock * obj);
+#define access_score_AlnBlock Wise2_access_score_AlnBlock
+boolean Wise2_replace_in_column_AlnUnit(AlnUnit * obj,boolean in_column);
+#define replace_in_column_AlnUnit Wise2_replace_in_column_AlnUnit
+AlnColumn * Wise2_access_next_AlnColumn(AlnColumn * obj);
+#define access_next_AlnColumn Wise2_access_next_AlnColumn
+boolean Wise2_access_in_column_AlnUnit(AlnUnit * obj);
+#define access_in_column_AlnUnit Wise2_access_in_column_AlnUnit
+int Wise2_access_start_AlnUnit(AlnUnit * obj);
+#define access_start_AlnUnit Wise2_access_start_AlnUnit
+boolean Wise2_replace_seq_AlnUnit(AlnUnit * obj,AlnSequence * seq);
+#define replace_seq_AlnUnit Wise2_replace_seq_AlnUnit
 int Wise2_access_end_AlnUnit(AlnUnit * obj);
 #define access_end_AlnUnit Wise2_access_end_AlnUnit
+AlnSequence * Wise2_access_seq_AlnUnit(AlnUnit * obj);
+#define access_seq_AlnUnit Wise2_access_seq_AlnUnit
+int Wise2_access_bio_end_AlnSequence(AlnSequence * obj);
+#define access_bio_end_AlnSequence Wise2_access_bio_end_AlnSequence
+boolean Wise2_replace_start_AlnSequence(AlnSequence * obj,AlnUnit * start);
+#define replace_start_AlnSequence Wise2_replace_start_AlnSequence
+boolean Wise2_replace_length_AlnBlock(AlnBlock * obj,int length);
+#define replace_length_AlnBlock Wise2_replace_length_AlnBlock
+AlnUnit * Wise2_access_start_AlnSequence(AlnSequence * obj);
+#define access_start_AlnSequence Wise2_access_start_AlnSequence
+boolean Wise2_replace_next_AlnColumn(AlnColumn * obj,AlnColumn * next);
+#define replace_next_AlnColumn Wise2_replace_next_AlnColumn
+boolean Wise2_replace_data_type_AlnSequence(AlnSequence * obj,int data_type);
+#define replace_data_type_AlnSequence Wise2_replace_data_type_AlnSequence
+boolean Wise2_replace_end_AlnUnit(AlnUnit * obj,int end);
+#define replace_end_AlnUnit Wise2_replace_end_AlnUnit
+int Wise2_access_data_type_AlnSequence(AlnSequence * obj);
+#define access_data_type_AlnSequence Wise2_access_data_type_AlnSequence
+AlnSequence * Wise2_access_seq_AlnBlock(AlnBlock * obj,int i);
+#define access_seq_AlnBlock Wise2_access_seq_AlnBlock
+boolean Wise2_replace_data_AlnSequence(AlnSequence * obj,void * data);
+#define replace_data_AlnSequence Wise2_replace_data_AlnSequence
+boolean Wise2_replace_start_AlnUnit(AlnUnit * obj,int start);
+#define replace_start_AlnUnit Wise2_replace_start_AlnUnit
+void * Wise2_access_data_AlnSequence(AlnSequence * obj);
+#define access_data_AlnSequence Wise2_access_data_AlnSequence
+boolean Wise2_replace_score_AlnBlock(AlnBlock * obj,int score);
+#define replace_score_AlnBlock Wise2_replace_score_AlnBlock
 boolean Wise2_replace_bio_start_AlnSequence(AlnSequence * obj,int bio_start);
 #define replace_bio_start_AlnSequence Wise2_replace_bio_start_AlnSequence
+boolean Wise2_replace_label_AlnUnit(AlnUnit * obj,int label);
+#define replace_label_AlnUnit Wise2_replace_label_AlnUnit
+int Wise2_access_bio_start_AlnSequence(AlnSequence * obj);
+#define access_bio_start_AlnSequence Wise2_access_bio_start_AlnSequence
 boolean Wise2_can_swallow_AlnColumn(AlnColumn * master,AlnColumn * eaten,boolean (*comp_func)(char *,char *));
 #define can_swallow_AlnColumn Wise2_can_swallow_AlnColumn
 boolean Wise2_identical_labels_in_AlnColumn(AlnColumn * one,AlnColumn * two,boolean (*comp_func)(char *,char *));
