@@ -118,6 +118,9 @@ AlnRange * AlnRange_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->starti = 0; 
     out->startj = 0; 
     out->startstate = 0; 
@@ -145,6 +148,7 @@ AlnRange * AlnRange_alloc(void)
  */
 AlnRange * free_AlnRange(AlnRange * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -153,10 +157,18 @@ AlnRange * free_AlnRange(AlnRange * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -410,6 +422,9 @@ AlnRangeSet * AlnRangeSet_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->score = 0;  
     out->alr = NULL; 
     out->len = out->maxlen = 0;  
@@ -432,6 +447,7 @@ AlnRangeSet * AlnRangeSet_alloc(void)
  */
 AlnRangeSet * free_AlnRangeSet(AlnRangeSet * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -441,10 +457,18 @@ AlnRangeSet * free_AlnRangeSet(AlnRangeSet * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->alr != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->alr[i] != NULL) 
