@@ -463,6 +463,9 @@ cDNADB * cDNADB_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->is_single_seq = FALSE;  
     out->done_forward = FALSE;   
     out->forward_only = FALSE;   
@@ -492,6 +495,7 @@ cDNADB * cDNADB_alloc(void)
  */
 cDNADB * free_cDNADB(cDNADB * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -500,10 +504,18 @@ cDNADB * free_cDNADB(cDNADB * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->forw != NULL)   
       free_ComplexSequence(obj->forw);   
     if( obj->rev != NULL)    
