@@ -3,6 +3,7 @@ extern "C" {
 #endif
 #include "codon.h"
 
+
 /* Function:  read_CodonTable_file(file)
  *
  * Descrip:    Opens filename, reads it as if a Ewan style
@@ -14,7 +15,7 @@ extern "C" {
  * Return [OWNER]  A codon-table, NULL if error [CodonTable *]
  *
  */
-# line 77 "codon.dy"
+# line 80 "codon.dy"
 CodonTable * read_CodonTable_file(char * file)
 {
   FILE * ifp;
@@ -49,7 +50,7 @@ CodonTable * read_CodonTable_file(char * file)
  * Return [UNKN ]  Undocumented return value [CodonTable *]
  *
  */
-# line 106 "codon.dy"
+# line 109 "codon.dy"
 CodonTable * read_CodonTable(FILE * ifp)
 {
   char buffer[MAXLINE];
@@ -92,7 +93,7 @@ CodonTable * read_CodonTable(FILE * ifp)
  * Return [UNKN ]  Undocumented return value [char *]
  *
  */
-# line 144 "codon.dy"
+# line 147 "codon.dy"
 char * alloc_aminoacid_from_seq(CodonTable * ct,char * seq)
 {
   char buf[2];
@@ -118,7 +119,7 @@ char * alloc_aminoacid_from_seq(CodonTable * ct,char * seq)
  * Return [UNKN ]  an amino acid char (A-Z) [aa]
  *
  */
-# line 165 "codon.dy"
+# line 168 "codon.dy"
 aa aminoacid_from_seq(CodonTable * ct,char * seq)
 {
   return aminoacid_from_codon(ct,codon_from_seq(seq));
@@ -135,7 +136,7 @@ aa aminoacid_from_seq(CodonTable * ct,char * seq)
  * Return [READ ]  aminoacid that is this codon (X for ambiguous, * for stop) [aa]
  *
  */
-# line 177 "codon.dy"
+# line 180 "codon.dy"
 aa aminoacid_from_codon(CodonTable * ct,codon c)
 {
   return ct->codon_str[c];
@@ -153,7 +154,7 @@ aa aminoacid_from_codon(CodonTable * ct,codon c)
  * Return [READ ]  aminoacid number [0-26] for this codon [int]
  *
  */
-# line 190 "codon.dy"
+# line 193 "codon.dy"
 int aminoacid_no_from_codon(CodonTable * ct,codon c)
 {
   return (ct->codon_str[c] - 'A');
@@ -171,7 +172,7 @@ int aminoacid_no_from_codon(CodonTable * ct,codon c)
  * Return [UNKN ]  TRUE if is stop, FALSE otherwise [boolean]
  *
  */
-# line 203 "codon.dy"
+# line 206 "codon.dy"
 boolean is_stop_codon(codon c,CodonTable * ct)
 {
   aa a;
@@ -195,7 +196,7 @@ boolean is_stop_codon(codon c,CodonTable * ct)
  * Return [UNKN ]  TRUE if real codon, FALSE if contains N's [boolean]
  *
  */
-# line 222 "codon.dy"
+# line 225 "codon.dy"
 boolean is_non_ambiguous_codon_seq(char * seq)
 {
   if( *seq == '\0' || *(seq+1) == '\0' || *(seq+2) == '\0') {
@@ -225,7 +226,7 @@ boolean is_non_ambiguous_codon_seq(char * seq)
  * Return [UNKN ]  TRUE if valid, FALSE if not. [boolean]
  *
  */
-# line 247 "codon.dy"
+# line 250 "codon.dy"
 boolean is_valid_aminoacid(CodonTable * ct,char c)
 {
   if( strchr(ct->codon_str,c) != NULL )
@@ -243,7 +244,7 @@ boolean is_valid_aminoacid(CodonTable * ct,char c)
  * Return [UNKN ]  TRUE if (ATGCN) FALSE otherwise [boolean]
  *
  */
-# line 260 "codon.dy"
+# line 263 "codon.dy"
 boolean is_valid_base_char(char c)
 {
   if( c == 'A' || c == 'T' || c == 'G' || c == 'C' || c == 'N')
@@ -261,7 +262,7 @@ boolean is_valid_base_char(char c)
  * Return [UNKN ]  Undocumented return value [codon]
  *
  */
-# line 270 "codon.dy"
+# line 273 "codon.dy"
 codon codon_from_base4_codon(int c)
 {
   base one;
@@ -290,7 +291,7 @@ codon codon_from_base4_codon(int c)
  * Return [UNKN ]  base 4 codon (0-63) [int]
  *
  */
-# line 294 "codon.dy"
+# line 297 "codon.dy"
 int base4_codon_from_codon(codon c)
 {
   base one;
@@ -319,7 +320,7 @@ int base4_codon_from_codon(codon c)
  * Return [UNKN ]  TRUE if has N's , FALSE otherwise [boolean]
  *
  */
-# line 318 "codon.dy"
+# line 321 "codon.dy"
 boolean has_random_bases(codon c)
 {
   base o;
@@ -354,7 +355,7 @@ boolean has_random_bases(codon c)
  * Return [UNKN ]  codon number  [codon]
  *
  */
-# line 348 "codon.dy"
+# line 351 "codon.dy"
 codon permute_possible_random_bases(codon c,base one,base two,base three)
 {
   base o;
@@ -390,7 +391,7 @@ codon permute_possible_random_bases(codon c,base one,base two,base three)
  * Arg:        three [UNKN ] Undocumented argument [base *]
  *
  */
-# line 375 "codon.dy"
+# line 378 "codon.dy"
 void all_bases_from_codon(codon c,base * one,base * two,base * three)
 {
   base o;
@@ -414,6 +415,33 @@ void all_bases_from_codon(codon c,base * one,base * two,base * three)
 
 }
 
+/* Function:  reverse_codon(c)
+ *
+ * Descrip:    Reverses codon. Takes a forward codon number and
+ *             builds the inverted codon number
+ *
+ *
+ * Arg:        c [UNKN ] Undocumented argument [codon]
+ *
+ * Return [UNKN ]  Undocumented return value [codon]
+ *
+ */
+# line 405 "codon.dy"
+codon reverse_codon(codon c)
+{
+  base one;
+  base two;
+  base three;
+
+  all_bases_from_codon(c,&one,&two,&three);
+
+  one = complement_base(one);
+  two = complement_base(two);
+  three = complement_base(three);
+  
+  return three*125 + two * 25 + one;
+}
+
 
 /* Function:  base_from_codon(c,pos)
  *
@@ -428,7 +456,7 @@ void all_bases_from_codon(codon c,base * one,base * two,base * three)
  * Return [UNKN ]  Undocumented return value [base]
  *
  */
-# line 404 "codon.dy"
+# line 426 "codon.dy"
 base  base_from_codon(codon c,int pos)
 {
   base one;
@@ -469,16 +497,23 @@ base  base_from_codon(codon c,int pos)
  * Return [UNKN ]  Undocumented return value [codon]
  *
  */
-# line 439 "codon.dy"
+# line 461 "codon.dy"
 codon codon_from_seq(char * seq)
 {
   base one;
   base two;
   base three;
 
-  one = base_from_char(*seq);
-  two = base_from_char(*(seq+1));
-  three = base_from_char(*(seq+2));
+  if( !is_base(*seq) || !is_base(*(seq+1)) || !is_base (*(seq+2)) ) {
+    warn("Attempting to translate bad base %c%c%c",seq[0],seq[1],seq[2]);
+    one = BASE_N;
+    two = BASE_N;
+    three = BASE_N;
+  } else {
+    one = base_from_char(*seq);
+    two = base_from_char(*(seq+1));
+    three = base_from_char(*(seq+2));
+  }
 
   return one*25+two*5+three;
 }
@@ -496,7 +531,7 @@ codon codon_from_seq(char * seq)
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 460 "codon.dy"
+# line 489 "codon.dy"
 int base4_codon_from_seq(char * seq)
 {
   base one;
@@ -523,7 +558,7 @@ int base4_codon_from_seq(char * seq)
  * Return [UNKN ]  Undocumented return value [char]
  *
  */
-# line 479 "codon.dy"
+# line 508 "codon.dy"
 char char_from_base(base b)
 {
   switch(b) {
@@ -549,7 +584,7 @@ char char_from_base(base b)
  * Return [UNKN ]  Undocumented return value [base]
  *
  */
-# line 497 "codon.dy"
+# line 526 "codon.dy"
 base base_from_char(char c)
 {
   c = (char)toupper((int)c);
@@ -576,9 +611,13 @@ base base_from_char(char c)
  * Return [UNKN ]  Undocumented return value [char]
  *
  */
-# line 516 "codon.dy"
+# line 545 "codon.dy"
 char char_complement_base(char c)
 {
+  if( c == '-' || c == '.' || c == '~' ) {
+    return c;
+  }
+
   return char_from_base(complement_base(base_from_char(c)));
 }
 
@@ -593,7 +632,7 @@ char char_complement_base(char c)
  * Return [UNKN ]  Undocumented return value [base]
  *
  */
-# line 525 "codon.dy"
+# line 558 "codon.dy"
 base complement_base(base b)
 {
   switch(b) {
@@ -609,8 +648,57 @@ base complement_base(base b)
 
 
 
+/* Function:  four_fold_sites_CodonTable(*ct,seq)
+ *
+ * Descrip:    returns the number of four fold degenerate
+ *             sites in this codon
+ *
+ *
+ * Arg:        *ct [UNKN ] Undocumented argument [CodonTable]
+ * Arg:        seq [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [int]
+ *
+ */
+# line 577 "codon.dy"
+int four_fold_sites_CodonTable(CodonTable *ct,char * seq)
+{
+  int site_count;
+  int i;
+  int j;
+  char codon[3];
+  char aa;
+  
 
-# line 568 "codon.c"
+  assert(ct);
+
+  codon[0] = seq[0];
+  codon[1] = seq[1];
+  codon[2] = seq[2];
+
+  aa = aminoacid_from_seq(ct,codon);
+  
+  site_count = 0;
+
+  for(i=0;i<3;i++) {
+    for( j=0;j<4;j++) {
+      codon[i] = char_from_base(j);
+      if( aa != aminoacid_from_seq(ct,codon) ) {
+	break;
+      }
+    }
+    if( j >= 4 ) {
+      site_count++;
+    }
+    codon[i] = seq[i];
+  }
+
+  return site_count;
+
+}
+
+
+# line 652 "codon.c"
 /* Function:  hard_link_CodonTable(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -653,6 +741,9 @@ CodonTable * CodonTable_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* codon_str[125] is an array: no default possible */ 
     out->name = NULL;    
 
@@ -674,6 +765,7 @@ CodonTable * CodonTable_alloc(void)
  */
 CodonTable * free_CodonTable(CodonTable * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -682,10 +774,18 @@ CodonTable * free_CodonTable(CodonTable * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
 
