@@ -331,6 +331,9 @@ ComplexConsensusWord * ComplexConsensusWord_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->pattern = NULL; 
     out->score = 0;  
     out->p = 0.0;    
@@ -353,6 +356,7 @@ ComplexConsensusWord * ComplexConsensusWord_alloc(void)
  */
 ComplexConsensusWord * free_ComplexConsensusWord(ComplexConsensusWord * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -361,10 +365,18 @@ ComplexConsensusWord * free_ComplexConsensusWord(ComplexConsensusWord * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->pattern != NULL)    
       ckfree(obj->pattern);  
 
@@ -620,6 +632,9 @@ ComplexConsensi * ComplexConsensi_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->ccw = NULL; 
     out->len = out->maxlen = 0;  
     out->name = NULL;    
@@ -642,6 +657,7 @@ ComplexConsensi * ComplexConsensi_alloc(void)
  */
 ComplexConsensi * free_ComplexConsensi(ComplexConsensi * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -651,10 +667,18 @@ ComplexConsensi * free_ComplexConsensi(ComplexConsensi * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->ccw != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->ccw[i] != NULL) 
