@@ -4,7 +4,80 @@ extern "C" {
 #include "complexsequence.h"
 
 
+/* Function:  reversed_ComplexSequence(forward,cses)
+ *
+ * Descrip: No Description
+ *
+ * Arg:        forward [UNKN ] Undocumented argument [Sequence *]
+ * Arg:           cses [UNKN ] Undocumented argument [ComplexSequenceEvalSet *]
+ *
+ * Return [UNKN ]  Undocumented return value [ComplexSequence *]
+ *
+ */
+# line 102 "complexsequence.dy"
+ComplexSequence * reversed_ComplexSequence(Sequence * forward,ComplexSequenceEvalSet * cses) 
+{
+  ComplexSequence * out;
+  register int i;
+  register int j;
+  register int * poi;
+  Sequence * rev;
+  int seqlen;
 
+  if( forward == NULL ) {
+    warn("Trying to make a complex sequence from a NULL sequence - impossible!");
+    return NULL;
+  }
+
+  if( can_evaluate_this_Sequence(cses,forward) == FALSE ) {
+    warn("Could not evaluate these sequences Sequence type [%d][%s] Evaluation type [%d][%s]",forward->type,Sequence_type_to_string(forward->type),cses->type,Sequence_type_to_string(cses->type));
+    return NULL;
+  }
+
+  if( cses->has_been_prepared == FALSE) {
+    warn("Trappable error: you have not prepared this ComplexSequenceEvalSet before using. Please do so in the future");
+    prepare_ComplexSequenceEvalSet(cses);
+  }
+
+  rev = reverse_complement_Sequence(forward);
+
+  out = ComplexSequence_alloc();
+  if( out == NULL )
+    return NULL;
+
+  out->creator = hard_link_ComplexSequenceEvalSet(cses);
+
+  out->datastore = (int *) ckcalloc((forward->len+cses->left_lookback)*cses->len,sizeof(int));
+
+  if( out->datastore == NULL ) {
+    warn("Could not allocate data pointer of length %d for ComplexSequence",forward->len*cses->len);
+    free_ComplexSequence(out);
+    return NULL;
+  }
+
+  out->data  = out->datastore + (cses->left_lookback * cses->len);
+
+  for(i=0;i<cses->left_lookback;i++) {
+    for(j=0;j<cses->len;j++) 
+      out->datastore[(i*cses->len)+j] = cses->cse[j]->outside_score;
+      }
+
+  out->depth = cses->len;
+  seqlen = forward->len;
+
+  for(i=0,poi = out->data;i<forward->len;i++,poi = next_ComplexSequence_data(out,poi)) {
+    for(j=0;j<cses->len;j++) {
+      /*      fprintf(stderr,"Calling with i at %d vs %d\n",i,cses->cse[j]->left_window); */
+      if( i < cses->cse[j]->right_window || (i + cses->cse[j]->left_window) >= seqlen)
+	poi[j] = cses->cse[j]->outside_score;
+      else poi[j] = (*cses->cse[j]->eval_func)(cses->cse[j]->data_type,cses->cse[j]->data,rev->seq+seqlen-i);
+    }
+  }
+
+  
+  return out;
+
+}
 
 /* Function:  new_ComplexSequence(seq,cses)
  *
@@ -20,7 +93,7 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [ComplexSequence *]
  *
  */
-# line 107 "complexsequence.dy"
+# line 175 "complexsequence.dy"
 ComplexSequence * new_ComplexSequence(Sequence * seq,ComplexSequenceEvalSet * cses)
 {
   ComplexSequence * out;
@@ -94,11 +167,14 @@ ComplexSequence * new_ComplexSequence(Sequence * seq,ComplexSequenceEvalSet * cs
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 175 "complexsequence.dy"
+# line 243 "complexsequence.dy"
 void show_ComplexSequence(ComplexSequence * cs,FILE * ofp)
 {
   register int i;
-
+	
+  assert(cs);
+  assert(ofp);
+  assert(cs->seq);
   fprintf(ofp,"ComplexSequence %s\n",cs->seq->name);
   for(i=0;i<cs->length;i++) {
     show_one_position_ComplexSequence(cs,i,ofp);
@@ -116,18 +192,25 @@ void show_ComplexSequence(ComplexSequence * cs,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 191 "complexsequence.dy"
+# line 262 "complexsequence.dy"
 void show_one_position_ComplexSequence(ComplexSequence * cs,int pos,FILE * ofp)
 {
   int i;
 
+  assert(cs);
+  assert(ofp);
+
   fprintf(ofp,"%4d  %c [",pos,cs->seq->seq[pos]);
 
   for(i=0;i<cs->depth;i++) {
-    if( cs->creator->cse[i]->score_type == CseScoreType_Index ) {
-      fprintf(ofp,"%4d%c",ComplexSequence_data(cs,pos,i),i == cs->depth-1 ? ']' : ',');
+    if( cs->creator != NULL ) {
+      if( cs->creator->cse[i]->score_type == CseScoreType_Index ) {
+	fprintf(ofp,"%4d%c",ComplexSequence_data(cs,pos,i),i == cs->depth-1 ? ']' : ',');
+      } else {
+	fprintf(ofp,"% 6d (%0+2.2f)%c",ComplexSequence_data(cs,pos,i),Score2Bits(ComplexSequence_data(cs,pos,i)), i == cs->depth-1 ? ']' : ',');
+      }
     } else {
-      fprintf(ofp,"% 6d (%0+2.2f)%c",ComplexSequence_data(cs,pos,i),Score2Bits(ComplexSequence_data(cs,pos,i)), i == cs->depth-1 ? ']' : ',');
+      fprintf(ofp,"%4d%c",ComplexSequence_data(cs,pos,i),i == cs->depth-1 ? ']' : ',');
     }
   }
   fprintf(ofp,"\n");
@@ -150,7 +233,7 @@ void show_one_position_ComplexSequence(ComplexSequence * cs,int pos,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 218 "complexsequence.dy"
+# line 296 "complexsequence.dy"
 boolean prepare_ComplexSequenceEvalSet(ComplexSequenceEvalSet * cses)
 {
   register int i;
@@ -189,7 +272,7 @@ boolean prepare_ComplexSequenceEvalSet(ComplexSequenceEvalSet * cses)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 249 "complexsequence.dy"
+# line 327 "complexsequence.dy"
 boolean can_evaluate_this_Sequence(ComplexSequenceEvalSet * cses,Sequence * s)
 {
   return can_evaluate_this_type(cses,s->type);
@@ -207,7 +290,7 @@ boolean can_evaluate_this_Sequence(ComplexSequenceEvalSet * cses,Sequence * s)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 260 "complexsequence.dy"
+# line 338 "complexsequence.dy"
 boolean can_evaluate_this_type(ComplexSequenceEvalSet * cses,int type)
 {
   switch (type) {
@@ -226,7 +309,7 @@ boolean can_evaluate_this_type(ComplexSequenceEvalSet * cses,int type)
 
     
 
-# line 216 "complexsequence.c"
+# line 297 "complexsequence.c"
 /* Function:  hard_link_ComplexSequenceEval(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -269,6 +352,9 @@ ComplexSequenceEval * ComplexSequenceEval_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->type = 0;   
     out->sequence_type = 0;  
     out->left_window = 0;    
@@ -297,6 +383,7 @@ ComplexSequenceEval * ComplexSequenceEval_alloc(void)
  */
 ComplexSequenceEval * free_ComplexSequenceEval(ComplexSequenceEval * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -305,10 +392,18 @@ ComplexSequenceEval * free_ComplexSequenceEval(ComplexSequenceEval * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     /* obj->data is linked in */ 
     /* obj->eval_func is a function pointer */ 
 
@@ -564,6 +659,9 @@ ComplexSequenceEvalSet * ComplexSequenceEvalSet_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->type = 0;   
     out->has_been_prepared = FALSE;  
     out->left_window = 0;    
@@ -590,6 +688,7 @@ ComplexSequenceEvalSet * ComplexSequenceEvalSet_alloc(void)
  */
 ComplexSequenceEvalSet * free_ComplexSequenceEvalSet(ComplexSequenceEvalSet * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -599,10 +698,18 @@ ComplexSequenceEvalSet * free_ComplexSequenceEvalSet(ComplexSequenceEvalSet * ob
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->cse != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->cse[i] != NULL) 
@@ -659,6 +766,9 @@ ComplexSequence * ComplexSequence_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->type = 0;   
     out->seq = NULL; 
     out->datastore = NULL;   
@@ -684,6 +794,7 @@ ComplexSequence * ComplexSequence_alloc(void)
  */
 ComplexSequence * free_ComplexSequence(ComplexSequence * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -692,10 +803,18 @@ ComplexSequence * free_ComplexSequence(ComplexSequence * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->seq != NULL)    
       free_Sequence(obj->seq);   
     /* obj->data is linked in */ 
