@@ -4,6 +4,111 @@ extern "C" {
 #include "compmat.h"
 
 
+/* Function:  simple_CompProb(match,rnd)
+ *
+ * Descrip:    Makes a simple CompProb matrix 
+ *
+ *
+ * Arg:        match [UNKN ] Undocumented argument [Probability]
+ * Arg:          rnd [UNKN ] Undocumented argument [Probability]
+ *
+ * Return [UNKN ]  Undocumented return value [CompProb *]
+ *
+ */
+# line 50 "compmat.dy"
+CompProb * simple_CompProb(Probability match,Probability rnd)
+{
+  CompProb * p;
+  Probability rem;
+  int i,j;
+
+  p = CompProb_alloc();
+  rem = (1.0-match)/20;
+
+  for(i=0;i<26;i++) {
+    for(j=i;j<26;j++) {
+      if( i == j ) {
+	p->comp[i][j] = match/rnd;
+      } else {
+	p->comp[i][j] = rem/rnd;
+	p->comp[j][i] = rem/rnd;
+      }
+    }
+  }
+
+  return p;
+}
+
+/* Function:  fold_column_RandomModel_CompProb(cp,rm)
+ *
+ * Descrip:    Folds a random model in over the columns
+ *
+ *
+ * Arg:        cp [UNKN ] Undocumented argument [CompProb *]
+ * Arg:        rm [UNKN ] Undocumented argument [RandomModel *]
+ *
+ */
+# line 76 "compmat.dy"
+void fold_column_RandomModel_CompProb(CompProb * cp,RandomModel * rm)
+{
+  int i,j;
+
+  for(i=0;i<26;i++) {
+    for(j=0;j<26;j++) {
+      if( rm->aminoacid[j] < 0.00000000000001 ) {
+        continue;
+      }
+      cp->comp[i][j] = cp->comp[i][j] / rm->aminoacid[j];
+    }
+  }
+}
+
+
+/* Function:  simple_aa_CompProb(match,set,rnd)
+ *
+ * Descrip:    Makes a simple CompProb with simple aa rules
+ *
+ *
+ * Arg:        match [UNKN ] Undocumented argument [Probability]
+ * Arg:          set [UNKN ] Undocumented argument [Probability]
+ * Arg:          rnd [UNKN ] Undocumented argument [Probability]
+ *
+ * Return [UNKN ]  Undocumented return value [CompProb *]
+ *
+ */
+# line 94 "compmat.dy"
+CompProb * simple_aa_CompProb(Probability match,Probability set,Probability rnd)
+{
+  CompProb * p;
+  Probability rem;
+  int i,j,k;
+  char * aa_set[] = { "FYW","LIV","RK","DE" };
+
+  p = CompProb_alloc();
+  rem = (1.0-match)/20;
+
+  for(i=0;i<26;i++) {
+    for(j=i;j<26;j++) {
+      if( i == j ) {
+	p->comp[i][j] = match/rnd;
+      } else {
+	for( k = 0;k<4;k++) {
+	  if( strchr(aa_set[k],'A'+i) != NULL && strchr(aa_set[k],'A'+j) != NULL ) {
+	    p->comp[i][j] = set/rnd;
+	    p->comp[j][i] = set/rnd;
+	    fprintf(stderr,"Setting %c,%c to %.2f\n",'A'+i,'A'+j,set/rnd);
+	  } else {
+	    p->comp[i][j] = rem/rnd;
+	    p->comp[j][i] = rem/rnd;
+	  }
+	}
+      }
+    }
+  }
+
+  return p;
+}
+
 /* Function:  CompMat_from_CompProb(cp)
  *
  * Descrip:    Maps a CompProb to a CompMat going through
@@ -15,7 +120,7 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [CompMat *]
  *
  */
-# line 49 "compmat.dy"
+# line 130 "compmat.dy"
 CompMat * CompMat_from_CompProb(CompProb * cp)
 {
   int i,j;
@@ -46,7 +151,7 @@ CompMat * CompMat_from_CompProb(CompProb * cp)
  * Return [UNKN ]  Undocumented return value [CompProb *]
  *
  */
-# line 72 "compmat.dy"
+# line 153 "compmat.dy"
 CompProb * CompProb_from_halfbit(CompMat * cm)
 {
   CompProb * out;
@@ -74,7 +179,7 @@ CompProb * CompProb_from_halfbit(CompMat * cm)
  * Return [UNKN ]  Undocumented return value [CompMat *]
  *
  */
-# line 92 "compmat.dy"
+# line 173 "compmat.dy"
 CompMat * CompMat_from_halfbit(CompMat * cm)
 {
   CompMat * out;
@@ -98,7 +203,7 @@ CompMat * CompMat_from_halfbit(CompMat * cm)
  * Arg:        factor [UNKN ] amount to multiple by [int]
  *
  */
-# line 112 "compmat.dy"
+# line 193 "compmat.dy"
 void factor_CompMat(CompMat * cm,int factor)
 {
   int i,j;
@@ -123,7 +228,7 @@ void factor_CompMat(CompMat * cm,int factor)
  * Return [UNKN ]  Undocumented return value [Score]
  *
  */
-# line 131 "compmat.dy"
+# line 212 "compmat.dy"
 Score fail_safe_CompMat_access(CompMat * cm,int aa1,int aa2)
 {
   if( cm == NULL) {
@@ -151,7 +256,7 @@ Score fail_safe_CompMat_access(CompMat * cm,int aa1,int aa2)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 153 "compmat.dy"
+# line 234 "compmat.dy"
 boolean write_Blast_CompMat(CompMat * cm,FILE * ofp)
 {
   return write_Blast_CompMat_alphabet(cm,"ARNDCQEGHILKMFPSTWYVBZX",ofp);
@@ -167,7 +272,7 @@ boolean write_Blast_CompMat(CompMat * cm,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 162 "compmat.dy"
+# line 243 "compmat.dy"
 boolean bad_CompMat_alphabet(char * al)
 {
   char * runner;
@@ -194,7 +299,7 @@ boolean bad_CompMat_alphabet(char * al)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 183 "compmat.dy"
+# line 264 "compmat.dy"
 boolean write_Blast_CompMat_alphabet(CompMat * cm,char * alphabet,FILE * ofp)
 {
   char * runner;
@@ -262,7 +367,7 @@ boolean write_Blast_CompMat_alphabet(CompMat * cm,char * alphabet,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [CompMat *]
  *
  */
-# line 243 "compmat.dy"
+# line 324 "compmat.dy"
 CompMat * read_Blast_file_CompMat(char * filename)
 {
   CompMat * out;
@@ -296,7 +401,7 @@ CompMat * read_Blast_file_CompMat(char * filename)
  * Return [UNKN ]  Undocumented return value [CompMat *]
  *
  */
-# line 269 "compmat.dy"
+# line 350 "compmat.dy"
 CompMat * read_Blast_CompMat(FILE * ifp)
 {
   char buffer[MAXLINE];
@@ -356,6 +461,38 @@ CompMat * read_Blast_CompMat(FILE * ifp)
   return out;
 
 }
+
+/* Function:  read_Blast_file_CompProb(filename)
+ *
+ * Descrip:    Reads a BLAST format comp prob from file
+ *
+ *
+ * Arg:        filename [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [CompProb *]
+ *
+ */
+# line 413 "compmat.dy"
+CompProb * read_Blast_file_CompProb(char * filename)
+{
+  FILE * ifp;
+  CompProb * out;
+
+  ifp = openfile(filename,"r");
+
+  if( ifp == NULL ) {
+    warn("Could not open %s for compprob",filename);
+    return NULL;
+  }
+
+  out = read_Blast_CompProb(ifp);
+
+  fclose(ifp);
+
+  return out;
+}
+
+
 /* Function:  read_Blast_CompProb(ifp)
  *
  * Descrip:    reads a BLAST format matrix and
@@ -367,7 +504,7 @@ CompMat * read_Blast_CompMat(FILE * ifp)
  * Return [UNKN ]  newly allocated CompProb [CompProb *]
  *
  */
-# line 335 "compmat.dy"
+# line 440 "compmat.dy"
 CompProb * read_Blast_CompProb(FILE * ifp)
 {
   char buffer[MAXLINE];
@@ -437,7 +574,7 @@ CompProb * read_Blast_CompProb(FILE * ifp)
  * Return [UNKN ]  Undocumented return value [CompMat *]
  *
  */
-# line 398 "compmat.dy"
+# line 503 "compmat.dy"
 CompMat * blank_CompMat(void)
 {
   register int i;
@@ -465,7 +602,7 @@ CompMat * blank_CompMat(void)
  * Return [UNKN ]  Undocumented return value [CompProb *]
  *
  */
-# line 419 "compmat.dy"
+# line 524 "compmat.dy"
 CompProb * blank_CompProb(void)
 {
   register int i;
@@ -489,7 +626,7 @@ CompProb * blank_CompProb(void)
 
 
 
-# line 465 "compmat.c"
+# line 594 "compmat.c"
 /* Function:  hard_link_CompProb(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -532,6 +669,9 @@ CompProb * CompProb_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* comp[26][26] is an array: no default possible */ 
     out->name = NULL;    
 
@@ -553,6 +693,7 @@ CompProb * CompProb_alloc(void)
  */
 CompProb * free_CompProb(CompProb * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -561,10 +702,18 @@ CompProb * free_CompProb(CompProb * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
 
@@ -616,6 +765,9 @@ CompMat * CompMat_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* comp[26][26] is an array: no default possible */ 
     out->name = NULL;    
 
@@ -637,6 +789,7 @@ CompMat * CompMat_alloc(void)
  */
 CompMat * free_CompMat(CompMat * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -645,10 +798,18 @@ CompMat * free_CompMat(CompMat * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
 
