@@ -11,10 +11,12 @@
 
 #include "packaln.h"
 #include "basematrix.h"
+#include "shattermatrix.h"
 #include "dpenvelope.h"
 #include "dbsearchimpl.h"
 #include "dprunimpl.h"
 #include "matrixdebug.h"
+#include "shattermatrix.h"
 
 #include "sequence.h"
 #include "seqalign.h"
@@ -22,6 +24,8 @@
 #include "database.h"
 #include "hscore.h"
 #include "histogram.h"
+#include "searchstatinterface.h"
+#include "searchstatlookup.h"
 #include "aln.h"
 #include "linesubs.h"
 #include "complexsequence.h"
@@ -41,6 +45,10 @@
 #include "cdnadb.h"
 #include "dna.h"
 
+#include "seqlookup.h"
+#include "arrayseqlookup.h"
+#include "proteinstreamedindex.h"
+
 #include "embl.h"
 #include "genomicregion.h"
 #include "gene.h"
