@@ -726,6 +726,9 @@ Gene * Gene_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->start = 0;  
     out->end = 0;    
     out->genomic = NULL; 
@@ -754,6 +757,7 @@ Gene * Gene_alloc(void)
  */
 Gene * free_Gene(Gene * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -763,10 +767,18 @@ Gene * free_Gene(Gene * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     /* obj->parent is linked in */ 
     if( obj->genomic != NULL)    
       free_Genomic(obj->genomic);    
