@@ -31,7 +31,10 @@ Genomic * truncate_Genomic(Genomic * gen,int start,int stop)
   int tst,tend;
   int i;
 
+
   out = Genomic_from_Sequence(magic_trunc_Sequence(gen->baseseq,start,stop));
+  assert(out);
+
   for(i=0;i<gen->len;i++) {
     if( start < stop ) {
       /* then we have to figure out if this repeat overlaps, and if so, put it in */
@@ -91,7 +94,7 @@ Genomic * truncate_Genomic(Genomic * gen,int start,int stop)
  * Return [UNKN ]  Undocumented return value [Genomic *]
  *
  */
-# line 122 "genomic.dy"
+# line 125 "genomic.dy"
 Genomic * reverse_complement_Genomic(Genomic * gen)
 {
   return truncate_Genomic(gen,gen->baseseq->len,0);
@@ -110,16 +113,18 @@ Genomic * reverse_complement_Genomic(Genomic * gen)
  * Return [UNKN ]  Undocumented return value [Genomic *]
  *
  */
-# line 135 "genomic.dy"
+# line 138 "genomic.dy"
 Genomic * read_fasta_file_Genomic(char * filename,int length_of_N)
 {
   Sequence * seq;
 
   seq = read_fasta_file_Sequence(filename);
   if( seq == NULL ) {
-    warn("Unable to read sequence from %s, so no genomic",filename);
+    warn("Unable to read sequence from %s [%d], so no genomic",filename,seq);
     return NULL;
   }
+
+
   if( length_of_N < 0 ) 
     return Genomic_from_Sequence(seq);
   else 
@@ -140,7 +145,7 @@ Genomic * read_fasta_file_Genomic(char * filename,int length_of_N)
  * Return [UNKN ]  Undocumented return value [Genomic *]
  *
  */
-# line 159 "genomic.dy"
+# line 164 "genomic.dy"
 Genomic * read_fasta_Genomic(FILE * ifp,int length_of_N)
 {
   Sequence * seq;
@@ -168,7 +173,7 @@ Genomic * read_fasta_Genomic(FILE * ifp,int length_of_N)
  * Return [UNKN ]  Undocumented return value [char *]
  *
  */
-# line 180 "genomic.dy"
+# line 185 "genomic.dy"
 char * Genomic_name(Genomic * gen)
 {
   return gen->baseseq->name;
@@ -184,7 +189,7 @@ char * Genomic_name(Genomic * gen)
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 189 "genomic.dy"
+# line 194 "genomic.dy"
 int Genomic_length(Genomic * gen)
 {
   return gen->baseseq->len;
@@ -201,7 +206,7 @@ int Genomic_length(Genomic * gen)
  * Return [UNKN ]  Undocumented return value [char]
  *
  */
-# line 200 "genomic.dy"
+# line 205 "genomic.dy"
 char Genomic_seqchar(Genomic * gen,int pos)
 {
   return gen->baseseq->seq[pos];
@@ -221,7 +226,7 @@ char Genomic_seqchar(Genomic * gen,int pos)
  * Return [UNKN ]  Undocumented return value [Genomic *]
  *
  */
-# line 211 "genomic.dy"
+# line 216 "genomic.dy"
 Genomic * Genomic_from_Sequence_Nheuristic(Sequence * seq,int length_of_N)
 {
   Genomic * out;
@@ -272,7 +277,7 @@ Genomic * Genomic_from_Sequence_Nheuristic(Sequence * seq,int length_of_N)
  * Return [UNKN ]  Undocumented return value [Genomic *]
  *
  */
-# line 256 "genomic.dy"
+# line 261 "genomic.dy"
 Genomic * Genomic_from_Sequence(Sequence * seq)
 {
   Genomic * out;
@@ -314,7 +319,7 @@ Genomic * Genomic_from_Sequence(Sequence * seq)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 292 "genomic.dy"
+# line 297 "genomic.dy"
 void show_Genomic(Genomic * gen,FILE * ofp)
 {
   int i;
@@ -331,7 +336,7 @@ void show_Genomic(Genomic * gen,FILE * ofp)
   
 }
 
-# line 313 "genomic.c"
+# line 318 "genomic.c"
 /* Function:  hard_link_GenomicRepeat(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -374,6 +379,9 @@ GenomicRepeat * GenomicRepeat_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->start = 0;  
     out->end = 0;    
     out->type = NULL;    
@@ -396,6 +404,7 @@ GenomicRepeat * GenomicRepeat_alloc(void)
  */
 GenomicRepeat * free_GenomicRepeat(GenomicRepeat * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -404,10 +413,18 @@ GenomicRepeat * free_GenomicRepeat(GenomicRepeat * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->type != NULL)   
       ckfree(obj->type);     
 
@@ -663,6 +680,9 @@ Genomic * Genomic_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->baseseq = NULL; 
     out->repeat = NULL;  
     out->len = out->maxlen = 0;  
@@ -685,6 +705,7 @@ Genomic * Genomic_alloc(void)
  */
 Genomic * free_Genomic(Genomic * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -694,10 +715,18 @@ Genomic * free_Genomic(Genomic * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->baseseq != NULL)    
       free_Sequence(obj->baseseq);   
     if( obj->repeat != NULL) {  
