@@ -138,7 +138,8 @@ ComplexSequence * init_GenomicDB(GenomicDB * gendb,int * return_status)
   seq = init_SequenceDB(gendb->sdb,return_status);
 
   if( seq == NULL || *return_status == DB_RETURN_ERROR || *return_status == DB_RETURN_END ) {
-    return NULL; /** error already reported **/
+    warn("On initiation of the genomicdb, immedaitely got an end scenario");
+    return NULL; 
   }
 
   /* check dna status. We assumme someone knows what he is doing when he makes a genomic db!*/
@@ -175,7 +176,7 @@ ComplexSequence * init_GenomicDB(GenomicDB * gendb,int * return_status)
  * Return [UNKN ]  Undocumented return value [ComplexSequence *]
  *
  */
-# line 205 "genomicdb.dy"
+# line 206 "genomicdb.dy"
 ComplexSequence * reload_GenomicDB(ComplexSequence * last,GenomicDB * gendb,int * return_status)
 {
   ComplexSequence * cs;
@@ -300,7 +301,7 @@ ComplexSequence * reload_GenomicDB(ComplexSequence * last,GenomicDB * gendb,int
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 324 "genomicdb.dy"
+# line 325 "genomicdb.dy"
 boolean close_GenomicDB(ComplexSequence * cs,GenomicDB * gendb) 
 {
   if( gendb->is_single_seq == TRUE ) {
@@ -329,7 +330,7 @@ boolean close_GenomicDB(ComplexSequence * cs,GenomicDB * gendb)
  * Return [UNKN ]  Undocumented return value [GenomicDB *]
  *
  */
-# line 345 "genomicdb.dy"
+# line 346 "genomicdb.dy"
 GenomicDB * new_GenomicDB_from_single_seq(Genomic * gen,ComplexSequenceEvalSet * cses,int score_in_repeat_coding)
 {
   ComplexSequence * cs,*cs_rev;
@@ -360,7 +361,7 @@ GenomicDB * new_GenomicDB_from_single_seq(Genomic * gen,ComplexSequenceEvalSet *
  * Return [UNKN ]  Undocumented return value [GenomicDB *]
  *
  */
-# line 369 "genomicdb.dy"
+# line 370 "genomicdb.dy"
 GenomicDB * new_GenomicDB_from_forrev_cseq(ComplexSequence * cs,ComplexSequence * cs_rev)
 {
   GenomicDB * out;
@@ -389,7 +390,7 @@ GenomicDB * new_GenomicDB_from_forrev_cseq(ComplexSequence * cs,ComplexSequence
  * Return [UNKN ]  Undocumented return value [GenomicDB *]
  *
  */
-# line 390 "genomicdb.dy"
+# line 391 "genomicdb.dy"
 GenomicDB * new_GenomicDB(SequenceDB * seqdb,ComplexSequenceEvalSet * cses,int length_of_N,int repeat_in_cds_score)
 {
   GenomicDB * out;
@@ -419,7 +420,7 @@ GenomicDB * new_GenomicDB(SequenceDB * seqdb,ComplexSequenceEvalSet * cses,int l
 }
 
  
-# line 403 "genomicdb.c"
+# line 404 "genomicdb.c"
 /* Function:  hard_link_GenomicDB(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -462,6 +463,9 @@ GenomicDB * GenomicDB_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->is_single_seq = FALSE;  
     out->done_forward = FALSE;   
     out->forw = NULL;    
@@ -493,6 +497,7 @@ GenomicDB * GenomicDB_alloc(void)
  */
 GenomicDB * free_GenomicDB(GenomicDB * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -501,10 +506,18 @@ GenomicDB * free_GenomicDB(GenomicDB * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->forw != NULL)   
       free_ComplexSequence(obj->forw);   
     if( obj->rev != NULL)    
