@@ -3,6 +3,228 @@ extern "C" {
 #endif
 #include "genomicregion.h"
 
+/* Function:  new_GenomicRegion_discard_short(gr,multiexon,singleexon)
+ *
+ * Descrip:    Makes a new genomic region with genes
+ *             greater than XXX length going through
+ *
+ *
+ * Arg:                gr [UNKN ] Undocumented argument [GenomicRegion *]
+ * Arg:         multiexon [UNKN ] Undocumented argument [int]
+ * Arg:        singleexon [UNKN ] Undocumented argument [int]
+ *
+ * Return [UNKN ]  Undocumented return value [GenomicRegion *]
+ *
+ */
+# line 75 "genomicregion.dy"
+GenomicRegion * new_GenomicRegion_discard_short(GenomicRegion * gr,int multiexon,int singleexon)
+{
+  GenomicRegion * out;
+  int i;
+
+  assert(gr);
+  assert(gr->genomic);
+
+  out = GenomicRegion_alloc_std();
+  assert(out);
+
+  if( gr->genomic != NULL ) {
+    out->genomic = hard_link_Genomic(gr->genomic);
+  }
+
+  for(i=0;i<gr->len;i++) {
+    if( gr->gene[i]->len == 0 ) {
+      continue;
+    }
+
+    assert(gr->gene[i]->transcript[0]);
+
+    if( gr->gene[i]->transcript[0]->ex_len > 1 ) {
+      if( length_Transcript(gr->gene[i]->transcript[0]) > multiexon ) {
+	add_GenomicRegion(out,hard_link_Gene(gr->gene[i]));
+      }
+    } else {
+      if( length_Transcript(gr->gene[i]->transcript[0]) > singleexon ) {
+	add_GenomicRegion(out,hard_link_Gene(gr->gene[i]));
+      }
+    }
+
+  }
+
+  return out;
+
+}
+
+/* Function:  show_GenomicRegionOptions(sgro,gr,ct,dividestr,ofp)
+ *
+ * Descrip:    Actually shows a genomic region wrt to the options
+ *
+ *
+ * Arg:             sgro [UNKN ] Undocumented argument [ShowGenomicRegionOptions *]
+ * Arg:               gr [UNKN ] Undocumented argument [GenomicRegion *]
+ * Arg:               ct [UNKN ] Undocumented argument [CodonTable *]
+ * Arg:        dividestr [UNKN ] Undocumented argument [char *]
+ * Arg:              ofp [UNKN ] Undocumented argument [FILE *]
+ *
+ */
+# line 116 "genomicregion.dy"
+void show_GenomicRegionOptions(ShowGenomicRegionOptions * sgro,GenomicRegion * gr,CodonTable * ct,char * dividestr,FILE * ofp)
+{
+  if( sgro->show_raw == TRUE ) {
+    show_GenomicRegion(gr,ofp);
+    fprintf(ofp,"%s\n",dividestr);
+  }
+
+  if( sgro->show_trans == TRUE ) {
+    dump_translations_GenomicRegion(gr,ct,ofp);
+    fprintf(ofp,"%s\n",dividestr);
+  }
+
+  if( sgro->show_gene_str == TRUE ) {
+    show_pretty_GenomicRegion(gr,0,ofp);
+    fprintf(ofp,"%s\n",dividestr);
+  }
+
+  if( sgro->show_gene_supp == TRUE ) {
+    show_pretty_GenomicRegion(gr,1,ofp);
+    fprintf(ofp,"%s\n",dividestr);
+  }
+    
+}
+
+/* Function:  show_help_ShowGenomicRegionOptions(ofp)
+ *
+ * Descrip:    Help for ShowGenomicRegionOptions
+ *
+ *
+ * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
+ *
+ */
+# line 143 "genomicregion.dy"
+void show_help_ShowGenomicRegionOptions(FILE * ofp)
+{
+  fprintf(ofp,"Gene Structure Output options\n");
+  fprintf(ofp,"   -trans        show translation\n");
+  fprintf(ofp,"   -cdna         show virtual cdna\n");
+  fprintf(ofp,"   -genes        show gene structure\n");
+  fprintf(ofp,"   -genesf       show gene structure with supporting evidence\n");
+  fprintf(ofp,"   -gener        show raw (offset) gene structure\n");
+  fprintf(ofp,"   -ace          show ace file\n");
+  
+}
+
+
+/* Function:  new_ShowGenomicRegionOptions_from_argv(argc,argv)
+ *
+ * Descrip:    Makes a ShowGenomicRegionOptions from command line
+ *
+ *
+ * Arg:        argc [UNKN ] Undocumented argument [int *]
+ * Arg:        argv [UNKN ] Undocumented argument [char **]
+ *
+ * Return [UNKN ]  Undocumented return value [ShowGenomicRegionOptions *]
+ *
+ */
+# line 159 "genomicregion.dy"
+ShowGenomicRegionOptions * new_ShowGenomicRegionOptions_from_argv(int * argc,char ** argv)
+{
+  ShowGenomicRegionOptions * out;
+
+  out = ShowGenomicRegionOptions_alloc();
+
+  out->show_trans    = strip_out_boolean_argument(argc,argv,"trans");
+  out->show_raw      = strip_out_boolean_argument(argc,argv,"gener");
+  out->show_gene_str = strip_out_boolean_argument(argc,argv,"genes");
+  out->show_gene_supp = strip_out_boolean_argument(argc,argv,"genesf");
+  out->show_cdna     = strip_out_boolean_argument(argc,argv,"cdna");
+  out->show_ace     = strip_out_boolean_argument(argc,argv,"ace");
+  out->show_ace     = strip_out_boolean_argument(argc,argv,"aceh");
+
+
+  return out;
+}
+
+/* Function:  read_genes_GenomicRegion(ifp)
+ *
+ * Descrip:    read genes output
+ *
+ *
+ * Arg:        ifp [UNKN ] Undocumented argument [FILE *]
+ *
+ * Return [UNKN ]  Undocumented return value [GenomicRegion *]
+ *
+ */
+# line 180 "genomicregion.dy"
+GenomicRegion * read_genes_GenomicRegion(FILE * ifp)
+{
+  char buffer[MAXLINE];
+  GenomicRegion * out;
+  Gene * g;
+  Transcript * t;
+  Translation * ts;
+  Exon * e;
+  int is_start = 1;
+
+
+  out = GenomicRegion_alloc_std();
+
+  
+  while( fgets(buffer,MAXLINE,ifp) ) {
+    if( strstartcmp(buffer,"//") == 0 ) {
+      break;
+    }
+    
+    if( strstartcmp(buffer,"Gene") == 0 ) {
+      if( is_start == 1 ) {
+	is_start =0;
+	continue; /* next gene triggered */
+      }
+
+      /* Line is Gene xxx yyy here */
+      
+      g = Gene_alloc_std();
+
+      add_GenomicRegion(out,g);
+
+      sscanf(buffer,"Gene %d %d",&g->start,&g->end);
+      g->start = g->start -1;
+      t = Transcript_alloc_std();
+      add_Gene(g,t);
+      g->parent = out;
+
+      while( fgets(buffer,MAXLINE,ifp) != NULL ) {
+	if( strstartcmp(buffer,"Gene") == 0 ) {
+	  break;
+	}
+	if( strstartcmp(buffer,"//") == 0 ) {
+	  break;
+	}
+	e = Exon_alloc_std();
+	add_ex_Transcript(t,e);
+
+	sscanf(buffer," Exon %d %d",&e->start,&e->end);
+	e->start = e->start - g->start -1;
+	e->end   = e->end   - g->start;
+      }
+
+      ts = Translation_alloc();
+      ts->start = 0;
+      ts->end = length_Transcript(t);
+      ts->parent = t;
+      add_Transcript(t,ts);
+      t->parent = g;
+      
+      if( strstartcmp(buffer,"//") == 0 ) {
+	break;
+      }
+    }
+  }
+      
+  
+    return out;
+}
+
+
 /* Function:  show_GenomicOverlapResults(gor,ofp)
  *
  * Descrip:    shows overlap resuls vaguely humanely
@@ -12,7 +234,7 @@ extern "C" {
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 62 "genomicregion.dy"
+# line 253 "genomicregion.dy"
 void show_GenomicOverlapResults(GenomicOverlapResults * gor,FILE * ofp)
 {
   int i;
@@ -31,7 +253,7 @@ void show_GenomicOverlapResults(GenomicOverlapResults * gor,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 75 "genomicregion.dy"
+# line 266 "genomicregion.dy"
 void show_GenomicOverlapGene(GenomicOverlapGene * gog,FILE * ofp)
 {
   fprintf(ofp,"Perfect      exons %d\n",gog->exon_perfect);
@@ -54,7 +276,7 @@ void show_GenomicOverlapGene(GenomicOverlapGene * gog,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [GenomicOverlapResults *]
  *
  */
-# line 89 "genomicregion.dy"
+# line 280 "genomicregion.dy"
 GenomicOverlapResults * Genomic_overlap(GenomicRegion * query,GenomicRegion * truth)
 {
   GenomicOverlapResults * out;
@@ -106,7 +328,7 @@ GenomicOverlapResults * Genomic_overlap(GenomicRegion * query,GenomicRegion * tr
  * Return [UNKN ]  Undocumented return value [GenomicOverlapGene *]
  *
  */
-# line 132 "genomicregion.dy"
+# line 323 "genomicregion.dy"
 GenomicOverlapGene * Gene_overlap_forward(Gene * test,Gene * truth)
 {
   int i;
@@ -214,7 +436,7 @@ GenomicOverlapGene * Gene_overlap_forward(Gene * test,Gene * truth)
  * Return [UNKN ]  Undocumented return value [GenomicOverlapGene *]
  *
  */
-# line 231 "genomicregion.dy"
+# line 422 "genomicregion.dy"
 GenomicOverlapGene * Gene_overlap_backward(Gene * test,Gene * truth)
 {
   int i;
@@ -325,7 +547,7 @@ GenomicOverlapGene * Gene_overlap_backward(Gene * test,Gene * truth)
  * Return [UNKN ]  Undocumented return value [GenomicRegion *]
  *
  */
-# line 332 "genomicregion.dy"
+# line 523 "genomicregion.dy"
 GenomicRegion * simple_merged_GenomicRegion(GenomicRegion * gr,double bits_cutoff,int max_ext)
 {
   GenomicRegion * out;
@@ -389,7 +611,7 @@ GenomicRegion * simple_merged_GenomicRegion(GenomicRegion * gr,double bits_cutof
  * Arg:        gr [UNKN ] Undocumented argument [GenomicRegion *]
  *
  */
-# line 390 "genomicregion.dy"
+# line 581 "genomicregion.dy"
 void sort_GenomicRegion_absolute(GenomicRegion * gr)
 {
   sort_GenomicRegion(gr,compare_Gene_absolute);
@@ -406,7 +628,7 @@ void sort_GenomicRegion_absolute(GenomicRegion * gr)
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 398 "genomicregion.dy"
+# line 589 "genomicregion.dy"
 int compare_Gene_absolute(Gene * one,Gene * two)
 {
   int startone;
@@ -438,7 +660,7 @@ int compare_Gene_absolute(Gene * one,Gene * two)
  * Return [SOFT ]  a Genomic sequence [Genomic *]
  *
  */
-# line 425 "genomicregion.dy"
+# line 616 "genomicregion.dy"
 Genomic * get_Genomic_from_GenomicRegion(GenomicRegion * gr)
 {
   return gr->genomic;
@@ -454,7 +676,7 @@ Genomic * get_Genomic_from_GenomicRegion(GenomicRegion * gr)
  * Return [UNKN ]  Undocumented return value [GenomicRegion *]
  *
  */
-# line 433 "genomicregion.dy"
+# line 624 "genomicregion.dy"
 GenomicRegion * read_EMBL_GenomicRegion_efetch(char * efetch) 
 {
   FILE * ifp;
@@ -493,7 +715,7 @@ GenomicRegion * read_EMBL_GenomicRegion_efetch(char * efetch)
  * Return [UNKN ]  Undocumented return value [GenomicRegion *]
  *
  */
-# line 464 "genomicregion.dy"
+# line 655 "genomicregion.dy"
 GenomicRegion * read_EMBL_GenomicRegion_SRS(char * srsquery) 
 {
   FILE * ifp;
@@ -525,7 +747,7 @@ GenomicRegion * read_EMBL_GenomicRegion_SRS(char * srsquery)
  * Return [UNKN ]  Undocumented return value [GenomicRegion *]
  *
  */
-# line 488 "genomicregion.dy"
+# line 679 "genomicregion.dy"
 GenomicRegion * read_EMBL_GenomicRegion_file(char * filename)
 {
   FILE * ifp;
@@ -554,7 +776,7 @@ GenomicRegion * read_EMBL_GenomicRegion_file(char * filename)
  * Return [UNKN ]  Undocumented return value [GenomicRegion *]
  *
  */
-# line 509 "genomicregion.dy"
+# line 700 "genomicregion.dy"
 GenomicRegion * read_EMBL_GenomicRegion(FILE * ifp)
 {
   GenomicRegion * out;
@@ -617,7 +839,7 @@ GenomicRegion * read_EMBL_GenomicRegion(FILE * ifp)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 561 "genomicregion.dy"
+# line 752 "genomicregion.dy"
 boolean read_EMBL_FT_into_GenomicRegion(char * buffer,int maxlen,GenomicRegion * gr,FILE * ifp)
 {
   Gene * gene;
@@ -651,7 +873,7 @@ boolean read_EMBL_FT_into_GenomicRegion(char * buffer,int maxlen,GenomicRegion *
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 588 "genomicregion.dy"
+# line 779 "genomicregion.dy"
 void show_GenomicRegion(GenomicRegion * gr,FILE * ofp)
 {
   int i;
@@ -674,7 +896,7 @@ void show_GenomicRegion(GenomicRegion * gr,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 603 "genomicregion.dy"
+# line 794 "genomicregion.dy"
 void dump_translations_GenomicRegion(GenomicRegion * gr,CodonTable * ct,FILE * ofp)
 {
   int i,j,k;
@@ -697,7 +919,7 @@ void dump_translations_GenomicRegion(GenomicRegion * gr,CodonTable * ct,FILE * o
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 619 "genomicregion.dy"
+# line 810 "genomicregion.dy"
 void dump_transcripts_GenomicRegion(GenomicRegion * gr,FILE * ofp)
 {
   int i,j;
@@ -721,7 +943,7 @@ void dump_transcripts_GenomicRegion(GenomicRegion * gr,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [GenomicRegion *]
  *
  */
-# line 635 "genomicregion.dy"
+# line 826 "genomicregion.dy"
 GenomicRegion * new_GenomicRegion(Genomic * gen)
 {
   GenomicRegion * out;
@@ -744,7 +966,7 @@ GenomicRegion * new_GenomicRegion(Genomic * gen)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 652 "genomicregion.dy"
+# line 843 "genomicregion.dy"
 boolean add_Gene_to_GenomicRegion(GenomicRegion * gr,Gene * gene)
 {
   gene->parent = gr;
@@ -766,7 +988,7 @@ boolean add_Gene_to_GenomicRegion(GenomicRegion * gr,Gene * gene)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 667 "genomicregion.dy"
+# line 858 "genomicregion.dy"
 void write_Embl_FT_GenomicRegion(GenomicRegion * gr,FILE * ofp)
 {
   int i;
@@ -799,7 +1021,7 @@ void write_Embl_FT_GenomicRegion(GenomicRegion * gr,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 693 "genomicregion.dy"
+# line 884 "genomicregion.dy"
 void write_Diana_FT_GenomicRegion(GenomicRegion * gr,FILE * ofp)
 {
   int i;
@@ -833,7 +1055,7 @@ void write_Diana_FT_GenomicRegion(GenomicRegion * gr,FILE * ofp)
  * Arg:             ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 719 "genomicregion.dy"
+# line 910 "genomicregion.dy"
 void show_ace_GenomicRegion(GenomicRegion * gr,char * seq_name,FILE * ofp)
 {
   int i,j;
@@ -886,7 +1108,7 @@ void show_ace_GenomicRegion(GenomicRegion * gr,char * seq_name,FILE * ofp)
  * Arg:                ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 760 "genomicregion.dy"
+# line 951 "genomicregion.dy"
 void show_halfwise_GenomicRegion(GenomicRegion * gr,char * seq_name,char * method,char * db,boolean doweb,char * weblocation,FILE * ofp)
 {
   int i,j;
@@ -937,7 +1159,7 @@ void show_halfwise_GenomicRegion(GenomicRegion * gr,char * seq_name,char * metho
  * Arg:             ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 802 "genomicregion.dy"
+# line 993 "genomicregion.dy"
 void show_GFF_GenomicRegion(GenomicRegion * gr,char * seq_name,char * source,FILE * ofp)
 {
   int i,j,k,phase,len;
@@ -1019,7 +1241,7 @@ void show_GFF_GenomicRegion(GenomicRegion * gr,char * seq_name,char * source,FIL
  * Arg:                    ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 878 "genomicregion.dy"
+# line 1069 "genomicregion.dy"
 void show_pretty_GenomicRegion(GenomicRegion * gr,boolean show_supporting,FILE * ofp)
 {
   int i;
@@ -1032,7 +1254,7 @@ void show_pretty_GenomicRegion(GenomicRegion * gr,boolean show_supporting,FILE *
 
 
 
-# line 984 "genomicregion.c"
+# line 1196 "genomicregion.c"
 /* Function:  swap_GenomicRegion(list,i,j)
  *
  * Descrip:    swap function: an internal for qsort_GenomicRegion
@@ -1279,6 +1501,9 @@ GenomicRegion * GenomicRegion_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->gene = NULL;    
     out->len = out->maxlen = 0;  
     out->genomic = NULL; 
@@ -1301,6 +1526,7 @@ GenomicRegion * GenomicRegion_alloc(void)
  */
 GenomicRegion * free_GenomicRegion(GenomicRegion * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1310,10 +1536,18 @@ GenomicRegion * free_GenomicRegion(GenomicRegion * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->gene != NULL)   {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->gene[i] != NULL)    
@@ -1372,6 +1606,9 @@ GenomicOverlapGene * GenomicOverlapGene_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->exon_perfect = 0;   
     out->exon_truncated = 0; 
     out->exon_partial = 0;   
@@ -1399,6 +1636,7 @@ GenomicOverlapGene * GenomicOverlapGene_alloc(void)
  */
 GenomicOverlapGene * free_GenomicOverlapGene(GenomicOverlapGene * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1407,10 +1645,18 @@ GenomicOverlapGene * free_GenomicOverlapGene(GenomicOverlapGene * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->truth != NULL)  
       free_Gene(obj->truth);     
     if( obj->test != NULL)   
@@ -1668,6 +1914,9 @@ GenomicOverlapResults * GenomicOverlapResults_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->gene_overlap = 0;   
     out->gog = NULL; 
     out->len = out->maxlen = 0;  
@@ -1690,6 +1939,7 @@ GenomicOverlapResults * GenomicOverlapResults_alloc(void)
  */
 GenomicOverlapResults * free_GenomicOverlapResults(GenomicOverlapResults * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1699,10 +1949,18 @@ GenomicOverlapResults * free_GenomicOverlapResults(GenomicOverlapResults * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->gog != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->gog[i] != NULL) 
@@ -1717,6 +1975,106 @@ GenomicOverlapResults * free_GenomicOverlapResults(GenomicOverlapResults * obj)
 }    
 
 
+/* Function:  hard_link_ShowGenomicRegionOptions(obj)
+ *
+ * Descrip:    Bumps up the reference count of the object
+ *             Meaning that multiple pointers can 'own' it
+ *
+ *
+ * Arg:        obj [UNKN ] Object to be hard linked [ShowGenomicRegionOptions *]
+ *
+ * Return [UNKN ]  Undocumented return value [ShowGenomicRegionOptions *]
+ *
+ */
+ShowGenomicRegionOptions * hard_link_ShowGenomicRegionOptions(ShowGenomicRegionOptions * obj) 
+{
+    if( obj == NULL )    {  
+      warn("Trying to hard link to a ShowGenomicRegionOptions object: passed a NULL object");    
+      return NULL;   
+      }  
+    obj->dynamite_hard_link++;   
+    return obj;  
+}    
+
+
+/* Function:  ShowGenomicRegionOptions_alloc(void)
+ *
+ * Descrip:    Allocates structure: assigns defaults if given 
+ *
+ *
+ *
+ * Return [UNKN ]  Undocumented return value [ShowGenomicRegionOptions *]
+ *
+ */
+ShowGenomicRegionOptions * ShowGenomicRegionOptions_alloc(void) 
+{
+    ShowGenomicRegionOptions * out; /* out is exported at end of function */ 
+
+
+    /* call ckalloc and see if NULL */ 
+    if((out=(ShowGenomicRegionOptions *) ckalloc (sizeof(ShowGenomicRegionOptions))) == NULL)    {  
+      warn("ShowGenomicRegionOptions_alloc failed ");    
+      return NULL;  /* calling function should respond! */ 
+      }  
+    out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
+    out->show_trans = 0; 
+    out->show_raw = 0;   
+    out->show_cdna = 0;  
+    out->show_ace = 0;   
+    out->show_ace_halfwise = 0;  
+    out->show_GFF = 0;   
+    out->show_gene_str = 0;  
+    out->show_gene_supp = 0; 
+
+
+    return out;  
+}    
+
+
+/* Function:  free_ShowGenomicRegionOptions(obj)
+ *
+ * Descrip:    Free Function: removes the memory held by obj
+ *             Will chain up to owned members and clear all lists
+ *
+ *
+ * Arg:        obj [UNKN ] Object that is free'd [ShowGenomicRegionOptions *]
+ *
+ * Return [UNKN ]  Undocumented return value [ShowGenomicRegionOptions *]
+ *
+ */
+ShowGenomicRegionOptions * free_ShowGenomicRegionOptions(ShowGenomicRegionOptions * obj) 
+{
+    int return_early = 0;    
+
+
+    if( obj == NULL) {  
+      warn("Attempting to free a NULL pointer to a ShowGenomicRegionOptions obj. Should be trappable");  
+      return NULL;   
+      }  
+
+
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
+    if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
+      obj->dynamite_hard_link--; 
+      }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
+
+
+    ckfree(obj); 
+    return NULL; 
+}    
+
+
 /* Function:  access_gene_GenomicRegion(obj,i)
  *
  * Descrip:    Access members stored in the gene list
