@@ -44,6 +44,9 @@ extern "C" {
  */
 struct Wise2_Histogram {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     int*   histogram;   /*  counts of hits                      */ 
     int min;    /*  elem 0 of histogram == min          */ 
     int max;    /*  last elem of histogram == max       */ 
