@@ -571,6 +571,9 @@ DataEntry * DataEntry_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     /* data[DATAENTRYSTDPOINTS] is an array: no default possible */ 
     out->is_reversed = FALSE;    
@@ -594,6 +597,7 @@ DataEntry * DataEntry_alloc(void)
  */
 DataEntry * free_DataEntry(DataEntry * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -602,10 +606,18 @@ DataEntry * free_DataEntry(DataEntry * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     /* obj->filename is linked in */ 
@@ -658,6 +670,9 @@ DataScore * DataScore_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->query = NULL;   
     out->target = NULL;  
     out->score = 0;  
@@ -711,6 +726,9 @@ DataScoreStorage * DataScoreStorage_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* score_array[DATASCORESTORAGE_LENGTH] is an array: no default possible */ 
     /* query_array[DATASCORESTORAGE_LENGTH] is an array: no default possible */ 
     /* target_array[DATASCORESTORAGE_LENGTH] is an array: no default possible */ 
@@ -1130,6 +1148,9 @@ Hscore * Hscore_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->ds = NULL;  
     out->len = out->maxlen = 0;  
     out->store = NULL;   
@@ -1159,6 +1180,7 @@ Hscore * Hscore_alloc(void)
  */
 Hscore * free_Hscore(Hscore * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1168,10 +1190,18 @@ Hscore * free_Hscore(Hscore * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->ds != NULL) {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->ds[i] != NULL)  
