@@ -33,6 +33,9 @@ typedef long BytePosition;
  */
 struct Wise2_DataEntry {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * name;    /*  name of the entry */ 
     int data[DATAENTRYSTDPOINTS];   /*  space for algorithms to use  */ 
     boolean is_reversed;    /*  for sequences. handy */ 
@@ -57,6 +60,9 @@ typedef struct Wise2_DataEntry Wise2_DataEntry;
  */
 struct Wise2_DataScore {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     DataEntry * query;   
     DataEntry * target;  
     int score;   
@@ -86,6 +92,9 @@ typedef struct Wise2_DataScore Wise2_DataScore;
  */
 struct Wise2_DataScoreStorage {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     DataScore score_array[DATASCORESTORAGE_LENGTH];  
     DataEntry query_array[DATASCORESTORAGE_LENGTH];  
     DataEntry target_array[DATASCORESTORAGE_LENGTH];     
@@ -123,6 +132,9 @@ typedef struct Wise2_DataScoreStorage Wise2_DataScoreStorage;
  */
 struct Wise2_Hscore {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     DataScore ** ds;     
     int len;/* len for above ds  */ 
     int maxlen; /* maxlen for above ds */ 
