@@ -3,6 +3,60 @@ extern "C" {
 #endif
 #include "matrixdebug.h"
 
+/* Function:  show_PackAln_Debug(de,pal,ofp)
+ *
+ * Descrip:    Shows PackAln in debug context
+ *
+ *
+ * Arg:         de [UNKN ] Undocumented argument [DebugMatrix *]
+ * Arg:        pal [UNKN ] Undocumented argument [PackAln *]
+ * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
+ *
+ */
+# line 63 "matrixdebug.dy"
+void show_PackAln_Debug(DebugMatrix * de,PackAln * pal,FILE * ofp)
+{
+  int i;
+  int j,k;
+  int cum;
+  PackAlnUnit * prev = pal->pau[0];
+  PackAlnUnit * c;
+  DebugState * state;
+  DebugTransition * trans;
+ 
+  for(i=1;i<pal->len;i++) {
+    c= pal->pau[i];
+    /* have to find the correct state_num */
+    if( c->state >= de->len ) {
+      /* special state - can't handle */
+      fprintf(ofp,"Special state\n");
+      prev = c;
+      continue;
+    }
+    if( prev->state >= de->len ) {
+      /* special state - can't handle */
+      fprintf(ofp,"From Special state\n");
+      prev = c;
+      continue;
+    }
+    
+    state = de->state[pal->pau[i]->state];
+    trans = NULL;
+    for(k=0;k<state->len;k++) {
+      if( state->trans[k]->from_state_num == prev->state ) {
+	trans = state->trans[k];
+      }
+    }
+
+    fprintf(ofp,"SCORE: %5d %5d %5d %s\n",c->score,c->i,c->j,state->statename);
+
+    if( trans != NULL ) {
+      (*trans->show_transition)(de->matrix,c->i,c->j,ofp);
+    }
+    prev = c;
+  }
+}
+
 /* Function:  show_DebugMatrix(de,buffer)
  *
  * Descrip:    Show function cell state transition
@@ -12,7 +66,7 @@ extern "C" {
  * Arg:        buffer [UNKN ] Undocumented argument [char *]
  *
  */
-# line 61 "matrixdebug.dy"
+# line 109 "matrixdebug.dy"
 void show_DebugMatrix(DebugMatrix * de,char * buffer)
 {
   char ** base;
@@ -62,7 +116,7 @@ void show_DebugMatrix(DebugMatrix * de,char * buffer)
  * Return [UNKN ]  Undocumented return value [DebugState *]
  *
  */
-# line 102 "matrixdebug.dy"
+# line 150 "matrixdebug.dy"
 DebugState * find_DebugState(DebugMatrix * de,char * name)
 {
   int i;
@@ -87,7 +141,7 @@ DebugState * find_DebugState(DebugMatrix * de,char * name)
  * Return [UNKN ]  Undocumented return value [DebugTransition *]
  *
  */
-# line 118 "matrixdebug.dy"
+# line 166 "matrixdebug.dy"
 DebugTransition * find_DebugTransition(DebugState * state,char * name)
 {
   int i;
@@ -112,7 +166,7 @@ DebugTransition * find_DebugTransition(DebugState * state,char * name)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 135 "matrixdebug.dy"
+# line 183 "matrixdebug.dy"
 boolean user_DebugMatrix(DebugMatrix * de)
 {
   char buffer[MAXLINE];
@@ -168,10 +222,11 @@ boolean user_DebugMatrix(DebugMatrix * de)
 	if( is_integer_string(*brk,&bp->posi) == FALSE ) {
 	  fprintf(out,">>>> i position not an integer.\n");
 	} 
+	brk++;
 	if( is_integer_string(*brk,&bp->posj) == FALSE ) {
 	  fprintf(out,">>>> j position not an integer.\n");
 	} 	
-	fprintf(out,"Adding cursor break point\n");
+	fprintf(out,"Adding cursor break point %d,%d\n",bp->posi,bp->posj);
 	add_bp_DebugMatrix(de,bp);
       }
       ckfree(base);
@@ -213,7 +268,7 @@ boolean user_DebugMatrix(DebugMatrix * de)
  * Return [UNKN ]  Undocumented return value [MatrixDebugBreakPoint_type]
  *
  */
-# line 228 "matrixdebug.dy"
+# line 277 "matrixdebug.dy"
 MatrixDebugBreakPoint_type should_break_DebugMatrix(DebugMatrix * de)
 {
   int i;
@@ -258,7 +313,7 @@ MatrixDebugBreakPoint_type should_break_DebugMatrix(DebugMatrix * de)
  * Return [UNKN ]  Undocumented return value [DebugMatrix *]
  *
  */
-# line 266 "matrixdebug.dy"
+# line 315 "matrixdebug.dy"
 DebugMatrix * std_DebugMatrix(void)
 {
   DebugMatrix * out;
@@ -286,7 +341,7 @@ DebugMatrix * std_DebugMatrix(void)
   return out;
 }
 
-# line 276 "matrixdebug.c"
+# line 329 "matrixdebug.c"
 /* Function:  hard_link_DebugTransition(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -329,6 +384,10 @@ DebugTransition * DebugTransition_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
+    out->from_state_num = 0; 
     out->show_transition = NULL; 
 
 
@@ -349,6 +408,7 @@ DebugTransition * DebugTransition_alloc(void)
  */
 DebugTransition * free_DebugTransition(DebugTransition * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -357,10 +417,18 @@ DebugTransition * free_DebugTransition(DebugTransition * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     /* obj->fromstate is linked in */ 
     /* obj->show_transition is a function pointer */ 
 
@@ -616,6 +684,9 @@ DebugState * DebugState_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->state_num = 0;  
     out->trans = NULL;   
     out->len = out->maxlen = 0;  
@@ -639,6 +710,7 @@ DebugState * DebugState_alloc(void)
  */
 DebugState * free_DebugState(DebugState * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -648,10 +720,18 @@ DebugState * free_DebugState(DebugState * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     /* obj->statename is linked in */ 
     if( obj->trans != NULL)  {  
       for(i=0;i<obj->len;i++)    {  
@@ -710,6 +790,9 @@ DebugBreakPoint * DebugBreakPoint_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->type = MDBP_Cursor; 
     out->posi = 0;   
     out->posj = 0;   
@@ -734,6 +817,7 @@ DebugBreakPoint * DebugBreakPoint_alloc(void)
  */
 DebugBreakPoint * free_DebugBreakPoint(DebugBreakPoint * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -742,10 +826,18 @@ DebugBreakPoint * free_DebugBreakPoint(DebugBreakPoint * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -1162,6 +1254,9 @@ DebugMatrix * DebugMatrix_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->currenti = -1;  
     out->currentj = -1;  
     out->max_score = 0;  
@@ -1194,6 +1289,7 @@ DebugMatrix * DebugMatrix_alloc(void)
  */
 DebugMatrix * free_DebugMatrix(DebugMatrix * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1203,10 +1299,18 @@ DebugMatrix * free_DebugMatrix(DebugMatrix * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     /* obj->matrix is linked in */ 
     if( obj->state != NULL)  {  
       for(i=0;i<obj->len;i++)    {  
