@@ -3,6 +3,74 @@ extern "C" {
 #endif
 #include "packaln.h"
 
+/* Function:  read_simple_PackAln_file(file)
+ *
+ * Descrip:    Reads in a PackAln from a file in show_simple_PackAln
+ *
+ *
+ * Arg:        file [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+# line 57 "packaln.dy"
+PackAln * read_simple_PackAln_file(char * file)
+{
+  FILE * ifp;
+  PackAln * out;
+
+  if( (ifp=openfile(file,"r")) == NULL ) {
+    warn("Could not open %s for PackAln file reading",file);
+  }
+
+  out = read_simple_PackAln(ifp);
+
+  fclose(ifp);
+
+  return out;
+  
+}
+
+/* Function:  read_simple_PackAln(ifp)
+ *
+ * Descrip:    Reads in PackAln from file format in show_simple_PackAln
+ *
+ *
+ * Arg:        ifp [UNKN ] Undocumented argument [FILE *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+# line 77 "packaln.dy"
+PackAln * read_simple_PackAln(FILE * ifp)
+{
+  PackAln * out;
+  PackAlnUnit * unit;
+  char buffer[MAXLINE];
+
+  assert(ifp);
+
+  out = PackAln_alloc_std();
+
+  /* score line */
+  fgets(buffer,MAXLINE,ifp);
+
+  while( fgets(buffer,MAXLINE,ifp) != NULL ) {
+    if( strstartcmp(buffer,"//") == 0 ) {
+      break;
+    }
+
+    unit = PackAlnUnit_alloc();
+    
+    sscanf(buffer,"Position i:[%d] j:[%d] State:[%d] Score: %d",&unit->i,&unit->j,&unit->state,&unit->score);
+
+    add_PackAln(out,unit);
+  }
+
+  return out;
+}
+
+
 /* Function:  show_simple_PackAlnUnit(pau,ofp)
  *
  * Descrip:    shows packalnunit very simply ;)
@@ -12,7 +80,7 @@ extern "C" {
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 58 "packaln.dy"
+# line 110 "packaln.dy"
 void show_simple_PackAlnUnit(PackAlnUnit * pau,FILE * ofp)
 {
   fprintf(ofp,"Position i:[%d] j:[%d] State:[%d] Score: %d\n",pau->i,pau->j,pau->state,pau->score);
@@ -28,7 +96,7 @@ void show_simple_PackAlnUnit(PackAlnUnit * pau,FILE * ofp)
  * Arg:                  ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 67 "packaln.dy"
+# line 119 "packaln.dy"
 void show_text_PackAlnUnit(PackAlnUnit * pau,char * (*state_to_char)(int),FILE * ofp)
 {
   fprintf(ofp,"Position i:[%4d] j:[%4d] State:[%2d] Score:[%3d] [%s]\n",pau->i,pau->j,pau->state,pau->score,(*state_to_char)(pau->state));
@@ -47,7 +115,7 @@ void show_text_PackAlnUnit(PackAlnUnit * pau,char * (*state_to_char)(int),FILE *
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 79 "packaln.dy"
+# line 131 "packaln.dy"
 void show_bits_and_cumlative_PackAln(PackAln * pal,FILE * ofp)
 {
   int i;
@@ -78,7 +146,7 @@ void show_bits_and_cumlative_PackAln(PackAln * pal,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 103 "packaln.dy"
+# line 155 "packaln.dy"
 void show_simple_PackAln(PackAln * pal,FILE * ofp)
 {
   register int i;
@@ -100,7 +168,7 @@ void show_simple_PackAln(PackAln * pal,FILE * ofp)
  * Arg:                  ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 117 "packaln.dy"
+# line 169 "packaln.dy"
 void show_text_PackAln(PackAln * pal,char * (*state_to_char)(int),FILE * ofp)
 {
   register int i;
@@ -120,7 +188,7 @@ void show_text_PackAln(PackAln * pal,char * (*state_to_char)(int),FILE * ofp)
  * Arg:        pal [UNKN ] PackAln to be inverted  [PackAln *]
  *
  */
-# line 133 "packaln.dy"
+# line 185 "packaln.dy"
 void invert_PackAln(PackAln * pal) 
 {
   PackAlnUnit ** temp;
@@ -140,7 +208,7 @@ void invert_PackAln(PackAln * pal)
 
 
 
-# line 130 "packaln.c"
+# line 194 "packaln.c"
 /* Function:  hard_link_PackAlnUnit(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -183,6 +251,9 @@ PackAlnUnit * PackAlnUnit_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->i = 0;  
     out->j = 0;  
     out->state = 0;  
@@ -206,6 +277,7 @@ PackAlnUnit * PackAlnUnit_alloc(void)
  */
 PackAlnUnit * free_PackAlnUnit(PackAlnUnit * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -214,10 +286,18 @@ PackAlnUnit * free_PackAlnUnit(PackAlnUnit * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -471,6 +551,9 @@ PackAln * PackAln_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->pau = NULL; 
     out->len = out->maxlen = 0;  
     out->score = 0;  
@@ -493,6 +576,7 @@ PackAln * PackAln_alloc(void)
  */
 PackAln * free_PackAln(PackAln * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -502,10 +586,18 @@ PackAln * free_PackAln(PackAln * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->pau != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->pau[i] != NULL) 
