@@ -361,6 +361,9 @@ ProteinDB * ProteinDB_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->is_single_seq = FALSE;  
     out->is_random_db = FALSE;   
     out->single = NULL;  
@@ -387,6 +390,7 @@ ProteinDB * ProteinDB_alloc(void)
  */
 ProteinDB * free_ProteinDB(ProteinDB * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -395,10 +399,18 @@ ProteinDB * free_ProteinDB(ProteinDB * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->single != NULL) 
       free_ComplexSequence(obj->single);     
     if( obj->sdb != NULL)    
