@@ -14,12 +14,13 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [Sequence *]
  *
  */
-# line 42 "randomdb.dy"
+# line 44 "randomdb.dy"
 Sequence * Sequence_from_RandomProteinDB(RandomProteinDB * rndp)
 {
   int i;
   Sequence * out;
   char * name = "RandomSequence";
+  char buffer[512];
   char * seq;
 
   if( rndp->use_flat_length == FALSE ) {
@@ -33,6 +34,7 @@ Sequence * Sequence_from_RandomProteinDB(RandomProteinDB * rndp)
     seq[i] = draw_random_aa_RandomModel(rndp->emission);
   }
 
+  sprintf(buffer,"%s%d",name,rndp->num++);
   out = new_Sequence_from_strings(name,seq);
   ckfree(seq);
   return out;
@@ -49,7 +51,7 @@ Sequence * Sequence_from_RandomProteinDB(RandomProteinDB * rndp)
  * Return [UNKN ]  Undocumented return value [RandomProteinDB *]
  *
  */
-# line 71 "randomdb.dy"
+# line 75 "randomdb.dy"
 RandomProteinDB * new_flat_RandomProteinDB(RandomModel * rm,int length)
 {
   RandomProteinDB * out;
@@ -74,12 +76,13 @@ RandomProteinDB * new_flat_RandomProteinDB(RandomModel * rm,int length)
  * Return [UNKN ]  Undocumented return value [Sequence *]
  *
  */
-# line 88 "randomdb.dy"
+# line 92 "randomdb.dy"
 Sequence * Sequence_from_RandomDNADB(RandomDNADB * rndp)
 {
   int i;
   Sequence * out;
   char * name = "RandomSequence";
+  char buffer[512];
   char * seq;
 
   if( rndp->use_flat_length == FALSE ) {
@@ -93,7 +96,8 @@ Sequence * Sequence_from_RandomDNADB(RandomDNADB * rndp)
     seq[i] = draw_random_base_RandomModelDNA(rndp->emission);
   }
 
-  out = new_Sequence_from_strings(name,seq);
+  sprintf(buffer,"%s%d",name,rndp->num++);
+  out = new_Sequence_from_strings(buffer,seq);
   ckfree(seq);
   return out;
 }
@@ -109,7 +113,7 @@ Sequence * Sequence_from_RandomDNADB(RandomDNADB * rndp)
  * Return [UNKN ]  Undocumented return value [RandomDNADB *]
  *
  */
-# line 117 "randomdb.dy"
+# line 123 "randomdb.dy"
 RandomDNADB * new_flat_RandomDNADB(RandomModelDNA * rm,int length)
 {
   RandomDNADB * out;
@@ -118,11 +122,12 @@ RandomDNADB * new_flat_RandomDNADB(RandomModelDNA * rm,int length)
   out->use_flat_length = TRUE;
   out->emission = hard_link_RandomModelDNA(rm);
   out->length = length;
+  out->num = 1;
 
   return out;
 }
 
-# line 116 "randomdb.c"
+# line 121 "randomdb.c"
 /* Function:  hard_link_RandomProteinDB(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -165,10 +170,14 @@ RandomProteinDB * RandomProteinDB_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->use_flat_length = TRUE; 
     out->length = 0; 
     out->length_dist = NULL; 
     out->emission = NULL;    
+    out->num = 0;    
 
 
     return out;  
@@ -188,6 +197,7 @@ RandomProteinDB * RandomProteinDB_alloc(void)
  */
 RandomProteinDB * free_RandomProteinDB(RandomProteinDB * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -196,10 +206,18 @@ RandomProteinDB * free_RandomProteinDB(RandomProteinDB * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->length_dist != NULL)    
       free_Histogram(obj->length_dist);  
     if( obj->emission != NULL)   
@@ -253,10 +271,14 @@ RandomDNADB * RandomDNADB_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->use_flat_length = TRUE; 
     out->length = 0; 
     out->length_dist = NULL; 
     out->emission = NULL;    
+    out->num = 0;    
 
 
     return out;  
@@ -276,6 +298,7 @@ RandomDNADB * RandomDNADB_alloc(void)
  */
 RandomDNADB * free_RandomDNADB(RandomDNADB * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -284,10 +307,18 @@ RandomDNADB * free_RandomDNADB(RandomDNADB * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->length_dist != NULL)    
       free_Histogram(obj->length_dist);  
     if( obj->emission != NULL)   
@@ -475,6 +506,50 @@ RandomModel * access_emission_RandomProteinDB(RandomProteinDB * obj)
 }    
 
 
+/* Function:  replace_num_RandomProteinDB(obj,num)
+ *
+ * Descrip:    Replace member variable num
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj [UNKN ] Object holding the variable [RandomProteinDB *]
+ * Arg:        num [OWNER] New value of the variable [int]
+ *
+ * Return [SOFT ]  member variable num [boolean]
+ *
+ */
+boolean replace_num_RandomProteinDB(RandomProteinDB * obj,int num) 
+{
+    if( obj == NULL)     {  
+      warn("In replacement function num for object RandomProteinDB, got a NULL object"); 
+      return FALSE;  
+      }  
+    obj->num = num;  
+    return TRUE; 
+}    
+
+
+/* Function:  access_num_RandomProteinDB(obj)
+ *
+ * Descrip:    Access member variable num
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj [UNKN ] Object holding the variable [RandomProteinDB *]
+ *
+ * Return [SOFT ]  member variable num [int]
+ *
+ */
+int access_num_RandomProteinDB(RandomProteinDB * obj) 
+{
+    if( obj == NULL)     {  
+      warn("In accessor function num for object RandomProteinDB, got a NULL object");    
+      return 0;  
+      }  
+    return obj->num;     
+}    
+
+
 /* Function:  replace_use_flat_length_RandomDNADB(obj,use_flat_length)
  *
  * Descrip:    Replace member variable use_flat_length
@@ -651,6 +726,50 @@ RandomModelDNA * access_emission_RandomDNADB(RandomDNADB * obj)
 }    
 
 
+/* Function:  replace_num_RandomDNADB(obj,num)
+ *
+ * Descrip:    Replace member variable num
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj [UNKN ] Object holding the variable [RandomDNADB *]
+ * Arg:        num [OWNER] New value of the variable [int]
+ *
+ * Return [SOFT ]  member variable num [boolean]
+ *
+ */
+boolean replace_num_RandomDNADB(RandomDNADB * obj,int num) 
+{
+    if( obj == NULL)     {  
+      warn("In replacement function num for object RandomDNADB, got a NULL object"); 
+      return FALSE;  
+      }  
+    obj->num = num;  
+    return TRUE; 
+}    
+
+
+/* Function:  access_num_RandomDNADB(obj)
+ *
+ * Descrip:    Access member variable num
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj [UNKN ] Object holding the variable [RandomDNADB *]
+ *
+ * Return [SOFT ]  member variable num [int]
+ *
+ */
+int access_num_RandomDNADB(RandomDNADB * obj) 
+{
+    if( obj == NULL)     {  
+      warn("In accessor function num for object RandomDNADB, got a NULL object");    
+      return 0;  
+      }  
+    return obj->num;     
+}    
+
+
 
 #ifdef _cplusplus
 }
