@@ -12,6 +12,9 @@ extern "C" {
 
 struct Wise2_RandomModel {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Probability aminoacid[26];   
     char * name;     
     } ;  
@@ -25,6 +28,9 @@ typedef struct Wise2_RandomModel Wise2_RandomModel;
 
 struct Wise2_RandomModelScoreaa {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Score aminoacid[26];     
     char * name;     
     } ;  
@@ -38,6 +44,9 @@ typedef struct Wise2_RandomModelScoreaa Wise2_RandomModelScoreaa;
 
 struct Wise2_RandomCodonScore {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Score codon[126];    
     char * name;     
     } ;  
@@ -51,6 +60,9 @@ typedef struct Wise2_RandomCodonScore Wise2_RandomCodonScore;
 
 struct Wise2_RandomCodon {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Probability codon[126];  
     char * name;     
     } ;  
@@ -64,6 +76,9 @@ typedef struct Wise2_RandomCodon Wise2_RandomCodon;
 
 struct Wise2_RandomModelDNA {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Probability base[5];     
     char * name;     
     } ;  
@@ -77,6 +92,9 @@ typedef struct Wise2_RandomModelDNA Wise2_RandomModelDNA;
 
 struct Wise2_RandomModelDNAScore {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Score base[5];   
     char * name;     
     } ;  
@@ -239,6 +257,20 @@ RandomCodon  * Wise2_RandomCodon_from_raw_CodonFrequency(double codon[64],CodonT
 #define RandomCodon_from_raw_CodonFrequency Wise2_RandomCodon_from_raw_CodonFrequency
 
 
+/* Function:  flat_RandomCodon(ct)
+ *
+ * Descrip:    Makes a flat RandomCodon from CodonTable
+ *
+ *
+ * Arg:        ct [UNKN ] Undocumented argument [CodonTable *]
+ *
+ * Return [UNKN ]  Undocumented return value [RandomCodon *]
+ *
+ */
+RandomCodon * Wise2_flat_RandomCodon(CodonTable * ct);
+#define flat_RandomCodon Wise2_flat_RandomCodon
+
+
 /* Function:  RandomModelDNAScore_from_RandomModelDNA(rmd)
  *
  * Descrip:    Gives you a log space RandomModelDNAScore
