@@ -3,6 +3,145 @@ extern "C" {
 #endif
 #include "seqalign.h"
 
+/* Function:  is_gapped_SeqAlign(sa,pos)
+ *
+ * Descrip:    Tells you whether either . or - characters occur at
+ *             a position. Uses C-style coordinates
+ *
+ *
+ * Arg:         sa [UNKN ] Undocumented argument [SeqAlign *]
+ * Arg:        pos [UNKN ] Undocumented argument [int]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+# line 46 "seqalign.dy"
+boolean is_gapped_SeqAlign(SeqAlign * sa,int pos)
+{
+   int i;
+
+   assert(sa != NULL);
+   assert(pos < sa->seq[0]->len);
+
+   for(i=0;i<sa->len;i++) {
+     if( sa->seq[i]->seq[pos] == '.' ||
+         sa->seq[i]->seq[pos] == '-' ) {
+        return TRUE;
+     }
+  }
+
+  return FALSE;
+
+}
+
+/* Function:  trim_from_N_SeqAlign(sa)
+ *
+ * Descrip:    trims sequence alignments to first and last
+ *             non N character (for DNA obviously)
+ *
+ *
+ * Arg:        sa [UNKN ] Undocumented argument [SeqAlign *]
+ *
+ * Return [UNKN ]  Undocumented return value [SeqAlign *]
+ *
+ */
+# line 68 "seqalign.dy"
+SeqAlign * trim_from_N_SeqAlign(SeqAlign * sa)
+{
+  int i;
+  int j;
+  int k;
+  int left = 0;
+  int right = 0;
+
+  SeqAlign * out;
+  Sequence * seq;
+
+  char buffer[MAXLINE];
+
+  assert(sa);
+  for(i=0;i<sa->len;i++) {
+    if( sa->seq[i]->len > left ) {
+     left = sa->seq[i]->len;
+    }
+  }
+
+  for(i=0;i<sa->len;i++) {
+   for(j=0;j<sa->seq[i]->len;j++) {
+     if( sa->seq[i]->seq[j] != 'N' ) {
+       break;
+     }
+   }
+   if( left > j ) {
+     left = j;
+   }
+
+   for(j=sa->seq[i]->len-1;j>=0;j--) {
+     if( sa->seq[i]->seq[j] != 'N' ) {
+       break;
+     }
+   }
+
+   if ( right < j ) {
+     right = j;
+   }
+ 
+  }
+
+
+  right++;
+  /* now we have left and right, make new sa */
+
+  out = SeqAlign_alloc_len(sa->len);
+  
+  for(i=0;i<sa->len;i++) {
+   seq = Sequence_alloc();
+   add_SeqAlign(out,seq);
+   seq->name = stringalloc(sa->seq[i]->name);
+   
+   for(j=left,k=0;j<right && j<sa->seq[i]->len;j++,k++) {
+     buffer[k] = sa->seq[i]->seq[j];
+   }
+   for(;j<right;j++,k++) {
+     buffer[k] = 'N';
+   }
+   buffer[k] = '\0';
+
+   seq->seq = stringalloc(buffer);
+   seq->len = k;
+   seq->type = SEQUENCE_DNA;
+ }
+
+
+ return out;
+
+}
+
+/* Function:  reweight_SeqAlign(sal)
+ *
+ * Descrip:    Provides reweighting for a seqalign so all the weights
+ *             add up to 1.0
+ *
+ *
+ * Arg:        sal [UNKN ] Undocumented argument [SeqAlign *]
+ *
+ */
+# line 143 "seqalign.dy"
+void reweight_SeqAlign(SeqAlign * sal)
+{
+  int i;
+  double total =0.0;
+
+  for(i=0;i<sal->len;i++) {
+    total += sal->seq[i]->weight;
+  }
+
+  for(i=0;i<sal->len;i++) {
+    sal->seq[i]->weight = sal->seq[i]->weight/ total;
+  }
+
+
+}
 
 /* Function:  ColumnCount_from_SeqAlign(sa,col)
  *
@@ -18,7 +157,7 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [ColumnCount *]
  *
  */
-# line 48 "seqalign.dy"
+# line 165 "seqalign.dy"
 ColumnCount * ColumnCount_from_SeqAlign(SeqAlign * sa,int col)
 {
   ColumnCount * out;
@@ -66,7 +205,7 @@ ColumnCount * ColumnCount_from_SeqAlign(SeqAlign * sa,int col)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 85 "seqalign.dy"
+# line 202 "seqalign.dy"
 boolean write_selex_SeqAlign(const SeqAlign * sa,int name_len,int block_len,FILE * ofp)
 {
   int col;
@@ -74,9 +213,16 @@ boolean write_selex_SeqAlign(const SeqAlign * sa,int name_len,int block_len,FILE
   int j;
   int k;
 
+  assert(sa);
+  assert(ofp);
+  assert(name_len > 1);
+  assert(block_len > 1);
+
 
   for(col=0;col<sa->seq[0]->len;) {
     for(j=0;j<sa->len;j++) {
+      assert(sa->seq[j]->name);
+
       for(k=0;k<name_len && sa->seq[j]->name[k] != '\0';k++) {
 	fputc(sa->seq[j]->name[k],ofp);
       }
@@ -93,6 +239,8 @@ boolean write_selex_SeqAlign(const SeqAlign * sa,int name_len,int block_len,FILE
 
       /* do the block */
 
+      assert(sa->seq[j]->seq);
+
       for(i=col;(i - col) < block_len && i < sa->seq[j]->len ; i++) {
 	fputc(sa->seq[j]->seq[i],ofp);
       }
@@ -106,6 +254,116 @@ boolean write_selex_SeqAlign(const SeqAlign * sa,int name_len,int block_len,FILE
   return TRUE;
 }
 
+/* Function:  read_fasta_SeqAlign_file(filename)
+ *
+ * Descrip:    Reads in fasta file opening the file
+ *
+ *
+ * Arg:        filename [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [SeqAlign *]
+ *
+ */
+# line 253 "seqalign.dy"
+SeqAlign * read_fasta_SeqAlign_file(char * filename)
+{
+  FILE * ifp;
+  SeqAlign * out;
+
+  ifp = openfile(filename,"r");
+  if( ifp == NULL ) {
+    warn("Cannot open file %s",filename);
+    return NULL;
+  }
+  
+  out = read_fasta_SeqAlign(ifp);
+
+  fclose(ifp);
+
+  return out;
+}
+
+/* Function:  read_fasta_SeqAlign(ifp)
+ *
+ * Descrip:    Reads in fasta file style alignments
+ *
+ *
+ * Arg:        ifp [UNKN ] Undocumented argument [FILE *]
+ *
+ * Return [UNKN ]  Undocumented return value [SeqAlign *]
+ *
+ */
+# line 274 "seqalign.dy"
+SeqAlign * read_fasta_SeqAlign(FILE * ifp)
+{
+  Sequence * seq;
+  SeqAlign * out;
+
+  out = SeqAlign_alloc_std();
+
+  while( (seq = read_fasta_Sequence(ifp)) != NULL ) {
+    fprintf(stderr,"Adding sequence %s\n",seq->name);
+    add_SeqAlign(out,seq);
+  }
+
+  /* in multi-fasta alignments, this indicates the end */
+  if( out->len == 0 ) {
+    free_SeqAlign(out);
+    return NULL;
+  }
+
+  return out;
+}
+
+/* Function:  write_fasta_SeqAlign(sa,ofp)
+ *
+ * Descrip:    writes out Fasta file
+ *
+ *
+ * Arg:         sa [UNKN ] Undocumented argument [SeqAlign *]
+ * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
+ *
+ */
+# line 298 "seqalign.dy"
+void write_fasta_SeqAlign(SeqAlign * sa,FILE * ofp)
+{
+  int i;
+
+  for(i=0;i<sa->len;i++) {
+    write_fasta_Sequence(sa->seq[i],ofp);
+  }
+}
+
+/* Function:  read_selex_SeqAlign_file(filename)
+ *
+ * Descrip:    Open files as well
+ *
+ *
+ * Arg:        filename [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [SeqAlign *]
+ *
+ */
+# line 310 "seqalign.dy"
+SeqAlign * read_selex_SeqAlign_file(char * filename)
+{
+  FILE * ifp;
+  SeqAlign * out;
+
+  ifp = openfile(filename,"r");
+  if( ifp == NULL ) {
+    warn("Cannot open file %s",filename);
+    return NULL;
+  }
+  
+  out = read_selex_SeqAlign(ifp);
+
+  fclose(ifp);
+
+  return out;
+}
+
+
 /* Function:  read_selex_SeqAlign(ifp)
  *
  * Descrip:    Reads in selex (Stockholm) alignment
@@ -122,7 +380,7 @@ boolean write_selex_SeqAlign(const SeqAlign * sa,int name_len,int block_len,FILE
  * Return [UNKN ]  Undocumented return value [SeqAlign *]
  *
  */
-# line 133 "seqalign.dy"
+# line 338 "seqalign.dy"
 SeqAlign * read_selex_SeqAlign(FILE * ifp)
 {
   char buffer[4096];
@@ -140,7 +398,7 @@ SeqAlign * read_selex_SeqAlign(FILE * ifp)
   out = SeqAlign_alloc_std();
 
   while( fgets(buffer,4096,ifp) != NULL ) {
-    fprintf(stderr,"reading %s in selex alignment\n",buffer);
+    /*fprintf(stderr,"(selex) reading %s in selex alignment\n",buffer);*/
     if( buffer[0] == '#' ) 
       continue; /* skips over # */
     if( buffer[0] == '/' && buffer[1] == '/' )
@@ -217,26 +475,35 @@ SeqAlign * read_selex_SeqAlign(FILE * ifp)
 	/* get next line */
 
 	while( (seq = fgets(buffer,4096,ifp)) != NULL ) {
+	  /*fprintf(stderr,"(selex - 2nd loop) reading %s in selex alignment\n",buffer);*/
 	  if( buffer[0] != '#' ) 
 	    break;
-
+	  
 	  if( buffer[0] == '/' && buffer[1] == '/' )
 	    break;
 
 	}
+	
+	/*fprintf(stderr,"Broken from second loop\n");*/
+
 	if( seq == NULL ) {
+	  /*fprintf(stderr,"End of sequence?\n");*/
+	  isend = TRUE;
+	  break;
+	}
+
+	if( buffer[0] == '/' && buffer[1] == '/' ) {
+	  /*fprintf(stderr,"double slash terminator?\n");*/
 	  isend = TRUE;
 	  break;
 	}
 
 	/* if it is blank, break out of block */
 	if( !isalnum(buffer[0]) ) {
+	  /*fprintf(stderr,"Back to loop?\n");*/
 	  break;
 	}
 
-	if( buffer[0] == '/' && buffer[1] == '/' )
-	  break;
-
       }
 
       if( isend == TRUE )
@@ -254,7 +521,7 @@ SeqAlign * read_selex_SeqAlign(FILE * ifp)
 
 
 
-# line 250 "seqalign.c"
+# line 503 "seqalign.c"
 /* Function:  swap_SeqAlign(list,i,j)
  *
  * Descrip:    swap function: an internal for qsort_SeqAlign
@@ -501,6 +768,9 @@ SeqAlign * SeqAlign_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->seq = NULL; 
     out->len = out->maxlen = 0;  
@@ -523,6 +793,7 @@ SeqAlign * SeqAlign_alloc(void)
  */
 SeqAlign * free_SeqAlign(SeqAlign * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -532,10 +803,18 @@ SeqAlign * free_SeqAlign(SeqAlign * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->seq != NULL)    {  
@@ -594,6 +873,9 @@ ColumnCount * ColumnCount_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* count[27] is an array: no default possible */ 
 
 
@@ -614,6 +896,7 @@ ColumnCount * ColumnCount_alloc(void)
  */
 ColumnCount * free_ColumnCount(ColumnCount * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -622,10 +905,18 @@ ColumnCount * free_ColumnCount(ColumnCount * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
