@@ -13,6 +13,9 @@ extern "C" {
 
 #define SEQUENCEBLOCK 8000
 
+#define SEQUENCE_STARTSIZE 512
+#define SEQUENCE_REALLOC_LINEAR (4096*4)
+
 enum SequenceType {
 SEQUENCE_UNKNOWN = 64,
 SEQUENCE_PROTEIN,
@@ -42,24 +45,24 @@ SEQUENCE_RNA };
  *        amount of memory allocated in obj->seq 
  *        mainly for parsing purposes.
  *
- *        You are strongly encouraged to used the
- *        typed counterparts of Sequence, namely,
- *        Protein, cDNA and Genomic. By doing this 
- *        you are much, much less likely to
- *        mess up algorithms which expect specific
- *        sequence types. 
  *
  *
  */
 struct Wise2_Sequence {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * name;    /*  name of the sequence */ 
     char * seq; /*  actual sequence */ 
     int len;    /*  length of the sequence */ 
     int maxlen; /*  internal counter, indicating how much space in seq there is */ 
     int offset; /*  start (in bio-coords) of the sequence. Not called start due to weird legacy. */ 
     int end;    /*  end (in bio-coords == C coords) of the sequence */ 
-    int type;   /*  guess of protein/dna type */ 
+    char type;  /*  guess of protein/dna type */ 
+    int tax_id; /*  taxonimic id of this */ 
+    double weight;   
+    char * desc;    /*  description - often this will be NULL */ 
     } ;  
 /* Sequence defined */ 
 #ifndef DYNAMITE_DEFINED_Sequence
@@ -78,6 +81,9 @@ typedef struct Wise2_Sequence Wise2_Sequence;
  */
 struct Wise2_SequenceSet {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Sequence ** set;     
     int len;/* len for above set  */ 
     int maxlen; /* maxlen for above set */ 
@@ -452,6 +458,21 @@ Sequence * Wise2_read_Sequence_EMBL_seq(char * buffer,int maxlen,FILE * ifp);
 
 
 /* Function:  read_fasta_Sequence(ifp)
+ *
+ * Descrip:    reads a fasta file assumming pretty 
+ *             standard sanity for the file layout
+ *
+ *
+ * Arg:        ifp [UNKN ] Undocumented argument [FILE *]
+ *
+ * Return [UNKN ]  Undocumented return value [Sequence *]
+ *
+ */
+Sequence * Wise2_read_fasta_Sequence(FILE * ifp);
+#define read_fasta_Sequence Wise2_read_fasta_Sequence
+
+
+/* Function:  read_old_fasta_Sequence(ifp)
  *
  * Descrip:    reads the fasta file: format is
  *
@@ -471,8 +492,8 @@ Sequence * Wise2_read_Sequence_EMBL_seq(char * buffer,int maxlen,FILE * ifp);
  * Return [OWNER]  new Sequence structure  [Sequence *]
  *
  */
-Sequence * Wise2_read_fasta_Sequence(FILE * ifp);
-#define read_fasta_Sequence Wise2_read_fasta_Sequence
+Sequence * Wise2_read_old_fasta_Sequence(FILE * ifp);
+#define read_old_fasta_Sequence Wise2_read_old_fasta_Sequence
 
 
 /* Function:  show_Sequence_residue_list(seq,start,end,ofp)
@@ -498,6 +519,22 @@ void Wise2_show_Sequence_residue_list(Sequence * seq,int start,int end,FILE * of
 #define show_Sequence_residue_list Wise2_show_Sequence_residue_list
 
 
+/* Function:  add_string_to_Sequence(seq,more)
+ *
+ * Descrip:    New add_string_to_Sequence which should be much better at
+ *             handling memory update
+ *
+ *
+ * Arg:         seq [UNKN ] Undocumented argument [Sequence *]
+ * Arg:        more [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_add_string_to_Sequence(Sequence * seq,char * more);
+#define add_string_to_Sequence Wise2_add_string_to_Sequence
+
+
 /* Function:  empty_Sequence_from_dynamic_memory(name)
  *
  * Descrip:    Only allocates sequence structure and name
@@ -542,6 +579,34 @@ void Wise2_write_fasta_Sequence(Sequence * seq,FILE * ofp);
 #define write_fasta_Sequence Wise2_write_fasta_Sequence
 
 
+/* Function:  read_fasta_SequenceSet(ifp)
+ *
+ * Descrip:    reads a fasta file as a sequence set
+ *
+ *
+ * Arg:        ifp [UNKN ] Undocumented argument [FILE *]
+ *
+ * Return [UNKN ]  Undocumented return value [SequenceSet *]
+ *
+ */
+SequenceSet * Wise2_read_fasta_SequenceSet(FILE * ifp);
+#define read_fasta_SequenceSet Wise2_read_fasta_SequenceSet
+
+
+/* Function:  read_fasta_SequenceSet_file(filename)
+ *
+ * Descrip:    opens file and reads in sequence set
+ *
+ *
+ * Arg:        filename [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [SequenceSet *]
+ *
+ */
+SequenceSet * Wise2_read_fasta_SequenceSet_file(char * filename);
+#define read_fasta_SequenceSet_file Wise2_read_fasta_SequenceSet_file
+
+
 /* Function:  hard_link_Sequence(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -694,40 +759,52 @@ SequenceSet * Wise2_free_SequenceSet(SequenceSet * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
-boolean Wise2_replace_name_Sequence(Sequence * obj,char * name);
-#define replace_name_Sequence Wise2_replace_name_Sequence
 boolean Wise2_replace_len_Sequence(Sequence * obj,int len);
 #define replace_len_Sequence Wise2_replace_len_Sequence
-int Wise2_access_len_Sequence(Sequence * obj);
-#define access_len_Sequence Wise2_access_len_Sequence
-boolean Wise2_replace_maxlen_Sequence(Sequence * obj,int maxlen);
-#define replace_maxlen_Sequence Wise2_replace_maxlen_Sequence
-int Wise2_access_maxlen_Sequence(Sequence * obj);
-#define access_maxlen_Sequence Wise2_access_maxlen_Sequence
-int Wise2_length_set_SequenceSet(SequenceSet * obj);
-#define length_set_SequenceSet Wise2_length_set_SequenceSet
-Sequence * Wise2_access_set_SequenceSet(SequenceSet * obj,int i);
-#define access_set_SequenceSet Wise2_access_set_SequenceSet
-char * Wise2_access_name_Sequence(Sequence * obj);
-#define access_name_Sequence Wise2_access_name_Sequence
 boolean Wise2_replace_offset_Sequence(Sequence * obj,int offset);
 #define replace_offset_Sequence Wise2_replace_offset_Sequence
-char * Wise2_access_seq_Sequence(Sequence * obj);
-#define access_seq_Sequence Wise2_access_seq_Sequence
 int Wise2_access_offset_Sequence(Sequence * obj);
 #define access_offset_Sequence Wise2_access_offset_Sequence
-int Wise2_access_type_Sequence(Sequence * obj);
-#define access_type_Sequence Wise2_access_type_Sequence
+int Wise2_length_set_SequenceSet(SequenceSet * obj);
+#define length_set_SequenceSet Wise2_length_set_SequenceSet
 boolean Wise2_replace_end_Sequence(Sequence * obj,int end);
 #define replace_end_Sequence Wise2_replace_end_Sequence
-boolean Wise2_replace_seq_Sequence(Sequence * obj,char * seq);
-#define replace_seq_Sequence Wise2_replace_seq_Sequence
-boolean Wise2_replace_type_Sequence(Sequence * obj,int type);
-#define replace_type_Sequence Wise2_replace_type_Sequence
+boolean Wise2_replace_name_Sequence(Sequence * obj,char * name);
+#define replace_name_Sequence Wise2_replace_name_Sequence
 int Wise2_access_end_Sequence(Sequence * obj);
 #define access_end_Sequence Wise2_access_end_Sequence
-boolean Wise2_add_string_to_Sequence(Sequence * seq,char * more);
-#define add_string_to_Sequence Wise2_add_string_to_Sequence
+boolean Wise2_replace_seq_Sequence(Sequence * obj,char * seq);
+#define replace_seq_Sequence Wise2_replace_seq_Sequence
+boolean Wise2_replace_type_Sequence(Sequence * obj,char type);
+#define replace_type_Sequence Wise2_replace_type_Sequence
+int Wise2_access_len_Sequence(Sequence * obj);
+#define access_len_Sequence Wise2_access_len_Sequence
+char Wise2_access_type_Sequence(Sequence * obj);
+#define access_type_Sequence Wise2_access_type_Sequence
+int Wise2_access_maxlen_Sequence(Sequence * obj);
+#define access_maxlen_Sequence Wise2_access_maxlen_Sequence
+boolean Wise2_replace_tax_id_Sequence(Sequence * obj,int tax_id);
+#define replace_tax_id_Sequence Wise2_replace_tax_id_Sequence
+Sequence * Wise2_access_set_SequenceSet(SequenceSet * obj,int i);
+#define access_set_SequenceSet Wise2_access_set_SequenceSet
+int Wise2_access_tax_id_Sequence(Sequence * obj);
+#define access_tax_id_Sequence Wise2_access_tax_id_Sequence
+char * Wise2_access_seq_Sequence(Sequence * obj);
+#define access_seq_Sequence Wise2_access_seq_Sequence
+boolean Wise2_replace_weight_Sequence(Sequence * obj,double weight);
+#define replace_weight_Sequence Wise2_replace_weight_Sequence
+char * Wise2_access_name_Sequence(Sequence * obj);
+#define access_name_Sequence Wise2_access_name_Sequence
+char * Wise2_access_desc_Sequence(Sequence * obj);
+#define access_desc_Sequence Wise2_access_desc_Sequence
+double Wise2_access_weight_Sequence(Sequence * obj);
+#define access_weight_Sequence Wise2_access_weight_Sequence
+boolean Wise2_replace_maxlen_Sequence(Sequence * obj,int maxlen);
+#define replace_maxlen_Sequence Wise2_replace_maxlen_Sequence
+boolean Wise2_replace_desc_Sequence(Sequence * obj,char * desc);
+#define replace_desc_Sequence Wise2_replace_desc_Sequence
+boolean Wise2_add_string_to_Sequence_old(Sequence * seq,char * more);
+#define add_string_to_Sequence_old Wise2_add_string_to_Sequence_old
 Sequence * Wise2_Sequence_from_static_memory (char * name,char * seq);
 #define Sequence_from_static_memory  Wise2_Sequence_from_static_memory 
 Sequence * Wise2_Sequence_from_dynamic_memory(char * name,char * seq);
