@@ -31,6 +31,12 @@
  * Wise2_access_end_Sequence
  * Wise2_replace_type_Sequence
  * Wise2_access_type_Sequence
+ * Wise2_replace_tax_id_Sequence
+ * Wise2_access_tax_id_Sequence
+ * Wise2_replace_weight_Sequence
+ * Wise2_access_weight_Sequence
+ * Wise2_replace_desc_Sequence
+ * Wise2_access_desc_Sequence
  * Wise2_free_Sequence [destructor]
  *
  */
@@ -243,22 +249,13 @@ Wise2_Sequence * Wise2_read_Sequence_EMBL_seq( char * buffer,int maxlen,FILE * i
 
 /* Function:  Wise2_read_fasta_Sequence(ifp)
  *
- * Descrip:    reads the fasta file: format is
- *
- *             >name
- *             sequence
- *
- *             allocates a structure and puts in the
- *             sequence. Calls /make_len_type_Sequence to
- *             check type and length.
- *
- *             It leaves the '>' on the next fasta sequence
- *             for multiple sequence reading
+ * Descrip:    reads a fasta file assumming pretty 
+ *             standard sanity for the file layout
  *
  *
- * Arg:        ifp          input file to read from [FILE *]
+ * Arg:        ifp          Undocumented argument [FILE *]
  *
- * Returns new Sequence structure  [Wise2_Sequence *]
+ * Returns Undocumented return value [Wise2_Sequence *]
  *
  */
 Wise2_Sequence * Wise2_read_fasta_Sequence( FILE * ifp);
@@ -510,12 +507,12 @@ int Wise2_access_end_Sequence( Wise2_Sequence * obj);
  *
  *
  * Arg:        obj          Object holding the variable [Wise2_Sequence *]
- * Arg:        type         New value of the variable [int]
+ * Arg:        type         New value of the variable [char]
  *
  * Returns member variable type [boolean]
  *
  */
-boolean Wise2_replace_type_Sequence( Wise2_Sequence * obj,int type);
+boolean Wise2_replace_type_Sequence( Wise2_Sequence * obj,char type);
 
 /* Function:  Wise2_access_type_Sequence(obj)
  *
@@ -525,10 +522,91 @@ boolean Wise2_replace_type_Sequence( Wise2_Sequence * obj,int type);
  *
  * Arg:        obj          Object holding the variable [Wise2_Sequence *]
  *
- * Returns member variable type [int]
+ * Returns member variable type [char]
  *
  */
-int Wise2_access_type_Sequence( Wise2_Sequence * obj);
+char Wise2_access_type_Sequence( Wise2_Sequence * obj);
+
+/* Function:  Wise2_replace_tax_id_Sequence(obj,tax_id)
+ *
+ * Descrip:    Replace member variable tax_id
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj          Object holding the variable [Wise2_Sequence *]
+ * Arg:        tax_id       New value of the variable [int]
+ *
+ * Returns member variable tax_id [boolean]
+ *
+ */
+boolean Wise2_replace_tax_id_Sequence( Wise2_Sequence * obj,int tax_id);
+
+/* Function:  Wise2_access_tax_id_Sequence(obj)
+ *
+ * Descrip:    Access member variable tax_id
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj          Object holding the variable [Wise2_Sequence *]
+ *
+ * Returns member variable tax_id [int]
+ *
+ */
+int Wise2_access_tax_id_Sequence( Wise2_Sequence * obj);
+
+/* Function:  Wise2_replace_weight_Sequence(obj,weight)
+ *
+ * Descrip:    Replace member variable weight
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj          Object holding the variable [Wise2_Sequence *]
+ * Arg:        weight       New value of the variable [double]
+ *
+ * Returns member variable weight [boolean]
+ *
+ */
+boolean Wise2_replace_weight_Sequence( Wise2_Sequence * obj,double weight);
+
+/* Function:  Wise2_access_weight_Sequence(obj)
+ *
+ * Descrip:    Access member variable weight
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj          Object holding the variable [Wise2_Sequence *]
+ *
+ * Returns member variable weight [double]
+ *
+ */
+double Wise2_access_weight_Sequence( Wise2_Sequence * obj);
+
+/* Function:  Wise2_replace_desc_Sequence(obj,desc)
+ *
+ * Descrip:    Replace member variable desc
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj          Object holding the variable [Wise2_Sequence *]
+ * Arg:        desc         New value of the variable [char *]
+ *
+ * Returns member variable desc [boolean]
+ *
+ */
+boolean Wise2_replace_desc_Sequence( Wise2_Sequence * obj,char * desc);
+
+/* Function:  Wise2_access_desc_Sequence(obj)
+ *
+ * Descrip:    Access member variable desc
+ *             For use principly by API functions
+ *
+ *
+ * Arg:        obj          Object holding the variable [Wise2_Sequence *]
+ *
+ * Returns member variable desc [char *]
+ *
+ */
+char * Wise2_access_desc_Sequence( Wise2_Sequence * obj);
 
 /* This is the destructor function, ie, call this to free object*/
 /* Function:  Wise2_free_Sequence(obj)
