@@ -20,7 +20,7 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [Sequence *]
  *
  */
-# line 74 "sequencedb.dy"
+# line 76 "sequencedb.dy"
 Sequence * get_Sequence_from_SequenceDB(SequenceDB * sdb,DataEntry * de)
 {
   FILE * ifp;
@@ -82,7 +82,7 @@ Sequence * get_Sequence_from_SequenceDB(SequenceDB * sdb,DataEntry * de)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 127 "sequencedb.dy"
+# line 129 "sequencedb.dy"
 boolean add_SequenceDB_info_DataEntry(SequenceDB * sdb,DataEntry * de)
 {
   if( sdb == NULL || de == NULL ) {
@@ -110,7 +110,7 @@ boolean add_SequenceDB_info_DataEntry(SequenceDB * sdb,DataEntry * de)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 149 "sequencedb.dy"
+# line 151 "sequencedb.dy"
 boolean close_SequenceDB(Sequence * last,SequenceDB * sdb)
 {
   if( last != NULL )
@@ -140,13 +140,22 @@ boolean close_SequenceDB(Sequence * last,SequenceDB * sdb)
  * Return [UNKN ]  Undocumented return value [Sequence *]
  *
  */
-# line 173 "sequencedb.dy"
+# line 175 "sequencedb.dy"
 Sequence * init_SequenceDB(SequenceDB * sdb,int * return_status)
 {
+  Sequence * temp = NULL;
+  int count;
+
   sdb->current_source = 0;
   sdb->sequence_no =0;
   load_next_fs_SequenceDB(sdb);
 
+  if( sdb->seq_start != -1 && sdb->seq_end != -1 ) {
+    for(count=0;count <= sdb->seq_start;count++) {
+      temp = reload_SequenceDB(temp,sdb,return_status);
+    }
+    return temp;
+  }
 
   return reload_SequenceDB(NULL,sdb,return_status);
 }
@@ -164,7 +173,7 @@ Sequence * init_SequenceDB(SequenceDB * sdb,int * return_status)
  * Return [UNKN ]  Undocumented return value [Sequence *]
  *
  */
-# line 191 "sequencedb.dy"
+# line 202 "sequencedb.dy"
 Sequence * reload_SequenceDB(Sequence * last,SequenceDB * sdb,int * return_status)
 {
   Sequence * out;
@@ -178,6 +187,16 @@ Sequence * reload_SequenceDB(Sequence * last,SequenceDB * sdb,int * return_statu
   if( last != NULL ) 
     free_Sequence(last);
 
+
+  /* if there is a seq_end, then see whether this is the end */
+
+  if( sdb->seq_end != -1 && sdb->sequence_no == sdb->seq_end ) {
+    /* end */
+    sdb->current_source = -1;
+    *return_status = DB_RETURN_END;
+    return NULL;
+  }
+
   /** see if we can read a Sequence now **/
 
 
@@ -249,7 +268,7 @@ Sequence * reload_SequenceDB(Sequence * last,SequenceDB * sdb,int * return_statu
  * Return [UNKN ]  Undocumented return value [Sequence *]
  *
  */
-# line 269 "sequencedb.dy"
+# line 290 "sequencedb.dy"
 Sequence * get_next_SequenceDB(SequenceDB * sdb)
 {
   
@@ -278,7 +297,7 @@ Sequence * get_next_SequenceDB(SequenceDB * sdb)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 291 "sequencedb.dy"
+# line 312 "sequencedb.dy"
 boolean SequenceDB_at_end(SequenceDB * sdb)
 {
   if( sdb->current_source == -1 ) {
@@ -306,7 +325,7 @@ boolean SequenceDB_at_end(SequenceDB * sdb)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 312 "sequencedb.dy"
+# line 333 "sequencedb.dy"
 boolean load_next_fs_SequenceDB(SequenceDB * sdb)
 {
   FileSource * fs;
@@ -347,7 +366,7 @@ boolean load_next_fs_SequenceDB(SequenceDB * sdb)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 346 "sequencedb.dy"
+# line 367 "sequencedb.dy"
 boolean close_last_fs_SequenceDB(SequenceDB * sdb)
 {
   FileSource * fs;
@@ -389,7 +408,7 @@ boolean close_last_fs_SequenceDB(SequenceDB * sdb)
  * Return [UNKN ]  Undocumented return value [SequenceDB *]
  *
  */
-# line 382 "sequencedb.dy"
+# line 403 "sequencedb.dy"
 SequenceDB * SequenceDB_from_FILE_and_format(FILE * input,int format)
 {
   SequenceDB * out;
@@ -416,7 +435,7 @@ SequenceDB * SequenceDB_from_FILE_and_format(FILE * input,int format)
  * Return [UNKN ]  Undocumented return value [FileSource *]
  *
  */
-# line 401 "sequencedb.dy"
+# line 422 "sequencedb.dy"
 FileSource * FileSource_from_FILE_and_format(FILE * input,int format)
 {
   FileSource * fs;
@@ -440,7 +459,7 @@ FileSource * FileSource_from_FILE_and_format(FILE * input,int format)
  * Return [UNKN ]  Undocumented return value [SequenceDB *]
  *
  */
-# line 419 "sequencedb.dy"
+# line 440 "sequencedb.dy"
 SequenceDB * single_fasta_SequenceDB(char * filename)
 {
   SequenceDB * out;
@@ -456,7 +475,8 @@ SequenceDB * single_fasta_SequenceDB(char * filename)
   fs->format   = SEQ_DB_FASTA;
 
   out = SequenceDB_alloc_len(1);
-
+  out->seq_start = -1;
+  out->seq_end   = -1;
   add_SequenceDB(out,fs);
 
   return out;
@@ -481,7 +501,7 @@ SequenceDB * single_fasta_SequenceDB(char * filename)
  * Return [UNKN ]  Undocumented return value [SequenceDB *]
  *
  */
-# line 454 "sequencedb.dy"
+# line 476 "sequencedb.dy"
 SequenceDB * read_SequenceDB_line(char * line,FILE * ifp)
 {
   SequenceDB * out = NULL;
@@ -529,7 +549,7 @@ SequenceDB * read_SequenceDB_line(char * line,FILE * ifp)
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 494 "sequencedb.dy"
+# line 516 "sequencedb.dy"
 int word_to_format(char * word)
 {
   if( strcmp(word,"fasta") == 0 ) {
@@ -554,7 +574,7 @@ int word_to_format(char * word)
  * Return [UNKN ]  Undocumented return value [FileSource *]
  *
  */
-# line 512 "sequencedb.dy"
+# line 534 "sequencedb.dy"
 FileSource * FileSource_from_line(char * line)
 {
   FileSource * out;
@@ -587,7 +607,7 @@ FileSource * FileSource_from_line(char * line)
 
 
 
-# line 559 "sequencedb.c"
+# line 579 "sequencedb.c"
 /* Function:  hard_link_FileSource(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -630,6 +650,9 @@ FileSource * FileSource_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->filename = NULL;    
     out->format = 0; 
     out->type = 0;   
@@ -652,6 +675,7 @@ FileSource * FileSource_alloc(void)
  */
 FileSource * free_FileSource(FileSource * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -660,10 +684,18 @@ FileSource * free_FileSource(FileSource * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->filename != NULL)   
       ckfree(obj->filename);     
     /* obj->input is linked in */ 
@@ -920,6 +952,9 @@ SequenceDB * SequenceDB_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->fs = NULL;  
     out->len = out->maxlen = 0;  
@@ -927,6 +962,8 @@ SequenceDB * SequenceDB_alloc(void)
     out->sequence_no = 0;    
     out->byte_position = 0;  
     out->has_warned_single = 0;  
+    out->seq_start = -1; 
+    out->seq_end = -1;   
 
 
     return out;  
@@ -946,6 +983,7 @@ SequenceDB * SequenceDB_alloc(void)
  */
 SequenceDB * free_SequenceDB(SequenceDB * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -955,10 +993,18 @@ SequenceDB * free_SequenceDB(SequenceDB * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->fs != NULL) {  
