@@ -195,6 +195,9 @@ SupportingFeature * SupportingFeature_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->start = 0;  
     out->end = 0;    
     out->hstart = 0; 
@@ -220,6 +223,7 @@ SupportingFeature * SupportingFeature_alloc(void)
  */
 SupportingFeature * free_SupportingFeature(SupportingFeature * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -228,10 +232,18 @@ SupportingFeature * free_SupportingFeature(SupportingFeature * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->hid != NULL)    
       ckfree(obj->hid);  
 
@@ -487,6 +499,9 @@ Exon * Exon_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->start = 0;  
     out->end = 0;    
     out->used = FALSE;   
@@ -513,6 +528,7 @@ Exon * Exon_alloc(void)
  */
 Exon * free_Exon(Exon * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -522,10 +538,18 @@ Exon * free_Exon(Exon * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->sf != NULL) {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->sf[i] != NULL)  
@@ -949,6 +973,9 @@ Transcript * Transcript_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->exon = NULL;    
     out->ex_len = out->ex_maxlen = 0;    
     out->translation = NULL; 
@@ -973,6 +1000,7 @@ Transcript * Transcript_alloc(void)
  */
 Transcript * free_Transcript(Transcript * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -982,10 +1010,18 @@ Transcript * free_Transcript(Transcript * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->exon != NULL)   {  
       for(i=0;i<obj->ex_len;i++) {  
         if( obj->exon[i] != NULL)    
