@@ -9,7 +9,11 @@ extern "C" {
 
 struct Wise2_CdnaWise10 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     ComplexSequence* query;  
@@ -56,6 +60,22 @@ struct thread_pool_holder_CdnaWise10 {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_CdnaWise10_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(CdnaWise10*,int,int,int); 
+    int (*access_special)(CdnaWise10*,int,int,int);  
+    } ;  
+/* CdnaWise10_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_CdnaWise10_access_func_holder
+typedef struct Wise2_CdnaWise10_access_func_holder Wise2_CdnaWise10_access_func_holder;
+#define CdnaWise10_access_func_holder Wise2_CdnaWise10_access_func_holder
+#define DYNAMITE_DEFINED_CdnaWise10_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -65,6 +85,35 @@ struct thread_pool_holder_CdnaWise10 {
 
 
 
+/* Function:  PackAln_read_Shatter_CdnaWise10(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [CdnaWise10 *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_CdnaWise10(CdnaWise10 * mat);
+#define PackAln_read_Shatter_CdnaWise10 Wise2_PackAln_read_Shatter_CdnaWise10
+
+
+/* Function:  calculate_shatter_CdnaWise10(mat,dpenv)
+ *
+ * Descrip:    This function calculates the CdnaWise10 matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [CdnaWise10 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_CdnaWise10(CdnaWise10 * mat,DPEnvelope * dpenv);
+#define calculate_shatter_CdnaWise10 Wise2_calculate_shatter_CdnaWise10
+
+
 /* Function:  search_CdnaWise10(dbsi,out,querydb,targetdb,gp,sc,rndcodon,utr,gap_open,gap_ext,utr_gap,intron_gap)
  *
  * Descrip:    This function makes a database search of CdnaWise10
@@ -150,7 +199,7 @@ PackAln * Wise2_PackAln_bestmemory_CdnaWise10(ComplexSequence* query,ComplexSequ
 #define PackAln_bestmemory_CdnaWise10 Wise2_PackAln_bestmemory_CdnaWise10
 
 
-/* Function:  allocate_Expl_CdnaWise10(query,target,gp,sc,rndcodon,utr,gap_open,gap_ext,utr_gap,intron_gap)
+/* Function:  allocate_Expl_CdnaWise10(query,target,gp,sc,rndcodon,utr,gap_open,gap_ext,utr_gap,intron_gap,dpri)
  *
  * Descrip:    This function allocates the CdnaWise10 structure
  *             and the basematrix area for explicit memory implementations
@@ -167,11 +216,12 @@ PackAln * Wise2_PackAln_bestmemory_CdnaWise10(ComplexSequence* query,ComplexSequ
  * Arg:           gap_ext [UNKN ] Resource [Score]
  * Arg:           utr_gap [UNKN ] Resource [Score]
  * Arg:        intron_gap [UNKN ] Resource [Score]
+ * Arg:              dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [CdnaWise10 *]
  *
  */
-CdnaWise10 * Wise2_allocate_Expl_CdnaWise10(ComplexSequence* query,ComplexSequence* target ,GeneParser4Score * gp,CodonMatrixScore * sc,RandomCodonScore * rndcodon,DnaMatrix* utr,Score gap_open,Score gap_ext,Score utr_gap,Score intron_gap);
+CdnaWise10 * Wise2_allocate_Expl_CdnaWise10(ComplexSequence* query,ComplexSequence* target ,GeneParser4Score * gp,CodonMatrixScore * sc,RandomCodonScore * rndcodon,DnaMatrix* utr,Score gap_open,Score gap_ext,Score utr_gap,Score intron_gap,DPRunImpl * dpri);
 #define allocate_Expl_CdnaWise10 Wise2_allocate_Expl_CdnaWise10
 
 
@@ -301,6 +351,21 @@ PackAln * Wise2_PackAln_read_Expl_CdnaWise10(CdnaWise10 * mat);
 #define PackAln_read_Expl_CdnaWise10 Wise2_PackAln_read_Expl_CdnaWise10
 
 
+/* Function:  PackAln_read_generic_CdnaWise10(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [CdnaWise10 *]
+ * Arg:          h [UNKN ] Undocumented argument [CdnaWise10_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_CdnaWise10(CdnaWise10 * mat,CdnaWise10_access_func_holder h);
+#define PackAln_read_generic_CdnaWise10 Wise2_PackAln_read_generic_CdnaWise10
+
+
 /* Function:  calculate_CdnaWise10(mat)
  *
  * Descrip:    This function calculates the CdnaWise10 matrix when in explicit mode
@@ -316,6 +381,21 @@ boolean Wise2_calculate_CdnaWise10(CdnaWise10 * mat);
 #define calculate_CdnaWise10 Wise2_calculate_CdnaWise10
 
 
+/* Function:  calculate_dpenv_CdnaWise10(mat,dpenv)
+ *
+ * Descrip:    This function calculates the CdnaWise10 matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] CdnaWise10 which contains explicit basematrix memory [CdnaWise10 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_CdnaWise10(CdnaWise10 * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_CdnaWise10 Wise2_calculate_dpenv_CdnaWise10
+
+
 /* Function:  CdnaWise10_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -352,6 +432,10 @@ CdnaWise10 * Wise2_free_CdnaWise10(CdnaWise10 * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_CdnaWise10_shatter_access_main(CdnaWise10 * mat,int i,int j,int state);
+#define CdnaWise10_shatter_access_main Wise2_CdnaWise10_shatter_access_main
+int Wise2_CdnaWise10_shatter_access_special(CdnaWise10 * mat,int i,int j,int state);
+#define CdnaWise10_shatter_access_special Wise2_CdnaWise10_shatter_access_special
 void * Wise2_thread_loop_CdnaWise10(void * ptr);
 #define thread_loop_CdnaWise10 Wise2_thread_loop_CdnaWise10
 int Wise2_score_only_CdnaWise10(ComplexSequence* query,ComplexSequence* target ,GeneParser4Score * gp,CodonMatrixScore * sc,RandomCodonScore * rndcodon,DnaMatrix* utr,Score gap_open,Score gap_ext,Score utr_gap,Score intron_gap);
@@ -388,21 +472,25 @@ void Wise2_run_up_dc_CdnaWise10(CdnaWise10 * mat,int starti,int stopi,int startj
 #define run_up_dc_CdnaWise10 Wise2_run_up_dc_CdnaWise10
 void Wise2_init_dc_CdnaWise10(CdnaWise10 * mat);
 #define init_dc_CdnaWise10 Wise2_init_dc_CdnaWise10
-boolean Wise2_dc_start_end_calculate_CdnaWise10(CdnaWise10 * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_CdnaWise10 Wise2_dc_start_end_calculate_CdnaWise10
 int Wise2_start_end_find_end_CdnaWise10(CdnaWise10 * mat,int * endj);
 #define start_end_find_end_CdnaWise10 Wise2_start_end_find_end_CdnaWise10
+boolean Wise2_dc_optimised_start_end_calc_CdnaWise10(CdnaWise10 *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_CdnaWise10 Wise2_dc_optimised_start_end_calc_CdnaWise10
 void Wise2_init_start_end_linear_CdnaWise10(CdnaWise10 * mat);
 #define init_start_end_linear_CdnaWise10 Wise2_init_start_end_linear_CdnaWise10
 AlnConvertSet * Wise2_AlnConvertSet_CdnaWise10(void);
 #define AlnConvertSet_CdnaWise10 Wise2_AlnConvertSet_CdnaWise10
-int Wise2_find_end_CdnaWise10(CdnaWise10 * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_CdnaWise10_explicit_access_main(CdnaWise10 * mat,int i,int j,int state);
+#define CdnaWise10_explicit_access_main Wise2_CdnaWise10_explicit_access_main
+int Wise2_CdnaWise10_explicit_access_special(CdnaWise10 * mat,int i,int j,int state);
+#define CdnaWise10_explicit_access_special Wise2_CdnaWise10_explicit_access_special
+int Wise2_find_end_CdnaWise10(CdnaWise10 * mat,int * ri,int * rj,int * state,boolean * isspecial,CdnaWise10_access_func_holder h);
 #define find_end_CdnaWise10 Wise2_find_end_CdnaWise10
 void Wise2_CdnaWise10_debug_show_matrix(CdnaWise10 * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define CdnaWise10_debug_show_matrix Wise2_CdnaWise10_debug_show_matrix
-int Wise2_max_calc_CdnaWise10(CdnaWise10 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_CdnaWise10(CdnaWise10 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,CdnaWise10_access_func_holder h);
 #define max_calc_CdnaWise10 Wise2_max_calc_CdnaWise10
-int Wise2_max_calc_special_CdnaWise10(CdnaWise10 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_CdnaWise10(CdnaWise10 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,CdnaWise10_access_func_holder h);
 #define max_calc_special_CdnaWise10 Wise2_max_calc_special_CdnaWise10
 
 #ifdef _cplusplus
