@@ -8,7 +8,11 @@ extern "C" {
 
 struct Wise2_CloneWise {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     MappedCloneSet * q;  
@@ -49,6 +53,22 @@ struct thread_pool_holder_CloneWise {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_CloneWise_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(CloneWise*,int,int,int);  
+    int (*access_special)(CloneWise*,int,int,int);   
+    } ;  
+/* CloneWise_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_CloneWise_access_func_holder
+typedef struct Wise2_CloneWise_access_func_holder Wise2_CloneWise_access_func_holder;
+#define CloneWise_access_func_holder Wise2_CloneWise_access_func_holder
+#define DYNAMITE_DEFINED_CloneWise_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -58,6 +78,35 @@ struct thread_pool_holder_CloneWise {
 
 
 
+/* Function:  PackAln_read_Shatter_CloneWise(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [CloneWise *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_CloneWise(CloneWise * mat);
+#define PackAln_read_Shatter_CloneWise Wise2_PackAln_read_Shatter_CloneWise
+
+
+/* Function:  calculate_shatter_CloneWise(mat,dpenv)
+ *
+ * Descrip:    This function calculates the CloneWise matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [CloneWise *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_CloneWise(CloneWise * mat,DPEnvelope * dpenv);
+#define calculate_shatter_CloneWise Wise2_calculate_shatter_CloneWise
+
+
 /* Function:  search_CloneWise(dbsi,out,q,t,match,target_skip_start,target_skip,query_skip_start,query_skip,spread,target_special_s)
  *
  * Descrip:    This function makes a database search of CloneWise
@@ -140,7 +189,7 @@ PackAln * Wise2_PackAln_bestmemory_CloneWise(MappedCloneSet * q,MappedCloneSet *
 #define PackAln_bestmemory_CloneWise Wise2_PackAln_bestmemory_CloneWise
 
 
-/* Function:  allocate_Expl_CloneWise(q,t,match,target_skip_start,target_skip,query_skip_start,query_skip,spread,target_special_s)
+/* Function:  allocate_Expl_CloneWise(q,t,match,target_skip_start,target_skip,query_skip_start,query_skip,spread,target_special_s,dpri)
  *
  * Descrip:    This function allocates the CloneWise structure
  *             and the basematrix area for explicit memory implementations
@@ -156,11 +205,12 @@ PackAln * Wise2_PackAln_bestmemory_CloneWise(MappedCloneSet * q,MappedCloneSet *
  * Arg:               query_skip [UNKN ] Resource [Score]
  * Arg:                   spread [UNKN ] Resource [int]
  * Arg:         target_special_s [UNKN ] Resource [int]
+ * Arg:                     dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [CloneWise *]
  *
  */
-CloneWise * Wise2_allocate_Expl_CloneWise(MappedCloneSet * q,MappedCloneSet * t ,MappedCloneMatch* match,Score target_skip_start,Score target_skip,Score query_skip_start,Score query_skip,int spread,int target_special_s);
+CloneWise * Wise2_allocate_Expl_CloneWise(MappedCloneSet * q,MappedCloneSet * t ,MappedCloneMatch* match,Score target_skip_start,Score target_skip,Score query_skip_start,Score query_skip,int spread,int target_special_s,DPRunImpl * dpri);
 #define allocate_Expl_CloneWise Wise2_allocate_Expl_CloneWise
 
 
@@ -289,6 +339,21 @@ PackAln * Wise2_PackAln_read_Expl_CloneWise(CloneWise * mat);
 #define PackAln_read_Expl_CloneWise Wise2_PackAln_read_Expl_CloneWise
 
 
+/* Function:  PackAln_read_generic_CloneWise(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [CloneWise *]
+ * Arg:          h [UNKN ] Undocumented argument [CloneWise_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_CloneWise(CloneWise * mat,CloneWise_access_func_holder h);
+#define PackAln_read_generic_CloneWise Wise2_PackAln_read_generic_CloneWise
+
+
 /* Function:  calculate_CloneWise(mat)
  *
  * Descrip:    This function calculates the CloneWise matrix when in explicit mode
@@ -304,6 +369,21 @@ boolean Wise2_calculate_CloneWise(CloneWise * mat);
 #define calculate_CloneWise Wise2_calculate_CloneWise
 
 
+/* Function:  calculate_dpenv_CloneWise(mat,dpenv)
+ *
+ * Descrip:    This function calculates the CloneWise matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] CloneWise which contains explicit basematrix memory [CloneWise *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_CloneWise(CloneWise * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_CloneWise Wise2_calculate_dpenv_CloneWise
+
+
 /* Function:  CloneWise_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -340,6 +420,12 @@ CloneWise * Wise2_free_CloneWise(CloneWise * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_CloneWise_shatter_access_main(CloneWise * mat,int i,int j,int state);
+#define CloneWise_shatter_access_main Wise2_CloneWise_shatter_access_main
+int Wise2_CloneWise_shatter_access_special(CloneWise * mat,int i,int j,int state);
+#define CloneWise_shatter_access_special Wise2_CloneWise_shatter_access_special
+void * Wise2_thread_loop_CloneWise(void * ptr);
+#define thread_loop_CloneWise Wise2_thread_loop_CloneWise
 int Wise2_score_only_CloneWise(MappedCloneSet * q,MappedCloneSet * t ,MappedCloneMatch* match,Score target_skip_start,Score target_skip,Score query_skip_start,Score query_skip,int spread,int target_special_s);
 #define score_only_CloneWise Wise2_score_only_CloneWise
 CloneWise * Wise2_allocate_CloneWise_only(MappedCloneSet * q,MappedCloneSet * t ,MappedCloneMatch* match,Score target_skip_start,Score target_skip,Score query_skip_start,Score query_skip,int spread,int target_special_s);
@@ -374,21 +460,25 @@ void Wise2_run_up_dc_CloneWise(CloneWise * mat,int starti,int stopi,int startj,i
 #define run_up_dc_CloneWise Wise2_run_up_dc_CloneWise
 void Wise2_init_dc_CloneWise(CloneWise * mat);
 #define init_dc_CloneWise Wise2_init_dc_CloneWise
-boolean Wise2_dc_start_end_calculate_CloneWise(CloneWise * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_CloneWise Wise2_dc_start_end_calculate_CloneWise
 int Wise2_start_end_find_end_CloneWise(CloneWise * mat,int * endj);
 #define start_end_find_end_CloneWise Wise2_start_end_find_end_CloneWise
+boolean Wise2_dc_optimised_start_end_calc_CloneWise(CloneWise *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_CloneWise Wise2_dc_optimised_start_end_calc_CloneWise
 void Wise2_init_start_end_linear_CloneWise(CloneWise * mat);
 #define init_start_end_linear_CloneWise Wise2_init_start_end_linear_CloneWise
 AlnConvertSet * Wise2_AlnConvertSet_CloneWise(void);
 #define AlnConvertSet_CloneWise Wise2_AlnConvertSet_CloneWise
-int Wise2_find_end_CloneWise(CloneWise * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_CloneWise_explicit_access_main(CloneWise * mat,int i,int j,int state);
+#define CloneWise_explicit_access_main Wise2_CloneWise_explicit_access_main
+int Wise2_CloneWise_explicit_access_special(CloneWise * mat,int i,int j,int state);
+#define CloneWise_explicit_access_special Wise2_CloneWise_explicit_access_special
+int Wise2_find_end_CloneWise(CloneWise * mat,int * ri,int * rj,int * state,boolean * isspecial,CloneWise_access_func_holder h);
 #define find_end_CloneWise Wise2_find_end_CloneWise
 void Wise2_CloneWise_debug_show_matrix(CloneWise * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define CloneWise_debug_show_matrix Wise2_CloneWise_debug_show_matrix
-int Wise2_max_calc_CloneWise(CloneWise * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_CloneWise(CloneWise * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,CloneWise_access_func_holder h);
 #define max_calc_CloneWise Wise2_max_calc_CloneWise
-int Wise2_max_calc_special_CloneWise(CloneWise * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_CloneWise(CloneWise * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,CloneWise_access_func_holder h);
 #define max_calc_special_CloneWise Wise2_max_calc_special_CloneWise
 
 #ifdef _cplusplus
