@@ -7,7 +7,11 @@ extern "C" {
 
 struct Wise2_DnaMatchBlock {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     ComplexSequence* query;  
@@ -30,6 +34,22 @@ typedef struct Wise2_DnaMatchBlock Wise2_DnaMatchBlock;
 #endif
 
 
+struct Wise2_DnaMatchBlock_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(DnaMatchBlock*,int,int,int);  
+    int (*access_special)(DnaMatchBlock*,int,int,int);   
+    } ;  
+/* DnaMatchBlock_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_DnaMatchBlock_access_func_holder
+typedef struct Wise2_DnaMatchBlock_access_func_holder Wise2_DnaMatchBlock_access_func_holder;
+#define DnaMatchBlock_access_func_holder Wise2_DnaMatchBlock_access_func_holder
+#define DYNAMITE_DEFINED_DnaMatchBlock_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -39,6 +59,35 @@ typedef struct Wise2_DnaMatchBlock Wise2_DnaMatchBlock;
 
 
 
+/* Function:  PackAln_read_Shatter_DnaMatchBlock(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [DnaMatchBlock *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_DnaMatchBlock(DnaMatchBlock * mat);
+#define PackAln_read_Shatter_DnaMatchBlock Wise2_PackAln_read_Shatter_DnaMatchBlock
+
+
+/* Function:  calculate_shatter_DnaMatchBlock(mat,dpenv)
+ *
+ * Descrip:    This function calculates the DnaMatchBlock matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [DnaMatchBlock *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_DnaMatchBlock(DnaMatchBlock * mat,DPEnvelope * dpenv);
+#define calculate_shatter_DnaMatchBlock Wise2_calculate_shatter_DnaMatchBlock
+
+
 /* Function:  search_DnaMatchBlock(dbsi,out,query,target,comp65,comp75,comp85,comp95,g,u,v,s,b)
  *
  * Descrip:    This function makes a database search of DnaMatchBlock
@@ -127,7 +176,7 @@ PackAln * Wise2_PackAln_bestmemory_DnaMatchBlock(ComplexSequence* query,ComplexS
 #define PackAln_bestmemory_DnaMatchBlock Wise2_PackAln_bestmemory_DnaMatchBlock
 
 
-/* Function:  allocate_Expl_DnaMatchBlock(query,target,comp65,comp75,comp85,comp95,g,u,v,s,b)
+/* Function:  allocate_Expl_DnaMatchBlock(query,target,comp65,comp75,comp85,comp95,g,u,v,s,b,dpri)
  *
  * Descrip:    This function allocates the DnaMatchBlock structure
  *             and the basematrix area for explicit memory implementations
@@ -145,11 +194,12 @@ PackAln * Wise2_PackAln_bestmemory_DnaMatchBlock(ComplexSequence* query,ComplexS
  * Arg:             v [UNKN ] Resource [Score]
  * Arg:             s [UNKN ] Resource [Score]
  * Arg:             b [UNKN ] Resource [Score]
+ * Arg:          dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [DnaMatchBlock *]
  *
  */
-DnaMatchBlock * Wise2_allocate_Expl_DnaMatchBlock(ComplexSequence* query,ComplexSequence* target ,DnaMatrix* comp65,DnaMatrix* comp75,DnaMatrix* comp85,DnaMatrix* comp95,Score g,Score u,Score v,Score s,Score b);
+DnaMatchBlock * Wise2_allocate_Expl_DnaMatchBlock(ComplexSequence* query,ComplexSequence* target ,DnaMatrix* comp65,DnaMatrix* comp75,DnaMatrix* comp85,DnaMatrix* comp95,Score g,Score u,Score v,Score s,Score b,DPRunImpl * dpri);
 #define allocate_Expl_DnaMatchBlock Wise2_allocate_Expl_DnaMatchBlock
 
 
@@ -280,6 +330,21 @@ PackAln * Wise2_PackAln_read_Expl_DnaMatchBlock(DnaMatchBlock * mat);
 #define PackAln_read_Expl_DnaMatchBlock Wise2_PackAln_read_Expl_DnaMatchBlock
 
 
+/* Function:  PackAln_read_generic_DnaMatchBlock(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [DnaMatchBlock *]
+ * Arg:          h [UNKN ] Undocumented argument [DnaMatchBlock_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_DnaMatchBlock(DnaMatchBlock * mat,DnaMatchBlock_access_func_holder h);
+#define PackAln_read_generic_DnaMatchBlock Wise2_PackAln_read_generic_DnaMatchBlock
+
+
 /* Function:  calculate_DnaMatchBlock(mat)
  *
  * Descrip:    This function calculates the DnaMatchBlock matrix when in explicit mode
@@ -295,6 +360,21 @@ boolean Wise2_calculate_DnaMatchBlock(DnaMatchBlock * mat);
 #define calculate_DnaMatchBlock Wise2_calculate_DnaMatchBlock
 
 
+/* Function:  calculate_dpenv_DnaMatchBlock(mat,dpenv)
+ *
+ * Descrip:    This function calculates the DnaMatchBlock matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] DnaMatchBlock which contains explicit basematrix memory [DnaMatchBlock *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_DnaMatchBlock(DnaMatchBlock * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_DnaMatchBlock Wise2_calculate_dpenv_DnaMatchBlock
+
+
 /* Function:  DnaMatchBlock_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -331,6 +411,10 @@ DnaMatchBlock * Wise2_free_DnaMatchBlock(DnaMatchBlock * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_DnaMatchBlock_shatter_access_main(DnaMatchBlock * mat,int i,int j,int state);
+#define DnaMatchBlock_shatter_access_main Wise2_DnaMatchBlock_shatter_access_main
+int Wise2_DnaMatchBlock_shatter_access_special(DnaMatchBlock * mat,int i,int j,int state);
+#define DnaMatchBlock_shatter_access_special Wise2_DnaMatchBlock_shatter_access_special
 int Wise2_score_only_DnaMatchBlock(ComplexSequence* query,ComplexSequence* target ,DnaMatrix* comp65,DnaMatrix* comp75,DnaMatrix* comp85,DnaMatrix* comp95,Score g,Score u,Score v,Score s,Score b);
 #define score_only_DnaMatchBlock Wise2_score_only_DnaMatchBlock
 DnaMatchBlock * Wise2_allocate_DnaMatchBlock_only(ComplexSequence* query,ComplexSequence* target ,DnaMatrix* comp65,DnaMatrix* comp75,DnaMatrix* comp85,DnaMatrix* comp95,Score g,Score u,Score v,Score s,Score b);
@@ -365,21 +449,25 @@ void Wise2_run_up_dc_DnaMatchBlock(DnaMatchBlock * mat,int starti,int stopi,int
 #define run_up_dc_DnaMatchBlock Wise2_run_up_dc_DnaMatchBlock
 void Wise2_init_dc_DnaMatchBlock(DnaMatchBlock * mat);
 #define init_dc_DnaMatchBlock Wise2_init_dc_DnaMatchBlock
-boolean Wise2_dc_start_end_calculate_DnaMatchBlock(DnaMatchBlock * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_DnaMatchBlock Wise2_dc_start_end_calculate_DnaMatchBlock
 int Wise2_start_end_find_end_DnaMatchBlock(DnaMatchBlock * mat,int * endj);
 #define start_end_find_end_DnaMatchBlock Wise2_start_end_find_end_DnaMatchBlock
+boolean Wise2_dc_optimised_start_end_calc_DnaMatchBlock(DnaMatchBlock *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_DnaMatchBlock Wise2_dc_optimised_start_end_calc_DnaMatchBlock
 void Wise2_init_start_end_linear_DnaMatchBlock(DnaMatchBlock * mat);
 #define init_start_end_linear_DnaMatchBlock Wise2_init_start_end_linear_DnaMatchBlock
 AlnConvertSet * Wise2_AlnConvertSet_DnaMatchBlock(void);
 #define AlnConvertSet_DnaMatchBlock Wise2_AlnConvertSet_DnaMatchBlock
-int Wise2_find_end_DnaMatchBlock(DnaMatchBlock * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_DnaMatchBlock_explicit_access_main(DnaMatchBlock * mat,int i,int j,int state);
+#define DnaMatchBlock_explicit_access_main Wise2_DnaMatchBlock_explicit_access_main
+int Wise2_DnaMatchBlock_explicit_access_special(DnaMatchBlock * mat,int i,int j,int state);
+#define DnaMatchBlock_explicit_access_special Wise2_DnaMatchBlock_explicit_access_special
+int Wise2_find_end_DnaMatchBlock(DnaMatchBlock * mat,int * ri,int * rj,int * state,boolean * isspecial,DnaMatchBlock_access_func_holder h);
 #define find_end_DnaMatchBlock Wise2_find_end_DnaMatchBlock
 void Wise2_DnaMatchBlock_debug_show_matrix(DnaMatchBlock * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define DnaMatchBlock_debug_show_matrix Wise2_DnaMatchBlock_debug_show_matrix
-int Wise2_max_calc_DnaMatchBlock(DnaMatchBlock * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_DnaMatchBlock(DnaMatchBlock * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,DnaMatchBlock_access_func_holder h);
 #define max_calc_DnaMatchBlock Wise2_max_calc_DnaMatchBlock
-int Wise2_max_calc_special_DnaMatchBlock(DnaMatchBlock * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_DnaMatchBlock(DnaMatchBlock * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,DnaMatchBlock_access_func_holder h);
 #define max_calc_special_DnaMatchBlock Wise2_max_calc_special_DnaMatchBlock
 
 #ifdef _cplusplus
