@@ -1,6 +1,17 @@
+
+/*
+ * this main is a real mess and needs to be completely 
+ * reworked. ugh! ugh!
+ *
+ * Will do this in 2.2.4 onwards
+ *
+ */
+
+
+
 /*  
-* $Revision: 1.5 $  
-* $Id: dbac.c,v 1.5 2001/09/07 08:17:39 birney Exp $  
+* $Revision: 1.11 $  
+* $Id: dbac.c,v 1.11 2003/04/09 08:04:53 birney Exp $  
 */  
   
 typedef int BOOL ;  
@@ -11,12 +22,16 @@ typedef int BOOL ;
 /*#define DOPROB*/  
   
 #include "dba.h"  
+#include "slimdba.h"
+#include "bigdba.h"
 #include "dbadisplay.h"  
 #include "version.h"  
+#include "seqalign.h"
   
 /*  
  * Dbb alignment. Idea by R. Durbin  
  */  
+static Probability MATCH55 = 0.55 ;  
 static Probability MATCH65 = 0.65 ;  
 static Probability MATCH75 = 0.75 ;  
 static Probability MATCH85 = 0.85 ;  
@@ -25,6 +40,9 @@ static Probability MATCHS = 0.8 ;
 static Probability GAP = 0.05 ;  
 static Probability BLOCKOPEN = 0.01 ;  
 static Probability UNMATCHED_PEN = 0.99;  
+
+static Probability GAP55 = 0.10;
+static Probability GAP65 = 0.10;
   
 int kbyte = 25000;  
   
@@ -124,7 +142,7 @@ void alnBlockSummary (AlnBlock *alb, Sequence *x, Sequence *y)
    inBlock = FALSE ;  
  }  
     }  
-    free(type);  
+  /*    free(type);  */
 }  
   
   
@@ -180,11 +198,6 @@ void showPFF (AlnBlock *alb, Sequence *x, Sequence *y)
         if (!inBlock) {   
             inBlock = TRUE ;  
             blockCount++;  
-/*WK             score = (((1-UNMATCHED_PEN)/(UNMATCHED_PEN*UNMATCHED_PEN)) * (BLOCKOPEN/UNMATCHED_PEN)) / 4; */  
-/*  
- * 010613 Nic: log2 function not available on all platforms  
- */  
-/*             tmpbitscore = log2((((1-UNMATCHED_PEN)/(UNMATCHED_PEN*UNMATCHED_PEN)) * (BLOCKOPEN/UNMATCHED_PEN)) / 4); */  
             tmpbitscore = (log((((1-UNMATCHED_PEN)/(UNMATCHED_PEN*UNMATCHED_PEN)) * (BLOCKOPEN/UNMATCHED_PEN)) / 4))/log(2);  
   
      if (xa->text_label[2] == '6') {  
@@ -218,13 +231,6 @@ void showPFF (AlnBlock *alb, Sequence *x, Sequence *y)
           xtmp[0] = x->seq[xa->end];  
           strcat(xSeq, xstr);  
           strcat(ySeq, ".");  
-/*WK           score *= GAP/UNMATCHED_PEN; */  
-  
-/*  
- * 010613 Nic: log2 function not available on all platforms  
- */  
-/*           tmpbitscore += log2(GAP/UNMATCHED_PEN);  
- */  
           tmpbitscore += (log(GAP/UNMATCHED_PEN))/log(2);  
             
         } else if (ya->text_label[1] == 'I') {  
@@ -232,12 +238,6 @@ void showPFF (AlnBlock *alb, Sequence *x, Sequence *y)
           strcat(xSeq, ".");  
           ytmp[0] = y->seq[ya->end];  
           strcat(ySeq, ystr);  
-/*WK           score *= GAP/UNMATCHED_PEN; */  
-/*  
- * 010613 Nic: log2 function not available on all platforms  
- */  
-/*           tmpbitscore += log2(GAP/UNMATCHED_PEN);  
- */  
           tmpbitscore += (log(GAP/UNMATCHED_PEN))/log(2);  
         } else if (x->seq[xa->end] == y->seq[ya->end]) {  
    ++nMatch ;  
@@ -246,16 +246,6 @@ void showPFF (AlnBlock *alb, Sequence *x, Sequence *y)
           strcat(xSeq, xstr);  
           ytmp[0] = y->seq[ya->end];  
           strcat(ySeq, ystr);  
-   /*WK   if(score<=MAXD)  
-     score *= (s/(UNMATCHED_PEN*UNMATCHED_PEN)) * (currMatch/0.25);  
-   */  
-  
-/*  
- * 010613 Nic: log2 function not available on all platforms  
- */  
-/*           tmpbitscore += log2( (s/(UNMATCHED_PEN*UNMATCHED_PEN))   
-                               *(currMatch/0.25));  
- */            
           tmpbitscore += (log( (s/(UNMATCHED_PEN*UNMATCHED_PEN))   
                                *(currMatch/0.25)))/log(2);  
   
@@ -265,19 +255,12 @@ void showPFF (AlnBlock *alb, Sequence *x, Sequence *y)
           strcat(xSeq, xstr);  
           ytmp[0] = y->seq[ya->end];  
           strcat(ySeq, ystr);  
-/*WK           score *= (s/(UNMATCHED_PEN*UNMATCHED_PEN)) * ( ((1-currMatch)/3)/0.25 ); */  
-/*  
- * 010613 Nic: log2 function not available on all platforms  
- */  
-/*           tmpbitscore += log2( (s/(UNMATCHED_PEN*UNMATCHED_PEN))   
-                               *( ((1-currMatch)/3)/0.25 ));  
- */            
           tmpbitscore += (log( (s/(UNMATCHED_PEN*UNMATCHED_PEN))   
                                *( ((1-currMatch)/3)/0.25 )))/log(2);  
         }  
       } else if (inBlock) {   
-/*WK  double bitscore = log2(score); */  
-  double bitscore = tmpbitscore;  
+
+	double bitscore = tmpbitscore;  
           
  /* print comment gff line for x */  
 /*         printf (">%s\tDBA\tBlock\t%d\t%d\t%.2f\t+\t.\tblock %2d; id %d; gaps %d; type %s\n",  
@@ -352,18 +335,22 @@ void usage (FILE * ofp)
   fprintf (ofp, " -matchB [%g]     match level B\n",MATCH75) ;  
   fprintf (ofp, " -matchC [%g]     match level C\n",MATCH85) ;  
   fprintf (ofp, " -matchD [%g]     match level D\n",MATCH95) ;  
+  fprintf (ofp, " -gapA   [%g]     (big only) Gap probability for A\n",GAP65) ;  
+  fprintf (ofp, " -gapAA  [%g]     (big only) Gap probability for AA\n",GAP55) ;  
   fprintf (ofp, " -gap [%g]       gap probability\n",GAP) ;  
   fprintf (ofp, " -blockopen [%g] block open probability\n",BLOCKOPEN) ;  
   fprintf (ofp, " -umatch [%g]    unmatched gap probability\n",UNMATCHED_PEN) ;  
-  fprintf (ofp, " -single           use only one match level, set with -matchA [%g]\n", MATCHS) ;  
-  fprintf (ofp, " -nomatchn         do not match N to any base\n") ;  
-  fprintf (ofp, " -align           show alignment for computer parsing\n") ;  
-  fprintf (ofp, " -pretty           show alignment for ASCII viewing\n") ;  
-  fprintf (ofp, " -pff           show phylogenetic footprinting format output\n") ;  
-  fprintf (ofp, "             (gapped fasta)\n") ;  
-  fprintf (ofp, " -label           show label alignment\n") ;  
+  fprintf (ofp, " -single         use only one match level, set with -matchA [%g]\n", MATCHS) ;  
+  fprintf (ofp, " -nomatchn       do not match N to any base\n") ;  
+  fprintf (ofp, " -align          show alignment for computer parsing\n") ;  
+  fprintf (ofp, " -anchor         show anchored alignment to first sequence\n") ;  
+  fprintf (ofp, " -pretty         show alignment for ASCII viewing\n") ;  
+  fprintf (ofp, " -pff            show phylogenetic footprinting format output\n") ;  
+  fprintf (ofp, "                 (gapped fasta)\n") ;  
+  fprintf (ofp, " -label            show label alignment\n") ;  
+  fprintf (ofp, " -slim             use slim DBA alignment\n") ;  
+  fprintf (ofp, " -big              use big DBA alignment with AA block\n") ;  
   fprintf (ofp, " -params           print parameters\n") ;  
-  fprintf (ofp, "       -kbyte [%d]    Kilobytes allowed for main matrix memory\n",kbyte);  
   show_help_DPRunImpl(ofp);  
   show_standard_options(ofp);  
     
@@ -379,6 +366,63 @@ void show_version(FILE * ofp)
   fprintf(ofp,"Copyright (c) 1998,1999,2000,2001 GRL ltd. It is distributed under a Gnu Public License\n");  
   fprintf(ofp,"See GNULICENSE in source directory for more information\n");  
 }  
+
+
+SeqAlign * make_SeqAlign_from_align(AlnBlock * alb,Sequence * one,Sequence * two)
+{
+  Sequence * one_align;
+  Sequence * two_align;
+  SeqAlign * out;
+  AlnColumn * alc;
+  int i;
+
+  char * seq;
+
+
+  /* one is anchor - easy! */
+
+  one_align = new_Sequence_from_strings(one->name,one->seq);
+
+  /* next one is harder! */
+
+  seq = calloc(one->len,sizeof(char));
+  
+  for(i=0,alc=alb->start;i<one->len;i++) {
+    seq[i] = '~';
+    if( alc == NULL ) {
+      continue;
+    }
+    if( alc->alu[0]->start+1 < i ) {
+      for(;alc->alu[0]->start+1 != i;alc = alc->next)
+	;
+    }
+
+    if( alc != NULL && alc->alu[0]->start+1 == i ) {
+      if( strstr(alc->alu[0]->text_label,"MM") != NULL ) {
+	seq[i] = two->seq[alc->alu[1]->start+1];
+      } else if ( strstr(alc->alu[0]->text_label,"MI") != NULL ) {
+	seq[i] = '-';
+      } else{
+	;
+      }
+      alc = alc->next;
+    }
+  }
+  
+  seq[one->len] = '\0';
+  fprintf(stderr,"Got %d with %d vs %d\n",i,strlen(seq),one->len);
+
+  two_align = new_Sequence_from_strings(two->name,seq);
+
+  out = SeqAlign_alloc_len(2);
+
+  add_SeqAlign(out,one_align);
+  add_SeqAlign(out,two_align);
+
+  return out;
+
+}
+    
   
   
 int main (int argc, char **argv)  
@@ -386,10 +430,12 @@ int main (int argc, char **argv)
   Sequence *one, *two ;  
   ComplexSequence *cone, *ctwo ;  
   ComplexSequenceEvalSet *cses ;  
+  DnaMatrix *dcm55 ;  
   DnaMatrix *dcm65 ;  
   DnaMatrix *dcm75 ;  
   DnaMatrix *dcm85 ;  
   DnaMatrix *dcm95 ;  
+  DnaProbMatrix * dpm55;  
   DnaProbMatrix * dpm65;  
   DnaProbMatrix * dpm75;  
   DnaProbMatrix * dpm85;  
@@ -405,7 +451,12 @@ int main (int argc, char **argv)
   int isSingle = 0;  
   int aSet = 0;  
   int show_pff = 0;  
-  
+  int show_anchor = 0;
+  int use_slim = 0;
+  int use_big  = 0;
+
+
+  SeqAlign * seqalign;
   DnaMatchBlock * dmat;  
   DPRunImpl * dpri;  
   
@@ -434,6 +485,10 @@ int main (int argc, char **argv)
        { MATCH85 = atof (*++argv) ; --argc ; }     
     else if (!strcmp (*argv, "-matchD"))  
        { MATCH95 = atof (*++argv) ; --argc ; }     
+    else if (!strcmp (*argv, "-gapA"))  
+       { GAP65 = atof (*++argv) ; --argc ; }     
+    else if (!strcmp (*argv, "-gapAA"))  
+       { GAP55 = atof (*++argv) ; --argc ; }     
     else if (!strcmp (*argv, "-umatch"))  
       { UNMATCHED_PEN = atof (*++argv) ; --argc ; }  
     else if (!strcmp (*argv, "-gap"))  
@@ -448,6 +503,12 @@ int main (int argc, char **argv)
       isNoMatchN = 1 ;   
     else if (!strcmp (*argv, "-pff"))   
       show_pff = 1 ;   
+    else if (!strcmp (*argv, "-anchor"))   
+      show_anchor = 1 ;   
+    else if (!strcmp (*argv, "-slim"))   
+      use_slim = 1 ;   
+    else if (!strcmp (*argv, "-big"))   
+      use_big = 1 ;   
     else  
       { fprintf (stderr, "option %s not recognized\n", *argv) ;  
  usage(stdout) ;  
@@ -509,6 +570,12 @@ int main (int argc, char **argv)
   
   /** make DnaProbMat's **/  
   
+  dpm55 = DnaProbMatrix_from_match(MATCH55,isNoMatchN == 1 ? NMaskType_BANNED : NMaskType_VARIABLE);  
+  if( dpm55 == NULL ) {  
+    fatal("Could not build DnaProbMatrix for MATCH65");  
+  }  
+  flat_null_DnaProbMatrix(dpm55);  
+
   dpm65 = DnaProbMatrix_from_match(MATCH65,isNoMatchN == 1 ? NMaskType_BANNED : NMaskType_VARIABLE);  
   if( dpm65 == NULL ) {  
     fatal("Could not build DnaProbMatrix for MATCH65");  
@@ -534,6 +601,11 @@ int main (int argc, char **argv)
   flat_null_DnaProbMatrix(dpm95);  
   
   
+  dcm55 = DnaMatrix_from_DnaProbMatrix(dpm55);  
+  if( dcm55 == NULL ) {  
+    fatal("Could not build DnaCompMat 55");  
+  }  
+
   dcm65 = DnaMatrix_from_DnaProbMatrix(dpm65);  
   if( dcm65 == NULL ) {  
     fatal("Could not build DnaCompMat 65");  
@@ -555,64 +627,106 @@ int main (int argc, char **argv)
   /** make the alignment **/  
   
   change_max_BaseMatrix_kbytes (kbyte) ; /* 25Mb */  
-  pal = PackAln_bestmemory_DnaMatchBlock (cone, ctwo,   
-       dcm65,  
-       dcm75,  
-       dcm85,  
-       dcm95,  
-    Probability2Score(GAP/UNMATCHED_PEN),   
-    Probability2Score(1),  
-       /* horrible hack here so that   
-          single searches are not / 4 */  
-  
-       Probability2Score(((1-UNMATCHED_PEN)/(UNMATCHED_PEN))/ (isSingle == TRUE ? 1 : 4)),   
-    Probability2Score((1.0 - (GAP + GAP + BLOCKOPEN))/(UNMATCHED_PEN*UNMATCHED_PEN)),  
-    Probability2Score(BLOCKOPEN/UNMATCHED_PEN),  
-       NULL,dpri    
+
+
+  if( use_big == 1 ) {
+    pal = PackAln_bestmemory_BigDnaMatchBlock (cone, ctwo,   
+					       dcm55,
+					       dcm65,  
+					       dcm75,
+					       dcm85,
+					       dcm95,
+					       Probability2Score(GAP/UNMATCHED_PEN),		       
+					       Probability2Score(GAP65/UNMATCHED_PEN),		       
+					       Probability2Score(GAP55/UNMATCHED_PEN),   
+					       Probability2Score(1),  
+					       /* horrible hack here so that   
+						  single searches are not / 4 */  
+					       
+					       Probability2Score(((1-UNMATCHED_PEN)/(UNMATCHED_PEN))/ (isSingle == TRUE ? 1 : 4)),   
+					       Probability2Score((1.0 - (GAP + GAP + BLOCKOPEN))/(UNMATCHED_PEN*UNMATCHED_PEN)),  
+					       Probability2Score(BLOCKOPEN/UNMATCHED_PEN),  
+					       NULL,dpri    
+					       );  
+    alb = convert_PackAln_to_AlnBlock_BigDnaMatchBlock(pal);  
+    
+  } else if( use_slim == 1 ) {
+
+    pal = PackAln_bestmemory_SlimDnaMatchBlock (cone, ctwo,   
+					    dcm65,  
+					    Probability2Score(GAP/UNMATCHED_PEN),   
+					    Probability2Score(1),  
+					    /* horrible hack here so that   
+					       single searches are not / 4 */  
+					    
+					    Probability2Score(((1-UNMATCHED_PEN)/(UNMATCHED_PEN))/ (isSingle == TRUE ? 1 : 4)),   
+					    Probability2Score((1.0 - (GAP + GAP + BLOCKOPEN))/(UNMATCHED_PEN*UNMATCHED_PEN)),  
+					    Probability2Score(BLOCKOPEN/UNMATCHED_PEN),  
+					    NULL,dpri    
       );  
-  
+    alb = convert_PackAln_to_AlnBlock_DnaMatchBlock(pal);  
+  } else {
+    pal = PackAln_bestmemory_DnaMatchBlock (cone, ctwo,   
+					    dcm65,  
+					    dcm75,  
+					    dcm85,  
+					    dcm95,  
+					    Probability2Score(GAP/UNMATCHED_PEN),   
+					    Probability2Score(1),  
+					    /* horrible hack here so that   
+					       single searches are not / 4 */  
+					    
+					    Probability2Score(((1-UNMATCHED_PEN)/(UNMATCHED_PEN))/ (isSingle == TRUE ? 1 : 4)),   
+					    Probability2Score((1.0 - (GAP + GAP + BLOCKOPEN))/(UNMATCHED_PEN*UNMATCHED_PEN)),  
+					    Probability2Score(BLOCKOPEN/UNMATCHED_PEN),  
+					    NULL,dpri    
+      );  
+    
 #ifdef DOPROB  
-  dmat = matrix_logsum_DnaMatchBlock(cone, ctwo,   
-       dcm65,  
-       dcm75,  
-       dcm85,  
-       dcm95,  
-    Probability2Score(GAP/UNMATCHED_PEN),   
-    Probability2Score(1),  
-       /* horrible hack here so that   
-          single searches are not / 4 */  
-  
-       Probability2Score(((1-UNMATCHED_PEN)/(UNMATCHED_PEN))/ (isSingle == TRUE ? 1 : 4)),   
-    Probability2Score((1.0 - (GAP + GAP + BLOCKOPEN))/(UNMATCHED_PEN*UNMATCHED_PEN)),  
-    Probability2Score(BLOCKOPEN/UNMATCHED_PEN)   );  
-  
-  fprintf(stdout,"And the score is %d %.2f\n",dmat->basematrix->matrix[dmat->basematrix->leni-1][dmat->basematrix->lenj-1],Score2Bits(dmat->basematrix->matrix[dmat->basematrix->leni-1][dmat->basematrix->lenj-1]));  
-  
-  score = score_only_logsum_DnaMatchBlock(cone, ctwo,   
-       dcm65,  
-       dcm75,  
-       dcm85,  
-       dcm95,  
-    Probability2Score(GAP/UNMATCHED_PEN),   
-    Probability2Score(1),  
-       /* horrible hack here so that   
-          single searches are not / 4 */  
-  
-       Probability2Score(((1-UNMATCHED_PEN)/(UNMATCHED_PEN))/ (isSingle == TRUE ? 1 : 4)),   
-    Probability2Score((1.0 - (GAP + GAP + BLOCKOPEN))/(UNMATCHED_PEN*UNMATCHED_PEN)),  
-    Probability2Score(BLOCKOPEN/UNMATCHED_PEN)   );  
-  
-  
-  fprintf(stdout,"And the score is %d %.2f\n",score,Score2Bits(score));  
-  
+    dmat = matrix_logsum_DnaMatchBlock(cone, ctwo,   
+				       dcm65,  
+				       dcm75,  
+				       dcm85,  
+				       dcm95,  
+				       Probability2Score(GAP/UNMATCHED_PEN),   
+				       Probability2Score(1),  
+				       /* horrible hack here so that   
+					  single searches are not / 4 */  
+				       
+				       Probability2Score(((1-UNMATCHED_PEN)/(UNMATCHED_PEN))/ (isSingle == TRUE ? 1 : 4)),   
+				       Probability2Score((1.0 - (GAP + GAP + BLOCKOPEN))/(UNMATCHED_PEN*UNMATCHED_PEN)),  
+				       Probability2Score(BLOCKOPEN/UNMATCHED_PEN)   );  
+    
+    fprintf(stdout,"And the score is %d %.2f\n",dmat->basematrix->matrix[dmat->basematrix->leni-1][dmat->basematrix->lenj-1],Score2Bits(dmat->basematrix->matrix[dmat->basematrix->leni-1][dmat->basematrix->lenj-1]));  
+    
+    score = score_only_logsum_DnaMatchBlock(cone, ctwo,   
+					    dcm65,  
+					    dcm75,  
+					    dcm85,  
+					    dcm95,  
+					    Probability2Score(GAP/UNMATCHED_PEN),   
+					    Probability2Score(1),  
+					    /* horrible hack here so that   
+					       single searches are not / 4 */  
+					    
+					    Probability2Score(((1-UNMATCHED_PEN)/(UNMATCHED_PEN))/ (isSingle == TRUE ? 1 : 4)),   
+					    Probability2Score((1.0 - (GAP + GAP + BLOCKOPEN))/(UNMATCHED_PEN*UNMATCHED_PEN)),  
+					    Probability2Score(BLOCKOPEN/UNMATCHED_PEN)   );  
+    
+    
+    fprintf(stdout,"And the score is %d %.2f\n",score,Score2Bits(score));  
+    
 #endif  
   
-  if( pal == NULL )   
-    fatal("Unable to build alignment!");  
+    if( pal == NULL )   
+      fatal("Unable to build alignment!");  
   
   
-  alb = convert_PackAln_to_AlnBlock_DnaMatchBlock(pal);  
-  
+    alb = convert_PackAln_to_AlnBlock_DnaMatchBlock(pal);  
+  }
+
+  /* alignment done! */
+
   if( show_label_align ) {  
     printf ("score = %.2f\n", Score2Bits(pal->score)) ;  
     mapped_ascii_AlnBlock(alb,Score2Probability,0,stdout);  
@@ -624,12 +738,24 @@ int main (int argc, char **argv)
     prettyAlnBlock (alb, one, two) ;  
   } else if (show_pff) {  
     showPFF (alb, one, two) ;  
+  } else if ( show_anchor ) {
+    seqalign = make_SeqAlign_from_align(alb,one,two);
+    write_fasta_SeqAlign(seqalign,stdout);
+    free_SeqAlign(seqalign);
   } else {  
     printf ("score = %.2f\n", Score2Bits(pal->score)) ;  
     alnBlockSummary (alb, one, two) ;  
   }  
   return 0;  
 }  
-  
+
+
   
 /******************* end of file ***************/  
+
+
+
+
+
+
+
