@@ -17,6 +17,9 @@ enum DnaStartEndEnum {
 
 struct Wise2_DnaStartEnd {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     int trans[DSE_NUMBER];  /*  start/end points possibilities */ 
     } ;  
 /* DnaStartEnd defined */ 
@@ -29,7 +32,11 @@ typedef struct Wise2_DnaStartEnd Wise2_DnaStartEnd;
 
 struct Wise2_DnaAlign {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     ComplexSequence* query;  
@@ -49,6 +56,22 @@ typedef struct Wise2_DnaAlign Wise2_DnaAlign;
 #endif
 
 
+struct Wise2_DnaAlign_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(DnaAlign*,int,int,int);   
+    int (*access_special)(DnaAlign*,int,int,int);    
+    } ;  
+/* DnaAlign_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_DnaAlign_access_func_holder
+typedef struct Wise2_DnaAlign_access_func_holder Wise2_DnaAlign_access_func_holder;
+#define DnaAlign_access_func_holder Wise2_DnaAlign_access_func_holder
+#define DYNAMITE_DEFINED_DnaAlign_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -142,6 +165,35 @@ DnaStartEnd * Wise2_free_DnaStartEnd(DnaStartEnd * obj);
 #define free_DnaStartEnd Wise2_free_DnaStartEnd
 
 
+/* Function:  PackAln_read_Shatter_DnaAlign(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [DnaAlign *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_DnaAlign(DnaAlign * mat);
+#define PackAln_read_Shatter_DnaAlign Wise2_PackAln_read_Shatter_DnaAlign
+
+
+/* Function:  calculate_shatter_DnaAlign(mat,dpenv)
+ *
+ * Descrip:    This function calculates the DnaAlign matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [DnaAlign *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_DnaAlign(DnaAlign * mat,DPEnvelope * dpenv);
+#define calculate_shatter_DnaAlign Wise2_calculate_shatter_DnaAlign
+
+
 /* Function:  search_DnaAlign(dbsi,out,query,target,comp,qgap,qext,tgap,text,startend)
  *
  * Descrip:    This function makes a database search of DnaAlign
@@ -221,7 +273,7 @@ PackAln * Wise2_PackAln_bestmemory_DnaAlign(ComplexSequence* query,ComplexSequen
 #define PackAln_bestmemory_DnaAlign Wise2_PackAln_bestmemory_DnaAlign
 
 
-/* Function:  allocate_Expl_DnaAlign(query,target,comp,qgap,qext,tgap,text,startend)
+/* Function:  allocate_Expl_DnaAlign(query,target,comp,qgap,qext,tgap,text,startend,dpri)
  *
  * Descrip:    This function allocates the DnaAlign structure
  *             and the basematrix area for explicit memory implementations
@@ -236,11 +288,12 @@ PackAln * Wise2_PackAln_bestmemory_DnaAlign(ComplexSequence* query,ComplexSequen
  * Arg:            tgap [UNKN ] Resource [Score]
  * Arg:            text [UNKN ] Resource [Score]
  * Arg:        startend [UNKN ] Resource [DnaStartEnd *]
+ * Arg:            dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [DnaAlign *]
  *
  */
-DnaAlign * Wise2_allocate_Expl_DnaAlign(ComplexSequence* query,ComplexSequence* target ,DnaMatrix* comp,Score qgap,Score qext,Score tgap,Score text,DnaStartEnd * startend);
+DnaAlign * Wise2_allocate_Expl_DnaAlign(ComplexSequence* query,ComplexSequence* target ,DnaMatrix* comp,Score qgap,Score qext,Score tgap,Score text,DnaStartEnd * startend,DPRunImpl * dpri);
 #define allocate_Expl_DnaAlign Wise2_allocate_Expl_DnaAlign
 
 
@@ -368,6 +421,21 @@ PackAln * Wise2_PackAln_read_Expl_DnaAlign(DnaAlign * mat);
 #define PackAln_read_Expl_DnaAlign Wise2_PackAln_read_Expl_DnaAlign
 
 
+/* Function:  PackAln_read_generic_DnaAlign(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [DnaAlign *]
+ * Arg:          h [UNKN ] Undocumented argument [DnaAlign_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_DnaAlign(DnaAlign * mat,DnaAlign_access_func_holder h);
+#define PackAln_read_generic_DnaAlign Wise2_PackAln_read_generic_DnaAlign
+
+
 /* Function:  calculate_DnaAlign(mat)
  *
  * Descrip:    This function calculates the DnaAlign matrix when in explicit mode
@@ -383,6 +451,21 @@ boolean Wise2_calculate_DnaAlign(DnaAlign * mat);
 #define calculate_DnaAlign Wise2_calculate_DnaAlign
 
 
+/* Function:  calculate_dpenv_DnaAlign(mat,dpenv)
+ *
+ * Descrip:    This function calculates the DnaAlign matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] DnaAlign which contains explicit basematrix memory [DnaAlign *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_DnaAlign(DnaAlign * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_DnaAlign Wise2_calculate_dpenv_DnaAlign
+
+
 /* Function:  DnaAlign_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -419,6 +502,10 @@ DnaAlign * Wise2_free_DnaAlign(DnaAlign * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_DnaAlign_shatter_access_main(DnaAlign * mat,int i,int j,int state);
+#define DnaAlign_shatter_access_main Wise2_DnaAlign_shatter_access_main
+int Wise2_DnaAlign_shatter_access_special(DnaAlign * mat,int i,int j,int state);
+#define DnaAlign_shatter_access_special Wise2_DnaAlign_shatter_access_special
 int Wise2_score_only_DnaAlign(ComplexSequence* query,ComplexSequence* target ,DnaMatrix* comp,Score qgap,Score qext,Score tgap,Score text,DnaStartEnd * startend);
 #define score_only_DnaAlign Wise2_score_only_DnaAlign
 DnaAlign * Wise2_allocate_DnaAlign_only(ComplexSequence* query,ComplexSequence* target ,DnaMatrix* comp,Score qgap,Score qext,Score tgap,Score text,DnaStartEnd * startend);
@@ -453,21 +540,25 @@ void Wise2_run_up_dc_DnaAlign(DnaAlign * mat,int starti,int stopi,int startj,int
 #define run_up_dc_DnaAlign Wise2_run_up_dc_DnaAlign
 void Wise2_init_dc_DnaAlign(DnaAlign * mat);
 #define init_dc_DnaAlign Wise2_init_dc_DnaAlign
-boolean Wise2_dc_start_end_calculate_DnaAlign(DnaAlign * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_DnaAlign Wise2_dc_start_end_calculate_DnaAlign
 int Wise2_start_end_find_end_DnaAlign(DnaAlign * mat,int * endj);
 #define start_end_find_end_DnaAlign Wise2_start_end_find_end_DnaAlign
+boolean Wise2_dc_optimised_start_end_calc_DnaAlign(DnaAlign *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_DnaAlign Wise2_dc_optimised_start_end_calc_DnaAlign
 void Wise2_init_start_end_linear_DnaAlign(DnaAlign * mat);
 #define init_start_end_linear_DnaAlign Wise2_init_start_end_linear_DnaAlign
 AlnConvertSet * Wise2_AlnConvertSet_DnaAlign(void);
 #define AlnConvertSet_DnaAlign Wise2_AlnConvertSet_DnaAlign
-int Wise2_find_end_DnaAlign(DnaAlign * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_DnaAlign_explicit_access_main(DnaAlign * mat,int i,int j,int state);
+#define DnaAlign_explicit_access_main Wise2_DnaAlign_explicit_access_main
+int Wise2_DnaAlign_explicit_access_special(DnaAlign * mat,int i,int j,int state);
+#define DnaAlign_explicit_access_special Wise2_DnaAlign_explicit_access_special
+int Wise2_find_end_DnaAlign(DnaAlign * mat,int * ri,int * rj,int * state,boolean * isspecial,DnaAlign_access_func_holder h);
 #define find_end_DnaAlign Wise2_find_end_DnaAlign
 void Wise2_DnaAlign_debug_show_matrix(DnaAlign * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define DnaAlign_debug_show_matrix Wise2_DnaAlign_debug_show_matrix
-int Wise2_max_calc_DnaAlign(DnaAlign * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_DnaAlign(DnaAlign * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,DnaAlign_access_func_holder h);
 #define max_calc_DnaAlign Wise2_max_calc_DnaAlign
-int Wise2_max_calc_special_DnaAlign(DnaAlign * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_DnaAlign(DnaAlign * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,DnaAlign_access_func_holder h);
 #define max_calc_special_DnaAlign Wise2_max_calc_special_DnaAlign
 
 #ifdef _cplusplus
