@@ -3,7 +3,7 @@ extern "C" {
 #endif
 #include "est_evidence.h"
 
-# line 31 "est_evidence.dy"
+# line 33 "est_evidence.dy"
 int indicate_intron_used(GenomeEvidenceSet * set,AlnBlock * alb)
 {
   AlnColumn * alc;
@@ -33,7 +33,7 @@ int indicate_intron_used(GenomeEvidenceSet * set,AlnBlock * alb)
 }
       
   
-# line 60 "est_evidence.dy"
+# line 62 "est_evidence.dy"
 GenomeEvidenceSet * read_est_evidence(FILE * ifp,CodonTable * ct)
 {
   char buffer[MAXLINE];
@@ -62,7 +62,11 @@ GenomeEvidenceSet * read_est_evidence(FILE * ifp,CodonTable * ct)
     } 
     if( strstartcmp(buffer,"exon") == 0 ) {
       exon = EstExon_alloc();
-      sscanf(buffer,"exon %d %d",&exon->start,&exon->end);
+      exon->intron_3_score = 0;
+      if( sscanf(buffer,"exon %d %d %d",&exon->start,&exon->end,&exon->intron_3_score) < 2 ) {
+	fatal("Unable to read exon line as evidence [%s]");
+      }
+
       exon->start--;
       exon->end--;
       add_EstEvidence(evi,exon);
@@ -96,7 +100,7 @@ GenomeEvidenceSet * read_est_evidence(FILE * ifp,CodonTable * ct)
 }
 
 
-# line 122 "est_evidence.dy"
+# line 128 "est_evidence.dy"
 GenomeEvidenceUnit * new_est_GenomeEvidenceUnit(EstEvidence * evi)
 {
   GenomeEvidenceUnit * in;
@@ -123,7 +127,7 @@ GenomeEvidenceUnit * new_est_GenomeEvidenceUnit(EstEvidence * evi)
 }
 
 
-# line 148 "est_evidence.dy"
+# line 154 "est_evidence.dy"
 int est_utr5_start(void * data,ComplexSequence *seq,int jposition)
 {
   EstEvidence * est;
@@ -137,7 +141,7 @@ int est_utr5_start(void * data,ComplexSequence *seq,int jposition)
 
 }
 
-# line 161 "est_evidence.dy"
+# line 167 "est_evidence.dy"
 int est_utr3_end(void * data,ComplexSequence *seq,int jposition)
 {
   EstEvidence * est;
@@ -151,7 +155,7 @@ int est_utr3_end(void * data,ComplexSequence *seq,int jposition)
 
 }
 
-# line 174 "est_evidence.dy"
+# line 180 "est_evidence.dy"
 int est_start_pot(void * data,ComplexSequence *seq,int jposition)
 {
   EstEvidence * est;
@@ -172,7 +176,7 @@ int est_start_pot(void * data,ComplexSequence *seq,int jposition)
 }
 
 
-# line 194 "est_evidence.dy"
+# line 200 "est_evidence.dy"
 int est_stop_pot(void * data,ComplexSequence *seq,int jposition)
 {
   EstEvidence * est;
@@ -187,7 +191,7 @@ int est_stop_pot(void * data,ComplexSequence *seq,int jposition)
 }
 
 
-# line 208 "est_evidence.dy"
+# line 214 "est_evidence.dy"
 int est_cds_frameshift(void * data,ComplexSequence * seq,int jposition,int jump)
 {
   EstEvidence * est;
@@ -205,7 +209,7 @@ int est_cds_frameshift(void * data,ComplexSequence * seq,int jposition,int jump)
 }
 
 
-# line 225 "est_evidence.dy"
+# line 231 "est_evidence.dy"
 int est_cds_3SS(void * data,ComplexSequence *seq,int jposition,int phase)
 {
   switch(phase) {
@@ -216,7 +220,7 @@ int est_cds_3SS(void * data,ComplexSequence *seq,int jposition,int phase)
   }
 }
 
-# line 235 "est_evidence.dy"
+# line 241 "est_evidence.dy"
 int est_cds_5SS(void * data,ComplexSequence *seq,int jposition,int phase)
 {
   switch(phase) {
@@ -228,7 +232,7 @@ int est_cds_5SS(void * data,ComplexSequence *seq,int jposition,int phase)
 }
 
 
-# line 246 "est_evidence.dy"
+# line 252 "est_evidence.dy"
 int est_intron_pot(void * data,ComplexSequence *seq,int jposition)
 {
   EstEvidence * est;
@@ -249,7 +253,7 @@ int est_intron_pot(void * data,ComplexSequence *seq,int jposition)
 }
 
 
-# line 266 "est_evidence.dy"
+# line 272 "est_evidence.dy"
 int est_cds_pot(void * data,ComplexSequence *seq,int jposition)
 {
   int i;
@@ -292,7 +296,7 @@ int est_cds_pot(void * data,ComplexSequence *seq,int jposition)
   return -1000000;
 }
 
-# line 308 "est_evidence.dy"
+# line 314 "est_evidence.dy"
 int est_3ss(void * data,ComplexSequence *seq,int jposition)
 {
   int i;
@@ -320,7 +324,7 @@ int est_3ss(void * data,ComplexSequence *seq,int jposition)
 }
 
 
-# line 335 "est_evidence.dy"
+# line 341 "est_evidence.dy"
 int est_5ss(void * data,ComplexSequence * seq,int jposition)
 {
   int i;
@@ -334,9 +338,9 @@ int est_5ss(void * data,ComplexSequence * seq,int jposition)
   for(i=0;i<est->len;i++) {
     if( jposition-1 == est->exon[i]->end ) {
       if( est->exon[i]->used == 0 ) {
-	return 500;
+	return 500+est->exon[i]->intron_3_score;
       } else {
-	return 10;
+	return 10+est->exon[i]->intron_3_score;
       }
     }
     
@@ -350,7 +354,7 @@ int est_5ss(void * data,ComplexSequence * seq,int jposition)
 }
  
 
-# line 364 "est_evidence.dy"
+# line 370 "est_evidence.dy"
 int est_utr_pot(void * data,ComplexSequence *seq,int jposition)
 {
   int i;
@@ -371,7 +375,7 @@ int est_utr_pot(void * data,ComplexSequence *seq,int jposition)
 
 
 
-# line 373 "est_evidence.c"
+# line 377 "est_evidence.c"
 /* Function:  hard_link_EstExon(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -414,11 +418,15 @@ EstExon * EstExon_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->start = 0;  
     out->end = 0;    
     out->is_coding = FALSE;  
     out->phase = 0;  
     out->used = 0;   
+    out->intron_3_score = 0; 
 
 
     return out;  
@@ -438,6 +446,7 @@ EstExon * EstExon_alloc(void)
  */
 EstExon * free_EstExon(EstExon * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -446,10 +455,18 @@ EstExon * free_EstExon(EstExon * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -499,6 +516,9 @@ EstIndel * EstIndel_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->start = 0;  
     out->end = 0;    
 
@@ -520,6 +540,7 @@ EstIndel * EstIndel_alloc(void)
  */
 EstIndel * free_EstIndel(EstIndel * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -528,10 +549,18 @@ EstIndel * free_EstIndel(EstIndel * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -948,6 +977,9 @@ EstEvidence * EstEvidence_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->exon = NULL;    
     out->len = out->maxlen = 0;  
     out->indel = NULL;   
@@ -973,6 +1005,7 @@ EstEvidence * EstEvidence_alloc(void)
  */
 EstEvidence * free_EstEvidence(EstEvidence * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -982,10 +1015,18 @@ EstEvidence * free_EstEvidence(EstEvidence * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->exon != NULL)   {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->exon[i] != NULL)    
