@@ -13,7 +13,11 @@ extern "C" {
 
 struct Wise2_EstFrag3 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     GeneWiseScore* query;    
@@ -30,6 +34,22 @@ typedef struct Wise2_EstFrag3 Wise2_EstFrag3;
 #endif
 
 
+struct Wise2_EstFrag3_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(EstFrag3*,int,int,int);   
+    int (*access_special)(EstFrag3*,int,int,int);    
+    } ;  
+/* EstFrag3_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_EstFrag3_access_func_holder
+typedef struct Wise2_EstFrag3_access_func_holder Wise2_EstFrag3_access_func_holder;
+#define EstFrag3_access_func_holder Wise2_EstFrag3_access_func_holder
+#define DYNAMITE_DEFINED_EstFrag3_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -39,6 +59,35 @@ typedef struct Wise2_EstFrag3 Wise2_EstFrag3;
 
 
 
+/* Function:  PackAln_read_Shatter_EstFrag3(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [EstFrag3 *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_EstFrag3(EstFrag3 * mat);
+#define PackAln_read_Shatter_EstFrag3 Wise2_PackAln_read_Shatter_EstFrag3
+
+
+/* Function:  calculate_shatter_EstFrag3(mat,dpenv)
+ *
+ * Descrip:    This function calculates the EstFrag3 matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [EstFrag3 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_EstFrag3(EstFrag3 * mat,DPEnvelope * dpenv);
+#define calculate_shatter_EstFrag3 Wise2_calculate_shatter_EstFrag3
+
+
 /* Function:  search_EstFrag3(dbsi,out,querydb,targetdb,cp,start_frag,end_frag)
  *
  * Descrip:    This function makes a database search of EstFrag3
@@ -109,7 +158,7 @@ PackAln * Wise2_PackAln_bestmemory_EstFrag3(GeneWiseScore* query,ComplexSequence
 #define PackAln_bestmemory_EstFrag3 Wise2_PackAln_bestmemory_EstFrag3
 
 
-/* Function:  allocate_Expl_EstFrag3(query,target,cp,start_frag,end_frag)
+/* Function:  allocate_Expl_EstFrag3(query,target,cp,start_frag,end_frag,dpri)
  *
  * Descrip:    This function allocates the EstFrag3 structure
  *             and the basematrix area for explicit memory implementations
@@ -121,11 +170,12 @@ PackAln * Wise2_PackAln_bestmemory_EstFrag3(GeneWiseScore* query,ComplexSequence
  * Arg:                cp [UNKN ] Resource [cDNAParserScore *]
  * Arg:        start_frag [UNKN ] Resource [Score]
  * Arg:          end_frag [UNKN ] Resource [Score]
+ * Arg:              dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [EstFrag3 *]
  *
  */
-EstFrag3 * Wise2_allocate_Expl_EstFrag3(GeneWiseScore* query,ComplexSequence* target ,cDNAParserScore * cp,Score start_frag,Score end_frag);
+EstFrag3 * Wise2_allocate_Expl_EstFrag3(GeneWiseScore* query,ComplexSequence* target ,cDNAParserScore * cp,Score start_frag,Score end_frag,DPRunImpl * dpri);
 #define allocate_Expl_EstFrag3 Wise2_allocate_Expl_EstFrag3
 
 
@@ -250,6 +300,21 @@ PackAln * Wise2_PackAln_read_Expl_EstFrag3(EstFrag3 * mat);
 #define PackAln_read_Expl_EstFrag3 Wise2_PackAln_read_Expl_EstFrag3
 
 
+/* Function:  PackAln_read_generic_EstFrag3(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [EstFrag3 *]
+ * Arg:          h [UNKN ] Undocumented argument [EstFrag3_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_EstFrag3(EstFrag3 * mat,EstFrag3_access_func_holder h);
+#define PackAln_read_generic_EstFrag3 Wise2_PackAln_read_generic_EstFrag3
+
+
 /* Function:  calculate_EstFrag3(mat)
  *
  * Descrip:    This function calculates the EstFrag3 matrix when in explicit mode
@@ -265,6 +330,21 @@ boolean Wise2_calculate_EstFrag3(EstFrag3 * mat);
 #define calculate_EstFrag3 Wise2_calculate_EstFrag3
 
 
+/* Function:  calculate_dpenv_EstFrag3(mat,dpenv)
+ *
+ * Descrip:    This function calculates the EstFrag3 matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] EstFrag3 which contains explicit basematrix memory [EstFrag3 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_EstFrag3(EstFrag3 * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_EstFrag3 Wise2_calculate_dpenv_EstFrag3
+
+
 /* Function:  EstFrag3_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -301,6 +381,10 @@ EstFrag3 * Wise2_free_EstFrag3(EstFrag3 * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_EstFrag3_shatter_access_main(EstFrag3 * mat,int i,int j,int state);
+#define EstFrag3_shatter_access_main Wise2_EstFrag3_shatter_access_main
+int Wise2_EstFrag3_shatter_access_special(EstFrag3 * mat,int i,int j,int state);
+#define EstFrag3_shatter_access_special Wise2_EstFrag3_shatter_access_special
 int Wise2_score_only_EstFrag3(GeneWiseScore* query,ComplexSequence* target ,cDNAParserScore * cp,Score start_frag,Score end_frag);
 #define score_only_EstFrag3 Wise2_score_only_EstFrag3
 EstFrag3 * Wise2_allocate_EstFrag3_only(GeneWiseScore* query,ComplexSequence* target ,cDNAParserScore * cp,Score start_frag,Score end_frag);
@@ -335,21 +419,25 @@ void Wise2_run_up_dc_EstFrag3(EstFrag3 * mat,int starti,int stopi,int startj,int
 #define run_up_dc_EstFrag3 Wise2_run_up_dc_EstFrag3
 void Wise2_init_dc_EstFrag3(EstFrag3 * mat);
 #define init_dc_EstFrag3 Wise2_init_dc_EstFrag3
-boolean Wise2_dc_start_end_calculate_EstFrag3(EstFrag3 * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_EstFrag3 Wise2_dc_start_end_calculate_EstFrag3
 int Wise2_start_end_find_end_EstFrag3(EstFrag3 * mat,int * endj);
 #define start_end_find_end_EstFrag3 Wise2_start_end_find_end_EstFrag3
+boolean Wise2_dc_optimised_start_end_calc_EstFrag3(EstFrag3 *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_EstFrag3 Wise2_dc_optimised_start_end_calc_EstFrag3
 void Wise2_init_start_end_linear_EstFrag3(EstFrag3 * mat);
 #define init_start_end_linear_EstFrag3 Wise2_init_start_end_linear_EstFrag3
 AlnConvertSet * Wise2_AlnConvertSet_EstFrag3(void);
 #define AlnConvertSet_EstFrag3 Wise2_AlnConvertSet_EstFrag3
-int Wise2_find_end_EstFrag3(EstFrag3 * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_EstFrag3_explicit_access_main(EstFrag3 * mat,int i,int j,int state);
+#define EstFrag3_explicit_access_main Wise2_EstFrag3_explicit_access_main
+int Wise2_EstFrag3_explicit_access_special(EstFrag3 * mat,int i,int j,int state);
+#define EstFrag3_explicit_access_special Wise2_EstFrag3_explicit_access_special
+int Wise2_find_end_EstFrag3(EstFrag3 * mat,int * ri,int * rj,int * state,boolean * isspecial,EstFrag3_access_func_holder h);
 #define find_end_EstFrag3 Wise2_find_end_EstFrag3
 void Wise2_EstFrag3_debug_show_matrix(EstFrag3 * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define EstFrag3_debug_show_matrix Wise2_EstFrag3_debug_show_matrix
-int Wise2_max_calc_EstFrag3(EstFrag3 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_EstFrag3(EstFrag3 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,EstFrag3_access_func_holder h);
 #define max_calc_EstFrag3 Wise2_max_calc_EstFrag3
-int Wise2_max_calc_special_EstFrag3(EstFrag3 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_EstFrag3(EstFrag3 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,EstFrag3_access_func_holder h);
 #define max_calc_special_EstFrag3 Wise2_max_calc_special_EstFrag3
 
 #ifdef _cplusplus
