@@ -12,7 +12,11 @@ extern "C" {
 
 struct Wise2_EstQuick3 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     GeneWiseScoreFlat* query;    
@@ -45,6 +49,22 @@ struct thread_pool_holder_EstQuick3 {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_EstQuick3_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(EstQuick3*,int,int,int);  
+    int (*access_special)(EstQuick3*,int,int,int);   
+    } ;  
+/* EstQuick3_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_EstQuick3_access_func_holder
+typedef struct Wise2_EstQuick3_access_func_holder Wise2_EstQuick3_access_func_holder;
+#define EstQuick3_access_func_holder Wise2_EstQuick3_access_func_holder
+#define DYNAMITE_DEFINED_EstQuick3_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -54,6 +74,35 @@ struct thread_pool_holder_EstQuick3 {
 
 
 
+/* Function:  PackAln_read_Shatter_EstQuick3(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [EstQuick3 *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_EstQuick3(EstQuick3 * mat);
+#define PackAln_read_Shatter_EstQuick3 Wise2_PackAln_read_Shatter_EstQuick3
+
+
+/* Function:  calculate_shatter_EstQuick3(mat,dpenv)
+ *
+ * Descrip:    This function calculates the EstQuick3 matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [EstQuick3 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_EstQuick3(EstQuick3 * mat,DPEnvelope * dpenv);
+#define calculate_shatter_EstQuick3 Wise2_calculate_shatter_EstQuick3
+
+
 /* Function:  search_EstQuick3(dbsi,out,querydb,targetdb,cp)
  *
  * Descrip:    This function makes a database search of EstQuick3
@@ -118,7 +167,7 @@ PackAln * Wise2_PackAln_bestmemory_EstQuick3(GeneWiseScoreFlat* query,ComplexSeq
 #define PackAln_bestmemory_EstQuick3 Wise2_PackAln_bestmemory_EstQuick3
 
 
-/* Function:  allocate_Expl_EstQuick3(query,target,cp)
+/* Function:  allocate_Expl_EstQuick3(query,target,cp,dpri)
  *
  * Descrip:    This function allocates the EstQuick3 structure
  *             and the basematrix area for explicit memory implementations
@@ -128,11 +177,12 @@ PackAln * Wise2_PackAln_bestmemory_EstQuick3(GeneWiseScoreFlat* query,ComplexSeq
  * Arg:         query [UNKN ] query data structure [GeneWiseScoreFlat*]
  * Arg:        target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            cp [UNKN ] Resource [cDNAParserScore *]
+ * Arg:          dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [EstQuick3 *]
  *
  */
-EstQuick3 * Wise2_allocate_Expl_EstQuick3(GeneWiseScoreFlat* query,ComplexSequence* target ,cDNAParserScore * cp);
+EstQuick3 * Wise2_allocate_Expl_EstQuick3(GeneWiseScoreFlat* query,ComplexSequence* target ,cDNAParserScore * cp,DPRunImpl * dpri);
 #define allocate_Expl_EstQuick3 Wise2_allocate_Expl_EstQuick3
 
 
@@ -255,6 +305,21 @@ PackAln * Wise2_PackAln_read_Expl_EstQuick3(EstQuick3 * mat);
 #define PackAln_read_Expl_EstQuick3 Wise2_PackAln_read_Expl_EstQuick3
 
 
+/* Function:  PackAln_read_generic_EstQuick3(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [EstQuick3 *]
+ * Arg:          h [UNKN ] Undocumented argument [EstQuick3_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_EstQuick3(EstQuick3 * mat,EstQuick3_access_func_holder h);
+#define PackAln_read_generic_EstQuick3 Wise2_PackAln_read_generic_EstQuick3
+
+
 /* Function:  calculate_EstQuick3(mat)
  *
  * Descrip:    This function calculates the EstQuick3 matrix when in explicit mode
@@ -270,6 +335,21 @@ boolean Wise2_calculate_EstQuick3(EstQuick3 * mat);
 #define calculate_EstQuick3 Wise2_calculate_EstQuick3
 
 
+/* Function:  calculate_dpenv_EstQuick3(mat,dpenv)
+ *
+ * Descrip:    This function calculates the EstQuick3 matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] EstQuick3 which contains explicit basematrix memory [EstQuick3 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_EstQuick3(EstQuick3 * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_EstQuick3 Wise2_calculate_dpenv_EstQuick3
+
+
 /* Function:  EstQuick3_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -306,6 +386,10 @@ EstQuick3 * Wise2_free_EstQuick3(EstQuick3 * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_EstQuick3_shatter_access_main(EstQuick3 * mat,int i,int j,int state);
+#define EstQuick3_shatter_access_main Wise2_EstQuick3_shatter_access_main
+int Wise2_EstQuick3_shatter_access_special(EstQuick3 * mat,int i,int j,int state);
+#define EstQuick3_shatter_access_special Wise2_EstQuick3_shatter_access_special
 void * Wise2_thread_loop_EstQuick3(void * ptr);
 #define thread_loop_EstQuick3 Wise2_thread_loop_EstQuick3
 int Wise2_score_only_EstQuick3(GeneWiseScoreFlat* query,ComplexSequence* target ,cDNAParserScore * cp);
@@ -342,21 +426,25 @@ void Wise2_run_up_dc_EstQuick3(EstQuick3 * mat,int starti,int stopi,int startj,i
 #define run_up_dc_EstQuick3 Wise2_run_up_dc_EstQuick3
 void Wise2_init_dc_EstQuick3(EstQuick3 * mat);
 #define init_dc_EstQuick3 Wise2_init_dc_EstQuick3
-boolean Wise2_dc_start_end_calculate_EstQuick3(EstQuick3 * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_EstQuick3 Wise2_dc_start_end_calculate_EstQuick3
 int Wise2_start_end_find_end_EstQuick3(EstQuick3 * mat,int * endj);
 #define start_end_find_end_EstQuick3 Wise2_start_end_find_end_EstQuick3
+boolean Wise2_dc_optimised_start_end_calc_EstQuick3(EstQuick3 *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_EstQuick3 Wise2_dc_optimised_start_end_calc_EstQuick3
 void Wise2_init_start_end_linear_EstQuick3(EstQuick3 * mat);
 #define init_start_end_linear_EstQuick3 Wise2_init_start_end_linear_EstQuick3
 AlnConvertSet * Wise2_AlnConvertSet_EstQuick3(void);
 #define AlnConvertSet_EstQuick3 Wise2_AlnConvertSet_EstQuick3
-int Wise2_find_end_EstQuick3(EstQuick3 * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_EstQuick3_explicit_access_main(EstQuick3 * mat,int i,int j,int state);
+#define EstQuick3_explicit_access_main Wise2_EstQuick3_explicit_access_main
+int Wise2_EstQuick3_explicit_access_special(EstQuick3 * mat,int i,int j,int state);
+#define EstQuick3_explicit_access_special Wise2_EstQuick3_explicit_access_special
+int Wise2_find_end_EstQuick3(EstQuick3 * mat,int * ri,int * rj,int * state,boolean * isspecial,EstQuick3_access_func_holder h);
 #define find_end_EstQuick3 Wise2_find_end_EstQuick3
 void Wise2_EstQuick3_debug_show_matrix(EstQuick3 * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define EstQuick3_debug_show_matrix Wise2_EstQuick3_debug_show_matrix
-int Wise2_max_calc_EstQuick3(EstQuick3 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_EstQuick3(EstQuick3 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,EstQuick3_access_func_holder h);
 #define max_calc_EstQuick3 Wise2_max_calc_EstQuick3
-int Wise2_max_calc_special_EstQuick3(EstQuick3 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_EstQuick3(EstQuick3 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,EstQuick3_access_func_holder h);
 #define max_calc_special_EstQuick3 Wise2_max_calc_special_EstQuick3
 
 #ifdef _cplusplus
