@@ -78,7 +78,7 @@ char * qend_str      = NULL;
 int qend             = -1;
 
 
-char * matrix_file      = "blosum62.bla";
+char * matrix_file      = "BLOSUM62.bla";
 CompMat * mat           = NULL;
 
 char * gap_str          = "12";
@@ -847,7 +847,7 @@ void show_version(FILE * ofp)
 void build_defaults(void)
 {
   codon_file = "codon.table";
-  matrix_file = "blosum62.bla";
+  matrix_file = "BLOSUM62.bla";
   
 
 }
