@@ -223,8 +223,10 @@ boolean protdna_btc_display(AlnBlock * alb,char * protsequence,char * protname_i
 	  
 	  if( strcmp(alc->alu[0]->text_label,"INSERT_STATE")== 0 ) {
 	    write_codon_match(btp,'-',' ',alc->alu[1]->start+1,aminoacid_from_seq(ct,dna->seq+alc->alu[1]->start+1),dna->seq+alc->alu[1]->start+1);
-	  }
-	  else {
+	  } else if( strcmp(alc->alu[0]->text_label,"BEFORE_MATCH") == 0 ||
+	             strcmp(alc->alu[0]->text_label,"AFTER_MATCH") == 0 ) {
+	    write_codon_match(btp,'~',' ',alc->alu[1]->start+1,aminoacid_from_seq(ct,dna->seq+alc->alu[1]->start+1),dna->seq+alc->alu[1]->start+1);
+	  } else {
 	    write_codon_match(btp,protsequence[alc->alu[0]->end],(*match_central_line)(protsequence[alc->alu[0]->end],alc->alu[0]->score[0],aminoacid_from_seq(ct,dna->seq+alc->alu[1]->start+1)),alc->alu[1]->start+1,aminoacid_from_seq(ct,dna->seq+alc->alu[1]->start+1),dna->seq+alc->alu[1]->start+1);
 	  }
 	  
@@ -292,7 +294,7 @@ boolean protdna_btc_display(AlnBlock * alb,char * protsequence,char * protname_i
 	   *
 	   */
 
-	  if( can_get_paste_area_btCanvas(btc,d_phase+7+13) == FALSE) {
+	  if( can_get_paste_area_btCanvas(btc,d_phase+7+17) == FALSE) {
 	    advance_line_btCanvas(btc);
 	    
 	    write_name_start_stuff(btc,protname,protoff,dnaname,dna,name,alc);
@@ -300,7 +302,7 @@ boolean protdna_btc_display(AlnBlock * alb,char * protsequence,char * protname_i
 	  
 	  /*** ok, if we can't get it now then we are fucked ***/
 	  
-	  if( can_get_paste_area_btCanvas(btc,d_phase+7+13) == FALSE) {
+	  if( can_get_paste_area_btCanvas(btc,d_phase+7+17) == FALSE) {
 	    warn("You have specified a length of your main canvas too small. I need at least 23 characters long.");
 	    advance_line_btCanvas(btc);
 	    return FALSE;
@@ -338,7 +340,7 @@ boolean protdna_btc_display(AlnBlock * alb,char * protsequence,char * protname_i
 	  write_5intron_match(btp,d_phase,7,dna->seq+alc->alu[1]->start+1);
 	  free_btPasteArea(btp);
 	  
-	  btp = get_paste_area_btCanvas(btc,13);
+	  btp = get_paste_area_btCanvas(btc,17);
 
 	  if( is_reversed == FALSE ) 
 	    write_intron_desc(btp,alc->alu[1]->start+1+d_phase+dna->offset,alc_temp->alu[1]->start+3+dna->offset,intron_number++,issplit,protc,transc,dnatemp);
@@ -439,7 +441,7 @@ boolean protdna_btc_display(AlnBlock * alb,char * protsequence,char * protname_i
   return TRUE;
 } 
 
-# line 435 "genedisplay.dy"
+# line 437 "genedisplay.dy"
 boolean write_alignment_separator(btCanvas * btc,int aln,int score)
 {
   char buffer[64];
@@ -458,7 +460,7 @@ boolean write_alignment_separator(btCanvas * btc,int aln,int score)
   return TRUE;
 }
 
-# line 453 "genedisplay.dy"
+# line 455 "genedisplay.dy"
 boolean write_name_start_stuff(btCanvas * btc,char * protname,int protoff,char * dnaname,Sequence * dna,int name_len,AlnColumn * alc)
 {
   char buffer[64];
@@ -485,26 +487,36 @@ boolean write_name_start_stuff(btCanvas * btc,char * protname,int protoff,char *
 }
 
 
-# line 479 "genedisplay.dy"
+# line 481 "genedisplay.dy"
 boolean write_intron_desc(btPasteArea * btp,int start,int stop,int in_number,boolean is_split,char prot,char trans,char * dna)
 {
-  char buffer[15];
+  char buffer[32];
 
   if( is_split ) {
     sprintf(buffer,"  %c:%c[%s]  ",prot,trans,dna);
     paste_string_btPasteArea(btp,0,2,buffer,BC_RIGHT,0);
   }
 
-  sprintf(buffer,"  Intron %-2d  ",in_number);
+  if( in_number < 1000 ) {
+    sprintf(buffer,"  Intron %-3d  ",in_number);
+  } else {
+    sprintf(buffer,"  Intron ???  ",in_number);
+  }
+
   paste_string_btPasteArea(btp,0,3,buffer,BC_RIGHT,0);
-  sprintf(buffer,"[%-5d:%5d]",start,stop);
+
+  if( start < 10000000 && stop < 10000000 ) 
+    sprintf(buffer,"[%-7d:%7d]",start,stop);
+  else 
+    sprintf(buffer,"[???????:???????]");
+
   paste_string_btPasteArea(btp,0,4,buffer,BC_RIGHT,0);
 
   return TRUE;
 }
 
 
-# line 497 "genedisplay.dy"
+# line 509 "genedisplay.dy"
 boolean write_3intron_match(btPasteArea * btp,int phase,int length,char * seq)
 {
   char buf[2];
@@ -537,7 +549,7 @@ boolean write_3intron_match(btPasteArea * btp,int phase,int length,char * seq)
 }
   
 
-# line 529 "genedisplay.dy"
+# line 541 "genedisplay.dy"
 boolean write_5intron_match(btPasteArea * btp,int phase,int length,char * seq)
 {
   char buf[2];
@@ -559,7 +571,7 @@ boolean write_5intron_match(btPasteArea * btp,int phase,int length,char * seq)
   return TRUE;
 }
 
-# line 550 "genedisplay.dy"
+# line 562 "genedisplay.dy"
 boolean write_codon_match(btPasteArea * btp,char match_letter,char midline,int c_start,char aa,char * seq) 
 {
   paste_char_btPasteArea(btp,0,0,match_letter,0);
@@ -602,7 +614,7 @@ boolean write_codon_match(btPasteArea * btp,char match_letter,char midline,int c
 
 
 
-# line 598 "genedisplay.c"
+# line 610 "genedisplay.c"
 
 #ifdef _cplusplus
 }
