@@ -1098,6 +1098,9 @@ GeneSingleCons * GeneSingleCons_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->string = NULL;  
     out->number = 0; 
 
@@ -1119,6 +1122,7 @@ GeneSingleCons * GeneSingleCons_alloc(void)
  */
 GeneSingleCons * free_GeneSingleCons(GeneSingleCons * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1127,10 +1131,18 @@ GeneSingleCons * free_GeneSingleCons(GeneSingleCons * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->string != NULL) 
       ckfree(obj->string);   
 
@@ -1386,6 +1398,9 @@ GeneConsensus * GeneConsensus_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->center = 0; 
     out->gsc = NULL; 
     out->len = out->maxlen = 0;  
@@ -1408,6 +1423,7 @@ GeneConsensus * GeneConsensus_alloc(void)
  */
 GeneConsensus * free_GeneConsensus(GeneConsensus * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1417,10 +1433,18 @@ GeneConsensus * free_GeneConsensus(GeneConsensus * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->gsc != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->gsc[i] != NULL) 
@@ -1477,6 +1501,9 @@ GeneFrequency21 * GeneFrequency21_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->ss5 = NULL; 
     out->ss3 = NULL; 
     /* codon[64] is an array: no default possible */ 
@@ -1504,6 +1531,7 @@ GeneFrequency21 * GeneFrequency21_alloc(void)
  */
 GeneFrequency21 * free_GeneFrequency21(GeneFrequency21 * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1512,10 +1540,18 @@ GeneFrequency21 * free_GeneFrequency21(GeneFrequency21 * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->ss5 != NULL)    
       free_GeneConsensus(obj->ss5);  
     if( obj->ss3 != NULL)    
