@@ -38,6 +38,9 @@ enum {
 
 struct Wise2_GeneSingleCons {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * string;   
     double number;   
     } ;  
@@ -51,6 +54,9 @@ typedef struct Wise2_GeneSingleCons Wise2_GeneSingleCons;
 
 struct Wise2_GeneConsensus {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     int center;  
     GeneSingleCons ** gsc;   
     int len;/* len for above gsc  */ 
@@ -66,6 +72,9 @@ typedef struct Wise2_GeneConsensus Wise2_GeneConsensus;
 
 struct Wise2_GeneFrequency21 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     GeneConsensus * ss5;     
     GeneConsensus * ss3;     
     double codon[64];    
