@@ -6,15 +6,19 @@ extern "C" {
 
 #include "dyna.h"
 #include "genewisemodel.h"
-
+#include "genewisemodeldb.h"
 
 
 struct Wise2_GeneLoop21 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
-    GeneWiseScore * query;   
+    GeneWiseScore* query;    
     ComplexSequence* target;     
     GeneParser21Score * gp21;    
     RandomCodonScore * rndcodon;     
@@ -30,7 +34,9 @@ typedef struct Wise2_GeneLoop21 Wise2_GeneLoop21;
 
 #ifdef PTHREAD
 struct thread_pool_holder_GeneLoop21 {  
-    GeneWiseScore * query;  /* Static query data: never free'd */ 
+    GeneWiseScore* query;   /* Query object placeholder */ 
+    GeneWiseDB* querydb;/* Query database object */ 
+    boolean query_init;  
     ComplexSequence* target;/* Target object placeholder */ 
     GenomicDB* targetdb;/* Target database object */ 
     boolean target_init; 
@@ -46,6 +52,22 @@ struct thread_pool_holder_GeneLoop21 {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_GeneLoop21_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(GeneLoop21*,int,int,int); 
+    int (*access_special)(GeneLoop21*,int,int,int);  
+    } ;  
+/* GeneLoop21_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_GeneLoop21_access_func_holder
+typedef struct Wise2_GeneLoop21_access_func_holder Wise2_GeneLoop21_access_func_holder;
+#define GeneLoop21_access_func_holder Wise2_GeneLoop21_access_func_holder
+#define DYNAMITE_DEFINED_GeneLoop21_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -55,7 +77,36 @@ struct thread_pool_holder_GeneLoop21 {
 
 
 
-/* Function:  search_GeneLoop21(dbsi,out,query,targetdb,gp21,rndcodon,rndbase)
+/* Function:  PackAln_read_Shatter_GeneLoop21(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneLoop21 *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_GeneLoop21(GeneLoop21 * mat);
+#define PackAln_read_Shatter_GeneLoop21 Wise2_PackAln_read_Shatter_GeneLoop21
+
+
+/* Function:  calculate_shatter_GeneLoop21(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneLoop21 matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [GeneLoop21 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_GeneLoop21(GeneLoop21 * mat,DPEnvelope * dpenv);
+#define calculate_shatter_GeneLoop21 Wise2_calculate_shatter_GeneLoop21
+
+
+/* Function:  search_GeneLoop21(dbsi,out,querydb,targetdb,gp21,rndcodon,rndbase)
  *
  * Descrip:    This function makes a database search of GeneLoop21
  *             It uses the dbsi structure to choose which implementation
@@ -65,7 +116,7 @@ struct thread_pool_holder_GeneLoop21 {
  *
  * Arg:            dbsi [UNKN ] Undocumented argument [DBSearchImpl *]
  * Arg:             out [UNKN ] Undocumented argument [Hscore *]
- * Arg:           query [UNKN ] Undocumented argument [GeneWiseScore *]
+ * Arg:         querydb [UNKN ] Undocumented argument [GeneWiseDB*]
  * Arg:        targetdb [UNKN ] Undocumented argument [GenomicDB*]
  * Arg:            gp21 [UNKN ] Undocumented argument [GeneParser21Score *]
  * Arg:        rndcodon [UNKN ] Undocumented argument [RandomCodonScore *]
@@ -74,18 +125,18 @@ struct thread_pool_holder_GeneLoop21 {
  * Return [UNKN ]  Undocumented return value [Search_Return_Type]
  *
  */
-Search_Return_Type Wise2_search_GeneLoop21(DBSearchImpl * dbsi,Hscore * out,GeneWiseScore * query,GenomicDB* targetdb ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
+Search_Return_Type Wise2_search_GeneLoop21(DBSearchImpl * dbsi,Hscore * out,GeneWiseDB* querydb,GenomicDB* targetdb ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
 #define search_GeneLoop21 Wise2_search_GeneLoop21
 
 
-/* Function:  serial_search_GeneLoop21(out,query,targetdb,gp21,rndcodon,rndbase)
+/* Function:  serial_search_GeneLoop21(out,querydb,targetdb,gp21,rndcodon,rndbase)
  *
  * Descrip:    This function makes a database search of GeneLoop21
  *             It is a single processor implementation
  *
  *
  * Arg:             out [UNKN ] Undocumented argument [Hscore *]
- * Arg:           query [UNKN ] Undocumented argument [GeneWiseScore *]
+ * Arg:         querydb [UNKN ] Undocumented argument [GeneWiseDB*]
  * Arg:        targetdb [UNKN ] Undocumented argument [GenomicDB*]
  * Arg:            gp21 [UNKN ] Undocumented argument [GeneParser21Score *]
  * Arg:        rndcodon [UNKN ] Undocumented argument [RandomCodonScore *]
@@ -94,7 +145,7 @@ Search_Return_Type Wise2_search_GeneLoop21(DBSearchImpl * dbsi,Hscore * out,Gene
  * Return [UNKN ]  Undocumented return value [Search_Return_Type]
  *
  */
-Search_Return_Type Wise2_serial_search_GeneLoop21(Hscore * out,GeneWiseScore * query,GenomicDB* targetdb ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
+Search_Return_Type Wise2_serial_search_GeneLoop21(Hscore * out,GeneWiseDB* querydb,GenomicDB* targetdb ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
 #define serial_search_GeneLoop21 Wise2_serial_search_GeneLoop21
 
 
@@ -110,7 +161,7 @@ Search_Return_Type Wise2_serial_search_GeneLoop21(Hscore * out,GeneWiseScore * q
  *             /convert_PackAln_to_AlnBlock_GeneLoop21
  *
  *
- * Arg:           query [UNKN ] query data structure [GeneWiseScore *]
+ * Arg:           query [UNKN ] query data structure [GeneWiseScore*]
  * Arg:          target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            gp21 [UNKN ] Resource [GeneParser21Score *]
  * Arg:        rndcodon [UNKN ] Resource [RandomCodonScore *]
@@ -121,27 +172,28 @@ Search_Return_Type Wise2_serial_search_GeneLoop21(Hscore * out,GeneWiseScore * q
  * Return [UNKN ]  Undocumented return value [PackAln *]
  *
  */
-PackAln * Wise2_PackAln_bestmemory_GeneLoop21(GeneWiseScore * query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase,DPEnvelope * dpenv,DPRunImpl * dpri);
+PackAln * Wise2_PackAln_bestmemory_GeneLoop21(GeneWiseScore* query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase,DPEnvelope * dpenv,DPRunImpl * dpri);
 #define PackAln_bestmemory_GeneLoop21 Wise2_PackAln_bestmemory_GeneLoop21
 
 
-/* Function:  allocate_Expl_GeneLoop21(query,target,gp21,rndcodon,rndbase)
+/* Function:  allocate_Expl_GeneLoop21(query,target,gp21,rndcodon,rndbase,dpri)
  *
  * Descrip:    This function allocates the GeneLoop21 structure
  *             and the basematrix area for explicit memory implementations
  *             It calls /allocate_GeneLoop21_only
  *
  *
- * Arg:           query [UNKN ] query data structure [GeneWiseScore *]
+ * Arg:           query [UNKN ] query data structure [GeneWiseScore*]
  * Arg:          target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            gp21 [UNKN ] Resource [GeneParser21Score *]
  * Arg:        rndcodon [UNKN ] Resource [RandomCodonScore *]
  * Arg:         rndbase [UNKN ] Resource [RandomModelDNAScore *]
+ * Arg:            dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [GeneLoop21 *]
  *
  */
-GeneLoop21 * Wise2_allocate_Expl_GeneLoop21(GeneWiseScore * query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
+GeneLoop21 * Wise2_allocate_Expl_GeneLoop21(GeneWiseScore* query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase,DPRunImpl * dpri);
 #define allocate_Expl_GeneLoop21 Wise2_allocate_Expl_GeneLoop21
 
 
@@ -166,7 +218,7 @@ void Wise2_recalculate_PackAln_GeneLoop21(PackAln * pal,GeneLoop21 * mat);
  *             It calls /allocate_GeneLoop21_only
  *
  *
- * Arg:           query [UNKN ] query data structure [GeneWiseScore *]
+ * Arg:           query [UNKN ] query data structure [GeneWiseScore*]
  * Arg:          target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            gp21 [UNKN ] Resource [GeneParser21Score *]
  * Arg:        rndcodon [UNKN ] Resource [RandomCodonScore *]
@@ -175,7 +227,7 @@ void Wise2_recalculate_PackAln_GeneLoop21(PackAln * pal,GeneLoop21 * mat);
  * Return [UNKN ]  Undocumented return value [GeneLoop21 *]
  *
  */
-GeneLoop21 * Wise2_allocate_Small_GeneLoop21(GeneWiseScore * query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
+GeneLoop21 * Wise2_allocate_Small_GeneLoop21(GeneWiseScore* query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
 #define allocate_Small_GeneLoop21 Wise2_allocate_Small_GeneLoop21
 
 
@@ -266,6 +318,21 @@ PackAln * Wise2_PackAln_read_Expl_GeneLoop21(GeneLoop21 * mat);
 #define PackAln_read_Expl_GeneLoop21 Wise2_PackAln_read_Expl_GeneLoop21
 
 
+/* Function:  PackAln_read_generic_GeneLoop21(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneLoop21 *]
+ * Arg:          h [UNKN ] Undocumented argument [GeneLoop21_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_GeneLoop21(GeneLoop21 * mat,GeneLoop21_access_func_holder h);
+#define PackAln_read_generic_GeneLoop21 Wise2_PackAln_read_generic_GeneLoop21
+
+
 /* Function:  calculate_GeneLoop21(mat)
  *
  * Descrip:    This function calculates the GeneLoop21 matrix when in explicit mode
@@ -281,6 +348,21 @@ boolean Wise2_calculate_GeneLoop21(GeneLoop21 * mat);
 #define calculate_GeneLoop21 Wise2_calculate_GeneLoop21
 
 
+/* Function:  calculate_dpenv_GeneLoop21(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneLoop21 matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] GeneLoop21 which contains explicit basematrix memory [GeneLoop21 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_GeneLoop21(GeneLoop21 * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_GeneLoop21 Wise2_calculate_dpenv_GeneLoop21
+
+
 /* Function:  GeneLoop21_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -317,9 +399,15 @@ GeneLoop21 * Wise2_free_GeneLoop21(GeneLoop21 * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
-int Wise2_score_only_GeneLoop21(GeneWiseScore * query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
+int Wise2_GeneLoop21_shatter_access_main(GeneLoop21 * mat,int i,int j,int state);
+#define GeneLoop21_shatter_access_main Wise2_GeneLoop21_shatter_access_main
+int Wise2_GeneLoop21_shatter_access_special(GeneLoop21 * mat,int i,int j,int state);
+#define GeneLoop21_shatter_access_special Wise2_GeneLoop21_shatter_access_special
+void * Wise2_thread_loop_GeneLoop21(void * ptr);
+#define thread_loop_GeneLoop21 Wise2_thread_loop_GeneLoop21
+int Wise2_score_only_GeneLoop21(GeneWiseScore* query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
 #define score_only_GeneLoop21 Wise2_score_only_GeneLoop21
-GeneLoop21 * Wise2_allocate_GeneLoop21_only(GeneWiseScore * query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
+GeneLoop21 * Wise2_allocate_GeneLoop21_only(GeneWiseScore* query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
 #define allocate_GeneLoop21_only Wise2_allocate_GeneLoop21_only
 void Wise2_init_GeneLoop21(GeneLoop21 * mat);
 #define init_GeneLoop21 Wise2_init_GeneLoop21
@@ -351,21 +439,25 @@ void Wise2_run_up_dc_GeneLoop21(GeneLoop21 * mat,int starti,int stopi,int startj
 #define run_up_dc_GeneLoop21 Wise2_run_up_dc_GeneLoop21
 void Wise2_init_dc_GeneLoop21(GeneLoop21 * mat);
 #define init_dc_GeneLoop21 Wise2_init_dc_GeneLoop21
-boolean Wise2_dc_start_end_calculate_GeneLoop21(GeneLoop21 * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_GeneLoop21 Wise2_dc_start_end_calculate_GeneLoop21
 int Wise2_start_end_find_end_GeneLoop21(GeneLoop21 * mat,int * endj);
 #define start_end_find_end_GeneLoop21 Wise2_start_end_find_end_GeneLoop21
+boolean Wise2_dc_optimised_start_end_calc_GeneLoop21(GeneLoop21 *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_GeneLoop21 Wise2_dc_optimised_start_end_calc_GeneLoop21
 void Wise2_init_start_end_linear_GeneLoop21(GeneLoop21 * mat);
 #define init_start_end_linear_GeneLoop21 Wise2_init_start_end_linear_GeneLoop21
 AlnConvertSet * Wise2_AlnConvertSet_GeneLoop21(void);
 #define AlnConvertSet_GeneLoop21 Wise2_AlnConvertSet_GeneLoop21
-int Wise2_find_end_GeneLoop21(GeneLoop21 * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_GeneLoop21_explicit_access_main(GeneLoop21 * mat,int i,int j,int state);
+#define GeneLoop21_explicit_access_main Wise2_GeneLoop21_explicit_access_main
+int Wise2_GeneLoop21_explicit_access_special(GeneLoop21 * mat,int i,int j,int state);
+#define GeneLoop21_explicit_access_special Wise2_GeneLoop21_explicit_access_special
+int Wise2_find_end_GeneLoop21(GeneLoop21 * mat,int * ri,int * rj,int * state,boolean * isspecial,GeneLoop21_access_func_holder h);
 #define find_end_GeneLoop21 Wise2_find_end_GeneLoop21
 void Wise2_GeneLoop21_debug_show_matrix(GeneLoop21 * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define GeneLoop21_debug_show_matrix Wise2_GeneLoop21_debug_show_matrix
-int Wise2_max_calc_GeneLoop21(GeneLoop21 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_GeneLoop21(GeneLoop21 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneLoop21_access_func_holder h);
 #define max_calc_GeneLoop21 Wise2_max_calc_GeneLoop21
-int Wise2_max_calc_special_GeneLoop21(GeneLoop21 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_GeneLoop21(GeneLoop21 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneLoop21_access_func_holder h);
 #define max_calc_special_GeneLoop21 Wise2_max_calc_special_GeneLoop21
 
 #ifdef _cplusplus
