@@ -6,15 +6,19 @@ extern "C" {
 #include "dyna.h"
 #include "geneparser4.h"
 #include "genewisemodel.h"
-
+#include "genewisemodeldb.h"
 
 
 struct Wise2_GeneLoop6 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
-    GeneWiseScore * query;   
+    GeneWiseScore* query;    
     ComplexSequence* target;     
     GeneParser4Score * gp;   
     } ;  
@@ -28,7 +32,9 @@ typedef struct Wise2_GeneLoop6 Wise2_GeneLoop6;
 
 #ifdef PTHREAD
 struct thread_pool_holder_GeneLoop6 {  
-    GeneWiseScore * query;  /* Static query data: never free'd */ 
+    GeneWiseScore* query;   /* Query object placeholder */ 
+    GeneWiseDB* querydb;/* Query database object */ 
+    boolean query_init;  
     ComplexSequence* target;/* Target object placeholder */ 
     GenomicDB* targetdb;/* Target database object */ 
     boolean target_init; 
@@ -42,6 +48,22 @@ struct thread_pool_holder_GeneLoop6 {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_GeneLoop6_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(GeneLoop6*,int,int,int);  
+    int (*access_special)(GeneLoop6*,int,int,int);   
+    } ;  
+/* GeneLoop6_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_GeneLoop6_access_func_holder
+typedef struct Wise2_GeneLoop6_access_func_holder Wise2_GeneLoop6_access_func_holder;
+#define GeneLoop6_access_func_holder Wise2_GeneLoop6_access_func_holder
+#define DYNAMITE_DEFINED_GeneLoop6_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -51,7 +73,36 @@ struct thread_pool_holder_GeneLoop6 {
 
 
 
-/* Function:  search_GeneLoop6(dbsi,out,query,targetdb,gp)
+/* Function:  PackAln_read_Shatter_GeneLoop6(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneLoop6 *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_GeneLoop6(GeneLoop6 * mat);
+#define PackAln_read_Shatter_GeneLoop6 Wise2_PackAln_read_Shatter_GeneLoop6
+
+
+/* Function:  calculate_shatter_GeneLoop6(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneLoop6 matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [GeneLoop6 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_GeneLoop6(GeneLoop6 * mat,DPEnvelope * dpenv);
+#define calculate_shatter_GeneLoop6 Wise2_calculate_shatter_GeneLoop6
+
+
+/* Function:  search_GeneLoop6(dbsi,out,querydb,targetdb,gp)
  *
  * Descrip:    This function makes a database search of GeneLoop6
  *             It uses the dbsi structure to choose which implementation
@@ -61,32 +112,32 @@ struct thread_pool_holder_GeneLoop6 {
  *
  * Arg:            dbsi [UNKN ] Undocumented argument [DBSearchImpl *]
  * Arg:             out [UNKN ] Undocumented argument [Hscore *]
- * Arg:           query [UNKN ] Undocumented argument [GeneWiseScore *]
+ * Arg:         querydb [UNKN ] Undocumented argument [GeneWiseDB*]
  * Arg:        targetdb [UNKN ] Undocumented argument [GenomicDB*]
  * Arg:              gp [UNKN ] Undocumented argument [GeneParser4Score *]
  *
  * Return [UNKN ]  Undocumented return value [Search_Return_Type]
  *
  */
-Search_Return_Type Wise2_search_GeneLoop6(DBSearchImpl * dbsi,Hscore * out,GeneWiseScore * query,GenomicDB* targetdb ,GeneParser4Score * gp);
+Search_Return_Type Wise2_search_GeneLoop6(DBSearchImpl * dbsi,Hscore * out,GeneWiseDB* querydb,GenomicDB* targetdb ,GeneParser4Score * gp);
 #define search_GeneLoop6 Wise2_search_GeneLoop6
 
 
-/* Function:  serial_search_GeneLoop6(out,query,targetdb,gp)
+/* Function:  serial_search_GeneLoop6(out,querydb,targetdb,gp)
  *
  * Descrip:    This function makes a database search of GeneLoop6
  *             It is a single processor implementation
  *
  *
  * Arg:             out [UNKN ] Undocumented argument [Hscore *]
- * Arg:           query [UNKN ] Undocumented argument [GeneWiseScore *]
+ * Arg:         querydb [UNKN ] Undocumented argument [GeneWiseDB*]
  * Arg:        targetdb [UNKN ] Undocumented argument [GenomicDB*]
  * Arg:              gp [UNKN ] Undocumented argument [GeneParser4Score *]
  *
  * Return [UNKN ]  Undocumented return value [Search_Return_Type]
  *
  */
-Search_Return_Type Wise2_serial_search_GeneLoop6(Hscore * out,GeneWiseScore * query,GenomicDB* targetdb ,GeneParser4Score * gp);
+Search_Return_Type Wise2_serial_search_GeneLoop6(Hscore * out,GeneWiseDB* querydb,GenomicDB* targetdb ,GeneParser4Score * gp);
 #define serial_search_GeneLoop6 Wise2_serial_search_GeneLoop6
 
 
@@ -102,7 +153,7 @@ Search_Return_Type Wise2_serial_search_GeneLoop6(Hscore * out,GeneWiseScore * qu
  *             /convert_PackAln_to_AlnBlock_GeneLoop6
  *
  *
- * Arg:         query [UNKN ] query data structure [GeneWiseScore *]
+ * Arg:         query [UNKN ] query data structure [GeneWiseScore*]
  * Arg:        target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            gp [UNKN ] Resource [GeneParser4Score *]
  * Arg:         dpenv [UNKN ] Undocumented argument [DPEnvelope *]
@@ -111,25 +162,26 @@ Search_Return_Type Wise2_serial_search_GeneLoop6(Hscore * out,GeneWiseScore * qu
  * Return [UNKN ]  Undocumented return value [PackAln *]
  *
  */
-PackAln * Wise2_PackAln_bestmemory_GeneLoop6(GeneWiseScore * query,ComplexSequence* target ,GeneParser4Score * gp,DPEnvelope * dpenv,DPRunImpl * dpri);
+PackAln * Wise2_PackAln_bestmemory_GeneLoop6(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp,DPEnvelope * dpenv,DPRunImpl * dpri);
 #define PackAln_bestmemory_GeneLoop6 Wise2_PackAln_bestmemory_GeneLoop6
 
 
-/* Function:  allocate_Expl_GeneLoop6(query,target,gp)
+/* Function:  allocate_Expl_GeneLoop6(query,target,gp,dpri)
  *
  * Descrip:    This function allocates the GeneLoop6 structure
  *             and the basematrix area for explicit memory implementations
  *             It calls /allocate_GeneLoop6_only
  *
  *
- * Arg:         query [UNKN ] query data structure [GeneWiseScore *]
+ * Arg:         query [UNKN ] query data structure [GeneWiseScore*]
  * Arg:        target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            gp [UNKN ] Resource [GeneParser4Score *]
+ * Arg:          dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [GeneLoop6 *]
  *
  */
-GeneLoop6 * Wise2_allocate_Expl_GeneLoop6(GeneWiseScore * query,ComplexSequence* target ,GeneParser4Score * gp);
+GeneLoop6 * Wise2_allocate_Expl_GeneLoop6(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp,DPRunImpl * dpri);
 #define allocate_Expl_GeneLoop6 Wise2_allocate_Expl_GeneLoop6
 
 
@@ -154,14 +206,14 @@ void Wise2_recalculate_PackAln_GeneLoop6(PackAln * pal,GeneLoop6 * mat);
  *             It calls /allocate_GeneLoop6_only
  *
  *
- * Arg:         query [UNKN ] query data structure [GeneWiseScore *]
+ * Arg:         query [UNKN ] query data structure [GeneWiseScore*]
  * Arg:        target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            gp [UNKN ] Resource [GeneParser4Score *]
  *
  * Return [UNKN ]  Undocumented return value [GeneLoop6 *]
  *
  */
-GeneLoop6 * Wise2_allocate_Small_GeneLoop6(GeneWiseScore * query,ComplexSequence* target ,GeneParser4Score * gp);
+GeneLoop6 * Wise2_allocate_Small_GeneLoop6(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp);
 #define allocate_Small_GeneLoop6 Wise2_allocate_Small_GeneLoop6
 
 
@@ -252,6 +304,21 @@ PackAln * Wise2_PackAln_read_Expl_GeneLoop6(GeneLoop6 * mat);
 #define PackAln_read_Expl_GeneLoop6 Wise2_PackAln_read_Expl_GeneLoop6
 
 
+/* Function:  PackAln_read_generic_GeneLoop6(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneLoop6 *]
+ * Arg:          h [UNKN ] Undocumented argument [GeneLoop6_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_GeneLoop6(GeneLoop6 * mat,GeneLoop6_access_func_holder h);
+#define PackAln_read_generic_GeneLoop6 Wise2_PackAln_read_generic_GeneLoop6
+
+
 /* Function:  calculate_GeneLoop6(mat)
  *
  * Descrip:    This function calculates the GeneLoop6 matrix when in explicit mode
@@ -267,6 +334,21 @@ boolean Wise2_calculate_GeneLoop6(GeneLoop6 * mat);
 #define calculate_GeneLoop6 Wise2_calculate_GeneLoop6
 
 
+/* Function:  calculate_dpenv_GeneLoop6(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneLoop6 matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] GeneLoop6 which contains explicit basematrix memory [GeneLoop6 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_GeneLoop6(GeneLoop6 * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_GeneLoop6 Wise2_calculate_dpenv_GeneLoop6
+
+
 /* Function:  GeneLoop6_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -303,9 +385,15 @@ GeneLoop6 * Wise2_free_GeneLoop6(GeneLoop6 * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
-int Wise2_score_only_GeneLoop6(GeneWiseScore * query,ComplexSequence* target ,GeneParser4Score * gp);
+int Wise2_GeneLoop6_shatter_access_main(GeneLoop6 * mat,int i,int j,int state);
+#define GeneLoop6_shatter_access_main Wise2_GeneLoop6_shatter_access_main
+int Wise2_GeneLoop6_shatter_access_special(GeneLoop6 * mat,int i,int j,int state);
+#define GeneLoop6_shatter_access_special Wise2_GeneLoop6_shatter_access_special
+void * Wise2_thread_loop_GeneLoop6(void * ptr);
+#define thread_loop_GeneLoop6 Wise2_thread_loop_GeneLoop6
+int Wise2_score_only_GeneLoop6(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp);
 #define score_only_GeneLoop6 Wise2_score_only_GeneLoop6
-GeneLoop6 * Wise2_allocate_GeneLoop6_only(GeneWiseScore * query,ComplexSequence* target ,GeneParser4Score * gp);
+GeneLoop6 * Wise2_allocate_GeneLoop6_only(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp);
 #define allocate_GeneLoop6_only Wise2_allocate_GeneLoop6_only
 void Wise2_init_GeneLoop6(GeneLoop6 * mat);
 #define init_GeneLoop6 Wise2_init_GeneLoop6
@@ -337,21 +425,25 @@ void Wise2_run_up_dc_GeneLoop6(GeneLoop6 * mat,int starti,int stopi,int startj,i
 #define run_up_dc_GeneLoop6 Wise2_run_up_dc_GeneLoop6
 void Wise2_init_dc_GeneLoop6(GeneLoop6 * mat);
 #define init_dc_GeneLoop6 Wise2_init_dc_GeneLoop6
-boolean Wise2_dc_start_end_calculate_GeneLoop6(GeneLoop6 * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_GeneLoop6 Wise2_dc_start_end_calculate_GeneLoop6
 int Wise2_start_end_find_end_GeneLoop6(GeneLoop6 * mat,int * endj);
 #define start_end_find_end_GeneLoop6 Wise2_start_end_find_end_GeneLoop6
+boolean Wise2_dc_optimised_start_end_calc_GeneLoop6(GeneLoop6 *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_GeneLoop6 Wise2_dc_optimised_start_end_calc_GeneLoop6
 void Wise2_init_start_end_linear_GeneLoop6(GeneLoop6 * mat);
 #define init_start_end_linear_GeneLoop6 Wise2_init_start_end_linear_GeneLoop6
 AlnConvertSet * Wise2_AlnConvertSet_GeneLoop6(void);
 #define AlnConvertSet_GeneLoop6 Wise2_AlnConvertSet_GeneLoop6
-int Wise2_find_end_GeneLoop6(GeneLoop6 * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_GeneLoop6_explicit_access_main(GeneLoop6 * mat,int i,int j,int state);
+#define GeneLoop6_explicit_access_main Wise2_GeneLoop6_explicit_access_main
+int Wise2_GeneLoop6_explicit_access_special(GeneLoop6 * mat,int i,int j,int state);
+#define GeneLoop6_explicit_access_special Wise2_GeneLoop6_explicit_access_special
+int Wise2_find_end_GeneLoop6(GeneLoop6 * mat,int * ri,int * rj,int * state,boolean * isspecial,GeneLoop6_access_func_holder h);
 #define find_end_GeneLoop6 Wise2_find_end_GeneLoop6
 void Wise2_GeneLoop6_debug_show_matrix(GeneLoop6 * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define GeneLoop6_debug_show_matrix Wise2_GeneLoop6_debug_show_matrix
-int Wise2_max_calc_GeneLoop6(GeneLoop6 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_GeneLoop6(GeneLoop6 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneLoop6_access_func_holder h);
 #define max_calc_GeneLoop6 Wise2_max_calc_GeneLoop6
-int Wise2_max_calc_special_GeneLoop6(GeneLoop6 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_GeneLoop6(GeneLoop6 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneLoop6_access_func_holder h);
 #define max_calc_special_GeneLoop6 Wise2_max_calc_special_GeneLoop6
 
 #ifdef _cplusplus
