@@ -69,7 +69,7 @@ GeneWiseCodonModel * GeneWiseCodonModel_from_GeneFrequencies(double * cds,GeneCo
   return out;
 }
 
-/* Function:  GeneParameter21_from_GeneModel(gm,ct,rnd_loop,cds_loop,rnd_to_model,link_loop,link_to_model)
+/* Function:  GeneParameter21_from_GeneModel(gm,ct,rnd_loop,cds_loop,rnd_to_model,link_loop,link_to_model,subs_error,indel_error)
  *
  * Descrip:    This actually makes the GeneParameter21 stuff from the
  *             new statistics
@@ -82,17 +82,20 @@ GeneWiseCodonModel * GeneWiseCodonModel_from_GeneFrequencies(double * cds,GeneCo
  * Arg:         rnd_to_model [UNKN ] Undocumented argument [Probability]
  * Arg:            link_loop [UNKN ] Undocumented argument [Probability]
  * Arg:        link_to_model [UNKN ] Undocumented argument [Probability]
+ * Arg:           subs_error [UNKN ] Undocumented argument [Probability]
+ * Arg:          indel_error [UNKN ] Undocumented argument [Probability]
  *
  * Return [UNKN ]  Undocumented return value [GeneParameter21 *]
  *
  */
 # line 127 "geneparameter.dy"
-GeneParameter21 * GeneParameter21_from_GeneModel(GeneModel * gm,CodonTable * ct,Probability rnd_loop,Probability cds_loop,Probability rnd_to_model,Probability link_loop,Probability link_to_model)
+GeneParameter21 * GeneParameter21_from_GeneModel(GeneModel * gm,CodonTable * ct,Probability rnd_loop,Probability cds_loop,Probability rnd_to_model,Probability link_loop,Probability link_to_model,Probability subs_error,Probability indel_error)
 {
   GeneParameter21 * out;
   CodonFrequency  * cf;
   RandomModelDNAScore * rmds;
   ComplexSequenceEval * cse;
+  int i;
 
   out = GeneParameter21_alloc_len(4);
 
@@ -103,6 +106,10 @@ GeneParameter21 * GeneParameter21_from_GeneModel(GeneModel * gm,CodonTable * ct,
   out->ct = hard_link_CodonTable(ct);
 
   out->gp = std_GeneParser21();
+	
+  for(i=0;i<5;i++) 
+    out->gp->central[i] = gm->rnd->base[i];
+
 
   GeneParser21_fold_in_RandomModelDNA(out->gp,gm->rnd);
 
@@ -125,10 +132,20 @@ GeneParameter21 * GeneParameter21_from_GeneModel(GeneModel * gm,CodonTable * ct,
   out->cses = new_ComplexSequenceEvalSet_from_GeneModel(gm);
 
 
+  /*** errors ***/
+
+  sprinkle_errors_over_CodonMapper(out->cm,subs_error);
+
+  add_flat_error_probabilities_GeneParser21(out->gp,indel_error);
+
+
+  fold_in_RandomModelDNA_into_RandomCodon(out->rc,gm->rnd);
+
+
   return out;
 }
 
-# line 168 "geneparameter.dy"
+# line 183 "geneparameter.dy"
 GeneParameter21 * GeneParameter21_from_GeneFrequency21(GeneFrequency21 * gf,CodonTable * ct,RandomModelDNA * rmd,Probability rnd_loop,Probability cds_loop,Probability rnd_to_model,Probability link_loop,Probability link_to_model)
 {
   GeneParameter21 * out;
@@ -240,7 +257,7 @@ GeneParameter21 * GeneParameter21_from_GeneFrequency21(GeneFrequency21 * gf,Codo
 }
 
 
-# line 240 "geneparameter.c"
+# line 257 "geneparameter.c"
 /* Function:  hard_link_GeneWiseCodonModel(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -283,6 +300,9 @@ GeneWiseCodonModel * GeneWiseCodonModel_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* in_donor[64] is an array: no default possible */ 
     /* in_acceptor[64] is an array: no default possible */ 
     /* in_cds[64] is an array: no default possible */ 
@@ -305,6 +325,7 @@ GeneWiseCodonModel * GeneWiseCodonModel_alloc(void)
  */
 GeneWiseCodonModel * free_GeneWiseCodonModel(GeneWiseCodonModel * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -313,10 +334,18 @@ GeneWiseCodonModel * free_GeneWiseCodonModel(GeneWiseCodonModel * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -570,6 +599,9 @@ GeneParameter21 * GeneParameter21_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->gp = NULL;  
     out->cm = NULL;  
     out->cses = NULL;    
@@ -599,6 +631,7 @@ GeneParameter21 * GeneParameter21_alloc(void)
  */
 GeneParameter21 * free_GeneParameter21(GeneParameter21 * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -608,10 +641,18 @@ GeneParameter21 * free_GeneParameter21(GeneParameter21 * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->gp != NULL) 
       free_GeneParser21(obj->gp);    
     if( obj->cm != NULL) 
