@@ -12,7 +12,7 @@ extern "C" {
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 88 "genestats.dy"
+# line 97 "genestats.dy"
 void show_help_GeneModelParam(FILE * ofp)
 {
 
@@ -20,7 +20,37 @@ void show_help_GeneModelParam(FILE * ofp)
   fprintf(ofp,"  -splice_max_collar      [5.0]  maximum Bits value for a splice site \n");
   fprintf(ofp,"  -splice_min_collar      [-5.0] minimum Bits value for a splice site \n");
   fprintf(ofp,"  -splice_score_offset    [%.1f]  score offset for splice sites\n",DEFAULT_SPLICE_OFFSET_SCORE);
+  fprintf(ofp,"  -[no]splice_gtag        make just gtag splice sites (default is gtag, ie no model)\n");  
+  fprintf(ofp,"  -splice_gtag_prob       [0.001] probability for gt/ag \n");
   fprintf(ofp,"  -genestats              [gene.stat]\n");
+
+}
+
+/* Function:  show_info_GeneModelParam(p,ofp)
+ *
+ * Descrip:    Shows genemodel param for info
+ *
+ *
+ * Arg:          p [UNKN ] Undocumented argument [GeneModelParam *]
+ * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
+ *
+ */
+# line 113 "genestats.dy"
+void show_info_GeneModelParam(GeneModelParam * p,FILE * ofp)
+{
+  fprintf(ofp,"Gene Parameter file: %s\n",p->gene_stats_file);
+  fprintf(ofp,"Splice site model:   %s\n",p->use_gtag_splice == TRUE ? "GT/AG only" : "Position Weight Matrix");
+  if( p->use_gtag_splice == FALSE ) {
+    fprintf(ofp,"Pseudo counts 5'SS   %1.2f\n",p->splice5_pseudo);
+    fprintf(ofp,"Pseudo counts 3'SS   %1.2f\n",p->splice3_pseudo);
+    fprintf(ofp,"Minimum SS collar    %2.2f\n",p->min_collar);
+    fprintf(ofp,"Maximum SS collar    %2.2f\n",p->max_collar);
+    fprintf(ofp,"Splice site offset   %2.2f\n",p->score_offset);
+  } else {
+
+    fprintf(ofp,"GT/AG bits penalty   %2.2f\n",Probability2Bits(p->prob_for_gtag));
+  }
+  
 }
 
 /* Function:  new_GeneModelParam_from_argv(argc,argv)
@@ -34,7 +64,7 @@ void show_help_GeneModelParam(FILE * ofp)
  * Return [UNKN ]  Undocumented return value [GeneModelParam *]
  *
  */
-# line 101 "genestats.dy"
+# line 133 "genestats.dy"
 GeneModelParam * new_GeneModelParam_from_argv(int * argc,char ** argv)
 {
   GeneModelParam * out;
@@ -50,6 +80,8 @@ GeneModelParam * new_GeneModelParam_from_argv(int * argc,char ** argv)
     } 
   }
 
+  strip_out_boolean_def_argument(argc,argv,"splice_gtag",&out->use_gtag_splice);
+
   if( (temp=strip_out_assigned_argument(argc,argv,"splice_max_collar")) != NULL ) {
     if( is_double_string(temp,&out->max_collar) == FALSE ) {
       warn("%s is not a floating point number. Can't be a splice_max_collar",temp);
@@ -74,10 +106,146 @@ GeneModelParam * new_GeneModelParam_from_argv(int * argc,char ** argv)
     out->gene_stats_file = stringalloc(temp);
   }
 
+  if( (temp=strip_out_assigned_argument(argc,argv,"splice_gtag_prob")) != NULL ) {
+    if( is_double_string(temp,&out->prob_for_gtag) == FALSE ) {
+      warn("%s is not a floating pointer number. Can't be a probability for gtag",temp);
+      free_GeneModelParam(out);
+      return NULL;
+    }
+  }
+
+
   return out;
 
 }
 
+/* Function:  vanilla_GeneralGeneModelScore(ct,start_odds,general_odds,stop_odds)
+ *
+ * Descrip:    Makes a vanilla general gene model score
+ *
+ *
+ * Arg:                  ct [UNKN ] Undocumented argument [CodonTable *]
+ * Arg:          start_odds [UNKN ] Undocumented argument [Probability]
+ * Arg:        general_odds [UNKN ] Undocumented argument [Probability]
+ * Arg:           stop_odds [UNKN ] Undocumented argument [Probability]
+ *
+ * Return [UNKN ]  Undocumented return value [GeneralGeneModelScore *]
+ *
+ */
+# line 190 "genestats.dy"
+GeneralGeneModelScore * vanilla_GeneralGeneModelScore(CodonTable * ct,Probability start_odds,Probability general_odds,Probability stop_odds)
+{
+  GeneralGeneModelScore * out;
+  RandomCodon * temp;
+  out = GeneralGeneModelScore_alloc();
+
+  temp = vanilla_start_RandomCodon(ct,start_odds);
+  out->start = RandomCodonScore_from_RandomCodon(temp);
+  free_RandomCodon(temp);
+
+  temp  = vanilla_stop_RandomCodon(ct,stop_odds);
+  out->stop = RandomCodonScore_from_RandomCodon(temp);
+  free_RandomCodon(temp);
+
+  temp  = vanilla_general_RandomCodon(ct,general_odds);
+  out->general = RandomCodonScore_from_RandomCodon(temp);
+  free_RandomCodon(temp);
+
+  return out;
+}
+
+/* Function:  vanilla_start_RandomCodon(ct,start_codon_odd_prob)
+ *
+ * Descrip:    Makes a vanilla (ie, all things equal) Met based
+ *             start codon rndscore
+ *
+ *
+ * Arg:                          ct [UNKN ] Undocumented argument [CodonTable *]
+ * Arg:        start_codon_odd_prob [UNKN ] Undocumented argument [Probability]
+ *
+ * Return [UNKN ]  Undocumented return value [RandomCodon *]
+ *
+ */
+# line 215 "genestats.dy"
+RandomCodon * vanilla_start_RandomCodon(CodonTable * ct,Probability start_codon_odd_prob)
+{
+  int i;
+  RandomCodon * out;
+
+  out = RandomCodon_alloc();
+  
+  for(i=0;i<125;i++) {
+    if( aminoacid_from_codon(ct,i) == 'M' ) {
+      out->codon[i] = start_codon_odd_prob;
+    } else {
+      out->codon[i] = 0.0;
+    }
+  }
+
+  return out;
+}
+
+/* Function:  vanilla_stop_RandomCodon(ct,stop_codon_odd_prob)
+ *
+ * Descrip:    Makes a vanilla (ie, all things equal) * based
+ *             stop codon rndscore
+ *
+ *
+ * Arg:                         ct [UNKN ] Undocumented argument [CodonTable *]
+ * Arg:        stop_codon_odd_prob [UNKN ] Undocumented argument [Probability]
+ *
+ * Return [UNKN ]  Undocumented return value [RandomCodon *]
+ *
+ */
+# line 237 "genestats.dy"
+RandomCodon * vanilla_stop_RandomCodon(CodonTable * ct,Probability stop_codon_odd_prob)
+{
+  int i;
+  RandomCodon * out;
+
+  out = RandomCodon_alloc();
+  
+  for(i=0;i<125;i++) {
+    if( aminoacid_from_codon(ct,i) == 'X' ) {
+      out->codon[i] = stop_codon_odd_prob;
+    } else {
+      out->codon[i] = 0.0;
+    }
+  }
+  return out;
+}
+
+/* Function:  vanilla_general_RandomCodon(ct,coding_odd_prob)
+ *
+ * Descrip:    Makes a vanilla (ie, all things equal) general non-stop
+ *
+ *
+ * Arg:                     ct [UNKN ] Undocumented argument [CodonTable *]
+ * Arg:        coding_odd_prob [UNKN ] Undocumented argument [Probability]
+ *
+ * Return [UNKN ]  Undocumented return value [RandomCodon *]
+ *
+ */
+# line 257 "genestats.dy"
+RandomCodon * vanilla_general_RandomCodon(CodonTable * ct,Probability coding_odd_prob)
+{
+  int i;
+  RandomCodon * out;
+
+  out = RandomCodon_alloc();
+  
+  for(i=0;i<125;i++) {
+    if( aminoacid_from_codon(ct,i) == 'X' ) {
+      out->codon[i] = 0.0;
+    } else {
+      out->codon[i] = coding_odd_prob;
+    }
+  }
+
+  return out;
+}
+
+
 /* Function:  std_GeneModelParam(void)
  *
  * Descrip:    Makes a standard GeneModelParam
@@ -87,7 +255,7 @@ GeneModelParam * new_GeneModelParam_from_argv(int * argc,char ** argv)
  * Return [UNKN ]  Undocumented return value [GeneModelParam *]
  *
  */
-# line 147 "genestats.dy"
+# line 279 "genestats.dy"
 GeneModelParam * std_GeneModelParam(void)
 {
   GeneModelParam * out;
@@ -103,6 +271,9 @@ GeneModelParam * std_GeneModelParam(void)
   out->max_collar   = +5.0;
   out->score_offset = DEFAULT_SPLICE_OFFSET_SCORE;
   out->gene_stats_file = stringalloc("gene.stat");
+  out->use_gtag_splice = TRUE;
+
+  out->prob_for_gtag = 0.001;
 
   return out;
 }
@@ -117,7 +288,7 @@ GeneModelParam * std_GeneModelParam(void)
  * Return [UNKN ]  Undocumented return value [GeneModel *]
  *
  */
-# line 169 "genestats.dy"
+# line 304 "genestats.dy"
 GeneModel * GeneModel_from_GeneModelParam(GeneModelParam * p)
 {
   GeneStats * st;
@@ -127,8 +298,12 @@ GeneModel * GeneModel_from_GeneModelParam(GeneModelParam * p)
 
   st = GeneStats_from_GeneModelParam(p);
 
+  assert(st);
+
   out = GeneModel_from_GeneStats(st,p);
 
+  assert(out);
+
   free_GeneStats(st);
 
   return out;
@@ -145,7 +320,7 @@ GeneModel * GeneModel_from_GeneModelParam(GeneModelParam * p)
  * Return [UNKN ]  Undocumented return value [GeneStats *]
  *
  */
-# line 189 "genestats.dy"
+# line 328 "genestats.dy"
 GeneStats * GeneStats_from_GeneModelParam(GeneModelParam * p)
 {
   GeneStats * gs;
@@ -177,7 +352,7 @@ GeneStats * GeneStats_from_GeneModelParam(GeneModelParam * p)
  * Return [UNKN ]  Undocumented return value [GeneModel *]
  *
  */
-# line 212 "genestats.dy"
+# line 351 "genestats.dy"
 GeneModel * GeneModel_from_GeneStats(GeneStats * gs,GeneModelParam * p)
 {
   GeneModel * out;
@@ -198,6 +373,8 @@ GeneModel * GeneModel_from_GeneStats(GeneStats * gs,GeneModelParam * p)
   }
   
   out->splice5 = pwmDNA_from_SeqAlign(gs->splice5,p->splice5_pseudo);
+  
+  /*  fprintf(stdout,"GS splice5 %d splice3 %d\n",gs->splice5,gs->splice3);*/
 
   fold_randommodel_pwmDNA(out->splice5,gs->rnd);
 
@@ -211,6 +388,7 @@ GeneModel * GeneModel_from_GeneStats(GeneStats * gs,GeneModelParam * p)
 
   out->splice3 = pwmDNA_from_SeqAlign(gs->splice3,p->splice3_pseudo);
   fold_randommodel_pwmDNA(out->splice3,gs->rnd);
+
   out->splice3score = SpliceSiteScore_alloc();
   out->splice3score->score = pwmDNAScore_from_pwmDNA(out->splice3);
   out->splice3score->offset = gs->splice3_offset;
@@ -218,7 +396,8 @@ GeneModel * GeneModel_from_GeneStats(GeneStats * gs,GeneModelParam * p)
   out->splice3score->max_collar   = Probability2Score(Bits2Probability(p->max_collar));
   out->splice3score->score_offset = Probability2Score(Bits2Probability(p->score_offset));
 
-
+  out->use_gtag_splice = p->use_gtag_splice;
+  out->score_for_gtag  = Probability2Score(p->prob_for_gtag);
 
   out->intron  = RandomModelDNA_alloc();
   for(total = 0.0,i=0;i<4;i++)
@@ -252,7 +431,7 @@ GeneModel * GeneModel_from_GeneStats(GeneStats * gs,GeneModelParam * p)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 280 "genestats.dy"
+# line 423 "genestats.dy"
 void show_GeneModel(GeneModel * gm,FILE * ofp)
 {
   fprintf(ofp,"Splice5\n");
@@ -271,7 +450,7 @@ void show_GeneModel(GeneModel * gm,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [ComplexSequenceEvalSet *]
  *
  */
-# line 291 "genestats.dy"
+# line 434 "genestats.dy"
 ComplexSequenceEvalSet * new_ComplexSequenceEvalSet_from_GeneModel(GeneModel * gm)
 {
   ComplexSequenceEvalSet * out;
@@ -284,8 +463,14 @@ ComplexSequenceEvalSet * new_ComplexSequenceEvalSet_from_GeneModel(GeneModel * g
 
   add_ComplexSequenceEvalSet(out,base_number_ComplexSequenceEval());
   add_ComplexSequenceEvalSet(out,codon_number_ComplexSequenceEval());
-  add_ComplexSequenceEvalSet(out,ComplexSequenceEval_from_pwmDNAScore_splice(gm->splice5score));
-  add_ComplexSequenceEvalSet(out,ComplexSequenceEval_from_pwmDNAScore_splice(gm->splice3score));
+  if( gm->use_gtag_splice == FALSE ) {
+    add_ComplexSequenceEvalSet(out,ComplexSequenceEval_from_pwmDNAScore_splice(gm->splice5score));
+    add_ComplexSequenceEvalSet(out,ComplexSequenceEval_from_pwmDNAScore_splice(gm->splice3score));
+  } else {
+    add_ComplexSequenceEvalSet(out,ComplexSequenceEval_for_scored_gt(&gm->score_for_gtag));
+    add_ComplexSequenceEvalSet(out,ComplexSequenceEval_for_scored_ag(&gm->score_for_gtag));
+  }
+
   add_ComplexSequenceEvalSet(out,flat_zero());
   add_ComplexSequenceEvalSet(out,flat_zero());
 
@@ -298,6 +483,107 @@ ComplexSequenceEvalSet * new_ComplexSequenceEvalSet_from_GeneModel(GeneModel * g
 }
 
 
+/* Function:  ComplexSequenceEval_for_scored_ag(score_for_ag)
+ *
+ * Descrip:    Makes a ComplexSequenceEval for a GT rule
+ *
+ *
+ * Arg:        score_for_ag [UNKN ] Undocumented argument [Score *]
+ *
+ * Return [UNKN ]  Undocumented return value [ComplexSequenceEval *]
+ *
+ */
+# line 470 "genestats.dy"
+ComplexSequenceEval * ComplexSequenceEval_for_scored_ag(Score * score_for_ag)
+{
+  ComplexSequenceEval * out;
+
+  out = ComplexSequenceEval_alloc();
+
+  out->left_window   = 3;
+  out->right_window  = 3;
+  out->left_lookback = 5;
+  out->outside_score = NEGI;
+
+  out->data = (void*) score_for_ag;
+  out->type = SEQUENCE_GENOMIC;
+  out->eval_func = scored_ag_eval_func;
+  out->score_type = CseScoreType_Bits;
+
+  return out;
+}
+
+/* Function:  scored_ag_eval_func(type,*data,seq)
+ *
+ * Descrip:    Function which actually does the evaluation for scored doners
+ *
+ *
+ * Arg:         type [UNKN ] Undocumented argument [int]
+ * Arg:        *data [UNKN ] Undocumented argument [void]
+ * Arg:          seq [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [int]
+ *
+ */
+# line 494 "genestats.dy"
+int scored_ag_eval_func(int type,void *data,char * seq)
+{
+  if( *(seq-1) == 'A' && *(seq) == 'G' ) 
+    return *(Score *)data;
+  else return NEGI;
+}
+
+/* Function:  ComplexSequenceEval_for_scored_gt(score_for_gt)
+ *
+ * Descrip:    Makes a ComplexSequenceEval for a GT rule
+ *
+ *
+ * Arg:        score_for_gt [UNKN ] Undocumented argument [Score *]
+ *
+ * Return [UNKN ]  Undocumented return value [ComplexSequenceEval *]
+ *
+ */
+# line 505 "genestats.dy"
+ComplexSequenceEval * ComplexSequenceEval_for_scored_gt(Score * score_for_gt)
+{
+  ComplexSequenceEval * out;
+
+  out = ComplexSequenceEval_alloc();
+
+  out->left_window   = 3;
+  out->right_window  = 7;
+  out->left_lookback = 8;
+  out->outside_score = NEGI;
+
+  out->data = (void*) score_for_gt;
+  out->type = SEQUENCE_GENOMIC;
+  out->eval_func = scored_gt_eval_func;
+  out->score_type = CseScoreType_Bits;
+
+  return out;
+}
+
+/* Function:  scored_gt_eval_func(type,*data,seq)
+ *
+ * Descrip:    Function which actually does the evaluation for scored doners
+ *
+ *
+ * Arg:         type [UNKN ] Undocumented argument [int]
+ * Arg:        *data [UNKN ] Undocumented argument [void]
+ * Arg:          seq [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [int]
+ *
+ */
+# line 529 "genestats.dy"
+int scored_gt_eval_func(int type,void *data,char * seq)
+{
+  if( *(seq) == 'G' && *(seq+1) == 'T' ) 
+    return *(Score *)data;
+  else return NEGI;
+}
+
+
 /* Function:  ComplexSequenceEval_from_pwmDNAScore_splice(score)
  *
  * Descrip:    Makes a ComplexSequenceEval for a splice site
@@ -309,11 +595,14 @@ ComplexSequenceEvalSet * new_ComplexSequenceEvalSet_from_GeneModel(GeneModel * g
  * Return [UNKN ]  Undocumented return value [ComplexSequenceEval *]
  *
  */
-# line 322 "genestats.dy"
+# line 542 "genestats.dy"
 ComplexSequenceEval * ComplexSequenceEval_from_pwmDNAScore_splice(SpliceSiteScore * score)
 {
   ComplexSequenceEval * out;
 
+
+  /*  printf("Making CSE from %d\n",score);*/
+
   out = ComplexSequenceEval_alloc();
 
   /* shouldn't really add ones, but this is ok anyway. 
@@ -325,9 +614,9 @@ ComplexSequenceEval * ComplexSequenceEval_from_pwmDNAScore_splice(SpliceSiteScor
    *STILL don't know precisely what is going on down here! ***/
 
   /*  out->left_window  = ssm->offset + ssm->pre_splice_site +1; */
-  out->left_window  = 10;
+  out->left_window   =10;
   /*  out->right_window = ssm->offset + ssm->post_splice_site +1; */
-  out->right_window = 10;
+  out->right_window  =10;
   out->left_lookback =10;
 
   out->outside_score= NEGI;
@@ -354,7 +643,7 @@ ComplexSequenceEval * ComplexSequenceEval_from_pwmDNAScore_splice(SpliceSiteScor
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 358 "genestats.dy"
+# line 581 "genestats.dy"
 int pwmDNA_splice_ComplexSequence_eval_func(int type,void * data,char * seq)
 {
   SpliceSiteScore * sc;
@@ -369,9 +658,16 @@ int pwmDNA_splice_ComplexSequence_eval_func(int type,void * data,char * seq)
   /* no idea what is happening here, but it works ;) */
 
 
+  if( seq[0] == 'N' && seq[1] == 'N' && seq[2] == 'N' ) {
+    return NEGI;
+  }
 
   score = score_pwmDNAScore_string(pds,seq-sc->offset+1);
 
+/*  printf("Offset is %d %c%c%c\n",sc->offset,seq[0],seq[1],seq[2]);*/
+
+/*  fprintf(stdout,"Before collaring, %d\n",score);*/
+
   if( score < sc->min_collar ) {
     score = sc->min_collar;
   }
@@ -379,8 +675,12 @@ int pwmDNA_splice_ComplexSequence_eval_func(int type,void * data,char * seq)
     score = sc->max_collar;
   }
 
+/*  fprintf(stdout,"Score %d before offset\n",score);*/
+
   score -= sc->score_offset;
 
+/*  fprintf(stdout,"Score %d after offset\n",score);*/
+
   /*  fprintf(stderr,"Scoring %d at some position\n",score);*/
 
   return score;
@@ -396,7 +696,7 @@ int pwmDNA_splice_ComplexSequence_eval_func(int type,void * data,char * seq)
  * Return [UNKN ]  Undocumented return value [GeneStats *]
  *
  */
-# line 392 "genestats.dy"
+# line 626 "genestats.dy"
 GeneStats * read_GeneStats(FILE * ifp)
 {
   char buffer[MAXLINE];
@@ -406,9 +706,10 @@ GeneStats * read_GeneStats(FILE * ifp)
   char **brk;
 
   out = GeneStats_alloc();
+  out->rnd = NULL;
 
   while( fgets(buffer,MAXLINE,ifp) != NULL ) {
-    fprintf(stderr,"Reading %s",buffer);
+    /*    fprintf(stderr,"Reading (main loop) %s",buffer); */
     if( buffer[0] == '#' )
       continue;
     
@@ -427,11 +728,13 @@ GeneStats * read_GeneStats(FILE * ifp)
 	warn("Could not read in selex alignment for splice5");
 	continue;
       }
+
       out->splice5 = temp;
       continue;
     }
 
     if( strstartcmp(buffer,"splice3") == 0 ) {
+
       base = brk = breakstring(buffer,spacestr);
       if( *brk == NULL || *(brk+1) == NULL || is_integer_string(*(brk+1),&out->splice3_offset) == 0) {
 	warn("Cannot read splice3 offset - must be splice3 <number>");
@@ -444,7 +747,9 @@ GeneStats * read_GeneStats(FILE * ifp)
 	warn("Could not read in selex alignment for splice5");
 	continue;
       }
+
       out->splice3 = temp;
+
       continue;
     }
     
@@ -501,8 +806,11 @@ GeneStats * read_GeneStats(FILE * ifp)
   
   }
 
-  fprintf(stderr,"About to leave function\n");
-
+  
+  assert(out);
+  assert(out->splice5);
+  assert(out->splice3);
+	
   return out;
 }
 
@@ -516,7 +824,7 @@ GeneStats * read_GeneStats(FILE * ifp)
  * Return [UNKN ]  Undocumented return value [RandomModelDNA *]
  *
  */
-# line 505 "genestats.dy"
+# line 747 "genestats.dy"
 RandomModelDNA * get_genestat_emission(char * buffer)
 {
   RandomModelDNA * out;
@@ -529,7 +837,7 @@ RandomModelDNA * get_genestat_emission(char * buffer)
 
   base = brk = breakstring(buffer,spacestr);
 
-  for(i=0;*brk != NULL && i < 4;i++, brk++){
+  for(i=0;*brk != NULL && i < 5;i++, brk++){
     if( is_double_string(*brk,out->base+i) == FALSE) {
       warn("For genestat word %s, not a double in emission!",*brk);
       return FALSE;
@@ -554,7 +862,7 @@ RandomModelDNA * get_genestat_emission(char * buffer)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 536 "genestats.dy"
+# line 778 "genestats.dy"
 void dump_GeneStats(GeneStats * st,FILE * ofp)
 {
   int i;
@@ -605,7 +913,7 @@ void dump_GeneStats(GeneStats * st,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 577 "genestats.dy"
+# line 819 "genestats.dy"
 boolean read_codon_GeneStats(double * codon_array,char* line,FILE * ifp)
 {
   boolean ret = TRUE;
@@ -659,7 +967,7 @@ boolean read_codon_GeneStats(double * codon_array,char* line,FILE * ifp)
 }
 
 
-# line 633 "genestats.c"
+# line 923 "genestats.c"
 /* Function:  hard_link_SpliceSiteScore(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -702,6 +1010,9 @@ SpliceSiteScore * SpliceSiteScore_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->score = NULL;   
     out->offset = 0; 
     out->min_collar = 0; 
@@ -726,6 +1037,7 @@ SpliceSiteScore * SpliceSiteScore_alloc(void)
  */
 SpliceSiteScore * free_SpliceSiteScore(SpliceSiteScore * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -734,10 +1046,18 @@ SpliceSiteScore * free_SpliceSiteScore(SpliceSiteScore * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->score != NULL)  
       free_pwmDNAScore(obj->score);  
 
@@ -789,6 +1109,9 @@ GeneStats * GeneStats_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->splice5 = NULL; 
     out->splice5_offset = 0; 
     out->splice3 = NULL; 
@@ -818,6 +1141,7 @@ GeneStats * GeneStats_alloc(void)
  */
 GeneStats * free_GeneStats(GeneStats * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -826,10 +1150,18 @@ GeneStats * free_GeneStats(GeneStats * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->splice5 != NULL)    
       free_SeqAlign(obj->splice5);   
     if( obj->splice3 != NULL)    
@@ -889,6 +1221,9 @@ GeneModelParam * GeneModelParam_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->splice5_pseudo = 0; 
     out->splice3_pseudo = 0; 
     out->intron_emission_pseudo = 0; 
@@ -897,6 +1232,8 @@ GeneModelParam * GeneModelParam_alloc(void)
     out->max_collar = 0; 
     out->score_offset = 0;   
     out->gene_stats_file = NULL; 
+    out->use_gtag_splice = FALSE;    
+    out->prob_for_gtag = 0;  
 
 
     return out;  
@@ -916,6 +1253,7 @@ GeneModelParam * GeneModelParam_alloc(void)
  */
 GeneModelParam * free_GeneModelParam(GeneModelParam * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -924,10 +1262,18 @@ GeneModelParam * free_GeneModelParam(GeneModelParam * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->gene_stats_file != NULL)    
       ckfree(obj->gene_stats_file);  
 
@@ -979,6 +1325,9 @@ GeneModel * GeneModel_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->splice5 = NULL; 
     out->splice5_offset = 0; 
     out->splice3 = NULL; 
@@ -991,6 +1340,8 @@ GeneModel * GeneModel_alloc(void)
     out->splice5score = NULL;    
     out->splice3score = NULL;    
     /* codon[64] is an array: no default possible */ 
+    out->use_gtag_splice = FALSE;    
+    out->score_for_gtag = 0; 
 
 
     return out;  
@@ -1010,6 +1361,7 @@ GeneModel * GeneModel_alloc(void)
  */
 GeneModel * free_GeneModel(GeneModel * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1018,10 +1370,18 @@ GeneModel * free_GeneModel(GeneModel * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->splice5 != NULL)    
       free_pwmDNA(obj->splice5);     
     if( obj->splice3 != NULL)    
@@ -1043,6 +1403,107 @@ GeneModel * free_GeneModel(GeneModel * obj)
 }    
 
 
+/* Function:  hard_link_GeneralGeneModelScore(obj)
+ *
+ * Descrip:    Bumps up the reference count of the object
+ *             Meaning that multiple pointers can 'own' it
+ *
+ *
+ * Arg:        obj [UNKN ] Object to be hard linked [GeneralGeneModelScore *]
+ *
+ * Return [UNKN ]  Undocumented return value [GeneralGeneModelScore *]
+ *
+ */
+GeneralGeneModelScore * hard_link_GeneralGeneModelScore(GeneralGeneModelScore * obj) 
+{
+    if( obj == NULL )    {  
+      warn("Trying to hard link to a GeneralGeneModelScore object: passed a NULL object");   
+      return NULL;   
+      }  
+    obj->dynamite_hard_link++;   
+    return obj;  
+}    
+
+
+/* Function:  GeneralGeneModelScore_alloc(void)
+ *
+ * Descrip:    Allocates structure: assigns defaults if given 
+ *
+ *
+ *
+ * Return [UNKN ]  Undocumented return value [GeneralGeneModelScore *]
+ *
+ */
+GeneralGeneModelScore * GeneralGeneModelScore_alloc(void) 
+{
+    GeneralGeneModelScore * out;/* out is exported at end of function */ 
+
+
+    /* call ckalloc and see if NULL */ 
+    if((out=(GeneralGeneModelScore *) ckalloc (sizeof(GeneralGeneModelScore))) == NULL)  {  
+      warn("GeneralGeneModelScore_alloc failed ");   
+      return NULL;  /* calling function should respond! */ 
+      }  
+    out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
+    out->start = NULL;   
+    out->stop = NULL;    
+    out->general = NULL; 
+
+
+    return out;  
+}    
+
+
+/* Function:  free_GeneralGeneModelScore(obj)
+ *
+ * Descrip:    Free Function: removes the memory held by obj
+ *             Will chain up to owned members and clear all lists
+ *
+ *
+ * Arg:        obj [UNKN ] Object that is free'd [GeneralGeneModelScore *]
+ *
+ * Return [UNKN ]  Undocumented return value [GeneralGeneModelScore *]
+ *
+ */
+GeneralGeneModelScore * free_GeneralGeneModelScore(GeneralGeneModelScore * obj) 
+{
+    int return_early = 0;    
+
+
+    if( obj == NULL) {  
+      warn("Attempting to free a NULL pointer to a GeneralGeneModelScore obj. Should be trappable"); 
+      return NULL;   
+      }  
+
+
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
+    if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
+      obj->dynamite_hard_link--; 
+      }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
+    if( obj->start != NULL)  
+      free_RandomCodonScore(obj->start);     
+    if( obj->stop != NULL)   
+      free_RandomCodonScore(obj->stop);  
+    if( obj->general != NULL)    
+      free_RandomCodonScore(obj->general);   
+
+
+    ckfree(obj); 
+    return NULL; 
+}    
+
+
 
 #ifdef _cplusplus
 }
