@@ -17,6 +17,9 @@ char * program_name = "genewise";
 #include "genedisplay.h"
 #include "matchsum.h"
 
+#include "phasemodel.h"
+#include "genephase6.h"
+
 
 /*
  * program specific variables
@@ -38,6 +41,7 @@ boolean do_both      = FALSE;
 
 boolean use_tsm      = FALSE;
 
+
 char * protein_file  = NULL;
 Protein * pro        = NULL;
 
@@ -46,7 +50,6 @@ ThreeStateModel * tsm = NULL;
 char * hmm_name       = NULL;
 
 char * potential_file = NULL;
-PotentialGene * pg    = NULL;
 
 char * qstart_str    = NULL;
 int qstart           = -1;
@@ -62,7 +65,10 @@ GeneStats * gs          = NULL;
 
 GeneModelParam * gmp    = NULL;
 
-boolean use_new_stats   = 0;
+PhasedProteinPara * ppp  = NULL;
+PhasedProtein * pp      = NULL;
+
+boolean use_new_stats   = 1;
 
 char * matrix_file      = NULL;
 CompMat * mat           = NULL;
@@ -73,8 +79,8 @@ int gap                 = 12;
 char * ext_str          = "2";
 int ext                 = 2;
 
-char * length_of_N_str  = "10";
-int length_of_N         = 10;
+char * length_of_N_str  = "0";
+int length_of_N         = 0;
 
 boolean flat_insert     = TRUE;
 
@@ -89,11 +95,11 @@ RandomModelDNA * rmd    = NULL;
 char * allN_string      = "1.0";
 Probability allN        = 1.0;
 
-char * subs_string      = "0.00001";
-double subs_error       = 0.00001;
+char * subs_string      = "0.000001";
+double subs_error       = 0.000001;
 
-char * indel_string      = "0.00001";
-double indel_error       = 0.00001;
+char * indel_string      = "0.000001";
+double indel_error       = 0.000001;
 
 char * cfreq_string      = "flat";
 boolean model_codon      = FALSE;
@@ -102,7 +108,7 @@ char * splice_string     = "model";
 boolean model_splice     = TRUE;
 
 char * startend_string   = "default";
-int startend             = TSM_default;
+TSM_StartEndMode startend   = TSM_default;
 
 char * null_string       = "syn";
 boolean use_syn          = FALSE;
@@ -116,6 +122,8 @@ char * alg_str           = NULL;
 char * kbyte_str         = NULL;
 int kbyte                = 10000; /* will be reset in build_defaults */
 
+char * pal_file          = NULL;
+
 boolean show_PackAln     = FALSE;
 boolean show_cumlative_PackAln = FALSE;
 boolean show_AlnBlock    = FALSE;
@@ -159,6 +167,8 @@ RandomModel * rm;
 
 DPRunImpl * dpri;
 
+GeneWiseRunPara * gwrp;
+
 void show_parameters(void)
 {
   fprintf(ofp,"%s %s (%s release)\n",program_name,VERSION_NUMBER,RELEASE_DAY);
@@ -176,13 +186,11 @@ void show_parameters(void)
   fprintf(ofp,"Target Sequence      %s\n",gen->baseseq->name);
   fprintf(ofp,"Strand:              %s\n",do_both == TRUE ? "both" : reverse == TRUE ? "reverse" : "forward");
   fprintf(ofp,"Start/End (protein)  %s\n",startend_string);
-  fprintf(ofp,"Gene Paras:          %s\n",gene_file);
+  show_info_GeneModelParam(gmp,ofp);
+
   fprintf(ofp,"Codon Table:         %s\n",codon_file);
   fprintf(ofp,"Subs error:          %2.2g\n",subs_error);
   fprintf(ofp,"Indel error:         %2.2g\n",indel_error);
-  fprintf(ofp,"Model splice?        %s\n",splice_string);
-  fprintf(ofp,"Model codon bias?    %s\n",cfreq_string);
-  fprintf(ofp,"Model intron bias?   %s\n",intron_string);
   fprintf(ofp,"Null model           %s\n",null_string);
   fprintf(ofp,"Algorithm            %s\n",alg_str);
 }
@@ -343,14 +351,17 @@ boolean build_alignment(void)
   GeneParameter21 * gpara;
   GeneModel * gm;
 
-  int is_global;
+  if( pal_file != NULL ) {
+    pal = read_simple_PackAln_file(pal_file);
+  }
+
 
   if( use_new_stats == 0 ) {
     gpara = GeneParameter21_wrap(gf,subs_error,indel_error,rmd,model_codon,model_splice,use_tied_model,ct,rnd_loop,cds_loop,rnd_to_model,link_loop,link_to_model);
   } else {
     gm = GeneModel_from_GeneStats(gs,gmp);
     
-    gpara= GeneParameter21_from_GeneModel(gm,ct,rnd_loop,cds_loop,rnd_to_model,link_loop,link_to_model);
+    gpara= GeneParameter21_from_GeneModel(gm,ct,rnd_loop,cds_loop,rnd_to_model,link_loop,link_to_model,subs_error,indel_error);
   }
   
 
@@ -359,22 +370,17 @@ boolean build_alignment(void)
     return FALSE;
   }
 
-  if( use_tsm == FALSE ) {
+  
+  /* phased based proteins use a different source */
 
-    /* Dirty hack to pass in the ability now to
-       set startend to TSM_endbiased */
-
-    if( startend == TSM_endbiased ) {
-      is_global = TSM_endbiased;
-    } else if ( startend == TSM_global ) {
-      is_global = 1;
-    } else {
-      is_global = 0;
-    }
-
-    alb =  AlnBlock_from_protein_genewise_wrap(pro,gen,mat,-gap,-ext,gpara,rmd,rmd,alg,is_global,use_syn,rm,allN,pg,dpri,&pal);
+  if( alg == GWWRAP_623P ) {
+    alb = AlnBlock_from_phased_protein_wrap(pro,tsm,gen,gpara->cm,rm,mat,ppp,gpara,dpri,&pal);
   } else {
-    alb =  AlnBlock_from_TSM_genewise_wrap(tsm,gen,gpara,rmd,rmd,use_syn,alg,allN,flat_insert,pg,dpri,&pal);
+    if( use_tsm == FALSE) {
+      alb =  AlnBlock_from_protein_genewise_wrap(pro,gen,mat,-gap,-ext,gpara,rmd,rmd,alg,use_syn,rm,allN,startend,dpri,&pal,gwrp);
+    } else {
+      alb =  AlnBlock_from_TSM_genewise_wrap(tsm,gen,gpara,rmd,rmd,use_syn,alg,allN,flat_insert,dpri,&pal,NULL);
+    }
   }
 
   free_GeneParameter21(gpara);
@@ -417,7 +423,9 @@ boolean free_io_objects(void)
   }
 
   free_CodonTable(ct);
-  free_GeneFrequency21(gf);
+  if( gf != NULL ) {
+    free_GeneFrequency21(gf);
+  }
   free_RandomModelDNA(rmd);
   if( is_embl ) {
     free_GenomicRegion(embl);
@@ -439,6 +447,8 @@ boolean build_objects(void)
 
 
 
+
+
   startend = threestatemodel_mode_from_string(startend_string);
   if( startend == TSM_unknown ) {
     warn("String %s was unable to converted into a start/end policy\n",startend_string);
@@ -490,12 +500,12 @@ boolean build_objects(void)
   }
 
   if( gen != NULL ) {
+
     if( tstart != -1 || tend != -1 ) {
       if( tstart == -1 )
 	tstart = 0;
       if( tend == -1 ) 
 	tend = gen->baseseq->len;
-      
       gen_temp = truncate_Genomic(gen,tstart-1,tend);
       if( gen_temp == NULL ){
 	ret = FALSE;
@@ -503,6 +513,8 @@ boolean build_objects(void)
 	free_Genomic(gen);
 	gen = gen_temp;
       }
+    } else {
+      /* no truncation required */
     }
   
 
@@ -541,6 +553,17 @@ boolean build_objects(void)
     gen->baseseq->offset = 1;
     gen->baseseq->end  = strlen(gen->baseseq->seq);
   }
+
+  if( alg_str != NULL ) {
+    alg = gwrap_alg_type_from_string(alg_str);
+  } else {
+    if( use_tsm == TRUE ) {
+      alg_str = "623L";
+    } else {
+      alg_str = "623";
+    }
+    alg = gwrap_alg_type_from_string(alg_str);
+  }
       
 
   if( qstart_str != NULL ) {
@@ -558,7 +581,6 @@ boolean build_objects(void)
   }
 
 
-
   if( use_tsm == FALSE ) {
     if( startend != TSM_default && startend != TSM_global && startend != TSM_local && startend != TSM_endbiased) {
       warn("Proteins can only have local/global/endbias startend policies set, not %s",startend_string);
@@ -568,12 +590,13 @@ boolean build_objects(void)
       ret = FALSE;
       warn("Could not read Protein sequence in %s",protein_file);
     } else {
+      
       if( qstart != -1 || qend != -1 ) {
 	if( qstart == -1 )
 	  qstart = 0;
 	if( qend == -1 ) 
 	  qend = pro->baseseq->len;
-
+	
 	pro_temp = truncate_Protein(pro,qstart-1,qend);
 	if( pro_temp == NULL ){
 	  ret = FALSE;
@@ -585,35 +608,36 @@ boolean build_objects(void)
     }
   } else {
     /** using a HMM **/
-
+    
     /*tsm = read_HMMer_1_7_ascii_file(hmm_file);*/
     /*tsm = Wise2_read_ThreeStateModel_from_hmmer1_file(hmm_file);*/
     tsm = HMMer2_read_ThreeStateModel(hmm_file);
-
-
-    if( tsm == NULL ) {
-      warn("Could not read hmm from %s\n",hmm_file);
-      ret = FALSE;
-    }  else {
-
-      display_char_in_ThreeStateModel(tsm);
-      if( hmm_name != NULL ) {
-	if( tsm->name != NULL ) 
-	  ckfree(tsm->name);
-	tsm->name = stringalloc(hmm_name);
-      }
-
-      if( tsm == NULL ) {
-	warn("Could not read %s as a hmm",hmm_file);
-      }
-      
-      /** have to set start/end **/
-      set_startend_policy_ThreeStateModel(tsm,startend,30,0.2);
-      
-    }
-  } /* end of else tsm != NULL */
     
+    
+      if( tsm == NULL ) {
+	warn("Could not read hmm from %s\n",hmm_file);
+	ret = FALSE;
+      }  else {
+	
+	display_char_in_ThreeStateModel(tsm);
+	if( hmm_name != NULL ) {
+	  if( tsm->name != NULL ) 
+	    ckfree(tsm->name);
+	  tsm->name = stringalloc(hmm_name);
+	}
+	
+	if( tsm == NULL ) {
+	  warn("Could not read %s as a hmm",hmm_file);
+	}
+	
+	/** have to set start/end **/
+	set_startend_policy_ThreeStateModel(tsm,startend,30,0.1);
+	
+      }
+  } /* end of else tsm != NULL */
+  
 
+  
   if( main_block_str != NULL ) {
     if( is_integer_string(main_block_str,&main_block) == FALSE ) {
       warn("Could not get maximum main_block number %s",main_block_str);
@@ -621,26 +645,6 @@ boolean build_objects(void)
     }
   }
    
-  
-  if( potential_file != NULL ) {
-    info("Potential gene file is read...\n");
-    pg = read_PotentialGene_file(potential_file);
-    if( pg == NULL ) {
-      warn("Could not build potential gene!");
-      return FALSE;
-    }
-  }
-
-  if( alg_str != NULL ) {
-    alg = gwrap_alg_type_from_string(alg_str);
-  } else {
-    if( use_tsm == TRUE ) {
-      alg_str = "623L";
-    } else {
-      alg_str = "623";
-    }
-    alg = gwrap_alg_type_from_string(alg_str);
-  }
 
 
   if( is_double_string(subs_string,&subs_error) == FALSE ) {
@@ -673,6 +677,7 @@ boolean build_objects(void)
     model_splice = TRUE;
   } else if ( strcmp(splice_string,"flat") == 0 ) {
     model_splice = FALSE;
+    gmp->use_gtag_splice = TRUE;
   } else {
     warn("Cannot interpret [%s] as a splice modelling parameter\n",splice_string);
     ret = FALSE;
@@ -709,7 +714,7 @@ boolean build_objects(void)
       warn("Could not read a GeneFrequency file in %s",gene_file);
     }
   } else {
-    if( (gs = GeneStats_from_GeneModelParam(gmp)) != NULL ){
+    if( (gs = GeneStats_from_GeneModelParam(gmp)) == NULL ){
       ret=FALSE;
       warn("Could not read gene statistics in %s",new_gene_file);
     }
@@ -745,7 +750,7 @@ void build_defaults(void)
   gene_file = "human.gf";
   new_gene_file = "gene.stat";
   codon_file = "codon.table";
-  matrix_file = "blosum62.bla";
+  matrix_file = "BLOSUM62.bla";
   
 
 }
@@ -771,13 +776,7 @@ void show_short_help(void)
   fprintf(stdout,"Copyright (c) GRL limited: portions of the code are from separate copyrights\n\n");
   fprintf(stdout,"genewise <protein-file> <dna-file> in fasta format\n");
   fprintf(stdout," Options. In any order, '-' as filename (for any input) means stdin\n");
-  fprintf(stdout," Dna      [-u,-v,-trev,-tabs,-fembl,-both]\n Protein  [-s,-t,-g,-e,-m]\n HMM      [-hmmer,-hname]\n");
-  fprintf(stdout," Model    [-codon,-gene,-cfreq,-splice,-subs,-indel,-intron,-null]\n Alg      [-kbyte,-alg]\n");
-  fprintf(stdout," Output   [-pretty,-genes,-genesf,-para,-sum,-cdna,-trans,-pep,-ace,-embl,-diana]\n");
-  fprintf(stdout,"  ..cont  [-gff,-gener,-alb,-pal,-block,-divide]\n");
-  fprintf(stdout," Standard [-help,-version,-silent,-quiet,-errorlog]\n");
   fprintf(stdout,"\nFor more help go %s -help.\n",program_name);
-  fprintf(stdout,"\nSee WWW help at http://www.sanger.ac.uk/Software/Wise2/\n");
   exit(63);   
 }
 
@@ -803,21 +802,24 @@ void show_help(FILE * ofp)
   fprintf(ofp,"HMM options\n");
   fprintf(ofp,"  -hmmer           Protein file is HMMer file (version 2 compatible)\n");
   fprintf(ofp,"  -hname           Use this as the name of the HMM, not filename\n");
-  fprintf(ofp,"Gene Model options\n");
+
+  show_help_GeneModelParam(ofp);
+  fprintf(ofp,"  -splice [model/flat] [LEGACY only for -splice flat. use -splice_gtag]\n");
+
+  show_help_PhasedProteinPara(ofp);
+
+  fprintf(ofp,"Other model options\n");
+  fprintf(ofp,"  -[no]newgene  use new gene stats (default), no for reverting to old system\n");
   fprintf(ofp,"  -init   [%s]  [default/global/local/wing/endbias] startend policy for the HMM/protein\n",startend_string);
   fprintf(ofp,"  -codon  [%s]  Codon file\n",codon_file);
-  fprintf(ofp,"  -gene   [%s]  Gene parameter file\n",gene_file);
   fprintf(ofp,"  -subs   [%2.2g] Substitution error rate\n",subs_error);
   fprintf(ofp,"  -indel  [%2.2g] Insertion/deletion error rate\n",indel_error);
-  fprintf(ofp,"  -cfreq  [model/flat] Using codon bias or not?     [default flat]\n");
-  fprintf(ofp,"  -splice [model/flat] Using splice model or GT/AG? [default model]\n");
-  fprintf(ofp,"  -intron [model/tied] Use tied model for introns   [default tied]\n");
   fprintf(ofp,"  -null   [syn/flat]   Random Model as synchronous or flat [default syn]\n");
-  fprintf(ofp,"  -pg     <file>       Potential Gene file (heurestic for speeding alignments)\n");
   fprintf(ofp,"  -alln   [%s]   Probability of matching a NNN codon\n",allN_string);
   fprintf(ofp,"  -insert [model/flat] Use protein insert model     [default flat]\n");
   fprintf(ofp,"Algorithm options\n");
-  fprintf(ofp,"  -alg    [623/623L/2193/2193L]  Algorithm used [default 623/623L]\n");
+  fprintf(ofp,"  -alg    [623/623L/623S/623P/2193/2193L]  Algorithm used [default 623/623L]\n");
+  fprintf(ofp,"  (normally use 623 for proteins, 623L for HMMs and 623P for Phased Proteins)\n");
   fprintf(ofp,"Output options [default -pretty -para]\n");
   fprintf(ofp,"  -pretty          show pretty ascii output\n");
   fprintf(ofp,"  -pseudo          Mark genes with frameshifts as pseudogenes\n");
@@ -842,25 +844,12 @@ void show_help(FILE * ofp)
   fprintf(ofp,"  -block  [%s]     Length of main block in pretty output\n",main_block_str);
   fprintf(ofp,"  -divide [%s]     divide string for multiple outputs\n",divide_str);
 
-  show_help_GeneModelParam(ofp);
+  show_help_GeneWiseRunPara(ofp);
 
+  
   show_help_DPRunImpl(ofp);
   show_standard_options(ofp);
 
-  fprintf(ofp,"\nSee WWW help at http://www.sanger.ac.uk/Software/Wise2/\n");
-
-  /*
-  fprintf(ofp,"\nPotential Gene file looks like\n");
-  fprintf(ofp,"pgene # stands for potential gene\n");
-  fprintf(ofp,"ptrans # stands for potential transcript\n");
-  fprintf(ofp,"pexon <start-in-dna> <end-in-dna> <start-in-protein> <end-in-protein>\n");
-  fprintf(ofp,"pexon <start-in-dna> <end-in-dna> <start-in-protein> <end-in-protein>\n");
-  fprintf(ofp,"...\n");
-  fprintf(ofp,"endptrans\n");
-  fprintf(ofp,"<another ptrans if you like>\n");
-  fprintf(ofp,"endpgene\n");
-  */
-
   exit(63);   
 }
 
@@ -889,6 +878,8 @@ int main(int argc,char ** argv)
 
   potential_file = strip_out_assigned_argument(&argc,argv,"pg");
 
+  pal_file = strip_out_assigned_argument(&argc,argv,"pal_file");
+
   if( (temp = strip_out_assigned_argument(&argc,argv,"gap")) != NULL )
     gap_str = temp;
 
@@ -922,7 +913,7 @@ int main(int argc,char ** argv)
   if( (strip_out_boolean_argument(&argc,argv,"trev")) == TRUE )
     reverse = TRUE;
 
-  if( (strip_out_boolean_argument(&argc,argv,"new")) == TRUE )
+  if( (strip_out_boolean_argument(&argc,argv,"[no]newgene")) == TRUE )
     use_new_stats = TRUE;
 
   if( (strip_out_boolean_argument(&argc,argv,"tfor")) == TRUE ){
@@ -976,8 +967,10 @@ int main(int argc,char ** argv)
   if( (temp = strip_out_assigned_argument(&argc,argv,"cfreq")) != NULL )
     cfreq_string = temp;
 
-  if( (temp = strip_out_assigned_argument(&argc,argv,"splice")) != NULL )
+  if( (temp = strip_out_assigned_argument(&argc,argv,"splice")) != NULL ) {
+    warn("deprecated command line option -splice. use -splice_gtag now");
     splice_string = temp;
+  }
 
   if( (temp = strip_out_assigned_argument(&argc,argv,"init")) != NULL )
     startend_string = temp;
@@ -1060,7 +1053,12 @@ int main(int argc,char ** argv)
 
   gmp  = new_GeneModelParam_from_argv(&argc,argv);
 
+  ppp = new_PhasedProteinPara_from_argv(&argc,argv);
+
+  gwrp = new_GeneWiseRunPara_from_argv(&argc,argv);
+
   strip_out_remaining_options_with_warning(&argc,argv);
+
   
 
   if( argc !=  3 ) {
@@ -1086,6 +1084,7 @@ int main(int argc,char ** argv)
   else 
     hmm_file  = argv[1];
 
+
   if( build_objects() == FALSE) 
     fatal("Could not build objects!");
 
