@@ -11,7 +11,11 @@ extern "C" {
 
 struct Wise2_GeneWise21 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     GeneWiseScore* query;    
@@ -48,6 +52,22 @@ struct thread_pool_holder_GeneWise21 {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_GeneWise21_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(GeneWise21*,int,int,int); 
+    int (*access_special)(GeneWise21*,int,int,int);  
+    } ;  
+/* GeneWise21_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_GeneWise21_access_func_holder
+typedef struct Wise2_GeneWise21_access_func_holder Wise2_GeneWise21_access_func_holder;
+#define GeneWise21_access_func_holder Wise2_GeneWise21_access_func_holder
+#define DYNAMITE_DEFINED_GeneWise21_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -57,6 +77,35 @@ struct thread_pool_holder_GeneWise21 {
 
 
 
+/* Function:  PackAln_read_Shatter_GeneWise21(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneWise21 *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_GeneWise21(GeneWise21 * mat);
+#define PackAln_read_Shatter_GeneWise21 Wise2_PackAln_read_Shatter_GeneWise21
+
+
+/* Function:  calculate_shatter_GeneWise21(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneWise21 matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [GeneWise21 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_GeneWise21(GeneWise21 * mat,DPEnvelope * dpenv);
+#define calculate_shatter_GeneWise21 Wise2_calculate_shatter_GeneWise21
+
+
 /* Function:  search_GeneWise21(dbsi,out,querydb,targetdb,gp21,rndcodon,rndbase)
  *
  * Descrip:    This function makes a database search of GeneWise21
@@ -127,7 +176,7 @@ PackAln * Wise2_PackAln_bestmemory_GeneWise21(GeneWiseScore* query,ComplexSequen
 #define PackAln_bestmemory_GeneWise21 Wise2_PackAln_bestmemory_GeneWise21
 
 
-/* Function:  allocate_Expl_GeneWise21(query,target,gp21,rndcodon,rndbase)
+/* Function:  allocate_Expl_GeneWise21(query,target,gp21,rndcodon,rndbase,dpri)
  *
  * Descrip:    This function allocates the GeneWise21 structure
  *             and the basematrix area for explicit memory implementations
@@ -139,11 +188,12 @@ PackAln * Wise2_PackAln_bestmemory_GeneWise21(GeneWiseScore* query,ComplexSequen
  * Arg:            gp21 [UNKN ] Resource [GeneParser21Score *]
  * Arg:        rndcodon [UNKN ] Resource [RandomCodonScore *]
  * Arg:         rndbase [UNKN ] Resource [RandomModelDNAScore *]
+ * Arg:            dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [GeneWise21 *]
  *
  */
-GeneWise21 * Wise2_allocate_Expl_GeneWise21(GeneWiseScore* query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
+GeneWise21 * Wise2_allocate_Expl_GeneWise21(GeneWiseScore* query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase,DPRunImpl * dpri);
 #define allocate_Expl_GeneWise21 Wise2_allocate_Expl_GeneWise21
 
 
@@ -268,6 +318,21 @@ PackAln * Wise2_PackAln_read_Expl_GeneWise21(GeneWise21 * mat);
 #define PackAln_read_Expl_GeneWise21 Wise2_PackAln_read_Expl_GeneWise21
 
 
+/* Function:  PackAln_read_generic_GeneWise21(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneWise21 *]
+ * Arg:          h [UNKN ] Undocumented argument [GeneWise21_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_GeneWise21(GeneWise21 * mat,GeneWise21_access_func_holder h);
+#define PackAln_read_generic_GeneWise21 Wise2_PackAln_read_generic_GeneWise21
+
+
 /* Function:  calculate_GeneWise21(mat)
  *
  * Descrip:    This function calculates the GeneWise21 matrix when in explicit mode
@@ -283,6 +348,21 @@ boolean Wise2_calculate_GeneWise21(GeneWise21 * mat);
 #define calculate_GeneWise21 Wise2_calculate_GeneWise21
 
 
+/* Function:  calculate_dpenv_GeneWise21(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneWise21 matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] GeneWise21 which contains explicit basematrix memory [GeneWise21 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_GeneWise21(GeneWise21 * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_GeneWise21 Wise2_calculate_dpenv_GeneWise21
+
+
 /* Function:  GeneWise21_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -319,6 +399,10 @@ GeneWise21 * Wise2_free_GeneWise21(GeneWise21 * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_GeneWise21_shatter_access_main(GeneWise21 * mat,int i,int j,int state);
+#define GeneWise21_shatter_access_main Wise2_GeneWise21_shatter_access_main
+int Wise2_GeneWise21_shatter_access_special(GeneWise21 * mat,int i,int j,int state);
+#define GeneWise21_shatter_access_special Wise2_GeneWise21_shatter_access_special
 void * Wise2_thread_loop_GeneWise21(void * ptr);
 #define thread_loop_GeneWise21 Wise2_thread_loop_GeneWise21
 int Wise2_score_only_GeneWise21(GeneWiseScore* query,ComplexSequence* target ,GeneParser21Score * gp21,RandomCodonScore * rndcodon,RandomModelDNAScore * rndbase);
@@ -355,21 +439,25 @@ void Wise2_run_up_dc_GeneWise21(GeneWise21 * mat,int starti,int stopi,int startj
 #define run_up_dc_GeneWise21 Wise2_run_up_dc_GeneWise21
 void Wise2_init_dc_GeneWise21(GeneWise21 * mat);
 #define init_dc_GeneWise21 Wise2_init_dc_GeneWise21
-boolean Wise2_dc_start_end_calculate_GeneWise21(GeneWise21 * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_GeneWise21 Wise2_dc_start_end_calculate_GeneWise21
 int Wise2_start_end_find_end_GeneWise21(GeneWise21 * mat,int * endj);
 #define start_end_find_end_GeneWise21 Wise2_start_end_find_end_GeneWise21
+boolean Wise2_dc_optimised_start_end_calc_GeneWise21(GeneWise21 *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_GeneWise21 Wise2_dc_optimised_start_end_calc_GeneWise21
 void Wise2_init_start_end_linear_GeneWise21(GeneWise21 * mat);
 #define init_start_end_linear_GeneWise21 Wise2_init_start_end_linear_GeneWise21
 AlnConvertSet * Wise2_AlnConvertSet_GeneWise21(void);
 #define AlnConvertSet_GeneWise21 Wise2_AlnConvertSet_GeneWise21
-int Wise2_find_end_GeneWise21(GeneWise21 * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_GeneWise21_explicit_access_main(GeneWise21 * mat,int i,int j,int state);
+#define GeneWise21_explicit_access_main Wise2_GeneWise21_explicit_access_main
+int Wise2_GeneWise21_explicit_access_special(GeneWise21 * mat,int i,int j,int state);
+#define GeneWise21_explicit_access_special Wise2_GeneWise21_explicit_access_special
+int Wise2_find_end_GeneWise21(GeneWise21 * mat,int * ri,int * rj,int * state,boolean * isspecial,GeneWise21_access_func_holder h);
 #define find_end_GeneWise21 Wise2_find_end_GeneWise21
 void Wise2_GeneWise21_debug_show_matrix(GeneWise21 * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define GeneWise21_debug_show_matrix Wise2_GeneWise21_debug_show_matrix
-int Wise2_max_calc_GeneWise21(GeneWise21 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_GeneWise21(GeneWise21 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneWise21_access_func_holder h);
 #define max_calc_GeneWise21 Wise2_max_calc_GeneWise21
-int Wise2_max_calc_special_GeneWise21(GeneWise21 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_GeneWise21(GeneWise21 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneWise21_access_func_holder h);
 #define max_calc_special_GeneWise21 Wise2_max_calc_special_GeneWise21
 
 #ifdef _cplusplus
