@@ -10,7 +10,11 @@ extern "C" {
 
 struct Wise2_GeneWise4 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     GeneWiseScore* query;    
@@ -43,6 +47,22 @@ struct thread_pool_holder_GeneWise4 {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_GeneWise4_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(GeneWise4*,int,int,int);  
+    int (*access_special)(GeneWise4*,int,int,int);   
+    } ;  
+/* GeneWise4_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_GeneWise4_access_func_holder
+typedef struct Wise2_GeneWise4_access_func_holder Wise2_GeneWise4_access_func_holder;
+#define GeneWise4_access_func_holder Wise2_GeneWise4_access_func_holder
+#define DYNAMITE_DEFINED_GeneWise4_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -52,6 +72,35 @@ struct thread_pool_holder_GeneWise4 {
 
 
 
+/* Function:  PackAln_read_Shatter_GeneWise4(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneWise4 *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_GeneWise4(GeneWise4 * mat);
+#define PackAln_read_Shatter_GeneWise4 Wise2_PackAln_read_Shatter_GeneWise4
+
+
+/* Function:  calculate_shatter_GeneWise4(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneWise4 matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [GeneWise4 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_GeneWise4(GeneWise4 * mat,DPEnvelope * dpenv);
+#define calculate_shatter_GeneWise4 Wise2_calculate_shatter_GeneWise4
+
+
 /* Function:  search_GeneWise4(dbsi,out,querydb,targetdb,gp)
  *
  * Descrip:    This function makes a database search of GeneWise4
@@ -116,7 +165,7 @@ PackAln * Wise2_PackAln_bestmemory_GeneWise4(GeneWiseScore* query,ComplexSequenc
 #define PackAln_bestmemory_GeneWise4 Wise2_PackAln_bestmemory_GeneWise4
 
 
-/* Function:  allocate_Expl_GeneWise4(query,target,gp)
+/* Function:  allocate_Expl_GeneWise4(query,target,gp,dpri)
  *
  * Descrip:    This function allocates the GeneWise4 structure
  *             and the basematrix area for explicit memory implementations
@@ -126,11 +175,12 @@ PackAln * Wise2_PackAln_bestmemory_GeneWise4(GeneWiseScore* query,ComplexSequenc
  * Arg:         query [UNKN ] query data structure [GeneWiseScore*]
  * Arg:        target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            gp [UNKN ] Resource [GeneParser4Score *]
+ * Arg:          dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [GeneWise4 *]
  *
  */
-GeneWise4 * Wise2_allocate_Expl_GeneWise4(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp);
+GeneWise4 * Wise2_allocate_Expl_GeneWise4(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp,DPRunImpl * dpri);
 #define allocate_Expl_GeneWise4 Wise2_allocate_Expl_GeneWise4
 
 
@@ -253,6 +303,21 @@ PackAln * Wise2_PackAln_read_Expl_GeneWise4(GeneWise4 * mat);
 #define PackAln_read_Expl_GeneWise4 Wise2_PackAln_read_Expl_GeneWise4
 
 
+/* Function:  PackAln_read_generic_GeneWise4(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneWise4 *]
+ * Arg:          h [UNKN ] Undocumented argument [GeneWise4_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_GeneWise4(GeneWise4 * mat,GeneWise4_access_func_holder h);
+#define PackAln_read_generic_GeneWise4 Wise2_PackAln_read_generic_GeneWise4
+
+
 /* Function:  calculate_GeneWise4(mat)
  *
  * Descrip:    This function calculates the GeneWise4 matrix when in explicit mode
@@ -268,6 +333,21 @@ boolean Wise2_calculate_GeneWise4(GeneWise4 * mat);
 #define calculate_GeneWise4 Wise2_calculate_GeneWise4
 
 
+/* Function:  calculate_dpenv_GeneWise4(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneWise4 matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] GeneWise4 which contains explicit basematrix memory [GeneWise4 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_GeneWise4(GeneWise4 * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_GeneWise4 Wise2_calculate_dpenv_GeneWise4
+
+
 /* Function:  GeneWise4_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -304,6 +384,10 @@ GeneWise4 * Wise2_free_GeneWise4(GeneWise4 * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_GeneWise4_shatter_access_main(GeneWise4 * mat,int i,int j,int state);
+#define GeneWise4_shatter_access_main Wise2_GeneWise4_shatter_access_main
+int Wise2_GeneWise4_shatter_access_special(GeneWise4 * mat,int i,int j,int state);
+#define GeneWise4_shatter_access_special Wise2_GeneWise4_shatter_access_special
 void * Wise2_thread_loop_GeneWise4(void * ptr);
 #define thread_loop_GeneWise4 Wise2_thread_loop_GeneWise4
 int Wise2_score_only_GeneWise4(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp);
@@ -340,21 +424,25 @@ void Wise2_run_up_dc_GeneWise4(GeneWise4 * mat,int starti,int stopi,int startj,i
 #define run_up_dc_GeneWise4 Wise2_run_up_dc_GeneWise4
 void Wise2_init_dc_GeneWise4(GeneWise4 * mat);
 #define init_dc_GeneWise4 Wise2_init_dc_GeneWise4
-boolean Wise2_dc_start_end_calculate_GeneWise4(GeneWise4 * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_GeneWise4 Wise2_dc_start_end_calculate_GeneWise4
 int Wise2_start_end_find_end_GeneWise4(GeneWise4 * mat,int * endj);
 #define start_end_find_end_GeneWise4 Wise2_start_end_find_end_GeneWise4
+boolean Wise2_dc_optimised_start_end_calc_GeneWise4(GeneWise4 *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_GeneWise4 Wise2_dc_optimised_start_end_calc_GeneWise4
 void Wise2_init_start_end_linear_GeneWise4(GeneWise4 * mat);
 #define init_start_end_linear_GeneWise4 Wise2_init_start_end_linear_GeneWise4
 AlnConvertSet * Wise2_AlnConvertSet_GeneWise4(void);
 #define AlnConvertSet_GeneWise4 Wise2_AlnConvertSet_GeneWise4
-int Wise2_find_end_GeneWise4(GeneWise4 * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_GeneWise4_explicit_access_main(GeneWise4 * mat,int i,int j,int state);
+#define GeneWise4_explicit_access_main Wise2_GeneWise4_explicit_access_main
+int Wise2_GeneWise4_explicit_access_special(GeneWise4 * mat,int i,int j,int state);
+#define GeneWise4_explicit_access_special Wise2_GeneWise4_explicit_access_special
+int Wise2_find_end_GeneWise4(GeneWise4 * mat,int * ri,int * rj,int * state,boolean * isspecial,GeneWise4_access_func_holder h);
 #define find_end_GeneWise4 Wise2_find_end_GeneWise4
 void Wise2_GeneWise4_debug_show_matrix(GeneWise4 * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define GeneWise4_debug_show_matrix Wise2_GeneWise4_debug_show_matrix
-int Wise2_max_calc_GeneWise4(GeneWise4 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_GeneWise4(GeneWise4 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneWise4_access_func_holder h);
 #define max_calc_GeneWise4 Wise2_max_calc_GeneWise4
-int Wise2_max_calc_special_GeneWise4(GeneWise4 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_GeneWise4(GeneWise4 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneWise4_access_func_holder h);
 #define max_calc_special_GeneWise4 Wise2_max_calc_special_GeneWise4
 
 #ifdef _cplusplus
