@@ -10,7 +10,11 @@ extern "C" {
 
 struct Wise2_GeneWise6 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     GeneWiseScore* query;    
@@ -43,6 +47,22 @@ struct thread_pool_holder_GeneWise6 {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_GeneWise6_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(GeneWise6*,int,int,int);  
+    int (*access_special)(GeneWise6*,int,int,int);   
+    } ;  
+/* GeneWise6_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_GeneWise6_access_func_holder
+typedef struct Wise2_GeneWise6_access_func_holder Wise2_GeneWise6_access_func_holder;
+#define GeneWise6_access_func_holder Wise2_GeneWise6_access_func_holder
+#define DYNAMITE_DEFINED_GeneWise6_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -52,6 +72,35 @@ struct thread_pool_holder_GeneWise6 {
 
 
 
+/* Function:  PackAln_read_Shatter_GeneWise6(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneWise6 *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_GeneWise6(GeneWise6 * mat);
+#define PackAln_read_Shatter_GeneWise6 Wise2_PackAln_read_Shatter_GeneWise6
+
+
+/* Function:  calculate_shatter_GeneWise6(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneWise6 matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [GeneWise6 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_GeneWise6(GeneWise6 * mat,DPEnvelope * dpenv);
+#define calculate_shatter_GeneWise6 Wise2_calculate_shatter_GeneWise6
+
+
 /* Function:  search_GeneWise6(dbsi,out,querydb,targetdb,gp)
  *
  * Descrip:    This function makes a database search of GeneWise6
@@ -116,7 +165,7 @@ PackAln * Wise2_PackAln_bestmemory_GeneWise6(GeneWiseScore* query,ComplexSequenc
 #define PackAln_bestmemory_GeneWise6 Wise2_PackAln_bestmemory_GeneWise6
 
 
-/* Function:  allocate_Expl_GeneWise6(query,target,gp)
+/* Function:  allocate_Expl_GeneWise6(query,target,gp,dpri)
  *
  * Descrip:    This function allocates the GeneWise6 structure
  *             and the basematrix area for explicit memory implementations
@@ -126,11 +175,12 @@ PackAln * Wise2_PackAln_bestmemory_GeneWise6(GeneWiseScore* query,ComplexSequenc
  * Arg:         query [UNKN ] query data structure [GeneWiseScore*]
  * Arg:        target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            gp [UNKN ] Resource [GeneParser4Score *]
+ * Arg:          dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [GeneWise6 *]
  *
  */
-GeneWise6 * Wise2_allocate_Expl_GeneWise6(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp);
+GeneWise6 * Wise2_allocate_Expl_GeneWise6(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp,DPRunImpl * dpri);
 #define allocate_Expl_GeneWise6 Wise2_allocate_Expl_GeneWise6
 
 
@@ -253,6 +303,21 @@ PackAln * Wise2_PackAln_read_Expl_GeneWise6(GeneWise6 * mat);
 #define PackAln_read_Expl_GeneWise6 Wise2_PackAln_read_Expl_GeneWise6
 
 
+/* Function:  PackAln_read_generic_GeneWise6(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneWise6 *]
+ * Arg:          h [UNKN ] Undocumented argument [GeneWise6_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_GeneWise6(GeneWise6 * mat,GeneWise6_access_func_holder h);
+#define PackAln_read_generic_GeneWise6 Wise2_PackAln_read_generic_GeneWise6
+
+
 /* Function:  calculate_GeneWise6(mat)
  *
  * Descrip:    This function calculates the GeneWise6 matrix when in explicit mode
@@ -268,6 +333,21 @@ boolean Wise2_calculate_GeneWise6(GeneWise6 * mat);
 #define calculate_GeneWise6 Wise2_calculate_GeneWise6
 
 
+/* Function:  calculate_dpenv_GeneWise6(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneWise6 matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] GeneWise6 which contains explicit basematrix memory [GeneWise6 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_GeneWise6(GeneWise6 * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_GeneWise6 Wise2_calculate_dpenv_GeneWise6
+
+
 /* Function:  GeneWise6_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -304,6 +384,10 @@ GeneWise6 * Wise2_free_GeneWise6(GeneWise6 * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_GeneWise6_shatter_access_main(GeneWise6 * mat,int i,int j,int state);
+#define GeneWise6_shatter_access_main Wise2_GeneWise6_shatter_access_main
+int Wise2_GeneWise6_shatter_access_special(GeneWise6 * mat,int i,int j,int state);
+#define GeneWise6_shatter_access_special Wise2_GeneWise6_shatter_access_special
 void * Wise2_thread_loop_GeneWise6(void * ptr);
 #define thread_loop_GeneWise6 Wise2_thread_loop_GeneWise6
 int Wise2_score_only_GeneWise6(GeneWiseScore* query,ComplexSequence* target ,GeneParser4Score * gp);
@@ -340,21 +424,25 @@ void Wise2_run_up_dc_GeneWise6(GeneWise6 * mat,int starti,int stopi,int startj,i
 #define run_up_dc_GeneWise6 Wise2_run_up_dc_GeneWise6
 void Wise2_init_dc_GeneWise6(GeneWise6 * mat);
 #define init_dc_GeneWise6 Wise2_init_dc_GeneWise6
-boolean Wise2_dc_start_end_calculate_GeneWise6(GeneWise6 * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_GeneWise6 Wise2_dc_start_end_calculate_GeneWise6
 int Wise2_start_end_find_end_GeneWise6(GeneWise6 * mat,int * endj);
 #define start_end_find_end_GeneWise6 Wise2_start_end_find_end_GeneWise6
+boolean Wise2_dc_optimised_start_end_calc_GeneWise6(GeneWise6 *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_GeneWise6 Wise2_dc_optimised_start_end_calc_GeneWise6
 void Wise2_init_start_end_linear_GeneWise6(GeneWise6 * mat);
 #define init_start_end_linear_GeneWise6 Wise2_init_start_end_linear_GeneWise6
 AlnConvertSet * Wise2_AlnConvertSet_GeneWise6(void);
 #define AlnConvertSet_GeneWise6 Wise2_AlnConvertSet_GeneWise6
-int Wise2_find_end_GeneWise6(GeneWise6 * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_GeneWise6_explicit_access_main(GeneWise6 * mat,int i,int j,int state);
+#define GeneWise6_explicit_access_main Wise2_GeneWise6_explicit_access_main
+int Wise2_GeneWise6_explicit_access_special(GeneWise6 * mat,int i,int j,int state);
+#define GeneWise6_explicit_access_special Wise2_GeneWise6_explicit_access_special
+int Wise2_find_end_GeneWise6(GeneWise6 * mat,int * ri,int * rj,int * state,boolean * isspecial,GeneWise6_access_func_holder h);
 #define find_end_GeneWise6 Wise2_find_end_GeneWise6
 void Wise2_GeneWise6_debug_show_matrix(GeneWise6 * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define GeneWise6_debug_show_matrix Wise2_GeneWise6_debug_show_matrix
-int Wise2_max_calc_GeneWise6(GeneWise6 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_GeneWise6(GeneWise6 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneWise6_access_func_holder h);
 #define max_calc_GeneWise6 Wise2_max_calc_GeneWise6
-int Wise2_max_calc_special_GeneWise6(GeneWise6 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_GeneWise6(GeneWise6 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneWise6_access_func_holder h);
 #define max_calc_special_GeneWise6 Wise2_max_calc_special_GeneWise6
 
 #ifdef _cplusplus
