@@ -41,6 +41,9 @@ SequenceDB * sdb  = NULL;
 Genomic * gen     = NULL;
 boolean use_single_dna = FALSE;
 
+int dna_seqdb_start = -1;
+int dna_seqdb_end   = -1;
+
 char * protein_file = NULL;
 
 Protein * pro   = NULL;
@@ -50,6 +53,9 @@ char * hmm_name       = NULL;
 
 ThreeStateDB * tsmdb  = NULL;
 
+int pro_seqdb_start = -1;
+int pro_seqdb_end   = -1;
+
 GeneWiseScore * gws = NULL;
 
 Hscore * hs       = NULL;
@@ -71,7 +77,7 @@ int qend             = -1;
 char * gene_file        = NULL;
 GeneFrequency21 * gf    = NULL;
 
-char * matrix_file      = "blosum62.bla";
+char * matrix_file      = "BLOSUM62.bla";
 CompMat * mat           = NULL;
 
 char * gap_str          = "12";
@@ -317,7 +323,7 @@ boolean show_output(void)
 
     temptsm = indexed_ThreeStateModel_ThreeStateDB(tsmdb,hs->ds[i]->query);
 
-    alb = AlnBlock_from_TSM_genewise_wrap(temptsm,gent,gpara,rmd,rmd,use_syn,aln_alg,allN,flat_insert,NULL,dpri,&pal);
+    alb = AlnBlock_from_TSM_genewise_wrap(temptsm,gent,gpara,rmd,rmd,use_syn,aln_alg,allN,flat_insert,dpri,&pal,NULL);
    
 
     gr = new_GenomicRegion(gent);
@@ -604,6 +610,13 @@ boolean build_objects(void)
       warn("Could not build a sequence database on %s",dna_seq_file);
       ret = FALSE;
     }
+
+    if( dna_seqdb_start != -1 && dna_seqdb_end != -1 ) {
+      sdb->seq_start = dna_seqdb_start;
+      sdb->seq_end   = dna_seqdb_end;
+    }
+						  
+
   }
 
   rm = default_RandomModel();
@@ -667,7 +680,6 @@ boolean build_objects(void)
   } else if ( use_pfam2 == TRUE ) {
     tsmdb = HMMer2_ThreeStateDB(protein_file);
 
-
     if( set_search_type_ThreeStateDB(tsmdb,startend_string) == FALSE) {
       warn("Unable to set global/local switch on threestatedb");
       ret = FALSE;
@@ -765,10 +777,11 @@ boolean build_objects(void)
     warn("No protein input file! Yikes!");
   }
 
-  /***
-  if( use_tsm == FALSE ) {
-  } else {
-  ****/
+
+  if( pro_seqdb_start != -1 && pro_seqdb_end != -1 ) {
+    tsmdb->hmm_model_start = pro_seqdb_start;
+    tsmdb->hmm_model_end   = pro_seqdb_end;
+  }
 
 
   if( main_block_str != NULL ) {
@@ -921,9 +934,13 @@ void show_help(FILE * ofp)
   fprintf(ofp,"  -pfam     pfam hmm library\n");
   fprintf(ofp,"  -pfam2    pfam old style model directory (2.1) \n");
   fprintf(ofp,"  -hmmer    single hmmer HMM (version 2 compatible)\n");
+  fprintf(ofp,"  -pro_db_start start position in protein/hmm database\n");
+  fprintf(ofp,"  -pro_db_end   end   position in protein/hmm database\n");
   fprintf(ofp,"DNA input type\n");
   fprintf(ofp,"  -dnadb    [default] dna fasta database\n");
   fprintf(ofp,"  -dnas     a single dna fasta sequence\n");
+  fprintf(ofp,"  -dna_db_start start position in dna database\n");
+  fprintf(ofp,"  -dna_db_end   end   position in dna database\n");
   fprintf(ofp,"Protein comparison options\n");
   fprintf(ofp,"  -gap      [%3d]  gap penalty\n",gap);
   fprintf(ofp,"  -ext      [%3d]  extension penalty\n",ext);
@@ -1000,7 +1017,7 @@ void build_defaults(void)
 {
   gene_file = "human.gf";
   codon_file = "codon.table";
-  matrix_file = "blosum62.bla";
+  matrix_file = "BLOSUM62.bla";
   
 
 }
@@ -1103,6 +1120,9 @@ int main(int argc,char ** argv)
   if( (strip_out_boolean_argument(&argc,argv,"dnadb")) == TRUE )
     use_single_dna = FALSE;
 
+  strip_out_integer_argument(&argc,argv,"dna_db_start",&dna_seqdb_start);
+  strip_out_integer_argument(&argc,argv,"dna_db_end",&dna_seqdb_end);
+
   if( (strip_out_boolean_argument(&argc,argv,"hmmer")) == TRUE )
     use_tsm = TRUE;
 
@@ -1119,6 +1139,11 @@ int main(int argc,char ** argv)
   if( (strip_out_boolean_argument(&argc,argv,"prodb")) == TRUE )
     use_db_pro = TRUE;
 
+  strip_out_integer_argument(&argc,argv,"pro_db_start",&pro_seqdb_start);
+  strip_out_integer_argument(&argc,argv,"pro_db_end",&pro_seqdb_end);
+
+
+
   if( (strip_out_boolean_argument(&argc,argv,"intie")) == TRUE )
     use_tied_model = TRUE;
 
@@ -1194,6 +1219,9 @@ int main(int argc,char ** argv)
   
   dpri = new_DPRunImpl_from_argv(&argc,argv);
 
+  assert(dpri);
+  assert(dbsi);
+
   strip_out_remaining_options_with_warning(&argc,argv);
   
 
