@@ -3,6 +3,27 @@ extern "C" {
 #endif
 #include "genewisemodel.h"
 
+
+/* Function:  pack_GeneWiseScore(gws)
+ *
+ * Descrip:    Packing up the GeneWise model into a byte structure
+ *
+ *
+ * Arg:        gws [UNKN ] Undocumented argument [GeneWiseScore *]
+ *
+ * Return [UNKN ]  Undocumented return value [char *]
+ *
+ */
+# line 110 "genewisemodel.dy"
+char * pack_GeneWiseScore(GeneWiseScore * gws)
+{
+  char * out;
+
+  return out;
+}
+
+
+
 /* Function:  GeneWiseScoreFlat_from_GeneWiseScore(gws)
  *
  * Descrip:    This produces a flattened GeneWiseSegment structure
@@ -16,7 +37,7 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [GeneWiseScoreFlat *]
  *
  */
-# line 112 "genewisemodel.dy"
+# line 125 "genewisemodel.dy"
 GeneWiseScoreFlat * GeneWiseScoreFlat_from_GeneWiseScore(GeneWiseScore * gws)
 {
   int i;
@@ -51,7 +72,7 @@ GeneWiseScoreFlat * GeneWiseScoreFlat_from_GeneWiseScore(GeneWiseScore * gws)
  * Return [UNKN ]  Undocumented return value [GeneWiseScoreFlat *]
  *
  */
-# line 140 "genewisemodel.dy"
+# line 153 "genewisemodel.dy"
 GeneWiseScoreFlat * free_GeneWiseScoreFlat(GeneWiseScoreFlat * obj)
 {
   if( obj == NULL ) {
@@ -84,7 +105,7 @@ GeneWiseScoreFlat * free_GeneWiseScoreFlat(GeneWiseScoreFlat * obj)
  * Arg:        cseq [UNKN ] Undocumented argument [ComplexSequence *]
  *
  */
-# line 165 "genewisemodel.dy"
+# line 178 "genewisemodel.dy"
 void map_phase0_codons_AlnBlock_GeneWise(AlnBlock * alb,GeneWiseScore * gws,ComplexSequence * cseq)
 {
   AlnColumn * alc;
@@ -141,7 +162,7 @@ void map_phase0_codons_AlnBlock_GeneWise(AlnBlock * alb,GeneWiseScore * gws,Comp
  * Arg:        gw [UNKN ] genewise model to flatten [GeneWise *]
  *
  */
-# line 218 "genewisemodel.dy"
+# line 231 "genewisemodel.dy"
 void flatten_balance_scores_GeneWise(GeneWise * gw)
 {
   int i;
@@ -170,7 +191,7 @@ void flatten_balance_scores_GeneWise(GeneWise * gw)
  * Return [UNKN ]  Undocumented return value [GeneWise *]
  *
  */
-# line 236 "genewisemodel.dy"
+# line 249 "genewisemodel.dy"
 GeneWise * GeneWise_from_ThreeStateModel_cdna(ThreeStateModel * tsm,cDNAParser * cp,CodonMapper * cm,Probability allN)
 {
   register int i;
@@ -193,6 +214,40 @@ GeneWise * GeneWise_from_ThreeStateModel_cdna(ThreeStateModel * tsm,cDNAParser *
 
   return out;
 }
+
+/* Function:  GeneWise_from_ThreeStateModel_setfactor(tsm,factor,cm,allN)
+ *
+ * Descrip:    This function makes a 
+ *             GeneWise model for the estwise
+ *             type algorithms
+ *
+ *
+ * Arg:           tsm [UNKN ] Undocumented argument [ThreeStateModel *]
+ * Arg:        factor [UNKN ] Undocumented argument [Probability]
+ * Arg:            cm [UNKN ] Undocumented argument [CodonMapper *]
+ * Arg:          allN [UNKN ] Undocumented argument [Probability]
+ *
+ * Return [UNKN ]  Undocumented return value [GeneWise *]
+ *
+ */
+# line 277 "genewisemodel.dy"
+GeneWise * GeneWise_from_ThreeStateModel_setfactor(ThreeStateModel * tsm,Probability factor,CodonMapper * cm,Probability allN)
+{
+  register int i;
+  GeneWise * out;
+
+  assert(tsm  != NULL);
+  assert(cm != NULL);
+
+  if( (out = GeneWise_alloc_len(tsm->len)) == NULL )
+    return NULL;
+
+  for(i=0;i<tsm->len;i++) 
+    add_GeneWise(out,GeneWiseSegment_from_ThreeStateUnit(tsm->unit[i],factor,cm,NULL,allN));
+
+
+  return out;
+}
   
   
 /* Function:  GeneWise_from_ThreeStateModel(tsm,gp,cm,allN,gwcm)
@@ -216,7 +271,7 @@ GeneWise * GeneWise_from_ThreeStateModel_cdna(ThreeStateModel * tsm,cDNAParser *
  * Return [UNKN ]  Undocumented return value [GeneWise *]
  *
  */
-# line 270 "genewisemodel.dy"
+# line 306 "genewisemodel.dy"
 GeneWise * GeneWise_from_ThreeStateModel(ThreeStateModel * tsm,GeneParser21 * gp,CodonMapper * cm,Probability allN,GeneWiseCodonModel * gwcm)
 {
   register int i;
@@ -224,7 +279,7 @@ GeneWise * GeneWise_from_ThreeStateModel(ThreeStateModel * tsm,GeneParser21 * gp
   Probability factor;
 
   assert(tsm);
-  assert(gp);
+  /*  assert(gp); */
   assert(cm);
   /*  assert(gwcm); */ /* can cope with null gwcm's */
 
@@ -235,8 +290,11 @@ GeneWise * GeneWise_from_ThreeStateModel(ThreeStateModel * tsm,GeneParser21 * gp
     out->name = stringalloc(tsm->name);
 
 
-
-  factor = (1.0 - removed_probability_from_cds(gp));
+  if( gp == NULL ) {
+    factor = 1.0;
+  } else {
+   factor = (1.0 - removed_probability_from_cds(gp));
+  }
 
   for(i=0;i<tsm->len;i++) {
     add_GeneWise(out,GeneWiseSegment_from_ThreeStateUnit(tsm->unit[i],factor,cm,gwcm,allN));
@@ -266,7 +324,7 @@ GeneWise * GeneWise_from_ThreeStateModel(ThreeStateModel * tsm,GeneParser21 * gp
  * Arg:        stop_codon_background [UNKN ] Undocumented argument [Probability]
  *
  */
-# line 310 "genewisemodel.dy"
+# line 349 "genewisemodel.dy"
 void GeneWise_fold_in_synchronised_RandomModel(GeneWise * gw,RandomModel * rm,CodonMapper * cm,CodonTable *ct,Probability stop_codon_background)
 {
   int i;
@@ -318,7 +376,7 @@ void GeneWise_fold_in_synchronised_RandomModel(GeneWise * gw,RandomModel * rm,Co
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 351 "genewisemodel.dy"
+# line 390 "genewisemodel.dy"
 boolean check_flat_insert(GeneWise * gw,boolean should_force,boolean should_warn,CodonTable * ct)
 {
   boolean ret = TRUE;
@@ -354,7 +412,7 @@ boolean check_flat_insert(GeneWise * gw,boolean should_force,boolean should_warn
  * Arg:        rmd [UNKN ] Undocumented argument [RandomModelDNA *]
  *
  */
-# line 380 "genewisemodel.dy"
+# line 419 "genewisemodel.dy"
 void GeneWise_fold_in_RandomModelDNA(GeneWise * gw,RandomModelDNA * rmd)
 {
   register int i;
@@ -392,7 +450,7 @@ void GeneWise_fold_in_RandomModelDNA(GeneWise * gw,RandomModelDNA * rmd)
  * Return [UNKN ]  Undocumented return value [Protein *]
  *
  */
-# line 405 "genewisemodel.dy"
+# line 444 "genewisemodel.dy"
 Protein * Protein_from_GeneWise_AlnColumn(Sequence * dna,AlnColumn * col,int position_in_aln,AlnColumn ** last_column,CodonTable * ct,boolean (*is_random_AlnColumn)(const AlnColumn *))
 {
   Sequence * out;
@@ -495,7 +553,7 @@ Protein * Protein_from_GeneWise_AlnColumn(Sequence * dna,AlnColumn * col,int pos
  * Return [UNKN ]  Undocumented return value [Probability]
  *
  */
-# line 500 "genewisemodel.dy"
+# line 539 "genewisemodel.dy"
 Probability probability_of_this_codon(int codon,RandomModelDNA * rmd)
 {
   base one;
@@ -516,7 +574,7 @@ Probability probability_of_this_codon(int codon,RandomModelDNA * rmd)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 515 "genewisemodel.dy"
+# line 554 "genewisemodel.dy"
 void show_GeneWise(GeneWise * gw,FILE * ofp)
 {
   register int i;
@@ -538,7 +596,7 @@ void show_GeneWise(GeneWise * gw,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 531 "genewisemodel.dy"
+# line 570 "genewisemodel.dy"
 void show_GeneWiseSegment(GeneWiseSegment * seg,FILE * ofp)
 {
   fprintf(ofp,"match=\" ");
@@ -564,7 +622,7 @@ void show_GeneWiseSegment(GeneWiseSegment * seg,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [GeneWiseSegment *]
  *
  */
-# line 546 "genewisemodel.dy"
+# line 585 "genewisemodel.dy"
 GeneWiseSegment * GeneWiseSegment_from_ThreeStateUnit(ThreeStateUnit * tsu,Probability factor,CodonMapper * cm,GeneWiseCodonModel * gwcm,Probability allN)
 {
   register int i;
@@ -664,7 +722,7 @@ GeneWiseSegment * GeneWiseSegment_from_ThreeStateUnit(ThreeStateUnit * tsu,Proba
  * Return [UNKN ]  Undocumented return value [Probability]
  *
  */
-# line 636 "genewisemodel.dy"
+# line 675 "genewisemodel.dy"
 Probability Probability_of_codon(int codon,CodonTable * ct,Probability * aminoacid_26_array,Probability stop)
 {
   register int i,j,k;
@@ -760,7 +818,7 @@ Probability Probability_of_codon(int codon,CodonTable * ct,Probability * aminoac
  * Return [UNKN ]  Undocumented return value [GeneWiseScore *]
  *
  */
-# line 724 "genewisemodel.dy"
+# line 763 "genewisemodel.dy"
 GeneWiseScore * GeneWiseScore_from_GeneWise(GeneWise * gw)
 {
   GeneWiseScore * out;
@@ -791,7 +849,7 @@ GeneWiseScore * GeneWiseScore_from_GeneWise(GeneWise * gw)
  * Return [UNKN ]  Undocumented return value [GeneWiseScoreSegment *]
  *
  */
-# line 747 "genewisemodel.dy"
+# line 786 "genewisemodel.dy"
 GeneWiseScoreSegment * GeneWiseScoreSegment_from_GeneWiseSegment(GeneWiseSegment * prev,GeneWiseSegment * seg)
 {
   GeneWiseScoreSegment * out;
@@ -845,7 +903,7 @@ GeneWiseScoreSegment * GeneWiseScoreSegment_from_GeneWiseSegment(GeneWiseSegment
 
 
 
-# line 813 "genewisemodel.c"
+# line 867 "genewisemodel.c"
 /* Function:  hard_link_GeneWiseSegment(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -888,6 +946,9 @@ GeneWiseSegment * GeneWiseSegment_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* match[GW_EMISSION_LEN] is an array: no default possible */ 
     /* insert[GW_EMISSION_LEN] is an array: no default possible */ 
     /* transition[GW_TRANSITION_LEN] is an array: no default possible */ 
@@ -910,6 +971,7 @@ GeneWiseSegment * GeneWiseSegment_alloc(void)
  */
 GeneWiseSegment * free_GeneWiseSegment(GeneWiseSegment * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -918,10 +980,18 @@ GeneWiseSegment * free_GeneWiseSegment(GeneWiseSegment * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -1175,6 +1245,9 @@ GeneWise * GeneWise_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->seg = NULL; 
     out->len = out->maxlen = 0;  
     out->name = NULL;    
@@ -1197,6 +1270,7 @@ GeneWise * GeneWise_alloc(void)
  */
 GeneWise * free_GeneWise(GeneWise * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1206,10 +1280,18 @@ GeneWise * free_GeneWise(GeneWise * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->seg != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->seg[i] != NULL) 
@@ -1268,6 +1350,9 @@ GeneWiseScoreSegment * GeneWiseScoreSegment_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* match[GW_EMISSION_LEN] is an array: no default possible */ 
     /* insert[GW_EMISSION_LEN] is an array: no default possible */ 
     /* transition[GW_TRANSITION_LEN] is an array: no default possible */ 
@@ -1290,6 +1375,7 @@ GeneWiseScoreSegment * GeneWiseScoreSegment_alloc(void)
  */
 GeneWiseScoreSegment * free_GeneWiseScoreSegment(GeneWiseScoreSegment * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1298,10 +1384,18 @@ GeneWiseScoreSegment * free_GeneWiseScoreSegment(GeneWiseScoreSegment * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -1555,6 +1649,9 @@ GeneWiseScore * GeneWiseScore_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->seg = NULL; 
     out->len = out->maxlen = 0;  
     out->name = NULL;    
@@ -1577,6 +1674,7 @@ GeneWiseScore * GeneWiseScore_alloc(void)
  */
 GeneWiseScore * free_GeneWiseScore(GeneWiseScore * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1586,10 +1684,18 @@ GeneWiseScore * free_GeneWiseScore(GeneWiseScore * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->seg != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->seg[i] != NULL) 
@@ -1648,6 +1754,9 @@ GeneWiseScoreFlat * GeneWiseScoreFlat_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->seg = NULL; 
     out->len = 0;    
 
