@@ -57,6 +57,9 @@ enum GeneWiseTransition {
  */
 struct Wise2_GeneWiseSegment {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Probability match[GW_EMISSION_LEN];  
     Probability insert[GW_EMISSION_LEN];     
     Probability transition[GW_TRANSITION_LEN];   
@@ -80,6 +83,9 @@ typedef struct Wise2_GeneWiseSegment Wise2_GeneWiseSegment;
  */
 struct Wise2_GeneWise {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     GeneWiseSegment ** seg;  
     int len;/* len for above seg  */ 
     int maxlen; /* maxlen for above seg */ 
@@ -102,6 +108,9 @@ typedef struct Wise2_GeneWise Wise2_GeneWise;
  */
 struct Wise2_GeneWiseScoreSegment {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Score match[GW_EMISSION_LEN];    
     Score insert[GW_EMISSION_LEN];   
     Score transition[GW_TRANSITION_LEN];     
@@ -123,6 +132,9 @@ typedef struct Wise2_GeneWiseScoreSegment Wise2_GeneWiseScoreSegment;
  */
 struct Wise2_GeneWiseScore {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     GeneWiseScoreSegment ** seg;     
     int len;/* len for above seg  */ 
     int maxlen; /* maxlen for above seg */ 
@@ -149,6 +161,9 @@ typedef struct Wise2_GeneWiseScore Wise2_GeneWiseScore;
  */
 struct Wise2_GeneWiseScoreFlat {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     GeneWiseScoreSegment * seg;  
     int len;     
     } ;  
@@ -169,6 +184,20 @@ typedef struct Wise2_GeneWiseScoreFlat Wise2_GeneWiseScoreFlat;
 
 
 
+/* Function:  pack_GeneWiseScore(gws)
+ *
+ * Descrip:    Packing up the GeneWise model into a byte structure
+ *
+ *
+ * Arg:        gws [UNKN ] Undocumented argument [GeneWiseScore *]
+ *
+ * Return [UNKN ]  Undocumented return value [char *]
+ *
+ */
+char * Wise2_pack_GeneWiseScore(GeneWiseScore * gws);
+#define pack_GeneWiseScore Wise2_pack_GeneWiseScore
+
+
 /* Function:  GeneWiseScoreFlat_from_GeneWiseScore(gws)
  *
  * Descrip:    This produces a flattened GeneWiseSegment structure
@@ -256,6 +285,25 @@ GeneWise * Wise2_GeneWise_from_ThreeStateModel_cdna(ThreeStateModel * tsm,cDNAPa
 #define GeneWise_from_ThreeStateModel_cdna Wise2_GeneWise_from_ThreeStateModel_cdna
 
 
+/* Function:  GeneWise_from_ThreeStateModel_setfactor(tsm,factor,cm,allN)
+ *
+ * Descrip:    This function makes a 
+ *             GeneWise model for the estwise
+ *             type algorithms
+ *
+ *
+ * Arg:           tsm [UNKN ] Undocumented argument [ThreeStateModel *]
+ * Arg:        factor [UNKN ] Undocumented argument [Probability]
+ * Arg:            cm [UNKN ] Undocumented argument [CodonMapper *]
+ * Arg:          allN [UNKN ] Undocumented argument [Probability]
+ *
+ * Return [UNKN ]  Undocumented return value [GeneWise *]
+ *
+ */
+GeneWise * Wise2_GeneWise_from_ThreeStateModel_setfactor(ThreeStateModel * tsm,Probability factor,CodonMapper * cm,Probability allN);
+#define GeneWise_from_ThreeStateModel_setfactor Wise2_GeneWise_from_ThreeStateModel_setfactor
+
+
 /* Function:  GeneWise_from_ThreeStateModel(tsm,gp,cm,allN,gwcm)
  *
  * Descrip:    This makes a genewise model from a 
