@@ -8,7 +8,11 @@ extern "C" {
 
 struct Wise2_GenomeWise9 {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     GenomeEvidenceSet* evi;  
@@ -47,6 +51,22 @@ struct thread_pool_holder_GenomeWise9 {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_GenomeWise9_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(GenomeWise9*,int,int,int);    
+    int (*access_special)(GenomeWise9*,int,int,int); 
+    } ;  
+/* GenomeWise9_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_GenomeWise9_access_func_holder
+typedef struct Wise2_GenomeWise9_access_func_holder Wise2_GenomeWise9_access_func_holder;
+#define GenomeWise9_access_func_holder Wise2_GenomeWise9_access_func_holder
+#define DYNAMITE_DEFINED_GenomeWise9_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -56,6 +76,35 @@ struct thread_pool_holder_GenomeWise9 {
 
 
 
+/* Function:  PackAln_read_Shatter_GenomeWise9(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GenomeWise9 *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_GenomeWise9(GenomeWise9 * mat);
+#define PackAln_read_Shatter_GenomeWise9 Wise2_PackAln_read_Shatter_GenomeWise9
+
+
+/* Function:  calculate_shatter_GenomeWise9(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GenomeWise9 matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [GenomeWise9 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_GenomeWise9(GenomeWise9 * mat,DPEnvelope * dpenv);
+#define calculate_shatter_GenomeWise9 Wise2_calculate_shatter_GenomeWise9
+
+
 /* Function:  search_GenomeWise9(dbsi,out,evi,targetdb,switchcost,newgenecost,non_start_codon,non_stop_codon,rndcodon)
  *
  * Descrip:    This function makes a database search of GenomeWise9
@@ -132,7 +181,7 @@ PackAln * Wise2_PackAln_bestmemory_GenomeWise9(GenomeEvidenceSet* evi,ComplexSeq
 #define PackAln_bestmemory_GenomeWise9 Wise2_PackAln_bestmemory_GenomeWise9
 
 
-/* Function:  allocate_Expl_GenomeWise9(evi,gen,switchcost,newgenecost,non_start_codon,non_stop_codon,rndcodon)
+/* Function:  allocate_Expl_GenomeWise9(evi,gen,switchcost,newgenecost,non_start_codon,non_stop_codon,rndcodon,dpri)
  *
  * Descrip:    This function allocates the GenomeWise9 structure
  *             and the basematrix area for explicit memory implementations
@@ -146,11 +195,12 @@ PackAln * Wise2_PackAln_bestmemory_GenomeWise9(GenomeEvidenceSet* evi,ComplexSeq
  * Arg:        non_start_codon [UNKN ] Resource [int]
  * Arg:         non_stop_codon [UNKN ] Resource [int]
  * Arg:               rndcodon [UNKN ] Resource [RandomCodonScore *]
+ * Arg:                   dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [GenomeWise9 *]
  *
  */
-GenomeWise9 * Wise2_allocate_Expl_GenomeWise9(GenomeEvidenceSet* evi,ComplexSequence* gen ,int switchcost,int newgenecost,int non_start_codon,int non_stop_codon,RandomCodonScore * rndcodon);
+GenomeWise9 * Wise2_allocate_Expl_GenomeWise9(GenomeEvidenceSet* evi,ComplexSequence* gen ,int switchcost,int newgenecost,int non_start_codon,int non_stop_codon,RandomCodonScore * rndcodon,DPRunImpl * dpri);
 #define allocate_Expl_GenomeWise9 Wise2_allocate_Expl_GenomeWise9
 
 
@@ -277,6 +327,21 @@ PackAln * Wise2_PackAln_read_Expl_GenomeWise9(GenomeWise9 * mat);
 #define PackAln_read_Expl_GenomeWise9 Wise2_PackAln_read_Expl_GenomeWise9
 
 
+/* Function:  PackAln_read_generic_GenomeWise9(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GenomeWise9 *]
+ * Arg:          h [UNKN ] Undocumented argument [GenomeWise9_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_GenomeWise9(GenomeWise9 * mat,GenomeWise9_access_func_holder h);
+#define PackAln_read_generic_GenomeWise9 Wise2_PackAln_read_generic_GenomeWise9
+
+
 /* Function:  calculate_GenomeWise9(mat)
  *
  * Descrip:    This function calculates the GenomeWise9 matrix when in explicit mode
@@ -292,6 +357,21 @@ boolean Wise2_calculate_GenomeWise9(GenomeWise9 * mat);
 #define calculate_GenomeWise9 Wise2_calculate_GenomeWise9
 
 
+/* Function:  calculate_dpenv_GenomeWise9(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GenomeWise9 matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] GenomeWise9 which contains explicit basematrix memory [GenomeWise9 *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_GenomeWise9(GenomeWise9 * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_GenomeWise9 Wise2_calculate_dpenv_GenomeWise9
+
+
 /* Function:  GenomeWise9_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -328,6 +408,12 @@ GenomeWise9 * Wise2_free_GenomeWise9(GenomeWise9 * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_GenomeWise9_shatter_access_main(GenomeWise9 * mat,int i,int j,int state);
+#define GenomeWise9_shatter_access_main Wise2_GenomeWise9_shatter_access_main
+int Wise2_GenomeWise9_shatter_access_special(GenomeWise9 * mat,int i,int j,int state);
+#define GenomeWise9_shatter_access_special Wise2_GenomeWise9_shatter_access_special
+void * Wise2_thread_loop_GenomeWise9(void * ptr);
+#define thread_loop_GenomeWise9 Wise2_thread_loop_GenomeWise9
 int Wise2_score_only_GenomeWise9(GenomeEvidenceSet* evi,ComplexSequence* gen ,int switchcost,int newgenecost,int non_start_codon,int non_stop_codon,RandomCodonScore * rndcodon);
 #define score_only_GenomeWise9 Wise2_score_only_GenomeWise9
 GenomeWise9 * Wise2_allocate_GenomeWise9_only(GenomeEvidenceSet* evi,ComplexSequence* gen ,int switchcost,int newgenecost,int non_start_codon,int non_stop_codon,RandomCodonScore * rndcodon);
@@ -362,21 +448,25 @@ void Wise2_run_up_dc_GenomeWise9(GenomeWise9 * mat,int starti,int stopi,int star
 #define run_up_dc_GenomeWise9 Wise2_run_up_dc_GenomeWise9
 void Wise2_init_dc_GenomeWise9(GenomeWise9 * mat);
 #define init_dc_GenomeWise9 Wise2_init_dc_GenomeWise9
-boolean Wise2_dc_start_end_calculate_GenomeWise9(GenomeWise9 * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_GenomeWise9 Wise2_dc_start_end_calculate_GenomeWise9
 int Wise2_start_end_find_end_GenomeWise9(GenomeWise9 * mat,int * endj);
 #define start_end_find_end_GenomeWise9 Wise2_start_end_find_end_GenomeWise9
+boolean Wise2_dc_optimised_start_end_calc_GenomeWise9(GenomeWise9 *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_GenomeWise9 Wise2_dc_optimised_start_end_calc_GenomeWise9
 void Wise2_init_start_end_linear_GenomeWise9(GenomeWise9 * mat);
 #define init_start_end_linear_GenomeWise9 Wise2_init_start_end_linear_GenomeWise9
 AlnConvertSet * Wise2_AlnConvertSet_GenomeWise9(void);
 #define AlnConvertSet_GenomeWise9 Wise2_AlnConvertSet_GenomeWise9
-int Wise2_find_end_GenomeWise9(GenomeWise9 * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_GenomeWise9_explicit_access_main(GenomeWise9 * mat,int i,int j,int state);
+#define GenomeWise9_explicit_access_main Wise2_GenomeWise9_explicit_access_main
+int Wise2_GenomeWise9_explicit_access_special(GenomeWise9 * mat,int i,int j,int state);
+#define GenomeWise9_explicit_access_special Wise2_GenomeWise9_explicit_access_special
+int Wise2_find_end_GenomeWise9(GenomeWise9 * mat,int * ri,int * rj,int * state,boolean * isspecial,GenomeWise9_access_func_holder h);
 #define find_end_GenomeWise9 Wise2_find_end_GenomeWise9
 void Wise2_GenomeWise9_debug_show_matrix(GenomeWise9 * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define GenomeWise9_debug_show_matrix Wise2_GenomeWise9_debug_show_matrix
-int Wise2_max_calc_GenomeWise9(GenomeWise9 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_GenomeWise9(GenomeWise9 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GenomeWise9_access_func_holder h);
 #define max_calc_GenomeWise9 Wise2_max_calc_GenomeWise9
-int Wise2_max_calc_special_GenomeWise9(GenomeWise9 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_GenomeWise9(GenomeWise9 * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GenomeWise9_access_func_holder h);
 #define max_calc_special_GenomeWise9 Wise2_max_calc_special_GenomeWise9
 
 #ifdef _cplusplus
