@@ -9,7 +9,11 @@ extern "C" {
 
 struct Wise2_GeneLiteModel {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     GwLiteScore* query;  
@@ -42,6 +46,22 @@ struct thread_pool_holder_GeneLiteModel {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_GeneLiteModel_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(GeneLiteModel*,int,int,int);  
+    int (*access_special)(GeneLiteModel*,int,int,int);   
+    } ;  
+/* GeneLiteModel_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_GeneLiteModel_access_func_holder
+typedef struct Wise2_GeneLiteModel_access_func_holder Wise2_GeneLiteModel_access_func_holder;
+#define GeneLiteModel_access_func_holder Wise2_GeneLiteModel_access_func_holder
+#define DYNAMITE_DEFINED_GeneLiteModel_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -51,6 +71,35 @@ struct thread_pool_holder_GeneLiteModel {
 
 
 
+/* Function:  PackAln_read_Shatter_GeneLiteModel(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneLiteModel *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_GeneLiteModel(GeneLiteModel * mat);
+#define PackAln_read_Shatter_GeneLiteModel Wise2_PackAln_read_Shatter_GeneLiteModel
+
+
+/* Function:  calculate_shatter_GeneLiteModel(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneLiteModel matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [GeneLiteModel *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_GeneLiteModel(GeneLiteModel * mat,DPEnvelope * dpenv);
+#define calculate_shatter_GeneLiteModel Wise2_calculate_shatter_GeneLiteModel
+
+
 /* Function:  search_GeneLiteModel(dbsi,out,querydb,targetdb,gp)
  *
  * Descrip:    This function makes a database search of GeneLiteModel
@@ -115,7 +164,7 @@ PackAln * Wise2_PackAln_bestmemory_GeneLiteModel(GwLiteScore* query,ComplexSeque
 #define PackAln_bestmemory_GeneLiteModel Wise2_PackAln_bestmemory_GeneLiteModel
 
 
-/* Function:  allocate_Expl_GeneLiteModel(query,target,gp)
+/* Function:  allocate_Expl_GeneLiteModel(query,target,gp,dpri)
  *
  * Descrip:    This function allocates the GeneLiteModel structure
  *             and the basematrix area for explicit memory implementations
@@ -125,11 +174,12 @@ PackAln * Wise2_PackAln_bestmemory_GeneLiteModel(GwLiteScore* query,ComplexSeque
  * Arg:         query [UNKN ] query data structure [GwLiteScore*]
  * Arg:        target [UNKN ] target data structure [ComplexSequence*]
  * Arg:            gp [UNKN ] Resource [GeneParser4Score *]
+ * Arg:          dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [GeneLiteModel *]
  *
  */
-GeneLiteModel * Wise2_allocate_Expl_GeneLiteModel(GwLiteScore* query,ComplexSequence* target ,GeneParser4Score * gp);
+GeneLiteModel * Wise2_allocate_Expl_GeneLiteModel(GwLiteScore* query,ComplexSequence* target ,GeneParser4Score * gp,DPRunImpl * dpri);
 #define allocate_Expl_GeneLiteModel Wise2_allocate_Expl_GeneLiteModel
 
 
@@ -252,6 +302,21 @@ PackAln * Wise2_PackAln_read_Expl_GeneLiteModel(GeneLiteModel * mat);
 #define PackAln_read_Expl_GeneLiteModel Wise2_PackAln_read_Expl_GeneLiteModel
 
 
+/* Function:  PackAln_read_generic_GeneLiteModel(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [GeneLiteModel *]
+ * Arg:          h [UNKN ] Undocumented argument [GeneLiteModel_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_GeneLiteModel(GeneLiteModel * mat,GeneLiteModel_access_func_holder h);
+#define PackAln_read_generic_GeneLiteModel Wise2_PackAln_read_generic_GeneLiteModel
+
+
 /* Function:  calculate_GeneLiteModel(mat)
  *
  * Descrip:    This function calculates the GeneLiteModel matrix when in explicit mode
@@ -267,6 +332,21 @@ boolean Wise2_calculate_GeneLiteModel(GeneLiteModel * mat);
 #define calculate_GeneLiteModel Wise2_calculate_GeneLiteModel
 
 
+/* Function:  calculate_dpenv_GeneLiteModel(mat,dpenv)
+ *
+ * Descrip:    This function calculates the GeneLiteModel matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] GeneLiteModel which contains explicit basematrix memory [GeneLiteModel *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_GeneLiteModel(GeneLiteModel * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_GeneLiteModel Wise2_calculate_dpenv_GeneLiteModel
+
+
 /* Function:  GeneLiteModel_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -303,6 +383,10 @@ GeneLiteModel * Wise2_free_GeneLiteModel(GeneLiteModel * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_GeneLiteModel_shatter_access_main(GeneLiteModel * mat,int i,int j,int state);
+#define GeneLiteModel_shatter_access_main Wise2_GeneLiteModel_shatter_access_main
+int Wise2_GeneLiteModel_shatter_access_special(GeneLiteModel * mat,int i,int j,int state);
+#define GeneLiteModel_shatter_access_special Wise2_GeneLiteModel_shatter_access_special
 void * Wise2_thread_loop_GeneLiteModel(void * ptr);
 #define thread_loop_GeneLiteModel Wise2_thread_loop_GeneLiteModel
 int Wise2_score_only_GeneLiteModel(GwLiteScore* query,ComplexSequence* target ,GeneParser4Score * gp);
@@ -339,21 +423,25 @@ void Wise2_run_up_dc_GeneLiteModel(GeneLiteModel * mat,int starti,int stopi,int
 #define run_up_dc_GeneLiteModel Wise2_run_up_dc_GeneLiteModel
 void Wise2_init_dc_GeneLiteModel(GeneLiteModel * mat);
 #define init_dc_GeneLiteModel Wise2_init_dc_GeneLiteModel
-boolean Wise2_dc_start_end_calculate_GeneLiteModel(GeneLiteModel * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_GeneLiteModel Wise2_dc_start_end_calculate_GeneLiteModel
 int Wise2_start_end_find_end_GeneLiteModel(GeneLiteModel * mat,int * endj);
 #define start_end_find_end_GeneLiteModel Wise2_start_end_find_end_GeneLiteModel
+boolean Wise2_dc_optimised_start_end_calc_GeneLiteModel(GeneLiteModel *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_GeneLiteModel Wise2_dc_optimised_start_end_calc_GeneLiteModel
 void Wise2_init_start_end_linear_GeneLiteModel(GeneLiteModel * mat);
 #define init_start_end_linear_GeneLiteModel Wise2_init_start_end_linear_GeneLiteModel
 AlnConvertSet * Wise2_AlnConvertSet_GeneLiteModel(void);
 #define AlnConvertSet_GeneLiteModel Wise2_AlnConvertSet_GeneLiteModel
-int Wise2_find_end_GeneLiteModel(GeneLiteModel * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_GeneLiteModel_explicit_access_main(GeneLiteModel * mat,int i,int j,int state);
+#define GeneLiteModel_explicit_access_main Wise2_GeneLiteModel_explicit_access_main
+int Wise2_GeneLiteModel_explicit_access_special(GeneLiteModel * mat,int i,int j,int state);
+#define GeneLiteModel_explicit_access_special Wise2_GeneLiteModel_explicit_access_special
+int Wise2_find_end_GeneLiteModel(GeneLiteModel * mat,int * ri,int * rj,int * state,boolean * isspecial,GeneLiteModel_access_func_holder h);
 #define find_end_GeneLiteModel Wise2_find_end_GeneLiteModel
 void Wise2_GeneLiteModel_debug_show_matrix(GeneLiteModel * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define GeneLiteModel_debug_show_matrix Wise2_GeneLiteModel_debug_show_matrix
-int Wise2_max_calc_GeneLiteModel(GeneLiteModel * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_GeneLiteModel(GeneLiteModel * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneLiteModel_access_func_holder h);
 #define max_calc_GeneLiteModel Wise2_max_calc_GeneLiteModel
-int Wise2_max_calc_special_GeneLiteModel(GeneLiteModel * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_GeneLiteModel(GeneLiteModel * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,GeneLiteModel_access_func_holder h);
 #define max_calc_special_GeneLiteModel Wise2_max_calc_special_GeneLiteModel
 
 #ifdef _cplusplus
