@@ -198,6 +198,9 @@ GwLiteSegment * GwLiteSegment_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* match[GWL_EMISSION_LEN] is an array: no default possible */ 
     /* insert[GWL_EMISSION_LEN] is an array: no default possible */ 
     /* transition[GWL_TRANSITION_LEN] is an array: no default possible */ 
@@ -220,6 +223,7 @@ GwLiteSegment * GwLiteSegment_alloc(void)
  */
 GwLiteSegment * free_GwLiteSegment(GwLiteSegment * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -228,10 +232,18 @@ GwLiteSegment * free_GwLiteSegment(GwLiteSegment * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -485,6 +497,9 @@ GwLite * GwLite_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->seg = NULL; 
     out->len = out->maxlen = 0;  
     out->name = NULL;    
@@ -507,6 +522,7 @@ GwLite * GwLite_alloc(void)
  */
 GwLite * free_GwLite(GwLite * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -516,10 +532,18 @@ GwLite * free_GwLite(GwLite * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->seg != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->seg[i] != NULL) 
@@ -578,6 +602,9 @@ GwLiteSegmentScore * GwLiteSegmentScore_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* match[GWL_EMISSION_LEN] is an array: no default possible */ 
     /* insert[GWL_EMISSION_LEN] is an array: no default possible */ 
     /* transition[GWL_TRANSITION_LEN] is an array: no default possible */ 
@@ -600,6 +627,7 @@ GwLiteSegmentScore * GwLiteSegmentScore_alloc(void)
  */
 GwLiteSegmentScore * free_GwLiteSegmentScore(GwLiteSegmentScore * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -608,10 +636,18 @@ GwLiteSegmentScore * free_GwLiteSegmentScore(GwLiteSegmentScore * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -865,6 +901,9 @@ GwLiteScore * GwLiteScore_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->seg = NULL; 
     out->len = out->maxlen = 0;  
     out->name = NULL;    
@@ -887,6 +926,7 @@ GwLiteScore * GwLiteScore_alloc(void)
  */
 GwLiteScore * free_GwLiteScore(GwLiteScore * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -896,10 +936,18 @@ GwLiteScore * free_GwLiteScore(GwLiteScore * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->seg != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->seg[i] != NULL) 
