@@ -8,7 +8,11 @@ extern "C" {
 
 struct Wise2_LocalCloneWise {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     MappedCloneSet * q;  
@@ -49,6 +53,22 @@ struct thread_pool_holder_LocalCloneWise {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_LocalCloneWise_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(LocalCloneWise*,int,int,int); 
+    int (*access_special)(LocalCloneWise*,int,int,int);  
+    } ;  
+/* LocalCloneWise_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_LocalCloneWise_access_func_holder
+typedef struct Wise2_LocalCloneWise_access_func_holder Wise2_LocalCloneWise_access_func_holder;
+#define LocalCloneWise_access_func_holder Wise2_LocalCloneWise_access_func_holder
+#define DYNAMITE_DEFINED_LocalCloneWise_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -58,6 +78,35 @@ struct thread_pool_holder_LocalCloneWise {
 
 
 
+/* Function:  PackAln_read_Shatter_LocalCloneWise(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [LocalCloneWise *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_LocalCloneWise(LocalCloneWise * mat);
+#define PackAln_read_Shatter_LocalCloneWise Wise2_PackAln_read_Shatter_LocalCloneWise
+
+
+/* Function:  calculate_shatter_LocalCloneWise(mat,dpenv)
+ *
+ * Descrip:    This function calculates the LocalCloneWise matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [LocalCloneWise *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_LocalCloneWise(LocalCloneWise * mat,DPEnvelope * dpenv);
+#define calculate_shatter_LocalCloneWise Wise2_calculate_shatter_LocalCloneWise
+
+
 /* Function:  search_LocalCloneWise(dbsi,out,q,t,match,target_skip_start,target_skip,query_skip_start,query_skip,spread,target_special_s)
  *
  * Descrip:    This function makes a database search of LocalCloneWise
@@ -140,7 +189,7 @@ PackAln * Wise2_PackAln_bestmemory_LocalCloneWise(MappedCloneSet * q,MappedClone
 #define PackAln_bestmemory_LocalCloneWise Wise2_PackAln_bestmemory_LocalCloneWise
 
 
-/* Function:  allocate_Expl_LocalCloneWise(q,t,match,target_skip_start,target_skip,query_skip_start,query_skip,spread,target_special_s)
+/* Function:  allocate_Expl_LocalCloneWise(q,t,match,target_skip_start,target_skip,query_skip_start,query_skip,spread,target_special_s,dpri)
  *
  * Descrip:    This function allocates the LocalCloneWise structure
  *             and the basematrix area for explicit memory implementations
@@ -156,11 +205,12 @@ PackAln * Wise2_PackAln_bestmemory_LocalCloneWise(MappedCloneSet * q,MappedClone
  * Arg:               query_skip [UNKN ] Resource [Score]
  * Arg:                   spread [UNKN ] Resource [int]
  * Arg:         target_special_s [UNKN ] Resource [int]
+ * Arg:                     dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [LocalCloneWise *]
  *
  */
-LocalCloneWise * Wise2_allocate_Expl_LocalCloneWise(MappedCloneSet * q,MappedCloneSet * t ,MappedCloneMatch* match,Score target_skip_start,Score target_skip,Score query_skip_start,Score query_skip,int spread,int target_special_s);
+LocalCloneWise * Wise2_allocate_Expl_LocalCloneWise(MappedCloneSet * q,MappedCloneSet * t ,MappedCloneMatch* match,Score target_skip_start,Score target_skip,Score query_skip_start,Score query_skip,int spread,int target_special_s,DPRunImpl * dpri);
 #define allocate_Expl_LocalCloneWise Wise2_allocate_Expl_LocalCloneWise
 
 
@@ -289,6 +339,21 @@ PackAln * Wise2_PackAln_read_Expl_LocalCloneWise(LocalCloneWise * mat);
 #define PackAln_read_Expl_LocalCloneWise Wise2_PackAln_read_Expl_LocalCloneWise
 
 
+/* Function:  PackAln_read_generic_LocalCloneWise(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [LocalCloneWise *]
+ * Arg:          h [UNKN ] Undocumented argument [LocalCloneWise_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_LocalCloneWise(LocalCloneWise * mat,LocalCloneWise_access_func_holder h);
+#define PackAln_read_generic_LocalCloneWise Wise2_PackAln_read_generic_LocalCloneWise
+
+
 /* Function:  calculate_LocalCloneWise(mat)
  *
  * Descrip:    This function calculates the LocalCloneWise matrix when in explicit mode
@@ -304,6 +369,21 @@ boolean Wise2_calculate_LocalCloneWise(LocalCloneWise * mat);
 #define calculate_LocalCloneWise Wise2_calculate_LocalCloneWise
 
 
+/* Function:  calculate_dpenv_LocalCloneWise(mat,dpenv)
+ *
+ * Descrip:    This function calculates the LocalCloneWise matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] LocalCloneWise which contains explicit basematrix memory [LocalCloneWise *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_LocalCloneWise(LocalCloneWise * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_LocalCloneWise Wise2_calculate_dpenv_LocalCloneWise
+
+
 /* Function:  LocalCloneWise_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -340,6 +420,12 @@ LocalCloneWise * Wise2_free_LocalCloneWise(LocalCloneWise * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_LocalCloneWise_shatter_access_main(LocalCloneWise * mat,int i,int j,int state);
+#define LocalCloneWise_shatter_access_main Wise2_LocalCloneWise_shatter_access_main
+int Wise2_LocalCloneWise_shatter_access_special(LocalCloneWise * mat,int i,int j,int state);
+#define LocalCloneWise_shatter_access_special Wise2_LocalCloneWise_shatter_access_special
+void * Wise2_thread_loop_LocalCloneWise(void * ptr);
+#define thread_loop_LocalCloneWise Wise2_thread_loop_LocalCloneWise
 int Wise2_score_only_LocalCloneWise(MappedCloneSet * q,MappedCloneSet * t ,MappedCloneMatch* match,Score target_skip_start,Score target_skip,Score query_skip_start,Score query_skip,int spread,int target_special_s);
 #define score_only_LocalCloneWise Wise2_score_only_LocalCloneWise
 LocalCloneWise * Wise2_allocate_LocalCloneWise_only(MappedCloneSet * q,MappedCloneSet * t ,MappedCloneMatch* match,Score target_skip_start,Score target_skip,Score query_skip_start,Score query_skip,int spread,int target_special_s);
@@ -374,21 +460,25 @@ void Wise2_run_up_dc_LocalCloneWise(LocalCloneWise * mat,int starti,int stopi,in
 #define run_up_dc_LocalCloneWise Wise2_run_up_dc_LocalCloneWise
 void Wise2_init_dc_LocalCloneWise(LocalCloneWise * mat);
 #define init_dc_LocalCloneWise Wise2_init_dc_LocalCloneWise
-boolean Wise2_dc_start_end_calculate_LocalCloneWise(LocalCloneWise * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_LocalCloneWise Wise2_dc_start_end_calculate_LocalCloneWise
 int Wise2_start_end_find_end_LocalCloneWise(LocalCloneWise * mat,int * endj);
 #define start_end_find_end_LocalCloneWise Wise2_start_end_find_end_LocalCloneWise
+boolean Wise2_dc_optimised_start_end_calc_LocalCloneWise(LocalCloneWise *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_LocalCloneWise Wise2_dc_optimised_start_end_calc_LocalCloneWise
 void Wise2_init_start_end_linear_LocalCloneWise(LocalCloneWise * mat);
 #define init_start_end_linear_LocalCloneWise Wise2_init_start_end_linear_LocalCloneWise
 AlnConvertSet * Wise2_AlnConvertSet_LocalCloneWise(void);
 #define AlnConvertSet_LocalCloneWise Wise2_AlnConvertSet_LocalCloneWise
-int Wise2_find_end_LocalCloneWise(LocalCloneWise * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_LocalCloneWise_explicit_access_main(LocalCloneWise * mat,int i,int j,int state);
+#define LocalCloneWise_explicit_access_main Wise2_LocalCloneWise_explicit_access_main
+int Wise2_LocalCloneWise_explicit_access_special(LocalCloneWise * mat,int i,int j,int state);
+#define LocalCloneWise_explicit_access_special Wise2_LocalCloneWise_explicit_access_special
+int Wise2_find_end_LocalCloneWise(LocalCloneWise * mat,int * ri,int * rj,int * state,boolean * isspecial,LocalCloneWise_access_func_holder h);
 #define find_end_LocalCloneWise Wise2_find_end_LocalCloneWise
 void Wise2_LocalCloneWise_debug_show_matrix(LocalCloneWise * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define LocalCloneWise_debug_show_matrix Wise2_LocalCloneWise_debug_show_matrix
-int Wise2_max_calc_LocalCloneWise(LocalCloneWise * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_LocalCloneWise(LocalCloneWise * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,LocalCloneWise_access_func_holder h);
 #define max_calc_LocalCloneWise Wise2_max_calc_LocalCloneWise
-int Wise2_max_calc_special_LocalCloneWise(LocalCloneWise * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_LocalCloneWise(LocalCloneWise * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,LocalCloneWise_access_func_holder h);
 #define max_calc_special_LocalCloneWise Wise2_max_calc_special_LocalCloneWise
 
 #ifdef _cplusplus
