@@ -470,6 +470,9 @@ MappedClone * MappedClone_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->clone_name = NULL;  
     out->accession = NULL;   
     out->contig = NULL;  
@@ -495,6 +498,7 @@ MappedClone * MappedClone_alloc(void)
  */
 MappedClone * free_MappedClone(MappedClone * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -503,10 +507,18 @@ MappedClone * free_MappedClone(MappedClone * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->clone_name != NULL) 
       ckfree(obj->clone_name);   
     if( obj->accession != NULL)  
@@ -766,6 +778,9 @@ MappedCloneSet * MappedCloneSet_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->clone = NULL;   
     out->len = out->maxlen = 0;  
     out->length = 0; 
@@ -789,6 +804,7 @@ MappedCloneSet * MappedCloneSet_alloc(void)
  */
 MappedCloneSet * free_MappedCloneSet(MappedCloneSet * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -798,10 +814,18 @@ MappedCloneSet * free_MappedCloneSet(MappedCloneSet * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->clone != NULL)  {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->clone[i] != NULL)   
@@ -963,6 +987,9 @@ MappedCloneMatch * MappedCloneMatch_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->matrix = NULL;  
     out->leni=out->maxleni=0;    
     out->lenj=out->maxlenj=0;    
@@ -987,6 +1014,7 @@ MappedCloneMatch * MappedCloneMatch_alloc(void)
  */
 MappedCloneMatch * free_MappedCloneMatch(MappedCloneMatch * obj) 
 {
+    int return_early = 0;    
     int i;   
     int j;   
 
@@ -997,10 +1025,18 @@ MappedCloneMatch * free_MappedCloneMatch(MappedCloneMatch * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->matrix != NULL) {  
       for(i=0;i<obj->leni;i++)   {  
         if( obj->matrix[i] != NULL)  
