@@ -302,6 +302,9 @@ MatchSummary * MatchSummary_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->bits = 0;   
     out->qname = NULL;   
     out->tname = NULL;   
@@ -332,6 +335,7 @@ MatchSummary * MatchSummary_alloc(void)
  */
 MatchSummary * free_MatchSummary(MatchSummary * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -340,10 +344,18 @@ MatchSummary * free_MatchSummary(MatchSummary * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->qname != NULL)  
       ckfree(obj->qname);    
     if( obj->tname != NULL)  
@@ -601,6 +613,9 @@ MatchSummarySet * MatchSummarySet_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->ms = NULL;  
     out->len = out->maxlen = 0;  
 
@@ -622,6 +637,7 @@ MatchSummarySet * MatchSummarySet_alloc(void)
  */
 MatchSummarySet * free_MatchSummarySet(MatchSummarySet * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -631,10 +647,18 @@ MatchSummarySet * free_MatchSummarySet(MatchSummarySet * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->ms != NULL) {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->ms[i] != NULL)  
