@@ -7,7 +7,11 @@ extern "C" {
 
 struct Wise2_ProteinBlockAligner {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     ComplexSequence* q;  
@@ -50,6 +54,22 @@ struct thread_pool_holder_ProteinBlockAligner {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_ProteinBlockAligner_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(ProteinBlockAligner*,int,int,int);    
+    int (*access_special)(ProteinBlockAligner*,int,int,int); 
+    } ;  
+/* ProteinBlockAligner_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_ProteinBlockAligner_access_func_holder
+typedef struct Wise2_ProteinBlockAligner_access_func_holder Wise2_ProteinBlockAligner_access_func_holder;
+#define ProteinBlockAligner_access_func_holder Wise2_ProteinBlockAligner_access_func_holder
+#define DYNAMITE_DEFINED_ProteinBlockAligner_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -59,6 +79,35 @@ struct thread_pool_holder_ProteinBlockAligner {
 
 
 
+/* Function:  PackAln_read_Shatter_ProteinBlockAligner(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [ProteinBlockAligner *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_ProteinBlockAligner(ProteinBlockAligner * mat);
+#define PackAln_read_Shatter_ProteinBlockAligner Wise2_PackAln_read_Shatter_ProteinBlockAligner
+
+
+/* Function:  calculate_shatter_ProteinBlockAligner(mat,dpenv)
+ *
+ * Descrip:    This function calculates the ProteinBlockAligner matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [ProteinBlockAligner *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_ProteinBlockAligner(ProteinBlockAligner * mat,DPEnvelope * dpenv);
+#define calculate_shatter_ProteinBlockAligner Wise2_calculate_shatter_ProteinBlockAligner
+
+
 /* Function:  search_ProteinBlockAligner(dbsi,out,querydb,targetdb,m,bentry,bexit,bfor_trans,b_self_trans,b3exit)
  *
  * Descrip:    This function makes a database search of ProteinBlockAligner
@@ -138,7 +187,7 @@ PackAln * Wise2_PackAln_bestmemory_ProteinBlockAligner(ComplexSequence* q,Comple
 #define PackAln_bestmemory_ProteinBlockAligner Wise2_PackAln_bestmemory_ProteinBlockAligner
 
 
-/* Function:  allocate_Expl_ProteinBlockAligner(q,t,m,bentry,bexit,bfor_trans,b_self_trans,b3exit)
+/* Function:  allocate_Expl_ProteinBlockAligner(q,t,m,bentry,bexit,bfor_trans,b_self_trans,b3exit,dpri)
  *
  * Descrip:    This function allocates the ProteinBlockAligner structure
  *             and the basematrix area for explicit memory implementations
@@ -153,11 +202,12 @@ PackAln * Wise2_PackAln_bestmemory_ProteinBlockAligner(ComplexSequence* q,Comple
  * Arg:          bfor_trans [UNKN ] Resource [Score]
  * Arg:        b_self_trans [UNKN ] Resource [Score]
  * Arg:              b3exit [UNKN ] Resource [Score]
+ * Arg:                dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [ProteinBlockAligner *]
  *
  */
-ProteinBlockAligner * Wise2_allocate_Expl_ProteinBlockAligner(ComplexSequence* q,ComplexSequence* t ,CompMat* m,Score bentry,Score bexit,Score bfor_trans,Score b_self_trans,Score b3exit);
+ProteinBlockAligner * Wise2_allocate_Expl_ProteinBlockAligner(ComplexSequence* q,ComplexSequence* t ,CompMat* m,Score bentry,Score bexit,Score bfor_trans,Score b_self_trans,Score b3exit,DPRunImpl * dpri);
 #define allocate_Expl_ProteinBlockAligner Wise2_allocate_Expl_ProteinBlockAligner
 
 
@@ -285,6 +335,21 @@ PackAln * Wise2_PackAln_read_Expl_ProteinBlockAligner(ProteinBlockAligner * mat)
 #define PackAln_read_Expl_ProteinBlockAligner Wise2_PackAln_read_Expl_ProteinBlockAligner
 
 
+/* Function:  PackAln_read_generic_ProteinBlockAligner(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [ProteinBlockAligner *]
+ * Arg:          h [UNKN ] Undocumented argument [ProteinBlockAligner_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_ProteinBlockAligner(ProteinBlockAligner * mat,ProteinBlockAligner_access_func_holder h);
+#define PackAln_read_generic_ProteinBlockAligner Wise2_PackAln_read_generic_ProteinBlockAligner
+
+
 /* Function:  calculate_ProteinBlockAligner(mat)
  *
  * Descrip:    This function calculates the ProteinBlockAligner matrix when in explicit mode
@@ -300,6 +365,21 @@ boolean Wise2_calculate_ProteinBlockAligner(ProteinBlockAligner * mat);
 #define calculate_ProteinBlockAligner Wise2_calculate_ProteinBlockAligner
 
 
+/* Function:  calculate_dpenv_ProteinBlockAligner(mat,dpenv)
+ *
+ * Descrip:    This function calculates the ProteinBlockAligner matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] ProteinBlockAligner which contains explicit basematrix memory [ProteinBlockAligner *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_ProteinBlockAligner(ProteinBlockAligner * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_ProteinBlockAligner Wise2_calculate_dpenv_ProteinBlockAligner
+
+
 /* Function:  ProteinBlockAligner_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -336,6 +416,10 @@ ProteinBlockAligner * Wise2_free_ProteinBlockAligner(ProteinBlockAligner * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_ProteinBlockAligner_shatter_access_main(ProteinBlockAligner * mat,int i,int j,int state);
+#define ProteinBlockAligner_shatter_access_main Wise2_ProteinBlockAligner_shatter_access_main
+int Wise2_ProteinBlockAligner_shatter_access_special(ProteinBlockAligner * mat,int i,int j,int state);
+#define ProteinBlockAligner_shatter_access_special Wise2_ProteinBlockAligner_shatter_access_special
 void * Wise2_thread_loop_ProteinBlockAligner(void * ptr);
 #define thread_loop_ProteinBlockAligner Wise2_thread_loop_ProteinBlockAligner
 int Wise2_score_only_ProteinBlockAligner(ComplexSequence* q,ComplexSequence* t ,CompMat* m,Score bentry,Score bexit,Score bfor_trans,Score b_self_trans,Score b3exit);
@@ -372,21 +456,25 @@ void Wise2_run_up_dc_ProteinBlockAligner(ProteinBlockAligner * mat,int starti,in
 #define run_up_dc_ProteinBlockAligner Wise2_run_up_dc_ProteinBlockAligner
 void Wise2_init_dc_ProteinBlockAligner(ProteinBlockAligner * mat);
 #define init_dc_ProteinBlockAligner Wise2_init_dc_ProteinBlockAligner
-boolean Wise2_dc_start_end_calculate_ProteinBlockAligner(ProteinBlockAligner * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_ProteinBlockAligner Wise2_dc_start_end_calculate_ProteinBlockAligner
 int Wise2_start_end_find_end_ProteinBlockAligner(ProteinBlockAligner * mat,int * endj);
 #define start_end_find_end_ProteinBlockAligner Wise2_start_end_find_end_ProteinBlockAligner
+boolean Wise2_dc_optimised_start_end_calc_ProteinBlockAligner(ProteinBlockAligner *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_ProteinBlockAligner Wise2_dc_optimised_start_end_calc_ProteinBlockAligner
 void Wise2_init_start_end_linear_ProteinBlockAligner(ProteinBlockAligner * mat);
 #define init_start_end_linear_ProteinBlockAligner Wise2_init_start_end_linear_ProteinBlockAligner
 AlnConvertSet * Wise2_AlnConvertSet_ProteinBlockAligner(void);
 #define AlnConvertSet_ProteinBlockAligner Wise2_AlnConvertSet_ProteinBlockAligner
-int Wise2_find_end_ProteinBlockAligner(ProteinBlockAligner * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_ProteinBlockAligner_explicit_access_main(ProteinBlockAligner * mat,int i,int j,int state);
+#define ProteinBlockAligner_explicit_access_main Wise2_ProteinBlockAligner_explicit_access_main
+int Wise2_ProteinBlockAligner_explicit_access_special(ProteinBlockAligner * mat,int i,int j,int state);
+#define ProteinBlockAligner_explicit_access_special Wise2_ProteinBlockAligner_explicit_access_special
+int Wise2_find_end_ProteinBlockAligner(ProteinBlockAligner * mat,int * ri,int * rj,int * state,boolean * isspecial,ProteinBlockAligner_access_func_holder h);
 #define find_end_ProteinBlockAligner Wise2_find_end_ProteinBlockAligner
 void Wise2_ProteinBlockAligner_debug_show_matrix(ProteinBlockAligner * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define ProteinBlockAligner_debug_show_matrix Wise2_ProteinBlockAligner_debug_show_matrix
-int Wise2_max_calc_ProteinBlockAligner(ProteinBlockAligner * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_ProteinBlockAligner(ProteinBlockAligner * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,ProteinBlockAligner_access_func_holder h);
 #define max_calc_ProteinBlockAligner Wise2_max_calc_ProteinBlockAligner
-int Wise2_max_calc_special_ProteinBlockAligner(ProteinBlockAligner * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_ProteinBlockAligner(ProteinBlockAligner * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,ProteinBlockAligner_access_func_holder h);
 #define max_calc_special_ProteinBlockAligner Wise2_max_calc_special_ProteinBlockAligner
 
 #ifdef _cplusplus
