@@ -247,6 +247,9 @@ PfamHmmer1Entry * PfamHmmer1Entry_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->entryname = NULL;   
     out->is_random = FALSE;  
     out->is_hmmls = FALSE;   
@@ -270,6 +273,7 @@ PfamHmmer1Entry * PfamHmmer1Entry_alloc(void)
  */
 PfamHmmer1Entry * free_PfamHmmer1Entry(PfamHmmer1Entry * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -278,10 +282,18 @@ PfamHmmer1Entry * free_PfamHmmer1Entry(PfamHmmer1Entry * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->entryname != NULL)  
       ckfree(obj->entryname);    
 
@@ -537,6 +549,9 @@ PfamHmmer1DB * PfamHmmer1DB_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->en = NULL;  
     out->len = out->maxlen = 0;  
     out->dirname = NULL; 
@@ -561,6 +576,7 @@ PfamHmmer1DB * PfamHmmer1DB_alloc(void)
  */
 PfamHmmer1DB * free_PfamHmmer1DB(PfamHmmer1DB * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -570,10 +586,18 @@ PfamHmmer1DB * free_PfamHmmer1DB(PfamHmmer1DB * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->en != NULL) {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->en[i] != NULL)  
