@@ -81,25 +81,54 @@ PhasedProteinPara * new_PhasedProteinPara_from_argv(int * argc,char ** argv)
     out->intron_file = stringalloc(temp);
   }
 
-
+  if( strip_out_boolean_argument(argc,argv,"phase_help") == TRUE ) {
+    fprintf(stdout,"Phased marks provide the ability to restrict the position of introns\n");
+    fprintf(stdout,"relative to the protein sequence; ie, assuming conserved introns. This\n");
+    fprintf(stdout,"is most useful for fast evolving genes inside of relatively consistent\n");
+    fprintf(stdout,"clades, eg for fast evolving genes, such as cytokines, in vertebrates\n");
+    fprintf(stdout,"As moving between clades - say between Human and Drosophila - the intron\n");
+    fprintf(stdout,"positions change, using these options would actively hinder good gene prediction\n");
+    fprintf(stdout,"\n");
+    fprintf(stdout,"This option can be used for either HMMs or proteins, although it is harder\n");
+    fprintf(stdout,"to coordinate the HMM intron position than the protein positions.\n");
+    fprintf(stdout,"Two things need to occur to use the phase information\n");
+    fprintf(stdout,"    provide a phase mark file as -phase_file <xxxxxx>\n");
+    fprintf(stdout,"    use the algorithm type 623P (6 states, 23 transitions, phased introns)\n");
+    fprintf(stdout,"\n");
+    fprintf(stdout,"The phase model attempts to make a ATG to STOP gene, even if the protein match\n");
+    fprintf(stdout,"is not present across the entire gene. One major headache in this are introns in first\n");
+    fprintf(stdout,"ATG, which is not handled at the moment\n\n");
+    fprintf(stdout,"Genewise uses the protein position, in 1 coordinates, (first amino acid is 1)\n");
+    fprintf(stdout,"for the definition of the intron. For phase 0 introns, it should be labeled as\n");
+    fprintf(stdout,"the amino acid before the intron. For phase 1 and 2 introns, this is on the intron\n\n");
+    fprintf(stdout,"We suggest using a small spread of positions to cope with intron positioning errors\n");
+    fprintf(stdout,"  eg, defining an intron at position 4, phase 0, make postions 3,4 and 5 with position 0\n\n");
+    fprintf(stdout,"The phase file format is\n");
+    fprintf(stdout,"# lines starting with hash are comments\n");
+    fprintf(stdout,"# three tab delimited columns\n");
+    fprintf(stdout,"# <protein-position>  <phase>\n");
+    fprintf(stdout,"# eg\n");
+    fprintf(stdout,"4 0\n");
+    exit(0);
+  }
 
   return out;
 
 }
 
 
-# line 141 "phasemodel.dy"
+# line 170 "phasemodel.dy"
 void show_help_PhasedProteinPara(FILE * ofp)
 {
   fprintf(ofp,"Phased Protein/HMM parameters (separate from other options)\n");
   fprintf(ofp,"  -phase_marked    [0.95]        Probability of marked introns\n");
   fprintf(ofp,"  -phase_unmarked  [0.00001]     Probability of unmarked introns\n");
   fprintf(ofp,"  -phase_file      [filename]    Intron positions and phases\n");
-  fprintf(ofp,"                  (lines starting # ignored, otherwise <position> <phase>, phase 0 the aa before the intron)\n");
+  fprintf(ofp,"  -phase_help      prints longer help on phase file and exits\n");
 }
 
 
-# line 151 "phasemodel.dy"
+# line 180 "phasemodel.dy"
 void write_fasta_PhasedProtein(PhasedProtein * pp,FILE * ofp)
 {
   int i;
@@ -132,7 +161,7 @@ void write_fasta_PhasedProtein(PhasedProtein * pp,FILE * ofp)
 }
 
 
-# line 183 "phasemodel.dy"
+# line 212 "phasemodel.dy"
 PhasedProtein * read_fasta_PhasedProtein_file(char * file)
 {
   PhasedProtein * pp;
@@ -148,7 +177,7 @@ PhasedProtein * read_fasta_PhasedProtein_file(char * file)
 
 }
 
-# line 198 "phasemodel.dy"
+# line 227 "phasemodel.dy"
 PhasedProtein * read_fasta_PhasedProtein(FILE * ifp)
 {
   PhasedProtein * pp;
@@ -197,7 +226,7 @@ PhasedProtein * read_fasta_PhasedProtein(FILE * ifp)
 
 
 
-# line 246 "phasemodel.dy"
+# line 275 "phasemodel.dy"
 GenePhaseModel * GenePhaseModel_from_ThreeStateModel(ThreeStateModel * tsm,CodonMapper * cm,RandomModel * rm,CompMat * mat,PhasedProteinPara * ppp)
 {
   GenePhaseModel * out;
@@ -268,7 +297,7 @@ GenePhaseModel * GenePhaseModel_from_ThreeStateModel(ThreeStateModel * tsm,Codon
 
 
 
-# line 316 "phasemodel.dy"
+# line 345 "phasemodel.dy"
 GenePhaseScore * GenePhaseScore_from_GenePhaseModel(GenePhaseModel * gpm)
 {
   int i;
@@ -294,7 +323,7 @@ GenePhaseScore * GenePhaseScore_from_GenePhaseModel(GenePhaseModel * gpm)
 
 
 
-# line 296 "phasemodel.c"
+# line 325 "phasemodel.c"
 /* Function:  hard_link_ProteinIntron(obj)
  *
  * Descrip:    Bumps up the reference count of the object
