@@ -7,7 +7,11 @@ extern "C" {
 
 struct Wise2_ProteinSW {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     BaseMatrix * basematrix;     
+    ShatterMatrix * shatter;     
     int leni;    
     int lenj;    
     ComplexSequence* query;  
@@ -44,6 +48,22 @@ struct thread_pool_holder_ProteinSW {
     DBSearchImpl * dbsi;     
     } ;  
 #endif /* PTHREAD */
+struct Wise2_ProteinSW_access_func_holder {  
+    int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
+    int (*access_main)(ProteinSW*,int,int,int);  
+    int (*access_special)(ProteinSW*,int,int,int);   
+    } ;  
+/* ProteinSW_access_func_holder defined */ 
+#ifndef DYNAMITE_DEFINED_ProteinSW_access_func_holder
+typedef struct Wise2_ProteinSW_access_func_holder Wise2_ProteinSW_access_func_holder;
+#define ProteinSW_access_func_holder Wise2_ProteinSW_access_func_holder
+#define DYNAMITE_DEFINED_ProteinSW_access_func_holder
+#endif
+
+
 
 
     /***************************************************/
@@ -53,6 +73,35 @@ struct thread_pool_holder_ProteinSW {
 
 
 
+/* Function:  PackAln_read_Shatter_ProteinSW(mat)
+ *
+ * Descrip:    Reads off PackAln from shatter matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [ProteinSW *]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_Shatter_ProteinSW(ProteinSW * mat);
+#define PackAln_read_Shatter_ProteinSW Wise2_PackAln_read_Shatter_ProteinSW
+
+
+/* Function:  calculate_shatter_ProteinSW(mat,dpenv)
+ *
+ * Descrip:    This function calculates the ProteinSW matrix when in shatter mode
+ *
+ *
+ * Arg:          mat [UNKN ] (null) [ProteinSW *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_shatter_ProteinSW(ProteinSW * mat,DPEnvelope * dpenv);
+#define calculate_shatter_ProteinSW Wise2_calculate_shatter_ProteinSW
+
+
 /* Function:  search_ProteinSW(dbsi,out,querydb,targetdb,comp,gap,ext)
  *
  * Descrip:    This function makes a database search of ProteinSW
@@ -123,7 +172,7 @@ PackAln * Wise2_PackAln_bestmemory_ProteinSW(ComplexSequence* query,ComplexSeque
 #define PackAln_bestmemory_ProteinSW Wise2_PackAln_bestmemory_ProteinSW
 
 
-/* Function:  allocate_Expl_ProteinSW(query,target,comp,gap,ext)
+/* Function:  allocate_Expl_ProteinSW(query,target,comp,gap,ext,dpri)
  *
  * Descrip:    This function allocates the ProteinSW structure
  *             and the basematrix area for explicit memory implementations
@@ -135,11 +184,12 @@ PackAln * Wise2_PackAln_bestmemory_ProteinSW(ComplexSequence* query,ComplexSeque
  * Arg:          comp [UNKN ] Resource [CompMat*]
  * Arg:           gap [UNKN ] Resource [int]
  * Arg:           ext [UNKN ] Resource [int]
+ * Arg:          dpri [UNKN ] Undocumented argument [DPRunImpl *]
  *
  * Return [UNKN ]  Undocumented return value [ProteinSW *]
  *
  */
-ProteinSW * Wise2_allocate_Expl_ProteinSW(ComplexSequence* query,ComplexSequence* target ,CompMat* comp,int gap,int ext);
+ProteinSW * Wise2_allocate_Expl_ProteinSW(ComplexSequence* query,ComplexSequence* target ,CompMat* comp,int gap,int ext,DPRunImpl * dpri);
 #define allocate_Expl_ProteinSW Wise2_allocate_Expl_ProteinSW
 
 
@@ -264,6 +314,21 @@ PackAln * Wise2_PackAln_read_Expl_ProteinSW(ProteinSW * mat);
 #define PackAln_read_Expl_ProteinSW Wise2_PackAln_read_Expl_ProteinSW
 
 
+/* Function:  PackAln_read_generic_ProteinSW(mat,h)
+ *
+ * Descrip:    Reads off PackAln from explicit matrix structure
+ *
+ *
+ * Arg:        mat [UNKN ] Undocumented argument [ProteinSW *]
+ * Arg:          h [UNKN ] Undocumented argument [ProteinSW_access_func_holder]
+ *
+ * Return [UNKN ]  Undocumented return value [PackAln *]
+ *
+ */
+PackAln * Wise2_PackAln_read_generic_ProteinSW(ProteinSW * mat,ProteinSW_access_func_holder h);
+#define PackAln_read_generic_ProteinSW Wise2_PackAln_read_generic_ProteinSW
+
+
 /* Function:  calculate_ProteinSW(mat)
  *
  * Descrip:    This function calculates the ProteinSW matrix when in explicit mode
@@ -279,6 +344,21 @@ boolean Wise2_calculate_ProteinSW(ProteinSW * mat);
 #define calculate_ProteinSW Wise2_calculate_ProteinSW
 
 
+/* Function:  calculate_dpenv_ProteinSW(mat,dpenv)
+ *
+ * Descrip:    This function calculates the ProteinSW matrix when in explicit mode, subject to the envelope
+ *
+ *
+ * Arg:          mat [UNKN ] ProteinSW which contains explicit basematrix memory [ProteinSW *]
+ * Arg:        dpenv [UNKN ] Undocumented argument [DPEnvelope *]
+ *
+ * Return [UNKN ]  Undocumented return value [boolean]
+ *
+ */
+boolean Wise2_calculate_dpenv_ProteinSW(ProteinSW * mat,DPEnvelope * dpenv);
+#define calculate_dpenv_ProteinSW Wise2_calculate_dpenv_ProteinSW
+
+
 /* Function:  ProteinSW_alloc(void)
  *
  * Descrip:    Allocates structure: assigns defaults if given 
@@ -315,6 +395,10 @@ ProteinSW * Wise2_free_ProteinSW(ProteinSW * obj);
     /* Internal functions                              */
     /* you are not expected to have to call these      */
     /***************************************************/
+int Wise2_ProteinSW_shatter_access_main(ProteinSW * mat,int i,int j,int state);
+#define ProteinSW_shatter_access_main Wise2_ProteinSW_shatter_access_main
+int Wise2_ProteinSW_shatter_access_special(ProteinSW * mat,int i,int j,int state);
+#define ProteinSW_shatter_access_special Wise2_ProteinSW_shatter_access_special
 void * Wise2_thread_loop_ProteinSW(void * ptr);
 #define thread_loop_ProteinSW Wise2_thread_loop_ProteinSW
 int Wise2_score_only_ProteinSW(ComplexSequence* query,ComplexSequence* target ,CompMat* comp,int gap,int ext);
@@ -351,21 +435,25 @@ void Wise2_run_up_dc_ProteinSW(ProteinSW * mat,int starti,int stopi,int startj,i
 #define run_up_dc_ProteinSW Wise2_run_up_dc_ProteinSW
 void Wise2_init_dc_ProteinSW(ProteinSW * mat);
 #define init_dc_ProteinSW Wise2_init_dc_ProteinSW
-boolean Wise2_dc_start_end_calculate_ProteinSW(ProteinSW * mat,DPEnvelope * dpenv);
-#define dc_start_end_calculate_ProteinSW Wise2_dc_start_end_calculate_ProteinSW
 int Wise2_start_end_find_end_ProteinSW(ProteinSW * mat,int * endj);
 #define start_end_find_end_ProteinSW Wise2_start_end_find_end_ProteinSW
+boolean Wise2_dc_optimised_start_end_calc_ProteinSW(ProteinSW *mat,DPEnvelope * dpenv);
+#define dc_optimised_start_end_calc_ProteinSW Wise2_dc_optimised_start_end_calc_ProteinSW
 void Wise2_init_start_end_linear_ProteinSW(ProteinSW * mat);
 #define init_start_end_linear_ProteinSW Wise2_init_start_end_linear_ProteinSW
 AlnConvertSet * Wise2_AlnConvertSet_ProteinSW(void);
 #define AlnConvertSet_ProteinSW Wise2_AlnConvertSet_ProteinSW
-int Wise2_find_end_ProteinSW(ProteinSW * mat,int * ri,int * rj,int * state,boolean * isspecial);
+int Wise2_ProteinSW_explicit_access_main(ProteinSW * mat,int i,int j,int state);
+#define ProteinSW_explicit_access_main Wise2_ProteinSW_explicit_access_main
+int Wise2_ProteinSW_explicit_access_special(ProteinSW * mat,int i,int j,int state);
+#define ProteinSW_explicit_access_special Wise2_ProteinSW_explicit_access_special
+int Wise2_find_end_ProteinSW(ProteinSW * mat,int * ri,int * rj,int * state,boolean * isspecial,ProteinSW_access_func_holder h);
 #define find_end_ProteinSW Wise2_find_end_ProteinSW
 void Wise2_ProteinSW_debug_show_matrix(ProteinSW * mat,int starti,int stopi,int startj,int stopj,FILE * ofp);
 #define ProteinSW_debug_show_matrix Wise2_ProteinSW_debug_show_matrix
-int Wise2_max_calc_ProteinSW(ProteinSW * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_ProteinSW(ProteinSW * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,ProteinSW_access_func_holder h);
 #define max_calc_ProteinSW Wise2_max_calc_ProteinSW
-int Wise2_max_calc_special_ProteinSW(ProteinSW * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore);
+int Wise2_max_calc_special_ProteinSW(ProteinSW * mat,int i,int j,int state,boolean isspecial,int * reti,int * retj,int * retstate,boolean * retspecial,int * cellscore,ProteinSW_access_func_holder h);
 #define max_calc_special_ProteinSW Wise2_max_calc_special_ProteinSW
 
 #ifdef _cplusplus
