@@ -3,6 +3,104 @@ extern "C" {
 #endif
 #include "pwmdna.h"
 
+
+/* Function:  max_prob_pwmDNA(in)
+ *
+ * Descrip:    maximum prob of pwmDNA
+ *
+ *
+ * Arg:        in [UNKN ] Undocumented argument [pwmDNA *]
+ *
+ * Return [UNKN ]  Undocumented return value [double]
+ *
+ */
+# line 64 "pwmdna.dy"
+double max_prob_pwmDNA(pwmDNA * in)
+{
+  int i,k;
+  double temp_max;
+  double score = 0.0;
+
+  for(i=0;i<in->len;i++) {
+    temp_max = in->pos[i]->emit[0];
+    for(k=1;k<4;k++) {
+      if( temp_max < in->pos[i]->emit[k] ) {
+	temp_max = in->pos[i]->emit[k];
+      }
+    }
+    score *= temp_max;
+  }
+
+  return score;
+}
+
+
+/* Function:  min_prob_pwmDNA(in)
+ *
+ * Descrip:    minimum prob of pwmDNA
+ *
+ *
+ * Arg:        in [UNKN ] Undocumented argument [pwmDNA *]
+ *
+ * Return [UNKN ]  Undocumented return value [double]
+ *
+ */
+# line 87 "pwmdna.dy"
+double min_prob_pwmDNA(pwmDNA * in)
+{
+  int i,k;
+  double temp_min;
+  double score = 0.0;
+
+  for(i=0;i<in->len;i++) {
+    temp_min = in->pos[i]->emit[0];
+    for(k=1;k<4;k++) {
+      if( temp_min > in->pos[i]->emit[k] ) {
+	temp_min = in->pos[i]->emit[k];
+      }
+    }
+    score *= temp_min;
+  }
+
+  return score;
+}
+
+/* Function:  circular_permuted_pwmDNA(in,rotate_number)
+ *
+ * Descrip:    Provides a rotated pwm for randomisation
+ *
+ *
+ * Arg:                   in [UNKN ] Undocumented argument [pwmDNA *]
+ * Arg:        rotate_number [UNKN ] Undocumented argument [int]
+ *
+ * Return [UNKN ]  Undocumented return value [pwmDNA *]
+ *
+ */
+# line 109 "pwmdna.dy"
+pwmDNA * circular_permuted_pwmDNA(pwmDNA * in,int rotate_number)
+{
+  int i;
+  pwmDNA * out;
+  pwmColProb * col;
+  int j;
+  int pos;
+
+  assert(in != NULL);
+
+  out = pwmDNA_alloc_len(in->len);
+  for(i=0;i<in->len;i++) {
+    pos = (i+rotate_number) % in->len;
+    col = pwmColProb_alloc();
+    add_pwmDNA(out,col);
+    for(j=0;j<5;j++) {
+      col->emit[j] = in->pos[pos]->emit[j];
+    }
+  }
+
+  return out;
+
+}
+
 /* Function:  score_pwmDNAScore_Sequence(pds,s,pos)
  *
  * Descrip:    This gives back a Score from a particular sequence and
@@ -16,7 +114,7 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [Score]
  *
  */
-# line 62 "pwmdna.dy"
+# line 137 "pwmdna.dy"
 Score score_pwmDNAScore_Sequence(pwmDNAScore * pds,Sequence * s,int pos)
 {
   int score;
@@ -26,7 +124,7 @@ Score score_pwmDNAScore_Sequence(pwmDNAScore * pds,Sequence * s,int pos)
   }
 
   score = score_pwmDNAScore_string(pds,s->seq+pos);
-  fprintf(stderr,"Making score %d for %d\n",pos);
+  fprintf(stderr,"Making score %d for %d\n",score,pos);
   return score;
 }
 
@@ -41,20 +139,54 @@ Score score_pwmDNAScore_Sequence(pwmDNAScore * pds,Sequence * s,int pos)
  * Return [UNKN ]  Undocumented return value [Score]
  *
  */
-# line 78 "pwmdna.dy"
+# line 153 "pwmdna.dy"
 Score score_pwmDNAScore_string(pwmDNAScore * pds,char * str)
 {
   int i;
   Score total = 0.0;
 
+  /*
   if( strlen(str) < pds->len ) {
     warn("String [%s] is shorter than the length of the pds. Should not be using it!",str);
     return NEGI;
   }
+  */
 
   for(i=0;i<pds->len;i++)
     total += pds->pos[i]->emit[base_from_char(str[i])];
 
+/*  printf("Score %c%c%c to score with %s  [%d]\n",str[0],str[1],str[2],pds->ref_align->seq[0]->seq,total);*/
+ 
+  return total;
+}
+
+/* Function:  prob_pwmDNA_string(pds,str)
+ *
+ * Descrip:    This gives back a Probability from a particular string
+ *
+ *
+ * Arg:        pds [UNKN ] Undocumented argument [pwmDNA *]
+ * Arg:        str [UNKN ] Undocumented argument [char *]
+ *
+ * Return [UNKN ]  Undocumented return value [Probability]
+ *
+ */
+# line 176 "pwmdna.dy"
+Probability prob_pwmDNA_string(pwmDNA * pds,char * str)
+{
+  int i;
+  Probability total = 1.0;
+
+
+  for(i=0;i<pds->len;i++) {
+    if( str[i] == '\0' ) {
+      return 0.0;
+    } else {
+      total *= pds->pos[i]->emit[base_from_char(str[i])];
+      /*      fprintf(stdout,"As %d, total %.6f with %c\n",i,total,str[i]);*/
+    }
+  }
+
   return total;
 }
 
@@ -67,7 +199,7 @@ Score score_pwmDNAScore_string(pwmDNAScore * pds,char * str)
  * Arg:        rmd [UNKN ] Undocumented argument [RandomModelDNA *]
  *
  */
-# line 97 "pwmdna.dy"
+# line 197 "pwmdna.dy"
 void fold_randommodel_pwmDNA(pwmDNA * pd,RandomModelDNA * rmd)
 {
   int i;
@@ -77,7 +209,11 @@ void fold_randommodel_pwmDNA(pwmDNA * pd,RandomModelDNA * rmd)
   assert(rmd);
   for(i=0;i<pd->len;i++)
     for(j=0;j<5;j++)
-      pd->pos[i]->emit[j] /= rmd->base[j];
+      if( rmd->base[j] < 0.00000000000000001 ) {
+	warn("Zero base %d, skipping",j);
+      } else {
+	pd->pos[i]->emit[j] /= rmd->base[j];
+      }
 
 }
 
@@ -95,7 +231,7 @@ void fold_randommodel_pwmDNA(pwmDNA * pd,RandomModelDNA * rmd)
  * Return [UNKN ]  Undocumented return value [pwmDNA *]
  *
  */
-# line 116 "pwmdna.dy"
+# line 220 "pwmdna.dy"
 pwmDNA * pwmDNA_from_SeqAlign(SeqAlign * sa,double simple_pseudocount)
 {
   pwmDNA * out;
@@ -103,8 +239,10 @@ pwmDNA * pwmDNA_from_SeqAlign(SeqAlign * sa,double simple_pseudocount)
   ColumnCount * cc;
   int i;
 
+  assert(sa);
 
   out = pwmDNA_alloc_std();
+  out->ref_align = hard_link_SeqAlign(sa);
 
   for(i=0;i<sa->seq[0]->len;i++) {
     cc = ColumnCount_from_SeqAlign(sa,i);
@@ -126,7 +264,7 @@ pwmDNA * pwmDNA_from_SeqAlign(SeqAlign * sa,double simple_pseudocount)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 140 "pwmdna.dy"
+# line 246 "pwmdna.dy"
 void show_pwmDNA_col(pwmDNA * pd,FILE * ofp)
 {
   char base[] = "ATGC";
@@ -159,7 +297,7 @@ void show_pwmDNA_col(pwmDNA * pd,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [pwmColProb *]
  *
  */
-# line 165 "pwmdna.dy"
+# line 271 "pwmdna.dy"
 pwmColProb * pwmColProb_from_ColumnCount(ColumnCount * cc,double simple_pseudocount)
 {
   pwmColProb * out;
@@ -191,13 +329,14 @@ pwmColProb * pwmColProb_from_ColumnCount(ColumnCount * cc,double simple_pseudoco
  * Return [UNKN ]  Undocumented return value [pwmDNAScore *]
  *
  */
-# line 188 "pwmdna.dy"
+# line 294 "pwmdna.dy"
 pwmDNAScore * pwmDNAScore_from_pwmDNA_RandomModelDNA(pwmDNA * pwm,RandomModelDNA * rmd)
 {
   int i;
   pwmDNAScore * out;
 
   out = pwmDNAScore_alloc_std();
+  out->ref_align = hard_link_SeqAlign(pwm->ref_align);
   for(i=0;i<pwm->len;i++) {
     add_pwmDNAScore(out,pwmColScore_from_pwmColProb_rmd(pwm->pos[i],rmd));
   }
@@ -216,13 +355,15 @@ pwmDNAScore * pwmDNAScore_from_pwmDNA_RandomModelDNA(pwmDNA * pwm,RandomModelDNA
  * Return [UNKN ]  Undocumented return value [pwmDNAScore *]
  *
  */
-# line 205 "pwmdna.dy"
+# line 312 "pwmdna.dy"
 pwmDNAScore * pwmDNAScore_from_pwmDNA(pwmDNA * pwm)
 {
   int i;
   pwmDNAScore * out;
 
   out = pwmDNAScore_alloc_std();
+  out->ref_align = hard_link_SeqAlign(pwm->ref_align);
+
   for(i=0;i<pwm->len;i++) {
     add_pwmDNAScore(out,pwmColScore_from_pwmColProb(pwm->pos[i]));
   }
@@ -241,7 +382,7 @@ pwmDNAScore * pwmDNAScore_from_pwmDNA(pwmDNA * pwm)
  * Return [UNKN ]  Undocumented return value [pwmColScore *]
  *
  */
-# line 223 "pwmdna.dy"
+# line 332 "pwmdna.dy"
 pwmColScore * pwmColScore_from_pwmColProb(pwmColProb * p)
 {
   pwmColScore * out;
@@ -265,7 +406,7 @@ pwmColScore * pwmColScore_from_pwmColProb(pwmColProb * p)
  * Return [UNKN ]  Undocumented return value [pwmColScore *]
  *
  */
-# line 239 "pwmdna.dy"
+# line 348 "pwmdna.dy"
 pwmColScore * pwmColScore_from_pwmColProb_rmd(pwmColProb * p,RandomModelDNA  * rmd)
 {
   pwmColScore * out;
@@ -278,7 +419,7 @@ pwmColScore * pwmColScore_from_pwmColProb_rmd(pwmColProb * p,RandomModelDNA  * r
   return out;
 }
 
-# line 260 "pwmdna.c"
+# line 393 "pwmdna.c"
 /* Function:  hard_link_pwmColScore(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -321,6 +462,9 @@ pwmColScore * pwmColScore_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* emit[5] is an array: no default possible */ 
 
 
@@ -341,6 +485,7 @@ pwmColScore * pwmColScore_alloc(void)
  */
 pwmColScore * free_pwmColScore(pwmColScore * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -349,10 +494,18 @@ pwmColScore * free_pwmColScore(pwmColScore * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -402,6 +555,9 @@ pwmColProb * pwmColProb_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* emit[5] is an array: no default possible */ 
 
 
@@ -422,6 +578,7 @@ pwmColProb * pwmColProb_alloc(void)
  */
 pwmColProb * free_pwmColProb(pwmColProb * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -430,10 +587,18 @@ pwmColProb * free_pwmColProb(pwmColProb * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -687,8 +852,12 @@ pwmDNA * pwmDNA_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->pos = NULL; 
     out->len = out->maxlen = 0;  
+    out->ref_align = NULL;   
 
 
     return out;  
@@ -708,6 +877,7 @@ pwmDNA * pwmDNA_alloc(void)
  */
 pwmDNA * free_pwmDNA(pwmDNA * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -717,10 +887,18 @@ pwmDNA * free_pwmDNA(pwmDNA * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->pos != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->pos[i] != NULL) 
@@ -728,6 +906,8 @@ pwmDNA * free_pwmDNA(pwmDNA * obj)
         }  
       ckfree(obj->pos);  
       }  
+    if( obj->ref_align != NULL)  
+      free_SeqAlign(obj->ref_align);     
 
 
     ckfree(obj); 
@@ -981,8 +1161,12 @@ pwmDNAScore * pwmDNAScore_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->pos = NULL; 
     out->len = out->maxlen = 0;  
+    out->ref_align = NULL;   
 
 
     return out;  
@@ -1002,6 +1186,7 @@ pwmDNAScore * pwmDNAScore_alloc(void)
  */
 pwmDNAScore * free_pwmDNAScore(pwmDNAScore * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1011,10 +1196,18 @@ pwmDNAScore * free_pwmDNAScore(pwmDNAScore * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->pos != NULL)    {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->pos[i] != NULL) 
@@ -1022,6 +1215,8 @@ pwmDNAScore * free_pwmDNAScore(pwmDNAScore * obj)
         }  
       ckfree(obj->pos);  
       }  
+    if( obj->ref_align != NULL)  
+      free_SeqAlign(obj->ref_align);     
 
 
     ckfree(obj); 
