@@ -316,6 +316,9 @@ SegmentHit * SegmentHit_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->qstart = 0; 
     out->qend = 0;   
@@ -342,6 +345,7 @@ SegmentHit * SegmentHit_alloc(void)
  */
 SegmentHit * free_SegmentHit(SegmentHit * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -350,10 +354,18 @@ SegmentHit * free_SegmentHit(SegmentHit * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->next_hit != NULL)   
@@ -407,6 +419,9 @@ SequenceHit * SequenceHit_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->start = NULL;   
 
@@ -428,6 +443,7 @@ SequenceHit * SequenceHit_alloc(void)
  */
 SequenceHit * free_SequenceHit(SequenceHit * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -436,10 +452,18 @@ SequenceHit * free_SequenceHit(SequenceHit * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->start != NULL)  
@@ -697,6 +721,9 @@ SegmentHitList * SegmentHitList_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->seghit = NULL;  
     out->len = out->maxlen = 0;  
 
@@ -718,6 +745,7 @@ SegmentHitList * SegmentHitList_alloc(void)
  */
 SegmentHitList * free_SegmentHitList(SegmentHitList * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -727,10 +755,18 @@ SegmentHitList * free_SegmentHitList(SegmentHitList * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->seghit != NULL) {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->seghit[i] != NULL)  
@@ -787,6 +823,9 @@ DnaSequenceHitList * DnaSequenceHitList_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->forward = NULL; 
     out->backward = NULL;    
 
@@ -808,6 +847,7 @@ DnaSequenceHitList * DnaSequenceHitList_alloc(void)
  */
 DnaSequenceHitList * free_DnaSequenceHitList(DnaSequenceHitList * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -816,10 +856,18 @@ DnaSequenceHitList * free_DnaSequenceHitList(DnaSequenceHitList * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->forward != NULL)    
       free_SegmentHitList(obj->forward);     
     if( obj->backward != NULL)   
