@@ -15,7 +15,7 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 89 "threestatedb.dy"
+# line 92 "threestatedb.dy"
 boolean open_for_indexing_ThreeStateDB(ThreeStateDB * mdb)
 {
   switch(mdb->dbtype) {
@@ -44,7 +44,7 @@ boolean open_for_indexing_ThreeStateDB(ThreeStateDB * mdb)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 110 "threestatedb.dy"
+# line 113 "threestatedb.dy"
 boolean close_for_indexing_ThreeStateDB(ThreeStateDB * mdb)
 {
   switch(mdb->dbtype) {
@@ -66,7 +66,7 @@ boolean close_for_indexing_ThreeStateDB(ThreeStateDB * mdb)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 125 "threestatedb.dy"
+# line 128 "threestatedb.dy"
 boolean set_search_type_ThreeStateDB(ThreeStateDB * tdb,char * type)
 {
   int ret;
@@ -99,7 +99,7 @@ boolean set_search_type_ThreeStateDB(ThreeStateDB * tdb,char * type)
  * Return [UNKN ]  Undocumented return value [ThreeStateModel *]
  *
  */
-# line 153 "threestatedb.dy"
+# line 156 "threestatedb.dy"
 ThreeStateModel * indexed_ThreeStateModel_ThreeStateDB(ThreeStateDB * mdb,DataEntry * en)
 {
   Sequence * seq;
@@ -178,7 +178,7 @@ ThreeStateModel * indexed_ThreeStateModel_ThreeStateDB(ThreeStateDB * mdb,DataEn
  * Return [UNKN ]  Undocumented return value [ThreeStateDB *]
  *
  */
-# line 226 "threestatedb.dy"
+# line 229 "threestatedb.dy"
 ThreeStateDB * new_proteindb_ThreeStateDB(SequenceDB * sdb,CompMat * comp,int gap,int ext)
 {
   ThreeStateDB * out;
@@ -207,7 +207,7 @@ ThreeStateDB * new_proteindb_ThreeStateDB(SequenceDB * sdb,CompMat * comp,int ga
  * Return [UNKN ]  Undocumented return value [ThreeStateDB *]
  *
  */
-# line 249 "threestatedb.dy"
+# line 252 "threestatedb.dy"
 ThreeStateDB * new_single_ThreeStateDB(ThreeStateModel * tsm,RandomModel * rm)
 {
   ThreeStateDB * out;
@@ -234,7 +234,7 @@ ThreeStateDB * new_single_ThreeStateDB(ThreeStateModel * tsm,RandomModel * rm)
  * Return [UNKN ]  Undocumented return value [ThreeStateDB *]
  *
  */
-# line 268 "threestatedb.dy"
+# line 271 "threestatedb.dy"
 ThreeStateDB * new_PfamHmmer1DB_ThreeStateDB(char * dirname)
 {
   ThreeStateDB * out;
@@ -261,7 +261,7 @@ ThreeStateDB * new_PfamHmmer1DB_ThreeStateDB(char * dirname)
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 285 "threestatedb.dy"
+# line 288 "threestatedb.dy"
 boolean dataentry_add_ThreeStateDB(DataEntry * de,ThreeStateScore * tss,ThreeStateDB * mdb)
 {
 
@@ -310,10 +310,15 @@ boolean dataentry_add_ThreeStateDB(DataEntry * de,ThreeStateScore * tss,ThreeSta
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 326 "threestatedb.dy"
+# line 329 "threestatedb.dy"
 boolean open_ThreeStateDB(ThreeStateDB * mdb)
 {
   int ret;
+  int return_status;
+  ThreeStateModel * temp;
+  int count;
+
+  mdb->current_no = 0;
 
   switch( mdb->dbtype ) {
 
@@ -325,7 +330,7 @@ boolean open_ThreeStateDB(ThreeStateDB * mdb)
       return FALSE;
     }
     mdb->phdb->cur = 0;
-    return TRUE;
+    break;
   case TSMDB_PROTEIN :
     if( mdb->sdb == NULL ) {
       warn("Attempting to open a protein tsm with no sequence db!");
@@ -338,16 +343,24 @@ boolean open_ThreeStateDB(ThreeStateDB * mdb)
     if( ret == DB_RETURN_END ) {
       warn("Due to some bad coding, can't cope with single protein databases in tsmdbs. oooops!");
     }
-    return TRUE;
+    break;
   case TSMDB_GENERIC :
-    return ((*mdb->open_generic)(mdb));
+    ((*mdb->open_generic)(mdb));
+    break;
   default :
     warn("Got an unrecognisable tsm db type in opening tsm %d",mdb->dbtype);
     return FALSE;
   }
     
+  if( mdb->hmm_model_start != -1 && mdb->hmm_model_end != -1 ) {
+    for(count=0;count<mdb->hmm_model_start;count++) {
+      temp = read_TSM_ThreeStateDB(mdb,&return_status);
+      free_ThreeStateModel(temp);
+    }
+  }
 
-  return FALSE;
+
+  return TRUE;
 }
 
 
@@ -365,14 +378,19 @@ boolean open_ThreeStateDB(ThreeStateDB * mdb)
  * Return [UNKN ]  Undocumented return value [ThreeStateModel *]
  *
  */
-# line 372 "threestatedb.dy"
+# line 388 "threestatedb.dy"
 ThreeStateModel * read_TSM_ThreeStateDB(ThreeStateDB * mdb,int * return_status)
 {
   ThreeStateModel * tsm;
   Protein * pro;
   Sequence * seq;
 
+  if( mdb->hmm_model_end != -1 && mdb->current_no == mdb->hmm_model_end ) {
+    *return_status = DB_RETURN_END;
+    return NULL;
+  }
 
+  mdb->current_no++;
 
   switch( mdb->dbtype ) {
 
@@ -464,7 +482,7 @@ ThreeStateModel * read_TSM_ThreeStateDB(ThreeStateDB * mdb,int * return_status)
  * Return [UNKN ]  Undocumented return value [ThreeStateScore *]
  *
  */
-# line 465 "threestatedb.dy"
+# line 486 "threestatedb.dy"
 ThreeStateScore * init_ThreeStateDB(ThreeStateDB * mdb,int * return_status)
 {
   ThreeStateModel * tsm;
@@ -510,7 +528,7 @@ ThreeStateScore * init_ThreeStateDB(ThreeStateDB * mdb,int * return_status)
  * Return [UNKN ]  Undocumented return value [ThreeStateScore *]
  *
  */
-# line 504 "threestatedb.dy"
+# line 525 "threestatedb.dy"
 ThreeStateScore * reload_ThreeStateDB(ThreeStateScore * prev,ThreeStateDB * mdb,int * return_status)
 {
   ThreeStateModel * tsm;
@@ -557,7 +575,7 @@ ThreeStateScore * reload_ThreeStateDB(ThreeStateScore * prev,ThreeStateDB * mdb,
  * Return [UNKN ]  Undocumented return value [boolean]
  *
  */
-# line 545 "threestatedb.dy"
+# line 566 "threestatedb.dy"
 boolean close_ThreeStateDB(ThreeStateScore * prev,ThreeStateDB * mdb)
 {
 
@@ -589,7 +607,7 @@ boolean close_ThreeStateDB(ThreeStateScore * prev,ThreeStateDB * mdb)
 
 }
 
-# line 565 "threestatedb.c"
+# line 583 "threestatedb.c"
 /* Function:  hard_link_ThreeStateDB(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -632,6 +650,9 @@ ThreeStateDB * ThreeStateDB_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->dbtype = TSMDB_UNKNOWN; 
     out->filename = NULL;    
     out->type = TSM_default; 
@@ -651,6 +672,9 @@ ThreeStateDB * ThreeStateDB_alloc(void)
     out->open_index_generic = NULL;  
     out->index_generic = NULL;   
     out->close_index_generic = NULL; 
+    out->hmm_model_start = -1;   
+    out->hmm_model_end = -1; 
+    out->current_no = 0; 
 
 
     return out;  
@@ -670,6 +694,7 @@ ThreeStateDB * ThreeStateDB_alloc(void)
  */
 ThreeStateDB * free_ThreeStateDB(ThreeStateDB * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -678,10 +703,18 @@ ThreeStateDB * free_ThreeStateDB(ThreeStateDB * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->filename != NULL)   
       ckfree(obj->filename);     
     /* obj->current_file is linked in */ 
