@@ -6,6 +6,35 @@ extern "C" {
  char * std_alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 
+/* Function:  information_from_ThreeStateUnit(tsu,rm)
+ *
+ * Descrip:     Gets the information content (K-L divergence) vs a background for a position
+ *
+ *
+ * Arg:        tsu [UNKN ] Undocumented argument [ThreeStateUnit *]
+ * Arg:         rm [UNKN ] Undocumented argument [RandomModel *]
+ *
+ * Return [UNKN ]  Undocumented return value [double]
+ *
+ */
+# line 117 "threestatemodel.dy"
+double information_from_ThreeStateUnit(ThreeStateUnit * tsu,RandomModel * rm)
+{
+  int i;
+  double info = 0.0;
+  double p;
+
+  for(i=0;i<ALPHABET_SIZE;i++) {
+    if( rm->aminoacid[i] < 1.0 ) {
+      p = rm->aminoacid[i]*Probability2Bits(rm->aminoacid[i]/tsu->match_emission[i]);
+      info += p;
+      /*  fprintf(stderr,"For amino acid %d, adding %f %f\n",i,p,rm->aminoacid[i]); */
+    }
+  }
+  
+  return info;
+}
+
 /* Function:  threestatemodel_mode_from_string(mode)
  *
  * Descrip:    gets out the mode from a string
@@ -16,7 +45,7 @@ extern "C" {
  * Return [UNKN ]  Undocumented return value [int]
  *
  */
-# line 117 "threestatemodel.dy"
+# line 137 "threestatemodel.dy"
 int threestatemodel_mode_from_string(char * mode)
 {
   int t;
@@ -41,13 +70,13 @@ int threestatemodel_mode_from_string(char * mode)
  *
  *
  * Arg:                  tsm [UNKN ] Undocumented argument [ThreeStateModel *]
- * Arg:                 mode [UNKN ] Undocumented argument [int]
+ * Arg:                 mode [UNKN ] Undocumented argument [TSM_StartEndMode]
  * Arg:          wing_length [UNKN ] Undocumented argument [int]
  * Arg:        internal_bias [UNKN ] Undocumented argument [Probability]
  *
  */
-# line 138 "threestatemodel.dy"
-void set_startend_policy_ThreeStateModel(ThreeStateModel * tsm,int mode,int wing_length,Probability internal_bias)
+# line 158 "threestatemodel.dy"
+void set_startend_policy_ThreeStateModel(ThreeStateModel * tsm,TSM_StartEndMode mode,int wing_length,Probability internal_bias)
 {
 
   switch(mode) {
@@ -76,7 +105,7 @@ void set_startend_policy_ThreeStateModel(ThreeStateModel * tsm,int mode,int wing
  * Arg:        internal [UNKN ] Undocumented argument [double]
  *
  */
-# line 160 "threestatemodel.dy"
+# line 180 "threestatemodel.dy"
 void force_endbias_model(ThreeStateModel * tsm,double startend,double internal)
 {
   int i;
@@ -114,7 +143,7 @@ void force_endbias_model(ThreeStateModel * tsm,double startend,double internal)
  * Arg:        prob_into_model [UNKN ] Probability to start the model: for true global will be 1.0 [double]
  *
  */
-# line 194 "threestatemodel.dy"
+# line 214 "threestatemodel.dy"
 void force_global_model(ThreeStateModel * tsm,double prob_into_model) 
 {
   force_weighted_local_model(tsm,prob_into_model,1.0,1.0);
@@ -134,7 +163,7 @@ void force_global_model(ThreeStateModel * tsm,double prob_into_model)
  * Arg:          wing_length [UNKN ] the rest of the probability spread over this distance into the wing [int]
  *
  */
-# line 209 "threestatemodel.dy"
+# line 229 "threestatemodel.dy"
 void force_wing_local_model(ThreeStateModel * tsm,double terminus_prob,int wing_length)
 {
   int i;
@@ -193,7 +222,7 @@ void force_wing_local_model(ThreeStateModel * tsm,double terminus_prob,int wing_
  * Arg:              ratio_end [UNKN ] ratio of prob to unit (last) to the rest (1.0 means all goes to the end) [double]
  *
  */
-# line 264 "threestatemodel.dy"
+# line 284 "threestatemodel.dy"
 void force_weighted_local_model(ThreeStateModel * tsm,double prob_into_model,double ratio_start,double ratio_end) 
 {
   int i;
@@ -229,7 +258,7 @@ void force_weighted_local_model(ThreeStateModel * tsm,double prob_into_model,dou
  * Arg:        tsm [UNKN ] ThreeStateModel to be 'forced' [ThreeStateModel *]
  *
  */
-# line 296 "threestatemodel.dy"
+# line 316 "threestatemodel.dy"
 void force_hmmfs_ThreeStateModel(ThreeStateModel * tsm)
 {
   int i;
@@ -269,7 +298,7 @@ void force_hmmfs_ThreeStateModel(ThreeStateModel * tsm)
  * Return [UNKN ]  Undocumented return value [ThreeStateScore     *]
  *
  */
-# line 330 "threestatemodel.dy"
+# line 350 "threestatemodel.dy"
 ThreeStateScore     * ThreeStateScore_from_ThreeStateModel(ThreeStateModel * tsm)
 {
   register int i;
@@ -303,7 +332,7 @@ ThreeStateScore     * ThreeStateScore_from_ThreeStateModel(ThreeStateModel * tsm
  * Return [UNKN ]  Undocumented return value [ThreeStateScoreUnit *]
  *
  */
-# line 356 "threestatemodel.dy"
+# line 376 "threestatemodel.dy"
 ThreeStateScoreUnit * ThreeStateScoreUnit_from_ThreeStateUnit(ThreeStateUnit * prev,ThreeStateUnit * tsu)
 {
   ThreeStateScoreUnit * out;
@@ -362,7 +391,7 @@ ThreeStateScoreUnit * ThreeStateScoreUnit_from_ThreeStateUnit(ThreeStateUnit * p
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 408 "threestatemodel.dy"
+# line 428 "threestatemodel.dy"
 void show_ThreeStateModel(ThreeStateModel * tsm,FILE * ofp)
 {
   register int i;
@@ -382,7 +411,7 @@ void show_ThreeStateModel(ThreeStateModel * tsm,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 422 "threestatemodel.dy"
+# line 442 "threestatemodel.dy"
 void show_ThreeStateUnit(ThreeStateUnit * tsu,FILE * ofp)
 {
   fprintf(ofp,"Match ");
@@ -407,7 +436,7 @@ void show_ThreeStateUnit(ThreeStateUnit * tsu,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [Protein *]
  *
  */
-# line 439 "threestatemodel.dy"
+# line 459 "threestatemodel.dy"
 Protein * pseudo_Protein_from_ThreeStateModel(ThreeStateModel * tsm)
 {
   int i;
@@ -448,7 +477,7 @@ Protein * pseudo_Protein_from_ThreeStateModel(ThreeStateModel * tsm)
  * Return [UNKN ]  Undocumented return value [ThreeStateModel *]
  *
  */
-# line 474 "threestatemodel.dy"
+# line 494 "threestatemodel.dy"
 ThreeStateModel * ThreeStateModel_from_half_bit_Sequence(Protein * pro,CompMat * mat,RandomModel * rm,int gap,int ext)
 {
   Sequence * seq;
@@ -501,14 +530,14 @@ ThreeStateModel * ThreeStateModel_from_half_bit_Sequence(Protein * pro,CompMat *
  * Return [UNKN ]  Undocumented return value [ThreeStateUnit *]
  *
  */
-# line 516 "threestatemodel.dy"
+# line 536 "threestatemodel.dy"
 ThreeStateUnit *  ThreeStateUnit_from_half_bit_aminoacid(char aa,CompMat * mat,RandomModel * rm,int gap,int ext)
 {
   ThreeStateUnit * tsu;
   register int i;
   Probability go;
   Probability ge;
-  Probability error;
+  Probability rnd_error;
 
   tsu = ThreeStateUnit_alloc();
   aa = toupper((int)aa);
@@ -520,11 +549,15 @@ ThreeStateUnit *  ThreeStateUnit_from_half_bit_aminoacid(char aa,CompMat * mat,R
     tsu->insert_emission[i] = rm->aminoacid[i];
   }
 
-  error = renormalise_Probability_array(tsu->match_emission,26);
+  rnd_error = renormalise_Probability_array(tsu->match_emission,26);
 
-  error = renormalise_Probability_array(tsu->insert_emission,26);
-  if( fabs(error) > 0.001 ) {
-    warn("Very bad problem - got a stupid error %f after renormalisation from the null model\n",error);
+  if( fabs(rnd_error) > 0.5 ) {
+    warn("Very bad problem - got a stupid error %.2f after renormalisation from the null model with the match state on aa %c\n",rnd_error,aa);
+  }
+
+  rnd_error = renormalise_Probability_array(tsu->insert_emission,26);
+  if( fabs(rnd_error) > 0.5 ) {
+    warn("Very bad problem - got a stupid error %.2f after renormalisation from the null model\n",rnd_error);
   }
 
 
@@ -569,7 +602,7 @@ ThreeStateUnit *  ThreeStateUnit_from_half_bit_aminoacid(char aa,CompMat * mat,R
  * Return [UNKN ]  Undocumented return value [ThreeStateModel *]
  *
  */
-# line 578 "threestatemodel.dy"
+# line 602 "threestatemodel.dy"
 ThreeStateModel * global_ThreeStateModel_from_half_bit_Sequence(Protein * pro,CompMat * mat,RandomModel * rm,int gap,int ext)
 {
 
@@ -620,7 +653,7 @@ ThreeStateModel * global_ThreeStateModel_from_half_bit_Sequence(Protein * pro,Co
  * Return [UNKN ]  Undocumented return value [char]
  *
  */
-# line 622 "threestatemodel.dy"
+# line 646 "threestatemodel.dy"
 char display_char_for_ThreeStateUnit(ThreeStateUnit * tsu) 
 {
   register int i;
@@ -648,7 +681,7 @@ char display_char_for_ThreeStateUnit(ThreeStateUnit * tsu)
  * Arg:        tsm [UNKN ] Undocumented argument [ThreeStateModel *]
  *
  */
-# line 644 "threestatemodel.dy"
+# line 668 "threestatemodel.dy"
 void display_char_in_ThreeStateModel(ThreeStateModel * tsm)
 {
   register int i;
@@ -668,7 +701,7 @@ void display_char_in_ThreeStateModel(ThreeStateModel * tsm)
  * Arg:         rm [UNKN ] Undocumented argument [RandomModel *]
  *
  */
-# line 657 "threestatemodel.dy"
+# line 681 "threestatemodel.dy"
 void fold_RandomModel_into_ThreeStateModel(ThreeStateModel * tsm,RandomModel * rm)
 {
   register int i;
@@ -702,7 +735,7 @@ void fold_RandomModel_into_ThreeStateModel(ThreeStateModel * tsm,RandomModel * r
  * Arg:        tsm [UNKN ] Undocumented argument [ThreeStateModel *]
  *
  */
-# line 685 "threestatemodel.dy"
+# line 709 "threestatemodel.dy"
 void add_sensible_start_end_global_for_HMMer(ThreeStateModel * tsm)
 {
   ThreeStateUnit * end;
@@ -734,7 +767,7 @@ void add_sensible_start_end_global_for_HMMer(ThreeStateModel * tsm)
  * Arg:        loop [UNKN ] Undocumented argument [Probability]
  *
  */
-# line 710 "threestatemodel.dy"
+# line 734 "threestatemodel.dy"
 void add_sensible_start_end_looping_for_HMMer(ThreeStateModel * tsm,Probability loop)
 {
   tsm->unit[0]->transition[TSM_START2MATCH] = loop * 0.5;
@@ -764,7 +797,7 @@ void add_sensible_start_end_looping_for_HMMer(ThreeStateModel * tsm,Probability
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 733 "threestatemodel.dy"
+# line 757 "threestatemodel.dy"
 void write_HMF_ThreeStateModel(ThreeStateModel * tsm,FILE * ofp)
 {
   int i;
@@ -791,7 +824,7 @@ void write_HMF_ThreeStateModel(ThreeStateModel * tsm,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 754 "threestatemodel.dy"
+# line 778 "threestatemodel.dy"
 void write_HMF_ThreeStateUnit(ThreeStateUnit * tsu,FILE * ofp)
 {
   fprintf(ofp,"UN %c\n",tsu->display_char);
@@ -815,7 +848,7 @@ void write_HMF_ThreeStateUnit(ThreeStateUnit * tsu,FILE * ofp)
  * Return [UNKN ]  Undocumented return value [ThreeStateModel *]
  *
  */
-# line 770 "threestatemodel.dy"
+# line 794 "threestatemodel.dy"
 ThreeStateModel * read_HMF_ThreeStateModel(FILE * ifp)
 {
   ThreeStateUnit * temp;
@@ -903,7 +936,7 @@ ThreeStateModel * read_HMF_ThreeStateModel(FILE * ifp)
  * Return [UNKN ]  Undocumented return value [ThreeStateUnit *]
  *
  */
-# line 853 "threestatemodel.dy"
+# line 877 "threestatemodel.dy"
 ThreeStateUnit * read_HMF_ThreeStateUnit(char * line,FILE * ifp)
 {
   ThreeStateUnit * out;
@@ -958,7 +991,7 @@ ThreeStateUnit * read_HMF_ThreeStateUnit(char * line,FILE * ifp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 901 "threestatemodel.dy"
+# line 925 "threestatemodel.dy"
 void write_HMMer_1_7_ascii_ThreeStateModel(ThreeStateModel * tsm,FILE * ofp)
 {
   register int i;
@@ -984,7 +1017,7 @@ void write_HMMer_1_7_ascii_ThreeStateModel(ThreeStateModel * tsm,FILE * ofp)
  * Arg:        ofp [UNKN ] Undocumented argument [FILE *]
  *
  */
-# line 920 "threestatemodel.dy"
+# line 944 "threestatemodel.dy"
 void write_HMMer_1_7_ascii_ThreeStateUnit(ThreeStateUnit * tsu,int no,FILE * ofp)
 {
   register int i;
@@ -1023,7 +1056,7 @@ void write_HMMer_1_7_ascii_ThreeStateUnit(ThreeStateUnit * tsu,int no,FILE * ofp
  * Return [UNKN ]  Undocumented return value [ThreeStateModel *]
  *
  */
-# line 953 "threestatemodel.dy"
+# line 977 "threestatemodel.dy"
 ThreeStateModel * read_HMMer_1_7_ascii_file(char * filename)
 {
   FILE * ifp;
@@ -1069,7 +1102,7 @@ ThreeStateModel * read_HMMer_1_7_ascii_file(char * filename)
  * Return [UNKN ]  Undocumented return value [ThreeStateModel *]
  *
  */
-# line 992 "threestatemodel.dy"
+# line 1016 "threestatemodel.dy"
 ThreeStateModel * convert_push_model_to_pull_model(ThreeStateModel * tsm)
 {
   register int i;
@@ -1100,7 +1133,7 @@ ThreeStateModel * convert_push_model_to_pull_model(ThreeStateModel * tsm)
  * Return [UNKN ]  Undocumented return value [ThreeStateUnit  *]
  *
  */
-# line 1015 "threestatemodel.dy"
+# line 1039 "threestatemodel.dy"
 ThreeStateUnit  * convert_push_unit_to_pull_unit(ThreeStateUnit * prev,ThreeStateUnit * this)
 {
   ThreeStateUnit * out;
@@ -1140,7 +1173,7 @@ ThreeStateUnit  * convert_push_unit_to_pull_unit(ThreeStateUnit * prev,ThreeStat
  * Return [UNKN ]  Undocumented return value [ThreeStateModel *]
  *
  */
-# line 1047 "threestatemodel.dy"
+# line 1071 "threestatemodel.dy"
 ThreeStateModel * read_HMMer_1_7_ascii(FILE * ifp) 
 {
   char buffer[MAXLINE];
@@ -1267,7 +1300,7 @@ ThreeStateModel * read_HMMer_1_7_ascii(FILE * ifp)
  * Return [UNKN ]  Undocumented return value [ThreeStateUnit *]
  *
  */
-# line 1166 "threestatemodel.dy"
+# line 1190 "threestatemodel.dy"
 ThreeStateUnit * read_HMMer_1_7_ThreeStateUnit(char * alphabet,FILE * ifp) 
 {
   char buffer[MAXLINE];
@@ -1487,7 +1520,7 @@ ThreeStateUnit * read_HMMer_1_7_ThreeStateUnit(char * alphabet,FILE * ifp)
  * Return [UNKN ]  Undocumented return value [ThreeStateUnit *]
  *
  */
-# line 1380 "threestatemodel.dy"
+# line 1404 "threestatemodel.dy"
 ThreeStateUnit * blank_ThreeStateUnit(void)
 {
   ThreeStateUnit * out;
@@ -1507,7 +1540,7 @@ ThreeStateUnit * blank_ThreeStateUnit(void)
 }
 
 
-# line 1445 "threestatemodel.c"
+# line 1476 "threestatemodel.c"
 /* Function:  hard_link_ThreeStateUnit(obj)
  *
  * Descrip:    Bumps up the reference count of the object
@@ -1550,6 +1583,9 @@ ThreeStateUnit * ThreeStateUnit_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* match_emission[ALPHABET_SIZE] is an array: no default possible */ 
     /* insert_emission[ALPHABET_SIZE] is an array: no default possible */ 
     /* transition[TRANSITION_LEN] is an array: no default possible */ 
@@ -1573,6 +1609,7 @@ ThreeStateUnit * ThreeStateUnit_alloc(void)
  */
 ThreeStateUnit * free_ThreeStateUnit(ThreeStateUnit * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1581,10 +1618,18 @@ ThreeStateUnit * free_ThreeStateUnit(ThreeStateUnit * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -1838,6 +1883,9 @@ ThreeStateModel * ThreeStateModel_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->name = NULL;    
     out->unit = NULL;    
     out->len = out->maxlen = 0;  
@@ -1864,6 +1912,7 @@ ThreeStateModel * ThreeStateModel_alloc(void)
  */
 ThreeStateModel * free_ThreeStateModel(ThreeStateModel * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -1873,10 +1922,18 @@ ThreeStateModel * free_ThreeStateModel(ThreeStateModel * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->name != NULL)   
       ckfree(obj->name);     
     if( obj->unit != NULL)   {  
@@ -1941,6 +1998,9 @@ ThreeStateScoreUnit * ThreeStateScoreUnit_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     /* match[ALPHABET_SIZE] is an array: no default possible */ 
     /* insert[ALPHABET_SIZE] is an array: no default possible */ 
     /* trans[TRANSITION_LEN] is an array: no default possible */ 
@@ -1963,6 +2023,7 @@ ThreeStateScoreUnit * ThreeStateScoreUnit_alloc(void)
  */
 ThreeStateScoreUnit * free_ThreeStateScoreUnit(ThreeStateScoreUnit * obj) 
 {
+    int return_early = 0;    
 
 
     if( obj == NULL) {  
@@ -1971,10 +2032,18 @@ ThreeStateScoreUnit * free_ThreeStateScoreUnit(ThreeStateScoreUnit * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
 
 
     ckfree(obj); 
@@ -2228,6 +2297,9 @@ ThreeStateScore * ThreeStateScore_alloc(void)
       return NULL;  /* calling function should respond! */ 
       }  
     out->dynamite_hard_link = 1; 
+#ifdef PTHREAD   
+    pthread_mutex_init(&(out->dynamite_mutex),NULL);     
+#endif   
     out->unit = NULL;    
     out->len = out->maxlen = 0;  
     out->name = NULL;    
@@ -2251,6 +2323,7 @@ ThreeStateScore * ThreeStateScore_alloc(void)
  */
 ThreeStateScore * free_ThreeStateScore(ThreeStateScore * obj) 
 {
+    int return_early = 0;    
     int i;   
 
 
@@ -2260,10 +2333,18 @@ ThreeStateScore * free_ThreeStateScore(ThreeStateScore * obj)
       }  
 
 
+#ifdef PTHREAD   
+    assert(pthread_mutex_lock(&(obj->dynamite_mutex)) == 0); 
+#endif   
     if( obj->dynamite_hard_link > 1)     {  
+      return_early = 1;  
       obj->dynamite_hard_link--; 
-      return NULL;   
       }  
+#ifdef PTHREAD   
+    assert(pthread_mutex_unlock(&(obj->dynamite_mutex)) == 0);   
+#endif   
+    if( return_early == 1)   
+      return NULL;   
     if( obj->unit != NULL)   {  
       for(i=0;i<obj->len;i++)    {  
         if( obj->unit[i] != NULL)    
