@@ -34,14 +34,14 @@ enum tsm_trans_type {
 extern char * std_alphabet;
 #define ALPHABET_SIZE 26
 
-enum TSM_StartEndMode {
+typedef enum TSM_StartEndMode {
   TSM_unknown = 26,
   TSM_default,
   TSM_local,
   TSM_global,
   TSM_wing,
   TSM_endbiased
-};
+} TSM_StartEndMode;
 
 
 /* Object ThreeStateUnit
@@ -54,6 +54,9 @@ enum TSM_StartEndMode {
  */
 struct Wise2_ThreeStateUnit {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Probability match_emission[ALPHABET_SIZE];   
     Probability insert_emission[ALPHABET_SIZE];  
     Probability transition[TRANSITION_LEN];  
@@ -76,6 +79,9 @@ typedef struct Wise2_ThreeStateUnit Wise2_ThreeStateUnit;
  */
 struct Wise2_ThreeStateModel {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     char * name;    /*  name of the model */ 
     ThreeStateUnit ** unit; /*  the actuall three state probs and emissions */ 
     int len;/* len for above unit  */ 
@@ -102,6 +108,9 @@ typedef struct Wise2_ThreeStateModel Wise2_ThreeStateModel;
  */
 struct Wise2_ThreeStateScoreUnit {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     Score match[ALPHABET_SIZE];  
     Score insert[ALPHABET_SIZE];     
     Score trans[TRANSITION_LEN];     
@@ -124,6 +133,9 @@ typedef struct Wise2_ThreeStateScoreUnit Wise2_ThreeStateScoreUnit;
  */
 struct Wise2_ThreeStateScore {  
     int dynamite_hard_link;  
+#ifdef PTHREAD   
+    pthread_mutex_t dynamite_mutex;  
+#endif   
     ThreeStateScoreUnit ** unit;     
     int len;/* len for above unit  */ 
     int maxlen; /* maxlen for above unit */ 
@@ -147,6 +159,21 @@ typedef struct Wise2_ThreeStateScore Wise2_ThreeStateScore;
 
 
 
+/* Function:  information_from_ThreeStateUnit(tsu,rm)
+ *
+ * Descrip:     Gets the information content (K-L divergence) vs a background for a position
+ *
+ *
+ * Arg:        tsu [UNKN ] Undocumented argument [ThreeStateUnit *]
+ * Arg:         rm [UNKN ] Undocumented argument [RandomModel *]
+ *
+ * Return [UNKN ]  Undocumented return value [double]
+ *
+ */
+double Wise2_information_from_ThreeStateUnit(ThreeStateUnit * tsu,RandomModel * rm);
+#define information_from_ThreeStateUnit Wise2_information_from_ThreeStateUnit
+
+
 /* Function:  threestatemodel_mode_from_string(mode)
  *
  * Descrip:    gets out the mode from a string
@@ -167,12 +194,12 @@ int Wise2_threestatemodel_mode_from_string(char * mode);
  *
  *
  * Arg:                  tsm [UNKN ] Undocumented argument [ThreeStateModel *]
- * Arg:                 mode [UNKN ] Undocumented argument [int]
+ * Arg:                 mode [UNKN ] Undocumented argument [TSM_StartEndMode]
  * Arg:          wing_length [UNKN ] Undocumented argument [int]
  * Arg:        internal_bias [UNKN ] Undocumented argument [Probability]
  *
  */
-void Wise2_set_startend_policy_ThreeStateModel(ThreeStateModel * tsm,int mode,int wing_length,Probability internal_bias);
+void Wise2_set_startend_policy_ThreeStateModel(ThreeStateModel * tsm,TSM_StartEndMode mode,int wing_length,Probability internal_bias);
 #define set_startend_policy_ThreeStateModel Wise2_set_startend_policy_ThreeStateModel
 
 
@@ -783,30 +810,30 @@ boolean Wise2_replace_name_ThreeStateModel(ThreeStateModel * obj,char * name);
 #define replace_name_ThreeStateModel Wise2_replace_name_ThreeStateModel
 char * Wise2_access_name_ThreeStateModel(ThreeStateModel * obj);
 #define access_name_ThreeStateModel Wise2_access_name_ThreeStateModel
-ThreeStateUnit * Wise2_access_unit_ThreeStateModel(ThreeStateModel * obj,int i);
-#define access_unit_ThreeStateModel Wise2_access_unit_ThreeStateModel
-RandomModel * Wise2_access_rm_ThreeStateModel(ThreeStateModel * obj);
-#define access_rm_ThreeStateModel Wise2_access_rm_ThreeStateModel
 int Wise2_length_unit_ThreeStateModel(ThreeStateModel * obj);
 #define length_unit_ThreeStateModel Wise2_length_unit_ThreeStateModel
 boolean Wise2_replace_alphabet_ThreeStateModel(ThreeStateModel * obj,char * alphabet);
 #define replace_alphabet_ThreeStateModel Wise2_replace_alphabet_ThreeStateModel
-boolean Wise2_replace_rm_ThreeStateModel(ThreeStateModel * obj,RandomModel * rm);
-#define replace_rm_ThreeStateModel Wise2_replace_rm_ThreeStateModel
+char Wise2_access_display_char_ThreeStateUnit(ThreeStateUnit * obj);
+#define access_display_char_ThreeStateUnit Wise2_access_display_char_ThreeStateUnit
 char * Wise2_access_alphabet_ThreeStateModel(ThreeStateModel * obj);
 #define access_alphabet_ThreeStateModel Wise2_access_alphabet_ThreeStateModel
 boolean Wise2_replace_display_char_ThreeStateUnit(ThreeStateUnit * obj,char display_char);
 #define replace_display_char_ThreeStateUnit Wise2_replace_display_char_ThreeStateUnit
 boolean Wise2_replace_accession_ThreeStateModel(ThreeStateModel * obj,char * accession);
 #define replace_accession_ThreeStateModel Wise2_replace_accession_ThreeStateModel
-char Wise2_access_display_char_ThreeStateUnit(ThreeStateUnit * obj);
-#define access_display_char_ThreeStateUnit Wise2_access_display_char_ThreeStateUnit
+RandomModel * Wise2_access_rm_ThreeStateModel(ThreeStateModel * obj);
+#define access_rm_ThreeStateModel Wise2_access_rm_ThreeStateModel
 char * Wise2_access_accession_ThreeStateModel(ThreeStateModel * obj);
 #define access_accession_ThreeStateModel Wise2_access_accession_ThreeStateModel
-double Wise2_access_threshold_ThreeStateModel(ThreeStateModel * obj);
-#define access_threshold_ThreeStateModel Wise2_access_threshold_ThreeStateModel
+ThreeStateUnit * Wise2_access_unit_ThreeStateModel(ThreeStateModel * obj,int i);
+#define access_unit_ThreeStateModel Wise2_access_unit_ThreeStateModel
 boolean Wise2_replace_threshold_ThreeStateModel(ThreeStateModel * obj,double threshold);
 #define replace_threshold_ThreeStateModel Wise2_replace_threshold_ThreeStateModel
+boolean Wise2_replace_rm_ThreeStateModel(ThreeStateModel * obj,RandomModel * rm);
+#define replace_rm_ThreeStateModel Wise2_replace_rm_ThreeStateModel
+double Wise2_access_threshold_ThreeStateModel(ThreeStateModel * obj);
+#define access_threshold_ThreeStateModel Wise2_access_threshold_ThreeStateModel
 ThreeStateScoreUnit * Wise2_ThreeStateScoreUnit_from_ThreeStateUnit(ThreeStateUnit * prev,ThreeStateUnit * tsu);
 #define ThreeStateScoreUnit_from_ThreeStateUnit Wise2_ThreeStateScoreUnit_from_ThreeStateUnit
 void Wise2_show_ThreeStateUnit(ThreeStateUnit * tsu,FILE * ofp);
